@@ -39,6 +39,7 @@ import org.apache.kudu.Type
 import org.apache.kudu.client.KuduPartitioner.KuduPartitionerBuilder
 import org.apache.kudu.client.PartitionSchema.HashBucketSchema
 import org.apache.kudu.client.PartitionSchema.RangeSchema
+import org.apache.kudu.client.PartitionSchema.RangeWithHashSchema
 import org.apache.yetus.audience.InterfaceAudience
 import org.apache.yetus.audience.InterfaceStability
 
@@ -69,6 +70,7 @@ object TableMetadata {
         .setCompression(col.getCompressionAlgorithm.toString)
         .setBlockSize(col.getDesiredBlockSize)
         .setComment(col.getComment)
+        .setIsAutoIncrementing(col.isAutoIncrementing)
       if (col.getTypeAttributes != null) {
         builder.setTypeAttributes(getTypeAttributesMetadata(col))
       }
@@ -121,13 +123,49 @@ object TableMetadata {
   private def getPartitionSchemaMetadata(table: KuduTable): PartitionSchemaMetadataPB = {
     val hashPartitions = getHashPartitionsMetadata(table)
     val rangePartitions = getRangePartitionMetadata(table)
+    val rangeAndHashPartitions = getRangeAndHashPartitionsMetadata(table)
     PartitionSchemaMetadataPB
       .newBuilder()
       .addAllHashPartitions(hashPartitions.asJava)
       .setRangePartitions(rangePartitions)
+      .addAllRangeAndHashPartitions(rangeAndHashPartitions.asJava)
       .build()
   }
 
+  private def getRangeAndHashPartitionsMetadata(
+      table: KuduTable): Seq[RangeAndHashPartitionMetadataPB] = {
+    val tableSchema = table.getSchema
+    val partitionSchema = table.getPartitionSchema
+    val rangeColumnNames = partitionSchema.getRangeSchema.getColumnIds.asScala.map { id =>
+      getColumnById(tableSchema, id).getName
+    }
+    partitionSchema.getRangesWithHashSchemas.asScala.map { rhs =>
+      val hashSchemas = rhs.hashSchemas.asScala.map { hs =>
+        val hashColumnNames = hs.getColumnIds.asScala.map { id =>
+          getColumnById(tableSchema, id).getName
+        }
+        HashPartitionMetadataPB
+          .newBuilder()
+          .addAllColumnNames(hashColumnNames.asJava)
+          .setNumBuckets(hs.getNumBuckets)
+          .setSeed(hs.getSeed)
+          .build()
+      }
+      val upperValues = getBoundValues(rhs.upperBound, rangeColumnNames, tableSchema)
+      val lowerValues = getBoundValues(rhs.lowerBound, rangeColumnNames, tableSchema)
+      val bounds = RangeBoundsMetadataPB
+        .newBuilder()
+        .addAllUpperBounds(upperValues.asJava)
+        .addAllLowerBounds(lowerValues.asJava)
+        .build()
+      RangeAndHashPartitionMetadataPB
+        .newBuilder()
+        .setBounds(bounds)
+        .addAllHashPartitions(hashSchemas.asJava)
+        .build()
+    }
+  }
+
   private def getHashPartitionsMetadata(table: KuduTable): Seq[HashPartitionMetadataPB] = {
     val tableSchema = table.getSchema
     val partitionSchema = table.getPartitionSchema
@@ -152,7 +190,7 @@ object TableMetadata {
     }
 
     val bounds = table
-      .getRangePartitions(table.getAsyncClient.getDefaultOperationTimeoutMs)
+      .getRangePartitionsWithTableHashSchema(table.getAsyncClient.getDefaultOperationTimeoutMs)
       .asScala
       .map { p =>
         val lowerValues = getBoundValues(p.getDecodedRangeKeyStart(table), columnNames, tableSchema)
@@ -199,36 +237,50 @@ object TableMetadata {
   }
 
   def getKuduSchema(metadata: TableMetadataPB): Schema = {
-    val columns = metadata.getColumnsList.asScala.map { col =>
-      val colType = Type.getTypeForName(col.getType)
-      val builder = new ColumnSchemaBuilder(col.getName, colType)
-        .key(col.getIsKey)
-        .nullable(col.getIsNullable)
-        .encoding(Encoding.valueOf(col.getEncoding))
-        .compressionAlgorithm(CompressionAlgorithm.valueOf(col.getCompression))
-        .desiredBlockSize(col.getBlockSize)
-        .comment(col.getComment)
-
-      if (col.hasDefaultValue) {
-        val value = valueFromString(col.getDefaultValue.getValue, colType)
-        builder.defaultValue(value)
+    var IsAutoIncrementingPresent = false
+    metadata.getColumnsList.asScala.foreach { col =>
+      if (col.getIsAutoIncrementing) {
+        IsAutoIncrementingPresent = true
       }
-
-      if (col.hasTypeAttributes) {
-        val attributes = col.getTypeAttributes
-        builder.typeAttributes(
-          new ColumnTypeAttributesBuilder()
-            .precision(attributes.getPrecision)
-            .scale(attributes.getScale)
-            .length(attributes.getLength)
-            .build()
-        )
-      }
-      builder.build()
     }
+    val columns = new util.ArrayList[ColumnSchema]()
+    val colIds = new util.ArrayList[Integer]()
     val toId = metadata.getColumnIdsMap.asScala
-    val colIds = metadata.getColumnsList.asScala.map(_.getName).map(toId)
-    new Schema(columns.asJava, colIds.asJava)
+    metadata.getColumnsList.asScala.foreach { col =>
+      if (!col.getIsAutoIncrementing) {
+        val colType = Type.getTypeForName(col.getType)
+        val builder = new ColumnSchemaBuilder(col.getName, colType)
+          .nullable(col.getIsNullable)
+          .encoding(Encoding.valueOf(col.getEncoding))
+          .compressionAlgorithm(CompressionAlgorithm.valueOf(col.getCompression))
+          .desiredBlockSize(col.getBlockSize)
+          .comment(col.getComment)
+        if (IsAutoIncrementingPresent) {
+          builder.nonUniqueKey(col.getIsKey)
+        } else {
+          builder.key(col.getIsKey)
+        }
+
+        if (col.hasDefaultValue) {
+          val value = valueFromString(col.getDefaultValue.getValue, colType)
+          builder.defaultValue(value)
+        }
+
+        if (col.hasTypeAttributes) {
+          val attributes = col.getTypeAttributes
+          builder.typeAttributes(
+            new ColumnTypeAttributesBuilder()
+              .precision(attributes.getPrecision)
+              .scale(attributes.getScale)
+              .length(attributes.getLength)
+              .build()
+          )
+        }
+        colIds.add(toId(col.getName))
+        columns.add(builder.build())
+      }
+    }
+    new Schema(columns, colIds)
   }
 
   private def getValue(row: PartialRow, columnName: String, colType: Type): Any = {
@@ -355,6 +407,22 @@ object TableMetadata {
     }
   }
 
+  def getRangeBoundsPartialRowsWithHashSchemas(
+      metadata: TableMetadataPB): Seq[RangeWithHashSchema] = {
+    val schema = getKuduSchema(metadata)
+    metadata.getPartitions.getRangeAndHashPartitionsList.asScala.map { rhp =>
+      val hashSchemas = rhp.getHashPartitionsList.asScala.map { hp =>
+        val colIds = hp.getColumnNamesList.asScala.map { name =>
+          new Integer(schema.getColumnIndex(schema.getColumnId(name)))
+        }
+        new HashBucketSchema(colIds.asJava, hp.getNumBuckets, hp.getSeed)
+      }
+      val lower = getPartialRow(rhp.getBounds.getLowerBoundsList.asScala, schema)
+      val upper = getPartialRow(rhp.getBounds.getUpperBoundsList.asScala, schema)
+      new RangeWithHashSchema(lower, upper, hashSchemas.asJava)
+    }
+  }
+
   def getPartitionSchema(metadata: TableMetadataPB): PartitionSchema = {
     val colNameToId = metadata.getColumnIdsMap.asScala
     val schema = getKuduSchema(metadata)
@@ -365,6 +433,16 @@ object TableMetadata {
       val colIds = hp.getColumnNamesList.asScala.map(colNameToId)
       new HashBucketSchema(colIds.asJava, hp.getNumBuckets, hp.getSeed)
     }
-    new PartitionSchema(rangeSchema, hashSchemas.asJava, schema)
+    val rangesWithHashSchemas =
+      metadata.getPartitions.getRangeAndHashPartitionsList.asScala.map { rhp =>
+        val rangeHashSchemas = rhp.getHashPartitionsList.asScala.map { hp =>
+          val colIds = hp.getColumnNamesList.asScala.map(colNameToId)
+          new HashBucketSchema(colIds.asJava, hp.getNumBuckets, hp.getSeed)
+        }
+        val lower = getPartialRow(rhp.getBounds.getLowerBoundsList.asScala, schema)
+        val upper = getPartialRow(rhp.getBounds.getUpperBoundsList.asScala, schema)
+        new RangeWithHashSchema(lower, upper, rangeHashSchemas.asJava)
+      }
+    new PartitionSchema(rangeSchema, hashSchemas.asJava, rangesWithHashSchemas.asJava, schema)
   }
 }
