@@ -38,13 +38,19 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
+import java.util.UUID;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import com.google.protobuf.Message;
 import com.google.protobuf.UnsafeByteOperations;
 import com.stumbleupon.async.Callback;
 import com.stumbleupon.async.Deferred;
+import io.netty.util.HashedWheelTimer;
+import io.netty.util.Timeout;
+import io.netty.util.TimerTask;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.yetus.audience.InterfaceStability;
 import org.slf4j.Logger;
@@ -284,6 +290,10 @@ public final class AsyncKuduScanner {
 
   final long scanRequestTimeout;
 
+  private String queryId;
+
+  private Timeout keepAliveTimeout;
+
   /**
    * UUID of the tserver which the scanner is bound with. The following scans of
    * this scanner will be sent to the tserver.
@@ -405,6 +415,27 @@ public final class AsyncKuduScanner {
     }
   }
 
+  AsyncKuduScanner(AsyncKuduClient client, KuduTable table, List<String> projectedNames,
+                   List<Integer> projectedIndexes, ReadMode readMode, boolean isFaultTolerant,
+                   long scanRequestTimeout,
+                   Map<String, KuduPredicate> predicates, long limit,
+                   boolean cacheBlocks, boolean prefetching,
+                   byte[] startPrimaryKey, byte[] endPrimaryKey,
+                   long startTimestamp, long htTimestamp,
+                   int batchSizeBytes, PartitionPruner pruner,
+                   ReplicaSelection replicaSelection, long keepAlivePeriodMs, String queryId) {
+    this(
+        client, table, projectedNames, projectedIndexes, readMode, isFaultTolerant,
+        scanRequestTimeout, predicates, limit, cacheBlocks, prefetching, startPrimaryKey,
+        endPrimaryKey, startTimestamp, htTimestamp, batchSizeBytes,
+        pruner, replicaSelection, keepAlivePeriodMs);
+    if (queryId.isEmpty()) {
+      this.queryId = UUID.randomUUID().toString().replace("-", "");
+    } else {
+      this.queryId = queryId;
+    }
+  }
+
   /**
    * Generates and returns a ColumnSchema for the virtual IS_DELETED column.
    * The column name is generated to ensure there is never a collision.
@@ -453,7 +484,11 @@ public final class AsyncKuduScanner {
    * @return true if there might be more data to scan, else false
    */
   public boolean hasMoreRows() {
-    return this.canRequestMore || cachedPrefetcherDeferred.get() != null;
+    boolean hasMore = this.canRequestMore || cachedPrefetcherDeferred.get() != null;
+    if (!hasMore) {
+      stopKeepAlivePeriodically();
+    }
+    return hasMore;
   }
 
   /**
@@ -813,6 +848,7 @@ public final class AsyncKuduScanner {
     if (closed) {
       return Deferred.fromResult(null);
     }
+    stopKeepAlivePeriodically();
     return client.closeScanner(this).addCallback(closedCallback()); // TODO errBack ?
   }
 
@@ -936,6 +972,41 @@ public final class AsyncKuduScanner {
     return client.keepAlive(this);
   }
 
+  /**
+   * Package-private access point for {@link AsyncKuduScanner}s to keep themselves
+   * alive on tablet servers by sending keep-alive requests periodically.
+   * @param keepAliveIntervalMS the interval of sending keep-alive requests.
+   * @return true if starting keep-alive timer successfully.
+   */
+  boolean startKeepAlivePeriodically(int keepAliveIntervalMS) {
+    if (closed) {
+      return false;
+    }
+    final class KeepAliveTimer implements TimerTask {
+      @Override
+      public void run(final Timeout timeout) {
+        keepAlive();
+        keepAliveTimeout = AsyncKuduClient.newTimeout(client.getTimer(), this, keepAliveIntervalMS);
+      }
+    }
+
+    keepAliveTimeout =
+        AsyncKuduClient.newTimeout(client.getTimer(), new KeepAliveTimer(), keepAliveIntervalMS);
+    return true;
+  }
+
+  /**
+   * Package-private access point for {@link AsyncKuduScanner}s to stop
+   * keep-alive timer.
+   * @return true if stopping keep-alive timer successfully.
+   */
+  boolean stopKeepAlivePeriodically() {
+    if (keepAliveTimeout != null) {
+      return keepAliveTimeout.cancel();
+    }
+    return true;
+  }
+
   /**
    * Returns an RPC to fetch the next rows.
    */
@@ -1226,6 +1297,7 @@ public final class AsyncKuduScanner {
         default:
           throw new RuntimeException("unreachable!");
       }
+      builder.setQueryId(UnsafeByteOperations.unsafeWrap(queryId.getBytes(UTF_8)));
 
       return builder.build();
     }
@@ -1338,7 +1410,7 @@ public final class AsyncKuduScanner {
           client, table, projectedColumnNames, projectedColumnIndexes, readMode, isFaultTolerant,
           scanRequestTimeout, predicates, limit, cacheBlocks, prefetching, lowerBoundPrimaryKey,
           upperBoundPrimaryKey, startTimestamp, htTimestamp, batchSizeBytes,
-          PartitionPruner.create(this), replicaSelection, keepAlivePeriodMs);
+          PartitionPruner.create(this), replicaSelection, keepAlivePeriodMs, queryId);
     }
   }
 }
