@@ -185,11 +185,23 @@ public class KuduTable {
   /**
    * Get a new upsert configured with this table's schema. The returned object should not be reused.
    * @return an upsert with this table's schema
+   * @throws UnsupportedOperationException if the table has auto-incrementing column
    */
   public Upsert newUpsert() {
     return new Upsert(this);
   }
 
+  /**
+   * Get a new upsert ignore configured with this table's schema. The operation ignores errors of
+   * updating immutable cells in a row. This is useful when upserting rows in a table with immutable
+   * columns.
+   * @return an upsert with this table's schema
+   * @throws UnsupportedOperationException if the table has auto-incrementing column
+   */
+  public UpsertIgnore newUpsertIgnore() {
+    return new UpsertIgnore(this);
+  }
+
   /**
    * Get a new insert ignore configured with this table's schema. An insert ignore will
    * ignore duplicate row errors. This is useful when the same insert may be sent multiple times.
@@ -202,7 +214,8 @@ public class KuduTable {
 
   /**
    * Get a new update ignore configured with this table's schema. An update ignore will
-   * ignore missing row errors. This is useful to update a row only if it exists.
+   * ignore missing row errors and updating on immutable columns errors. This is useful to
+   * update a row only if it exists, or update a row with immutable columns.
    * The returned object should not be reused.
    * @return an update ignore with this table's schema
    */
@@ -303,7 +316,27 @@ public class KuduTable {
     List<Partition> rangePartitions = getRangePartitions(timeout);
     List<String> formattedPartitions = new ArrayList<>();
     for (Partition partition : rangePartitions) {
-      formattedPartitions.add(partition.formatRangePartition(this));
+      formattedPartitions.add(partition.formatRangePartition(this, false));
+    }
+    return formattedPartitions;
+  }
+
+  /**
+   * Retrieves a formatted representation of this table's range partitions along
+   * with hash schema output for each range. The range partitions are returned
+   * in sorted order by value and contain no duplicates.
+   *
+   * @param timeout the timeout of the operation
+   * @return a list of the formatted range partitions with hash schema for each
+   */
+  @InterfaceAudience.LimitedPrivate("Impala")
+  @InterfaceStability.Unstable
+  public List<String> getFormattedRangePartitionsWithHashSchema(long timeout)
+      throws Exception {
+    List<Partition> rangePartitions = getRangePartitions(timeout);
+    List<String> formattedPartitions = new ArrayList<>();
+    for (Partition partition : rangePartitions) {
+      formattedPartitions.add(partition.formatRangePartition(this, true));
     }
     return formattedPartitions;
   }
@@ -320,6 +353,32 @@ public class KuduTable {
   public List<Partition> getRangePartitions(long timeout) throws Exception {
     // TODO: This could be moved into the RangeSchemaPB returned from server
     // to avoid an extra call to get the range partitions.
+    return getRangePartitionsHelper(timeout, false);
+  }
+
+  /**
+   * Only retrieves this table's range partitions that contain the table wide hash schema. The
+   * range partitions will be returned in sorted order by value, and will contain no duplicates.
+   *
+   * @param timeout the timeout of the operation
+   * @return a list of the formatted range partitions
+   */
+  @InterfaceAudience.Private
+  @InterfaceStability.Unstable
+  public List<Partition> getRangePartitionsWithTableHashSchema(long timeout) throws Exception {
+    return getRangePartitionsHelper(timeout, true);
+  }
+
+  /**
+   * Helper method that retrieves the table's range partitions. If onlyTableHashSchema is evaluated
+   * to true, then only range partitions that have the table wide hash schema will be returned. The
+   * range partitions will be returned in sorted order by value and will contain no duplicates.
+   * @param timeout the timeout of the operation
+   * @param onlyTableHashSchema whether to filter out the partitions with custom hash schema
+   * @return a list of the formatted range partitions
+   */
+  private List<Partition> getRangePartitionsHelper(long timeout,
+                                                   boolean onlyTableHashSchema) throws Exception {
     List<Partition> rangePartitions = new ArrayList<>();
     List<KuduScanToken> scanTokens = new KuduScanToken.KuduScanTokenBuilder(client, this)
         .setTimeout(timeout)
@@ -331,6 +390,12 @@ public class KuduTable {
       if (!Iterators.all(partition.getHashBuckets().iterator(), Predicates.equalTo(0))) {
         continue;
       }
+      // If onlyTableHashSchema is true, filter out any partitions
+      // that are part of a range that contains a custom hash schema.
+      if (onlyTableHashSchema && partitionSchema.getHashSchemaForRange(partition.rangeKeyStart) !=
+          partitionSchema.getHashBucketSchemas()) {
+        continue;
+      }
       rangePartitions.add(partition);
     }
     return rangePartitions;
