@@ -32,15 +32,14 @@ import java.net.InetAddress;
 import java.net.InetSocketAddress;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
-import java.security.PrivilegedExceptionAction;
 import java.security.cert.Certificate;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.Callable;
+import java.util.concurrent.CompletionException;
 import javax.net.ssl.SSLEngine;
 import javax.net.ssl.SSLException;
 import javax.net.ssl.SSLPeerUnverifiedException;
@@ -77,6 +76,7 @@ import org.ietf.jgss.GSSException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.kudu.client.internals.SecurityManagerCompatibility;
 import org.apache.kudu.rpc.RpcHeader;
 import org.apache.kudu.rpc.RpcHeader.AuthenticationTypePB;
 import org.apache.kudu.rpc.RpcHeader.NegotiatePB;
@@ -952,31 +952,34 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
   private byte[] evaluateChallenge(final byte[] challenge)
       throws SaslException, NonRecoverableException {
     try {
-      return Subject.doAs(securityContext.getSubject(),
-          new PrivilegedExceptionAction<byte[]>() {
+      return SecurityManagerCompatibility.get().callAs(securityContext.getSubject(),
+          new Callable<byte[]>() {
             @Override
-            public byte[] run() throws SaslException {
+            public byte[] call() throws SaslException {
               return saslClient.evaluateChallenge(challenge);
             }
           });
-    } catch (PrivilegedActionException e) {
-      // This cast is safe because the action above only throws checked SaslException.
-      SaslException saslException = (SaslException) e.getCause();
+    } catch (RuntimeException e) {
+      if (e.getCause() instanceof PrivilegedActionException) {
+        // This cast is safe because the action above only throws checked SaslException.
+        SaslException saslException = (SaslException) e.getCause().getCause();
 
-      // TODO(KUDU-2121): We should never get to this point if the client does not have
-      // Kerberos credentials, but it seems that on certain platforms it can happen.
-      // So, we try and determine whether the evaluateChallenge failed due to missing
-      // credentials, and return a nicer error message if so.
-      Throwable cause = saslException.getCause();
-      if (cause instanceof GSSException &&
-          ((GSSException) cause).getMajor() == GSSException.NO_CRED) {
-        throw new NonRecoverableException(
-            Status.ConfigurationError(
-                "Server requires Kerberos, but this client is not authenticated " +
-                "(missing or expired TGT)"),
-            saslException);
+        // TODO(KUDU-2121): We should never get to this point if the client does not have
+        // Kerberos credentials, but it seems that on certain platforms it can happen.
+        // So, we try and determine whether the evaluateChallenge failed due to missing
+        // credentials, and return a nicer error message if so.
+        Throwable cause = saslException.getCause();
+        if (cause instanceof GSSException &&
+            ((GSSException) cause).getMajor() == GSSException.NO_CRED) {
+          throw new NonRecoverableException(
+              Status.ConfigurationError(
+                  "Server requires Kerberos, but this client is not authenticated " +
+                      "(missing or expired TGT)"),
+              saslException);
+        }
+        throw saslException;
       }
-      throw saslException;
+      throw e;
     }
   }
 
@@ -1038,7 +1041,7 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
     }
 
     void resetAdded() {
-      Field addedField = AccessController.doPrivileged((PrivilegedAction<Field>) () -> {
+      Field addedField = SecurityManagerCompatibility.get().doPrivileged(() -> {
         try {
           Class<?> c = ChannelHandlerAdapter.class;
           Field added = c.getDeclaredField("added");
