@@ -36,6 +36,7 @@ import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.kudu.ColumnSchema;
 import org.apache.kudu.ColumnTypeAttributes;
 import org.apache.kudu.Common;
+import org.apache.kudu.RowOperations;
 import org.apache.kudu.Schema;
 import org.apache.kudu.Type;
 import org.apache.kudu.util.DateUtil;
@@ -106,6 +107,8 @@ public class ProtobufHelper {
         .setType(column.getWireType())
         .setIsKey(column.isKey())
         .setIsNullable(column.isNullable())
+        .setImmutable(column.isImmutable())
+        .setIsAutoIncrementing(column.isAutoIncrementing())
         .setCfileBlockSize(column.getDesiredBlockSize());
 
     if (!flags.contains(SchemaPBConversionFlags.SCHEMA_PB_WITHOUT_ID) && colId >= 0) {
@@ -148,25 +151,47 @@ public class ProtobufHelper {
   }
 
   public static ColumnSchema pbToColumnSchema(Common.ColumnSchemaPB pb) {
+    return pbToColumnSchema(pb, true);
+  }
+
+  public static ColumnSchema pbToColumnSchema(Common.ColumnSchemaPB pb,
+      boolean isKeyUnique) {
+    ColumnSchema.Encoding encoding = ColumnSchema.Encoding.valueOf(pb.getEncoding().name());
+    ColumnSchema.CompressionAlgorithm compressionAlgorithm =
+        ColumnSchema.CompressionAlgorithm.valueOf(pb.getCompression().name());
+    int desiredBlockSize = pb.getCfileBlockSize();
+
+    if (pb.getIsAutoIncrementing()) {
+      // Set encoding, compression algorithm, block size and comment from 'pb' parameter
+      return new ColumnSchema.AutoIncrementingColumnSchemaBuilder()
+                             .encoding(encoding)
+                             .compressionAlgorithm(compressionAlgorithm)
+                             .desiredBlockSize(desiredBlockSize)
+                             .comment(pb.getComment())
+                             .build();
+    }
+
     Type type = Type.getTypeForDataType(pb.getType());
     ColumnTypeAttributes typeAttributes = pb.hasTypeAttributes() ?
         pbToColumnTypeAttributes(pb.getTypeAttributes()) : null;
     Object defaultValue = pb.hasWriteDefaultValue() ?
         byteStringToObject(type, typeAttributes, pb.getWriteDefaultValue()) : null;
-    ColumnSchema.Encoding encoding = ColumnSchema.Encoding.valueOf(pb.getEncoding().name());
-    ColumnSchema.CompressionAlgorithm compressionAlgorithm =
-        ColumnSchema.CompressionAlgorithm.valueOf(pb.getCompression().name());
-    int desiredBlockSize = pb.getCfileBlockSize();
-    return new ColumnSchema.ColumnSchemaBuilder(pb.getName(), type)
-                           .key(pb.getIsKey())
-                           .nullable(pb.getIsNullable())
-                           .defaultValue(defaultValue)
-                           .encoding(encoding)
-                           .compressionAlgorithm(compressionAlgorithm)
-                           .desiredBlockSize(desiredBlockSize)
-                           .typeAttributes(typeAttributes)
-                           .comment(pb.getComment())
-                           .build();
+    ColumnSchema.ColumnSchemaBuilder csb =
+        new ColumnSchema.ColumnSchemaBuilder(pb.getName(), type);
+    if (pb.getIsKey() && isKeyUnique) {
+      csb.key(true);
+    } else {
+      csb.nonUniqueKey(pb.getIsKey());
+    }
+    return csb.nullable(pb.getIsNullable())
+              .immutable(pb.getImmutable())
+              .defaultValue(defaultValue)
+              .encoding(encoding)
+              .compressionAlgorithm(compressionAlgorithm)
+              .desiredBlockSize(desiredBlockSize)
+              .typeAttributes(typeAttributes)
+              .comment(pb.getComment())
+              .build();
   }
 
   public static ColumnTypeAttributes pbToColumnTypeAttributes(Common.ColumnTypeAttributesPB pb) {
@@ -185,10 +210,22 @@ public class ProtobufHelper {
   }
 
   public static Schema pbToSchema(Common.SchemaPB schema) {
+    // Since ColumnSchema.keyUnique in run-time structures is not persistent in Kudu
+    // server, we need to find if the table has auto-incrementing column first, and set
+    // all key columns as non unique key columns if the table has auto-incrementing
+    // column.
+    boolean hasAutoIncrementing = false;
+    for (Common.ColumnSchemaPB columnPb : schema.getColumnsList()) {
+      if (columnPb.getIsAutoIncrementing()) {
+        hasAutoIncrementing = true;
+        break;
+      }
+    }
     List<ColumnSchema> columns = new ArrayList<>(schema.getColumnsCount());
     List<Integer> columnIds = new ArrayList<>(schema.getColumnsCount());
     for (Common.ColumnSchemaPB columnPb : schema.getColumnsList()) {
-      columns.add(pbToColumnSchema(columnPb));
+      // Key is not unique if hasAutoIncrementing is true.
+      columns.add(pbToColumnSchema(columnPb, !hasAutoIncrementing));
       int id = columnPb.getId();
       if (id < 0) {
         throw new IllegalArgumentException("Illegal column ID: " + id);
@@ -211,7 +248,7 @@ public class ProtobufHelper {
     ImmutableList.Builder<PartitionSchema.HashBucketSchema> hashSchemas = ImmutableList.builder();
 
     for (Common.PartitionSchemaPB.HashBucketSchemaPB hashBucketSchemaPB
-        : pb.getHashBucketSchemasList()) {
+        : pb.getHashSchemaList()) {
       List<Integer> hashColumnIds = pbToIds(hashBucketSchemaPB.getColumnsList());
 
       PartitionSchema.HashBucketSchema hashSchema =
@@ -222,7 +259,35 @@ public class ProtobufHelper {
       hashSchemas.add(hashSchema);
     }
 
-    return new PartitionSchema(rangeSchema, hashSchemas.build(), schema);
+    // Populate the list of ranges with custom hash schemas.
+    ImmutableList.Builder<PartitionSchema.RangeWithHashSchema> rangesWithHashSchemas =
+        ImmutableList.builder();
+
+    for (Common.PartitionSchemaPB.RangeWithHashSchemaPB rhsPB :
+        pb.getCustomHashSchemaRangesList()) {
+      List<PartitionSchema.HashBucketSchema> rangeHashSchemas = new ArrayList<>();
+      for (Common.PartitionSchemaPB.HashBucketSchemaPB hbs : rhsPB.getHashSchemaList()) {
+        rangeHashSchemas.add(new PartitionSchema.HashBucketSchema(
+            pbToIds(hbs.getColumnsList()), hbs.getNumBuckets(), hbs.getSeed()));
+      }
+
+      // Decode RowOperationsPB into the range bounds.
+      final RowOperations.RowOperationsPB rangeBounds = rhsPB.getRangeBounds();
+      Operation.OperationsDecoder dec = new Operation.OperationsDecoder();
+      final List<RangePartition> partitions = dec.decodeRangePartitions(rangeBounds, schema);
+      if (partitions.size() != 1) {
+        throw new IllegalArgumentException("unexpected range bounds");
+      }
+      final RangePartition p = partitions.get(0);
+
+      PartitionSchema.RangeWithHashSchema rhs =
+          new PartitionSchema.RangeWithHashSchema(
+              p.lowerBound, p.upperBound, rangeHashSchemas);
+      rangesWithHashSchemas.add(rhs);
+    }
+
+    return new PartitionSchema(
+        rangeSchema, hashSchemas.build(), rangesWithHashSchemas.build(), schema);
   }
 
   public static Common.PartitionSchemaPB partitionSchemaToPb(PartitionSchema partitionSchema) {
@@ -235,7 +300,7 @@ public class ProtobufHelper {
                 .addAllColumns(idsToPb(hashBucketSchema.getColumnIds()))
                 .setNumBuckets(hashBucketSchema.getNumBuckets())
                 .setSeed(hashBucketSchema.getSeed());
-      builder.addHashBucketSchemas(hbsBuilder.build());
+      builder.addHashSchema(hbsBuilder.build());
     }
 
     Common.PartitionSchemaPB.RangeSchemaPB rangeSchemaPB =
@@ -244,6 +309,28 @@ public class ProtobufHelper {
               .build();
     builder.setRangeSchema(rangeSchemaPB);
 
+    // Based on the list of ranges with custom hash schemas, populate the
+    // PartitionSchemaPB.custom_hash_schema_ranges field.
+    for (PartitionSchema.RangeWithHashSchema rhs : partitionSchema.getRangesWithHashSchemas()) {
+      Common.PartitionSchemaPB.RangeWithHashSchemaPB.Builder rhsBuilder =
+          Common.PartitionSchemaPB.RangeWithHashSchemaPB.newBuilder();
+      for (PartitionSchema.HashBucketSchema hbs : rhs.hashSchemas) {
+        Common.PartitionSchemaPB.HashBucketSchemaPB.Builder hbsBuilder =
+            Common.PartitionSchemaPB.HashBucketSchemaPB.newBuilder()
+                .addAllColumns(idsToPb(hbs.getColumnIds()))
+                .setNumBuckets(hbs.getNumBuckets())
+                .setSeed(hbs.getSeed());
+        rhsBuilder.addHashSchema(hbsBuilder.build());
+      }
+
+      rhsBuilder.setRangeBounds(new Operation.OperationsEncoder().encodeLowerAndUpperBounds(
+          rhs.lowerBound,
+          rhs.upperBound,
+          RangePartitionBound.INCLUSIVE_BOUND,
+          RangePartitionBound.EXCLUSIVE_BOUND));
+      builder.addCustomHashSchemaRanges(rhsBuilder.build());
+    }
+
     return builder.build();
   }
 
