@@ -19,12 +19,10 @@ package org.apache.kudu.spark.kudu
 
 import java.security.AccessController
 import java.security.PrivilegedAction
-
 import javax.security.auth.Subject
 import javax.security.auth.login.AppConfigurationEntry
 import javax.security.auth.login.Configuration
 import javax.security.auth.login.LoginContext
-
 import scala.collection.JavaConverters._
 import scala.collection.mutable
 import org.apache.hadoop.util.ShutdownHookManager
@@ -48,6 +46,7 @@ import org.apache.kudu.spark.kudu.SparkUtil.kuduSchema
 import org.apache.kudu.spark.kudu.SparkUtil._
 import org.apache.kudu.Schema
 import org.apache.kudu.Type
+import org.apache.kudu.client.AsyncKuduClient.EncryptionPolicy
 
 /**
  * KuduContext is a serializable container for Kudu client connections.
@@ -63,7 +62,9 @@ class KuduContext(
     val kuduMaster: String,
     sc: SparkContext,
     val socketReadTimeoutMs: Option[Long],
-    val saslProtocolName: Option[String] = None)
+    val saslProtocolName: Option[String] = None,
+    val requireAuthentication: Boolean = false,
+    val encryptionPolicy: EncryptionPolicy = EncryptionPolicy.OPTIONAL)
     extends Serializable {
   val log: Logger = LoggerFactory.getLogger(getClass)
 
@@ -153,7 +154,8 @@ class KuduContext(
   @transient lazy val syncClient: KuduClient = asyncClient.syncClient()
 
   @transient lazy val asyncClient: AsyncKuduClient = {
-    val c = KuduClientCache.getAsyncClient(kuduMaster, saslProtocolName)
+    val c = KuduClientCache
+      .getAsyncClient(kuduMaster, saslProtocolName, requireAuthentication, encryptionPolicy)
     if (authnCredentials != null) {
       c.importAuthenticationCredentials(authnCredentials)
     }
@@ -327,6 +329,25 @@ class KuduContext(
     log.info(s"updated ${numUpdates.value.get(tableName)} rows in table '$tableName'")
   }
 
+  /**
+   * Deletes the rows of a [[DataFrame]] from a Kudu table, ignoring any none-existing
+   * rows.
+   *
+   * @param data the data to delete from Kudu
+   *             note that only the key columns should be specified for deletes
+   * @param tableName The Kudu tabe to delete from
+   * @param writeOptions the Kudu write options
+   */
+  def deleteIgnoreRows(
+      data: DataFrame,
+      tableName: String,
+      writeOptions: KuduWriteOptions = new KuduWriteOptions): Unit = {
+    log.info(s"deleting rows from table '$tableName'")
+    writeRows(data, tableName, DeleteIgnore, writeOptions)
+    log.info(
+      s"deleted up to ${numDeletes.value.get(tableName)} rows from table '$tableName' using DELETE_IGNORE")
+  }
+
   /**
    * Deletes the rows of a [[DataFrame]] from a Kudu table.
    *
@@ -611,13 +632,19 @@ private object KuduClientCache {
     clientCache.clear()
   }
 
-  def getAsyncClient(kuduMaster: String, saslProtocolName: Option[String]): AsyncKuduClient = {
+  def getAsyncClient(
+      kuduMaster: String,
+      saslProtocolName: Option[String],
+      requireAuthentication: Boolean = false,
+      encryptionPolicy: EncryptionPolicy = EncryptionPolicy.OPTIONAL): AsyncKuduClient = {
     clientCache.synchronized {
       if (!clientCache.contains(kuduMaster)) {
         val builder = new AsyncKuduClient.AsyncKuduClientBuilder(kuduMaster)
         if (saslProtocolName.nonEmpty) {
           builder.saslProtocolName(saslProtocolName.get)
         }
+        builder.requireAuthentication(requireAuthentication)
+        builder.encryptionPolicy(encryptionPolicy)
         val asyncClient = builder.build()
         val hookHandle = new Runnable {
           override def run(): Unit = asyncClient.close()
