@@ -490,4 +490,20 @@ public abstract class ClientTestUtil {
         ).build());
     return new Schema(columns);
   }
+
+  public static Schema createSchemaWithImmutableColumns() {
+    List<ColumnSchema> columns = new ArrayList<>(ClientTestUtil.getBasicSchema().getColumns());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("column5_i", Type.INT32)
+            .nullable(true).immutable(true).build());
+    return new Schema(columns);
+  }
+
+  public static Schema createSchemaWithNonUniqueKey() {
+    ArrayList<ColumnSchema> columns = new ArrayList<>();
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("key", Type.INT32).nonUniqueKey(true)
+        .build());
+    columns.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.INT32).nullable(true)
+        .build());
+    return new Schema(columns);
+  }
 }
