@@ -233,7 +233,7 @@ class CFileReader {
 // It is used to fill the data requested by the projection.
 class ColumnIterator {
  public:
-  virtual ~ColumnIterator() {}
+  virtual ~ColumnIterator() = default;
 
   // Seek to the given ordinal entry in the file.
   // Entry 0 is the first entry written to the file.
@@ -444,7 +444,7 @@ class CFileIterator final : public ColumnIterator {
 
     // Null bitmap and bitmap (RLE) decoder
     Slice rle_bitmap;
-    RleDecoder<bool> rle_decoder_;
+    RleDecoder<bool, 1> rle_decoder_;
 
     rowid_t last_row_idx() const {
       return first_row_idx() + num_rows_in_block_ - 1;
