@@ -40,6 +40,7 @@
 #include "kudu/common/types.h"
 #include "kudu/gutil/port.h"
 #include "kudu/util/array_view.h" // IWYU pragma: keep
+#include "kudu/util/bitmap.h"
 #include "kudu/util/coding.h"
 #include "kudu/util/coding-inl.h"
 #include "kudu/util/compression/compression_codec.h"
@@ -49,6 +50,7 @@
 #include "kudu/util/hexdump.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/pb_util.h"
+#include "kudu/util/rle-encoding.h"
 
 DEFINE_int32(cfile_default_block_size, 256*1024, "The default block size to use in cfiles");
 TAG_FLAG(cfile_default_block_size, advanced);
@@ -66,7 +68,6 @@ using kudu::fs::BlockCreationTransaction;
 using kudu::fs::BlockManager;
 using kudu::fs::WritableBlock;
 using std::accumulate;
-using std::pair;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -81,6 +82,41 @@ const size_t kChecksumSize = sizeof(uint32_t);
 
 static const size_t kMinBlockSize = 512;
 
+class NullBitmapBuilder {
+ public:
+  explicit NullBitmapBuilder(size_t initial_row_capacity)
+      : nitems_(0),
+        bitmap_(BitmapSize(initial_row_capacity)),
+        rle_encoder_(&bitmap_) {
+  }
+
+  size_t nitems() const {
+    return nitems_;
+  }
+
+  // If value parameter is true, it means that all values in this run are null
+  void AddRun(bool value, size_t run_length = 1) {
+    nitems_ += run_length;
+    rle_encoder_.Put(value, run_length);
+  }
+
+  // the returned Slice is only valid until this Builder is destroyed or Reset
+  Slice Finish() {
+    int len = rle_encoder_.Flush();
+    return Slice(bitmap_.data(), len);
+  }
+
+  void Reset() {
+    nitems_ = 0;
+    rle_encoder_.Clear();
+  }
+
+ private:
+  size_t nitems_;
+  faststring bitmap_;
+  RleEncoder<bool, 1> rle_encoder_;
+};
+
 ////////////////////////////////////////////////////////////
 // CFileWriter
 ////////////////////////////////////////////////////////////
@@ -90,17 +126,17 @@ CFileWriter::CFileWriter(WriterOptions options,
                          const TypeInfo* typeinfo,
                          bool is_nullable,
                          unique_ptr<WritableBlock> block)
-  : block_(std::move(block)),
-    off_(0),
-    value_count_(0),
-    options_(std::move(options)),
-    is_nullable_(is_nullable),
-    typeinfo_(typeinfo),
-    state_(kWriterInitialized) {
+    : options_(std::move(options)),
+      block_(std::move(block)),
+      off_(0),
+      value_count_(0),
+      is_nullable_(is_nullable),
+      typeinfo_(typeinfo),
+      state_(kWriterInitialized) {
   EncodingType encoding = options_.storage_attributes.encoding;
   Status s = TypeEncodingInfo::Get(typeinfo_, encoding, &type_encoding_info_);
-  if (!s.ok()) {
-    // TODO: we should somehow pass some contextual info about the
+  if (PREDICT_FALSE(!s.ok())) {
+    // TODO(af): we should somehow pass some contextual info about the
     // tablet here.
     WARN_NOT_OK(s, "Falling back to default encoding");
     s = TypeEncodingInfo::Get(typeinfo,
@@ -117,7 +153,7 @@ CFileWriter::CFileWriter(WriterOptions options,
   if (options_.storage_attributes.cfile_block_size <= 0) {
     options_.storage_attributes.cfile_block_size = FLAGS_cfile_default_block_size;
   }
-  if (options_.storage_attributes.cfile_block_size < kMinBlockSize) {
+  if (PREDICT_FALSE(options_.storage_attributes.cfile_block_size < kMinBlockSize)) {
     LOG(WARNING) << "Configured block size " << options_.storage_attributes.cfile_block_size
                  << " smaller than minimum allowed value " << kMinBlockSize
                  << ": using minimum.";
@@ -145,8 +181,7 @@ CFileWriter::~CFileWriter() {
 
 Status CFileWriter::Start() {
   TRACE_EVENT0("cfile", "CFileWriter::Start");
-  CHECK(state_ == kWriterInitialized) <<
-    "bad state for Start(): " << state_;
+  DCHECK(state_ == kWriterInitialized) << "bad state for Start(): " << state_;
 
   if (compression_ != NO_COMPRESSION) {
     const CompressionCodec* codec;
@@ -157,7 +192,7 @@ Status CFileWriter::Start() {
   CFileHeaderPB header;
   FlushMetadataToPB(header.mutable_metadata());
 
-  uint32_t pb_size = header.ByteSizeLong();
+  const uint32_t pb_size = header.ByteSizeLong();
 
   faststring header_str;
   // First the magic.
@@ -166,8 +201,7 @@ Status CFileWriter::Start() {
   PutFixed32(&header_str, pb_size);
   pb_util::AppendToString(header, &header_str);
 
-  vector<Slice> header_slices;
-  header_slices.emplace_back(header_str);
+  vector<Slice> header_slices { header_str };
 
   // Append header checksum.
   uint8_t checksum_buf[kChecksumSize];
@@ -202,8 +236,7 @@ Status CFileWriter::Finish() {
 
 Status CFileWriter::FinishAndReleaseBlock(BlockCreationTransaction* transaction) {
   TRACE_EVENT0("cfile", "CFileWriter::FinishAndReleaseBlock");
-  CHECK(state_ == kWriterWriting) <<
-    "Bad state for Finish(): " << state_;
+  DCHECK(state_ == kWriterWriting) << "Bad state for Finish(): " << state_;
 
   // Write out any pending values as the last data block.
   RETURN_NOT_OK(FinishCurDataBlock());
@@ -269,38 +302,36 @@ Status CFileWriter::FinishAndReleaseBlock(BlockCreationTransaction* transaction)
   return Status::OK();
 }
 
-void CFileWriter::AddMetadataPair(const Slice &key, const Slice &value) {
-  CHECK_NE(state_, kWriterFinished);
+void CFileWriter::AddMetadataPair(const Slice& key, const Slice& value) {
+  DCHECK_NE(state_, kWriterFinished);
 
   unflushed_metadata_.emplace_back(key.ToString(), value.ToString());
 }
 
 string CFileWriter::GetMetaValueOrDie(Slice key) const {
-  typedef pair<string, string> ss_pair;
-  for (const ss_pair& entry : unflushed_metadata_) {
-    if (Slice(entry.first) == key) {
-      return entry.second;
+  for (const auto& [m_key, m_val] : unflushed_metadata_) {
+    if (Slice(m_key) == key) {
+      return m_val;
     }
   }
   LOG(FATAL) << "Missing metadata entry: " << KUDU_REDACT(key.ToDebugString());
 }
 
-void CFileWriter::FlushMetadataToPB(RepeatedPtrField<FileMetadataPairPB> *field) {
-  typedef pair<string, string> ss_pair;
-  for (const ss_pair &entry : unflushed_metadata_) {
-    FileMetadataPairPB *pb = field->Add();
-    pb->set_key(entry.first);
-    pb->set_value(entry.second);
+void CFileWriter::FlushMetadataToPB(RepeatedPtrField<FileMetadataPairPB>* field) {
+  for (const auto& [key, val] : unflushed_metadata_) {
+    FileMetadataPairPB* pb = field->Add();
+    pb->set_key(key);
+    pb->set_value(val);
   }
   unflushed_metadata_.clear();
 }
 
-Status CFileWriter::AppendEntries(const void *entries, size_t count) {
+Status CFileWriter::AppendEntries(const void* entries, size_t count) {
   DCHECK(!is_nullable_);
 
   int rem = count;
 
-  const uint8_t *ptr = reinterpret_cast<const uint8_t *>(entries);
+  const uint8_t* ptr = reinterpret_cast<const uint8_t*>(entries);
 
   while (rem > 0) {
     int n = data_block_->Add(ptr, rem);
@@ -319,12 +350,12 @@ Status CFileWriter::AppendEntries(const void *entries, size_t count) {
   return Status::OK();
 }
 
-Status CFileWriter::AppendNullableEntries(const uint8_t *bitmap,
-                                          const void *entries,
+Status CFileWriter::AppendNullableEntries(const uint8_t* bitmap,
+                                          const void* entries,
                                           size_t count) {
   DCHECK(is_nullable_ && bitmap != nullptr);
 
-  const uint8_t *ptr = reinterpret_cast<const uint8_t *>(entries);
+  const uint8_t* ptr = reinterpret_cast<const uint8_t*>(entries);
 
   size_t nitems;
   bool is_non_null = false;
@@ -357,11 +388,8 @@ Status CFileWriter::AppendNullableEntries(const uint8_t *bitmap,
 }
 
 Status CFileWriter::FinishCurDataBlock() {
-  uint32_t num_elems_in_block = data_block_->Count();
-  if (is_nullable_) {
-    num_elems_in_block = non_null_bitmap_builder_->nitems();
-  }
-
+  const uint32_t num_elems_in_block =
+      is_nullable_ ? non_null_bitmap_builder_->nitems() : data_block_->Count();
   if (PREDICT_FALSE(num_elems_in_block == 0)) {
     return Status::OK();
   }
@@ -385,20 +413,24 @@ Status CFileWriter::FinishCurDataBlock() {
       kudu::HexDump(Slice(key_tmp_space, typeinfo_->size()));
   }
 
-  vector<Slice> v;
-  faststring null_headers;
-  if (is_nullable_) {
-    Slice non_null_bitmap = non_null_bitmap_builder_->Finish();
-    PutVarint32(&null_headers, num_elems_in_block);
-    PutVarint32(&null_headers, non_null_bitmap.size());
-    v.emplace_back(null_headers.data(), null_headers.size());
-    v.push_back(non_null_bitmap);
+  Status s;
+  {
+    vector<Slice> v;
+    v.reserve(data_slices.size() + (is_nullable_ ? 2 : 0));
+    faststring null_headers;
+    if (is_nullable_) {
+      const Slice non_null_bitmap = non_null_bitmap_builder_->Finish();
+      PutVarint32(&null_headers, num_elems_in_block);
+      PutVarint32(&null_headers, non_null_bitmap.size());
+      v.emplace_back(null_headers.data(), null_headers.size());
+      v.emplace_back(non_null_bitmap);
+    }
+    std::move(data_slices.begin(), data_slices.end(), std::back_inserter(v));
+    s = AppendRawBlock(std::move(v), first_elem_ord,
+                       reinterpret_cast<const void*>(key_tmp_space),
+                       Slice(last_key_),
+                       "data block");
   }
-  std::move(data_slices.begin(), data_slices.end(), std::back_inserter(v));
-  Status s = AppendRawBlock(v, first_elem_ord,
-                            reinterpret_cast<const void *>(key_tmp_space),
-                            Slice(last_key_),
-                            "data block");
 
   if (is_nullable_) {
     non_null_bitmap_builder_->Reset();
@@ -413,17 +445,17 @@ Status CFileWriter::FinishCurDataBlock() {
   return s;
 }
 
-Status CFileWriter::AppendRawBlock(const vector<Slice>& data_slices,
+Status CFileWriter::AppendRawBlock(vector<Slice> data_slices,
                                    size_t ordinal_pos,
-                                   const void *validx_curr,
+                                   const void* validx_curr,
                                    const Slice& validx_prev,
-                                   const char *name_for_log) {
-  CHECK_EQ(state_, kWriterWriting);
+                                   const char* name_for_log) {
+  DCHECK_EQ(state_, kWriterWriting);
 
   BlockPointer ptr;
-  Status s = AddBlock(data_slices, &ptr, name_for_log);
-  if (!s.ok()) {
-    LOG(WARNING) << "Unable to append block to file: " << s.ToString();
+  Status s = AddBlock(std::move(data_slices), &ptr, name_for_log);
+  if (PREDICT_FALSE(!s.ok())) {
+    LOG(ERROR) << "Unable to append block to file: " << s.ToString();
     return s;
   }
 
@@ -435,7 +467,7 @@ Status CFileWriter::AppendRawBlock(const vector<Slice>& data_slices,
   }
 
   if (validx_builder_ != nullptr) {
-    CHECK(validx_curr != nullptr) <<
+    DCHECK(validx_curr != nullptr) <<
       "must pass a key for raw block if validx is configured";
 
     (*options_.validx_key_encoder)(validx_curr, &tmp_buf_);
@@ -443,41 +475,40 @@ Status CFileWriter::AppendRawBlock(const vector<Slice>& data_slices,
     if (options_.optimize_index_keys) {
       GetSeparatingKey(validx_prev, &idx_key);
     }
-    VLOG(1) << "Appending validx entry\n" <<
-            kudu::HexDump(idx_key);
-    s = validx_builder_->Append(idx_key, ptr);
-    if (!s.ok()) {
-      LOG(WARNING) << "Unable to append to value index: " << s.ToString();
+    VLOG(1) << "Appending validx entry\n" << kudu::HexDump(idx_key);
+    Status s = validx_builder_->Append(idx_key, ptr);
+    if (PREDICT_FALSE(!s.ok())) {
+      LOG(ERROR) << "Unable to append to value index: " << s.ToString();
       return s;
     }
   }
 
-  return s;
+  return Status::OK();
 }
 
-Status CFileWriter::AddBlock(const vector<Slice> &data_slices,
-                             BlockPointer *block_ptr,
-                             const char *name_for_log) {
-  uint64_t start_offset = off_;
+Status CFileWriter::AddBlock(vector<Slice> data_slices,
+                             BlockPointer* block_ptr,
+                             const char* name_for_log) {
+  const uint64_t start_offset = off_;
   vector<Slice> out_slices;
 
   if (block_compressor_ != nullptr) {
     // Write compressed block
-    Status s = block_compressor_->Compress(data_slices, &out_slices);
-    if (!s.ok()) {
-      LOG(WARNING) << "Unable to compress block at offset " << off_
-                   << ": " << s.ToString();
+    Status s = block_compressor_->Compress(std::move(data_slices), &out_slices);
+    if (PREDICT_FALSE(!s.ok())) {
+      LOG(ERROR) << "Unable to compress block at offset " << start_offset
+                 << ": " << s.ToString();
       return s;
     }
   } else {
-    out_slices = data_slices;
+    out_slices = std::move(data_slices);
   }
 
   // Calculate and append a data checksum.
   uint8_t checksum_buf[kChecksumSize];
   if (FLAGS_cfile_write_checksums) {
     uint32_t checksum = 0;
-    for (const Slice &data : out_slices) {
+    for (const Slice& data : out_slices) {
       checksum = crc::Crc32c(data.data(), data.size(), checksum);
     }
     InlineEncodeFixed32(checksum_buf, checksum);
@@ -486,7 +517,8 @@ Status CFileWriter::AddBlock(const vector<Slice> &data_slices,
 
   RETURN_NOT_OK(WriteRawData(out_slices));
 
-  uint64_t total_size = off_ - start_offset;
+  DCHECK_GT(off_, start_offset);
+  const uint64_t total_size = off_ - start_offset;
 
   *block_ptr = BlockPointer(start_offset, total_size);
   VLOG(1) << "Appended " << name_for_log
@@ -500,10 +532,10 @@ Status CFileWriter::WriteRawData(const vector<Slice>& data) {
                                   return sum + curr.size();
                                 });
   Status s = block_->AppendV(data);
-  if (!s.ok()) {
-    LOG(WARNING) << "Unable to append data of size "
-                 << data_size << " at offset " << off_
-                 << ": " << s.ToString();
+  if (PREDICT_FALSE(!s.ok())) {
+    LOG(ERROR) << "Unable to append data of size "
+               << data_size << " at offset " << off_
+               << ": " << s.ToString();
   }
   off_ += data_size;
   return s;
