@@ -15,8 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
-#ifndef KUDU_CFILE_INDEX_BLOCK_H
-#define KUDU_CFILE_INDEX_BLOCK_H
+#pragma once
 
 #include <cstddef>
 #include <cstdint>
@@ -35,19 +34,16 @@ namespace cfile {
 // Forward decl.
 class IndexBlockIterator;
 
-struct WriterOptions;
-
 // Index Block Builder for a particular key type.
 // This works like the rest of the builders in the cfile package.
 // After repeatedly calling Add(), call Finish() to encode it
 // into a Slice, then you may Reset to re-use buffers.
 class IndexBlockBuilder {
  public:
-  explicit IndexBlockBuilder(const WriterOptions *options,
-                             bool is_leaf);
+  explicit IndexBlockBuilder(bool is_leaf);
 
   // Append an entry into the index.
-  void Add(const Slice &key, const BlockPointer &ptr);
+  void Add(const Slice& key, const BlockPointer& ptr);
 
   // Finish the current index block.
   // Returns a fully encoded Slice including the data
@@ -58,14 +54,19 @@ class IndexBlockBuilder {
 
   // Return the key of the first entry in this index block.
   // The pointed-to data is only valid until the next call to this builder.
-  Status GetFirstKey(Slice *key) const;
+  Status GetFirstKey(Slice* key) const;
 
-  // Return the number of entries already added to this index
-  // block.
+  // Return the number of entries already added to this index block.
   size_t count() const {
     return entry_offsets_.size();
   }
 
+  // Return 'true' if there aren't any entries added to this index block,
+  // 'false' otherwise.
+  bool empty() const {
+    return entry_offsets_.empty();
+  }
+
   // Return an estimate of the post-encoding size of this
   // index block. This estimate should be conservative --
   // it will over-estimate rather than under-estimate, and
@@ -78,16 +79,11 @@ class IndexBlockBuilder {
  private:
   DISALLOW_COPY_AND_ASSIGN(IndexBlockBuilder);
 
-#ifdef __clang__
-  __attribute__((__unused__))
-#endif
-  const WriterOptions *options_;
-
   // Is the builder currently between Finish() and Reset()
   bool finished_;
 
   // Is this a leaf block?
-  bool is_leaf_;
+  const bool is_leaf_;
 
   faststring buffer_;
   std::vector<uint32_t> entry_offsets_;
@@ -106,34 +102,35 @@ class IndexBlockReader {
   //
   // Note: this does not copy the data, so the slice must
   // remain valid for the lifetime of the reader (or until the next Parse call).
-  Status Parse(const Slice &data);
+  Status Parse(const Slice& data);
 
   size_t Count() const;
 
-  IndexBlockIterator *NewIterator() const;
+  IndexBlockIterator* NewIterator() const;
 
-  bool IsLeaf();
+  bool IsLeaf() const;
 
  private:
   friend class IndexBlockIterator;
 
-  int CompareKey(int idx_in_block, const Slice &search_key) const;
+  int CompareKey(size_t idx_in_block, const Slice& search_key) const;
 
-  Status ReadEntry(size_t idx, Slice *key, BlockPointer *block_ptr) const;
+  Status ReadEntry(size_t idx, Slice* key, BlockPointer* block_ptr) const;
 
-  // Set *ptr to the beginning of the index data for the given index
-  // entry.
+  // Set *ptr to the beginning of the index data for the given index entry.
   // Set *limit to the 'limit' pointer for that entry (i.e a pointer
   // beyond which the data no longer is part of that entry).
   //   - *limit can be used to prevent overrunning in the case of a
   //     corrupted length varint or length prefix
-  void GetKeyPointer(int idx_in_block, const uint8_t **ptr, const uint8_t **limit) const;
+  // Return Status::NotFound() if the given index isn't present.
+  // Return Status::Corruption() if inconsistency in block offset is detected.
+  Status GetKeyPointer(size_t idx_in_block,
+                       const uint8_t** ptr,
+                       const uint8_t** limit) const;
 
-  static const int kMaxTrailerSize = 64*1024;
   Slice data_;
-
   IndexBlockTrailerPB trailer_;
-  const uint8_t *key_offsets_;
+  const uint8_t* key_offsets_;
   bool parsed_;
 
   DISALLOW_COPY_AND_ASSIGN(IndexBlockReader);
@@ -141,7 +138,7 @@ class IndexBlockReader {
 
 class IndexBlockIterator {
  public:
-  explicit IndexBlockIterator(const IndexBlockReader *reader);
+  explicit IndexBlockIterator(const IndexBlockReader* reader);
 
   // Reset the state of this iterator. This should be used
   // after the associated 'reader' object parses a different block.
@@ -157,7 +154,7 @@ class IndexBlockIterator {
   // If this function returns an error, then the state of this
   // iterator is undefined (i.e it may or may not have moved
   // since the previous call)
-  Status SeekAtOrBefore(const Slice &search_key);
+  Status SeekAtOrBefore(const Slice& search_key);
 
   Status SeekToIndex(size_t idx);
 
@@ -165,12 +162,12 @@ class IndexBlockIterator {
 
   Status Next();
 
-  const BlockPointer &GetCurrentBlockPointer() const;
+  const BlockPointer& GetCurrentBlockPointer() const;
 
-  const Slice GetCurrentKey() const;
+  const Slice& GetCurrentKey() const;
 
  private:
-  const IndexBlockReader *reader_;
+  const IndexBlockReader* reader_;
   size_t cur_idx_;
   Slice cur_key_;
   BlockPointer cur_ptr_;
@@ -181,4 +178,3 @@ class IndexBlockIterator {
 
 } // namespace cfile
 } // namespace kudu
-#endif
