@@ -14,9 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-
-#ifndef KUDU_CFILE_RLE_BLOCK_H
-#define KUDU_CFILE_RLE_BLOCK_H
+#pragma once
 
 #include <algorithm>
 #include <string>
@@ -51,12 +49,12 @@ enum {
 class RleBitMapBlockBuilder final : public BlockBuilder {
  public:
   explicit RleBitMapBlockBuilder(const WriterOptions* options)
-      : encoder_(&buf_, 1),
-        options_(options) {
+      : options_(options),
+        encoder_(&buf_) {
     Reset();
   }
 
-  virtual int Add(const uint8_t* vals, size_t count) OVERRIDE {
+  int Add(const uint8_t* vals, size_t count) override {
      for (const uint8_t* val = vals;
           val < vals + count;
           ++val) {
@@ -68,41 +66,41 @@ class RleBitMapBlockBuilder final : public BlockBuilder {
     return count;
   }
 
-  virtual bool IsBlockFull() const override {
+  bool IsBlockFull() const override {
     return encoder_.len() > options_->storage_attributes.cfile_block_size;
   }
 
-  virtual void Finish(rowid_t ordinal_pos, std::vector<Slice>* slices) OVERRIDE {
+  void Finish(rowid_t ordinal_pos, std::vector<Slice>* slices) override {
     InlineEncodeFixed32(&buf_[0], count_);
     InlineEncodeFixed32(&buf_[4], ordinal_pos);
     encoder_.Flush();
     *slices = { buf_ };
   }
 
-  virtual void Reset() OVERRIDE {
+  void Reset() override {
     count_ = 0;
     encoder_.Clear();
     encoder_.Reserve(kRleBitmapBlockHeaderSize, 0);
   }
 
-  virtual size_t Count() const OVERRIDE {
+  size_t Count() const override {
     return count_;
   }
 
-  // TODO Implement this method
-  virtual Status GetFirstKey(void* key) const OVERRIDE {
+  // TODO(afeinberg) Implement this method
+  Status GetFirstKey(void* key) const override {
     return Status::NotSupported("BOOL keys not supported");
   }
 
-  // TODO Implement this method
-  virtual Status GetLastKey(void* key) const OVERRIDE {
+  // TODO(afeinberg) Implement this method
+  Status GetLastKey(void* key) const override {
     return Status::NotSupported("BOOL keys not supported");
   }
 
  private:
-  faststring buf_;
-  RleEncoder<bool> encoder_;
   const WriterOptions* const options_;
+  faststring buf_;
+  RleEncoder<bool, 1> encoder_;
   size_t count_;
 };
 
@@ -120,10 +118,10 @@ class RleBitMapBlockDecoder final : public BlockDecoder {
         cur_idx_(0) {
   }
 
-  virtual Status ParseHeader() OVERRIDE {
-    CHECK(!parsed_);
+  Status ParseHeader() override {
+    DCHECK(!parsed_);
 
-    if (data_.size() < kRleBitmapBlockHeaderSize) {
+    if (PREDICT_FALSE(data_.size() < kRleBitmapBlockHeaderSize)) {
       return Status::Corruption(
           "not enough bytes for header in RleBitMapBlockDecoder");
     }
@@ -133,16 +131,16 @@ class RleBitMapBlockDecoder final : public BlockDecoder {
 
     parsed_ = true;
 
-    rle_decoder_ = RleDecoder<bool>(data_.data() + kRleBitmapBlockHeaderSize,
-                                    data_.size() - kRleBitmapBlockHeaderSize, 1);
+    rle_decoder_ = RleDecoder<bool, 1>(data_.data() + kRleBitmapBlockHeaderSize,
+                                       data_.size() - kRleBitmapBlockHeaderSize);
 
     SeekToPositionInBlock(0);
 
     return Status::OK();
   }
 
-  virtual void SeekToPositionInBlock(uint pos) OVERRIDE {
-    CHECK(parsed_) << "Must call ParseHeader()";
+  void SeekToPositionInBlock(uint pos) override {
+    DCHECK(parsed_) << "Must call ParseHeader()";
     DCHECK_LE(pos, num_elems_)
       << "Tried to seek to " << pos << " which is > number of elements ("
       << num_elems_ << ") in the block!";
@@ -160,14 +158,15 @@ class RleBitMapBlockDecoder final : public BlockDecoder {
     } else {
       // This approach is also used by CFileReader to
       // seek backwards in an RLE encoded block
-      rle_decoder_ = RleDecoder<bool>(data_.data() + kRleBitmapBlockHeaderSize,
-                                      data_.size() - kRleBitmapBlockHeaderSize, 1);
+      DCHECK_GE(data_.size(), kRleBitmapBlockHeaderSize);
+      rle_decoder_ = RleDecoder<bool, 1>(data_.data() + kRleBitmapBlockHeaderSize,
+                                         data_.size() - kRleBitmapBlockHeaderSize);
       rle_decoder_.Skip(pos);
     }
     cur_idx_ = pos;
   }
 
-  virtual Status CopyNextValues(size_t *n, ColumnDataView* dst) OVERRIDE {
+  Status CopyNextValues(size_t* n, ColumnDataView* dst) override {
     DCHECK(parsed_);
 
     DCHECK_LE(*n, dst->nrows());
@@ -195,18 +194,18 @@ class RleBitMapBlockDecoder final : public BlockDecoder {
     return Status::OK();
   }
 
-  virtual Status SeekAtOrAfterValue(const void *value,
-                                    bool *exact_match) OVERRIDE {
+  Status SeekAtOrAfterValue(const void* /*value*/,
+                            bool* /*exact_match*/) override {
     return Status::NotSupported("BOOL keys are not supported!");
   }
 
-  virtual bool HasNext() const OVERRIDE { return cur_idx_ < num_elems_; }
+  bool HasNext() const override { return cur_idx_ < num_elems_; }
 
-  virtual size_t Count() const OVERRIDE { return num_elems_; }
+  size_t Count() const override { return num_elems_; }
 
-  virtual size_t GetCurrentIndex() const OVERRIDE { return cur_idx_; }
+  size_t GetCurrentIndex() const override { return cur_idx_; }
 
-  virtual rowid_t GetFirstRowId() const OVERRIDE { return ordinal_pos_base_; }
+  rowid_t GetFirstRowId() const override { return ordinal_pos_base_; }
 
  private:
   scoped_refptr<BlockHandle> block_;
@@ -215,7 +214,7 @@ class RleBitMapBlockDecoder final : public BlockDecoder {
   uint32_t num_elems_;
   rowid_t ordinal_pos_base_;
   uint32_t cur_idx_;
-  RleDecoder<bool> rle_decoder_;
+  RleDecoder<bool, 1> rle_decoder_;
 };
 
 //
@@ -227,16 +226,16 @@ template <DataType IntType>
 class RleIntBlockBuilder final : public BlockBuilder {
  public:
   explicit RleIntBlockBuilder(const WriterOptions* options)
-      : rle_encoder_(&buf_, kCppTypeSize * 8),
-        options_(options) {
+      : options_(options),
+        rle_encoder_(&buf_) {
     Reset();
   }
 
-  virtual bool IsBlockFull() const OVERRIDE {
+  bool IsBlockFull() const override {
     return rle_encoder_.len() > options_->storage_attributes.cfile_block_size;
   }
 
-  virtual int Add(const uint8_t* vals_void, size_t count) OVERRIDE {
+  int Add(const uint8_t* vals_void, size_t count) override {
     DCHECK_EQ(reinterpret_cast<uintptr_t>(vals_void) & (alignof(CppType) - 1), 0)
         << "Pointer passed to Add() must be naturally-aligned";
 
@@ -254,24 +253,24 @@ class RleIntBlockBuilder final : public BlockBuilder {
     return count;
   }
 
-  virtual void Finish(rowid_t ordinal_pos, std::vector<Slice>* slices) OVERRIDE {
+  void Finish(rowid_t ordinal_pos, std::vector<Slice>* slices) override {
     InlineEncodeFixed32(&buf_[0], count_);
     InlineEncodeFixed32(&buf_[4], ordinal_pos);
     rle_encoder_.Flush();
     *slices = { Slice(buf_) };
   }
 
-  virtual void Reset() OVERRIDE {
+  void Reset() override {
     count_ = 0;
     rle_encoder_.Clear();
     rle_encoder_.Reserve(kRleBitmapBlockHeaderSize, 0);
   }
 
-  virtual size_t Count() const OVERRIDE {
+  size_t Count() const override {
     return count_;
   }
 
-  virtual Status GetFirstKey(void* key) const OVERRIDE {
+  Status GetFirstKey(void* key) const override {
     if (PREDICT_FALSE(count_ == 0)) {
       return Status::NotFound("No keys in the block");
     }
@@ -279,7 +278,7 @@ class RleIntBlockBuilder final : public BlockBuilder {
     return Status::OK();
   }
 
-  virtual Status GetLastKey(void* key) const OVERRIDE {
+  Status GetLastKey(void* key) const override {
     if (PREDICT_FALSE(count_ == 0)) {
       return Status::NotFound("No keys in the block");
     }
@@ -289,17 +288,16 @@ class RleIntBlockBuilder final : public BlockBuilder {
 
  private:
   typedef typename TypeTraits<IntType>::cpp_type CppType;
-
   enum {
     kCppTypeSize = TypeTraits<IntType>::size
   };
 
+  const WriterOptions* const options_;
   CppType first_key_;
   CppType last_key_;
   faststring buf_;
   size_t count_;
-  RleEncoder<CppType> rle_encoder_;
-  const WriterOptions* const options_;
+  RleEncoder<CppType, kCppTypeSize * 8> rle_encoder_;
 };
 
 //
@@ -321,10 +319,10 @@ class RleIntBlockDecoder final : public BlockDecoder {
         cur_idx_(0) {
   }
 
-  virtual Status ParseHeader() OVERRIDE {
-    CHECK(!parsed_);
+  Status ParseHeader() override {
+    DCHECK(!parsed_);
 
-    if (data_.size() < kRleBitmapBlockHeaderSize) {
+    if (PREDICT_FALSE(data_.size() < kRleBitmapBlockHeaderSize)) {
       return Status::Corruption(
           "not enough bytes for header in RleIntBlockDecoder");
     }
@@ -334,17 +332,17 @@ class RleIntBlockDecoder final : public BlockDecoder {
 
     parsed_ = true;
 
-    rle_decoder_ = RleDecoder<CppType>(data_.data() + kRleBitmapBlockHeaderSize,
-                                       data_.size() - kRleBitmapBlockHeaderSize,
-                                       kCppTypeSize * 8);
+    rle_decoder_ = RleDecoder<CppType, kCppTypeSize * 8>(
+        data_.data() + kRleBitmapBlockHeaderSize,
+        data_.size() - kRleBitmapBlockHeaderSize);
 
     SeekToPositionInBlock(0);
 
     return Status::OK();
   }
 
-  virtual void SeekToPositionInBlock(uint pos) OVERRIDE {
-    CHECK(parsed_) << "Must call ParseHeader()";
+  void SeekToPositionInBlock(uint pos) override {
+    DCHECK(parsed_) << "Must call ParseHeader()";
     DCHECK_LE(pos, num_elems_)
         << "Tried to seek to " << pos << " which is > number of elements ("
         << num_elems_ << ") in the block!";
@@ -360,15 +358,15 @@ class RleIntBlockDecoder final : public BlockDecoder {
       uint nskip = pos - cur_idx_;
       rle_decoder_.Skip(nskip);
     } else {
-      rle_decoder_ = RleDecoder<CppType>(data_.data() + kRleBitmapBlockHeaderSize,
-                                         data_.size() - kRleBitmapBlockHeaderSize,
-                                         kCppTypeSize * 8);
+      rle_decoder_ = RleDecoder<CppType, kCppTypeSize * 8>(
+          data_.data() + kRleBitmapBlockHeaderSize,
+          data_.size() - kRleBitmapBlockHeaderSize);
       rle_decoder_.Skip(pos);
     }
     cur_idx_ = pos;
   }
 
-  virtual Status SeekAtOrAfterValue(const void *value_void, bool *exact_match) OVERRIDE {
+  Status SeekAtOrAfterValue(const void* value_void, bool* exact_match) override {
     // Currently using linear search as we do not check whether a
     // mid-point of a buffer will fall on a literal or not.
     //
@@ -402,7 +400,7 @@ class RleIntBlockDecoder final : public BlockDecoder {
     return Status::NotFound("not in block");
   }
 
-  virtual Status CopyNextValues(size_t *n, ColumnDataView *dst) OVERRIDE {
+  Status CopyNextValues(size_t* n, ColumnDataView* dst) override {
     DCHECK(parsed_);
 
     DCHECK_LE(*n, dst->nrows());
@@ -460,7 +458,7 @@ class RleIntBlockDecoder final : public BlockDecoder {
           if (!sel->TestBit(row_idx)) {
             continue;
           }
-          *(reinterpret_cast<CppType *>(data_ptr)) = val;
+          *(reinterpret_cast<CppType*>(data_ptr)) = val;
         }
       } else {
         // Mark that the rows will not be returned.
@@ -477,19 +475,19 @@ class RleIntBlockDecoder final : public BlockDecoder {
     return Status::OK();
   }
 
-  virtual bool HasNext() const OVERRIDE {
+  bool HasNext() const override {
     return cur_idx_ < num_elems_;
   }
 
-  virtual size_t Count() const OVERRIDE {
+  size_t Count() const override {
     return num_elems_;
   }
 
-  virtual size_t GetCurrentIndex() const OVERRIDE {
+  size_t GetCurrentIndex() const override {
     return cur_idx_;
   }
 
-  virtual rowid_t GetFirstRowId() const OVERRIDE {
+  rowid_t GetFirstRowId() const override {
     return ordinal_pos_base_;
   };
  private:
@@ -505,10 +503,8 @@ class RleIntBlockDecoder final : public BlockDecoder {
   uint32_t num_elems_;
   rowid_t ordinal_pos_base_;
   size_t cur_idx_;
-  RleDecoder<CppType> rle_decoder_;
+  RleDecoder<CppType, kCppTypeSize * 8> rle_decoder_;
 };
 
 } // namespace cfile
 } // namespace kudu
-
-#endif
