@@ -23,7 +23,6 @@
 #include <functional>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <set>
@@ -162,7 +161,6 @@ struct tm;
 namespace kudu {
 
 class BlockBloomFilter;
-class simple_spinlock;
 
 namespace client {
 
@@ -324,6 +322,11 @@ KuduClientBuilder& KuduClientBuilder::trusted_certificate(const string& cert_pem
   return *this;
 }
 
+KuduClientBuilder& KuduClientBuilder::rpc_max_message_size(int64_t size) {
+  data_->rpc_max_message_size_ = size;
+  return *this;
+}
+
 KuduClientBuilder& KuduClientBuilder::num_reactors(int num_reactors) {
   data_->num_reactors_ = num_reactors;
   return *this;
@@ -387,6 +390,9 @@ Status KuduClientBuilder::Build(shared_ptr<KuduClient>* client) {
     builder.set_rpc_negotiation_timeout_ms(
         data_->connection_negotiation_timeout_.ToMilliseconds());
   }
+  if (data_->rpc_max_message_size_) {
+    builder.set_rpc_max_message_size(*data_->rpc_max_message_size_);
+  }
   if (data_->num_reactors_) {
     builder.set_num_reactors(*data_->num_reactors_);
   }
@@ -848,17 +854,17 @@ Status KuduClient::ExportAuthenticationCredentials(string* authn_creds) const {
 }
 
 string KuduClient::GetHiveMetastoreUris() const {
-  std::lock_guard<simple_spinlock> l(data_->leader_master_lock_);
+  std::lock_guard l(data_->leader_master_lock_);
   return data_->hive_metastore_uris_;
 }
 
 bool KuduClient::GetHiveMetastoreSaslEnabled() const {
-  std::lock_guard<simple_spinlock> l(data_->leader_master_lock_);
+  std::lock_guard l(data_->leader_master_lock_);
   return data_->hive_metastore_sasl_enabled_;
 }
 
 string KuduClient::GetHiveMetastoreUuid() const {
-  std::lock_guard<simple_spinlock> l(data_->leader_master_lock_);
+  std::lock_guard l(data_->leader_master_lock_);
   return data_->hive_metastore_uuid_;
 }
 
@@ -935,6 +941,12 @@ KuduTableCreator& KuduTableCreator::set_comment(const string& comment) {
   return *this;
 }
 
+KuduTableCreator& KuduTableCreator::set_allow_empty_partition(
+    bool allow_empty_partition) {
+  data_->allow_empty_partition_ = allow_empty_partition;
+  return *this;
+}
+
 KuduTableCreator& KuduTableCreator::split_rows(const vector<const KuduPartialRow*>& rows) {
   for (const KuduPartialRow* row : rows) {
     data_->range_partition_splits_.emplace_back(const_cast<KuduPartialRow*>(row));
@@ -1021,6 +1033,10 @@ Status KuduTableCreator::Create() {
   if (data_->comment_) {
     req.set_comment(*data_->comment_);
   }
+  if (data_->allow_empty_partition_) {
+    req.set_allow_empty_partition(*data_->allow_empty_partition_);
+  }
+
   RETURN_NOT_OK_PREPEND(SchemaToPB(*data_->schema_->schema_, req.mutable_schema(),
                                    SCHEMA_PB_WITHOUT_WRITE_DEFAULT),
                         "Invalid schema");
@@ -1053,7 +1069,6 @@ Status KuduTableCreator::Create() {
 
   auto* partition_schema = req.mutable_partition_schema();
   partition_schema->CopyFrom(data_->partition_schema_);
-
   for (const auto& p : data_->range_partitions_) {
     const auto* range = p->data_;
     if (!range->lower_bound_ || !range->upper_bound_) {
@@ -1817,6 +1832,17 @@ Status KuduScanner::SetProjectedColumns(const vector<string>& col_names) {
   return SetProjectedColumnNames(col_names);
 }
 
+Status KuduScanner::SetQueryId(const string& query_id) {
+  if (data_->open_) {
+    return Status::IllegalState("Query id must be set before Open()");
+  }
+  if (query_id.empty()) {
+    return Status::InvalidArgument("Query id should not be empty");
+  }
+  data_->next_req_.set_query_id(query_id);
+  return Status::OK();
+}
+
 Status KuduScanner::SetProjectedColumnNames(const vector<string>& col_names) {
   if (data_->open_) {
     return Status::IllegalState("Projection must be set before Open()");
@@ -2064,6 +2090,11 @@ Status KuduScanner::Open() {
   MonoTime deadline = MonoTime::Now() + data_->configuration().timeout();
   set<string> blacklist;
 
+  if (data_->next_req_.query_id().empty()) {
+    static ObjectIdGenerator oid_generator;
+    data_->next_req_.set_query_id(oid_generator.Next());
+  }
+
   RETURN_NOT_OK(data_->OpenNextTablet(deadline, &blacklist));
 
   data_->open_ = true;
@@ -2074,6 +2105,15 @@ Status KuduScanner::KeepAlive() {
   return data_->KeepAlive();
 }
 
+Status KuduScanner::StartKeepAlivePeriodically(uint64_t keep_alive_interval_ms) {
+  return data_->StartKeepAlivePeriodically(keep_alive_interval_ms,
+                                           data_->table_->client()->data_->messenger_);
+}
+
+void KuduScanner::StopKeepAlivePeriodically() {
+  data_->StopKeepAlivePeriodically();
+}
+
 void KuduScanner::Close() {
   if (!data_->open_) return;
 
@@ -2103,10 +2143,14 @@ void KuduScanner::Close() {
 
 bool KuduScanner::HasMoreRows() const {
   CHECK(data_->open_);
-  return !data_->short_circuit_ &&                 // The scan is not short circuited
+  bool has_more = !data_->short_circuit_ &&        // The scan is not short circuited
       (data_->data_in_open_ ||                     // more data in hand
        data_->last_response_.has_more_results() || // more data in this tablet
        data_->MoreTablets());                      // more tablets to scan, possibly with more data
+  if (!has_more) {
+    data_->StopKeepAlivePeriodically();
+  }
+  return has_more;
 }
 
 Status KuduScanner::NextBatch(vector<KuduRowResult>* rows) {
@@ -2361,6 +2405,14 @@ Status KuduScanTokenBuilder::SetCacheBlocks(bool cache_blocks) {
   return data_->mutable_configuration()->SetCacheBlocks(cache_blocks);
 }
 
+Status KuduScanTokenBuilder::SetQueryId(const string& query_id) {
+  if (query_id.empty()) {
+    return Status::InvalidArgument("Query id should not be empty");
+  }
+  data_->SetQueryId(query_id);
+  return Status::OK();
+}
+
 Status KuduScanTokenBuilder::Build(vector<KuduScanToken*>* tokens) {
   return data_->Build(tokens);
 }
