@@ -23,6 +23,7 @@
 #include <mutex>
 #include <ostream>
 #include <set>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -79,6 +80,7 @@ using kudu::security::SignedTokenPB;
 using kudu::tserver::TabletServerAdminServiceProxy;
 using kudu::tserver::TabletServerServiceProxy;
 using std::set;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
@@ -101,6 +103,11 @@ DEFINE_int32(client_tablet_locations_by_id_ttl_ms, 60 * 60 * 1000, // 60 minutes
 TAG_FLAG(client_tablet_locations_by_id_ttl_ms, advanced);
 TAG_FLAG(client_tablet_locations_by_id_ttl_ms, runtime);
 
+DEFINE_bool(prevent_kudu_3461_infinite_recursion, true,
+            "Whether or not to prevent infinite recursion caused due to stale "
+            "client metacache as described in KUDU-3461. Used for testing only!");
+TAG_FLAG(prevent_kudu_3461_infinite_recursion, unsafe);
+
 namespace kudu {
 namespace client {
 namespace internal {
@@ -137,7 +144,7 @@ void RemoteTabletServer::DnsResolutionFinished(const HostPort& hp,
         client->data_->messenger_, (*addrs)[0], hp.host());
 
   {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     proxy_ = std::move(proxy);
     admin_proxy_ = std::move(admin_proxy);
     proxy_->set_user_credentials(client->data_->user_credentials_);
@@ -192,7 +199,7 @@ void RemoteTabletServer::InitProxy(KuduClient* client, const StatusCallback& cb)
 void RemoteTabletServer::Update(const master::TSInfoPB& pb) {
   CHECK_EQ(pb.permanent_uuid(), uuid_);
 
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
 
   rpc_hostports_.clear();
   for (const HostPortPB& hostport_pb : pb.rpc_addresses()) {
@@ -211,25 +218,25 @@ const string& RemoteTabletServer::permanent_uuid() const {
 }
 
 string RemoteTabletServer::location() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return location_;
 }
 
 shared_ptr<TabletServerServiceProxy> RemoteTabletServer::proxy() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   CHECK(proxy_);
   return proxy_;
 }
 
 shared_ptr<TabletServerAdminServiceProxy> RemoteTabletServer::admin_proxy() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   DCHECK(admin_proxy_);
   return admin_proxy_;
 }
 
 string RemoteTabletServer::ToString() const {
   string ret = uuid_;
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (!rpc_hostports_.empty()) {
     strings::SubstituteAndAppend(&ret, " ($0)", rpc_hostports_[0].ToString());
   }
@@ -237,7 +244,7 @@ string RemoteTabletServer::ToString() const {
 }
 
 void RemoteTabletServer::GetHostPorts(vector<HostPort>* host_ports) const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   *host_ports = rpc_hostports_;
 }
 
@@ -271,7 +278,7 @@ Status RemoteTablet::Refresh(
   }
 
   // Adopt the data from the successful response.
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   replicas_ = std::move(replicas);
 
   stale_ = false;
@@ -279,6 +286,7 @@ Status RemoteTablet::Refresh(
 }
 
 void RemoteTablet::MarkStale() {
+  VLOG(2) << Substitute("Marking tablet stale, tablet id $0", tablet_id_);
   stale_ = true;
 }
 
@@ -288,7 +296,7 @@ bool RemoteTablet::stale() const {
 
 void RemoteTablet::MarkReplicaFailed(RemoteTabletServer *ts,
                                      const Status& status) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   VLOG(2) << "Tablet " << tablet_id_ << ": Current remote replicas in meta cache: "
           << ReplicasAsStringUnlocked();
   KLOG_EVERY_N_SECS(WARNING, 1) << Substitute(
@@ -303,7 +311,7 @@ void RemoteTablet::MarkReplicaFailed(RemoteTabletServer *ts,
 
 int RemoteTablet::GetNumFailedReplicas() const {
   int failed = 0;
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (const RemoteReplica& rep : replicas_) {
     if (rep.failed) {
       failed++;
@@ -313,7 +321,7 @@ int RemoteTablet::GetNumFailedReplicas() const {
 }
 
 RemoteTabletServer* RemoteTablet::LeaderTServer() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (const RemoteReplica& replica : replicas_) {
     if (!replica.failed && replica.role == RaftPeerPB::LEADER) {
       return replica.ts;
@@ -328,7 +336,7 @@ bool RemoteTablet::HasLeader() const {
 
 void RemoteTablet::GetRemoteTabletServers(vector<RemoteTabletServer*>* servers) const {
   servers->clear();
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (const RemoteReplica& replica : replicas_) {
     if (replica.failed) {
       continue;
@@ -339,7 +347,7 @@ void RemoteTablet::GetRemoteTabletServers(vector<RemoteTabletServer*>* servers)
 
 void RemoteTablet::GetRemoteReplicas(vector<RemoteReplica>* replicas) const {
   replicas->clear();
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (const auto& r : replicas_) {
     if (r.failed) {
       continue;
@@ -349,7 +357,7 @@ void RemoteTablet::GetRemoteReplicas(vector<RemoteReplica>* replicas) const {
 }
 
 void RemoteTablet::MarkTServerAsLeader(const RemoteTabletServer* server) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (RemoteReplica& replica : replicas_) {
     if (replica.ts == server) {
       replica.role = RaftPeerPB::LEADER;
@@ -361,7 +369,7 @@ void RemoteTablet::MarkTServerAsLeader(const RemoteTabletServer* server) {
 }
 
 void RemoteTablet::MarkTServerAsFollower(const RemoteTabletServer* server) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   for (RemoteReplica& replica : replicas_) {
     if (replica.ts == server) {
       replica.role = RaftPeerPB::FOLLOWER;
@@ -371,7 +379,7 @@ void RemoteTablet::MarkTServerAsFollower(const RemoteTabletServer* server) {
 }
 
 string RemoteTablet::ReplicasAsString() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return ReplicasAsStringUnlocked();
 }
 
@@ -461,9 +469,19 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
   RemoteTabletServer* leader = nullptr;
   if (!tablet_->stale()) {
     leader = tablet_->LeaderTServer();
+    if (VLOG_IS_ON(2)) {
+      if (leader) {
+        VLOG(2) << Substitute("Client entry of tablet $0 is fresh, leader uuid $1",
+                              tablet_->tablet_id(), leader->permanent_uuid());
+      } else {
+        VLOG(2) << Substitute("Client entry of tablet $0 is fresh (no leader).",
+                              tablet_->tablet_id());
+      }
+    }
+
     bool marked_as_follower = false;
     {
-      std::lock_guard<simple_spinlock> lock(lock_);
+      std::lock_guard lock(lock_);
       marked_as_follower = ContainsKey(followers_, leader);
     }
     if (leader && marked_as_follower) {
@@ -484,7 +502,7 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
       tablet_->GetRemoteTabletServers(&replicas);
       set<RemoteTabletServer*> followers_copy;
       {
-        std::lock_guard<simple_spinlock> lock(lock_);
+        std::lock_guard lock(lock_);
         followers_copy = followers_;
 
       }
@@ -501,6 +519,8 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
             << "Preemptively marking tserver " << leader->ToString()
             << " as leader in the meta cache.";
         tablet_->MarkTServerAsLeader(leader);
+      } else {
+        VLOG(1) << "Tablet " << tablet_->tablet_id() << ": No valid leader.";
       }
     }
   }
@@ -518,6 +538,32 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
   // shift the write to another tablet (i.e. if it's since been split).
   if (!leader) {
     if (table_) {
+      VLOG(2) << Substitute("Table $0: No valid leader, lookup tablet by key.",
+                            table_->name());
+      if (PREDICT_TRUE(FLAGS_prevent_kudu_3461_infinite_recursion)) {
+        // First check metacache for the tablet
+        scoped_refptr<RemoteTablet> remote_tablet;
+        Status fastpath_status = meta_cache_->FastLookupTabletByKey(table_,
+                                                                    tablet_->partition().begin(),
+                                                                    MetaCache::LookupType::kPoint,
+                                                                    &remote_tablet);
+        if (!fastpath_status.IsIncomplete()) {
+          VLOG(2) << Substitute("Explicit fastpath lookup succeeded(maybe), "
+                                "proceed with callback, table: $0",
+                                table_->name());
+          const auto& known_tablet_id = tablet_->tablet_id();
+          if (remote_tablet && remote_tablet->tablet_id() != known_tablet_id) {
+            // Skip further processing if tablet in question has turned invalid
+            LOG(INFO) << Substitute(
+                "tablet seems to be replaced: former ID $0, new ID $1 (table $2)",
+                known_tablet_id, remote_tablet->tablet_id(), table_->name());
+            callback(Status::InvalidArgument(
+                         Substitute("tablet ID $0 is not valid", known_tablet_id)),
+                     nullptr);
+            return;
+          }
+        }
+      }
       meta_cache_->LookupTabletByKey(
           table_,
           tablet_->partition().begin(),
@@ -528,6 +574,7 @@ void MetaCacheServerPicker::PickLeader(const ServerPickedCallback& callback,
             this->LookUpTabletCb(callback, deadline, s);
           });
     } else {
+      VLOG(1) << "No valid table or leader: lookup tablet by ID " << tablet_->tablet_id();
       meta_cache_->LookupTabletById(
           client_,
           tablet_->tablet_id(),
@@ -553,7 +600,7 @@ void MetaCacheServerPicker::MarkServerFailed(RemoteTabletServer* replica, const
 
 void MetaCacheServerPicker::MarkReplicaNotLeader(RemoteTabletServer* replica) {
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     followers_.insert(CHECK_NOTNULL(replica));
   }
 }
@@ -568,12 +615,14 @@ void MetaCacheServerPicker::LookUpTabletCb(const ServerPickedCallback& callback,
                                            const Status& status) {
   // Whenever we lookup the tablet, clear the set of followers.
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     followers_.clear();
   }
 
   // If we couldn't lookup the tablet call the user callback immediately.
   if (!status.ok()) {
+    VLOG(2) << Substitute("Tablet lookup failed, tablet id: $0, status: $1",
+                          tablet_->tablet_id(), status.ToString());
     callback(status, nullptr);
     return;
   }
@@ -916,6 +965,8 @@ void LookupRpc::SendRpcSlowPath() {
     req_.set_replica_type_filter(master::ANY_REPLICA);
   }
 
+  VLOG(2) << Substitute("Slowpathing RPC $0: refreshing our metadata from the Master",
+                        ToString());
   // Actually send the request.
   AsyncLeaderMasterRpc::SendRpc();
 }
@@ -1001,7 +1052,7 @@ Status MetaCache::ProcessGetTabletLocationsResponse(const string& tablet_id,
   const auto& tablet_lower_bound = tablet.partition().partition_key_start();
   const auto& tablet_upper_bound = tablet.partition().partition_key_end();
 
-  std::lock_guard<percpu_rwlock> l(lock_);
+  std::lock_guard l(lock_);
   for (const auto& ts_info : ts_infos) {
     UpdateTabletServerUnlocked(ts_info);
   }
@@ -1047,7 +1098,7 @@ Status MetaCache::ProcessGetTableLocationsResponse(const KuduTable* table,
   MonoTime expiration_time = MonoTime::Now() +
       MonoDelta::FromMilliseconds(resp.ttl_millis());
 
-  std::lock_guard<percpu_rwlock> l(lock_);
+  std::lock_guard l(lock_);
   TabletMap& tablets_by_key = LookupOrInsert(&tablets_by_table_and_key_,
                                              table->id(), TabletMap());
 
@@ -1212,17 +1263,22 @@ bool MetaCache::LookupEntryByKeyFastPath(const KuduTable* table,
   const TabletMap* tablets = FindOrNull(tablets_by_table_and_key_, table->id());
   if (PREDICT_FALSE(!tablets)) {
     // No cache available for this table.
+    VLOG(2) << Substitute("No cache available for table $0", table->name());
     return false;
   }
 
   const MetaCacheEntry* e = FindFloorOrNull(*tablets, partition_key);
   if (PREDICT_FALSE(!e)) {
     // No tablets with a start partition key lower than 'partition_key'.
+    VLOG(2) << Substitute("Table $0: No tablets found with a start key lower than input key.",
+                          table->name());
     return false;
   }
 
   // Stale entries must be re-fetched.
   if (e->stale()) {
+    VLOG(2) << Substitute("Table $0: Stale entry for tablet $1 found, must be re-fetched.",
+                          table->name(), e->tablet()->tablet_id());
     return false;
   }
 
@@ -1231,18 +1287,28 @@ bool MetaCache::LookupEntryByKeyFastPath(const KuduTable* table,
     return true;
   }
 
+  VLOG(2) << Substitute("Table $0: Fastpath lookup by key failed!", table->name());
   return false;
 }
 
+Status MetaCache::FastLookupTabletByKey(const KuduTable* table,
+                                        PartitionKey partition_key,
+                                        MetaCache::LookupType lookup_type,
+                                        scoped_refptr<RemoteTablet>* remote_tablet) {
+  return DoFastPathLookup(table, &partition_key, lookup_type, remote_tablet);
+}
+
 Status MetaCache::DoFastPathLookup(const KuduTable* table,
                                    PartitionKey* partition_key,
                                    MetaCache::LookupType lookup_type,
                                    scoped_refptr<RemoteTablet>* remote_tablet) {
+  static const string err_str = "No tablet covering the requested range partition";
   MetaCacheEntry entry;
   while (PREDICT_TRUE(LookupEntryByKeyFastPath(table, *partition_key, &entry))
          && (entry.is_non_covered_range() || entry.tablet()->HasLeader())) {
-    VLOG(4) << "Fast lookup: found " << entry.DebugString(table) << " for "
-            << DebugLowerBoundPartitionKey(table, *partition_key);
+    VLOG(4) << Substitute("Fast lookup: found $0 for $1",
+                          entry.DebugString(table),
+                          DebugLowerBoundPartitionKey(table, *partition_key));
     if (!entry.is_non_covered_range()) {
       if (remote_tablet) {
         *remote_tablet = entry.tablet();
@@ -1250,11 +1316,12 @@ Status MetaCache::DoFastPathLookup(const KuduTable* table,
       return Status::OK();
     }
     if (lookup_type == LookupType::kPoint || entry.upper_bound_partition_key().empty()) {
-      return Status::NotFound("No tablet covering the requested range partition",
-                              entry.DebugString(table));
+      VLOG(2) << Substitute(err_str);
+      return Status::NotFound(err_str, entry.DebugString(table));
     }
     *partition_key = entry.upper_bound_partition_key();
   }
+  VLOG(2) << Substitute("Fastpath lookup failed with incomplete status");
   return Status::Incomplete("");
 }
 
@@ -1291,7 +1358,7 @@ Status MetaCache::DoFastPathLookupById(const string& tablet_id,
 
 void MetaCache::ClearNonCoveredRangeEntries(const std::string& table_id) {
   VLOG(3) << "Clearing non-covered range entries of table " << table_id;
-  std::lock_guard<percpu_rwlock> l(lock_);
+  std::lock_guard l(lock_);
 
   TabletMap* tablets = FindOrNull(tablets_by_table_and_key_, table_id);
   if (PREDICT_FALSE(!tablets)) {
@@ -1310,7 +1377,7 @@ void MetaCache::ClearNonCoveredRangeEntries(const std::string& table_id) {
 
 void MetaCache::ClearCache() {
   VLOG(3) << "Clearing cache";
-  std::lock_guard<percpu_rwlock> l(lock_);
+  std::lock_guard l(lock_);
   ts_cache_.clear();
   tablets_by_id_.clear();
   tablets_by_table_and_key_.clear();
@@ -1409,6 +1476,9 @@ void MetaCache::LookupTabletByKey(const KuduTable* table,
     return;
   }
 
+  VLOG(2) << Substitute("Fastpath lookup failed with $0. Proceed with RPC lookup,"
+                        " table: $1",
+                        fastpath_status.ToString(), table->name());
   LookupRpc* rpc = new LookupRpc(this,
                                  callback,
                                  table,
