@@ -19,10 +19,11 @@
 
 #include <algorithm>
 #include <cstdint>
+#include <functional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
-#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -45,11 +46,13 @@
 #include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/connection.h"
+#include "kudu/rpc/messenger.h" // IWYU pragma: keep
+#include "kudu/rpc/periodic.h"
 #include "kudu/rpc/rpc.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/rpc/rpc_header.pb.h"
 #include "kudu/security/token.pb.h"
-#include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/async_util.h"
 #include "kudu/util/bitmap.h"
 #include "kudu/util/hexdump.h"
@@ -61,7 +64,9 @@ using google::protobuf::FieldDescriptor;
 using google::protobuf::Reflection;
 using kudu::rpc::ComputeExponentialBackoff;
 using kudu::rpc::CredentialsPolicy;
+using kudu::rpc::Messenger;
 using kudu::rpc::RpcController;
+using kudu::rpc::PeriodicTimer;
 using kudu::security::SignedTokenPB;
 using kudu::tserver::NewScanRequestPB;
 using kudu::tserver::RowFormatFlags;
@@ -90,6 +95,53 @@ KuduScanner::Data::Data(KuduTable* table)
 }
 
 KuduScanner::Data::~Data() {
+  if (keep_alive_timer_) {
+    keep_alive_timer_->Stop();
+  }
+}
+
+void KuduScanner::Data::KeepAliveResponseCallback::Run() {
+  if (!controller.status().ok()) {
+    LOG(WARNING) << Substitute("Failed to send keep-alive request: $0",
+                               controller.status().ToString());
+  }
+  delete this;
+}
+
+Status KuduScanner::Data::StartKeepAlivePeriodically(uint64_t keep_alive_interval_ms,
+                                                     std::shared_ptr<Messenger> messenger) {
+  if (!open_) return Status::IllegalState("Scanner was not open.");
+  if (keep_alive_timer_ && keep_alive_timer_->started()) {
+    return Status::OK();
+  }
+  keep_alive_timer_ = PeriodicTimer::Create(
+      std::move(messenger),
+      [&]() {
+        if (!open_) return Status::IllegalState("Scanner was not open.");
+        // If there is no scanner to keep alive, we still return Status::OK().
+        if (!last_response_.IsInitialized() || !last_response_.has_more_results() ||
+            !next_req_.has_scanner_id()) {
+          return Status::OK();
+        }
+        // 'cb' deletes itself upon completion.
+        auto* cb = new KeepAliveResponseCallback();
+        cb->request.set_scanner_id(next_req_.scanner_id());
+        cb->controller.set_timeout(configuration_.timeout());
+        // PeriodicTimer does not allow waiting, so calling ScannerKeepAliveAsync()
+        // instead of ScannerKeepAlive().
+        proxy_->ScannerKeepAliveAsync(cb->request, &cb->response, &cb->controller,
+                                      [cb]() { cb->Run(); });
+        return Status::OK();
+      },
+      MonoDelta::FromMilliseconds(keep_alive_interval_ms));
+  keep_alive_timer_->Start();
+  return Status::OK();
+}
+
+void KuduScanner::Data::StopKeepAlivePeriodically() {
+  if (keep_alive_timer_) {
+    keep_alive_timer_->Stop();
+  }
 }
 
 Status KuduScanner::Data::EnrichStatusMessage(Status s) const {
