@@ -50,6 +50,9 @@ DEFINE_bool(show_attributes, false,
 DEFINE_bool(show_column_comment, false,
             "Whether to show column comment.");
 
+DEFINE_bool(show_column_id, false,
+            "Whether to show column id.");
+
 MAKE_ENUM_LIMITS(kudu::client::KuduColumnStorageAttributes::EncodingType,
                  kudu::client::KuduColumnStorageAttributes::AUTO_ENCODING,
                  kudu::client::KuduColumnStorageAttributes::RLE);
@@ -638,9 +641,8 @@ KuduSchemaBuilder* KuduSchemaBuilder::SetNonUniquePrimaryKey(const vector<string
 }
 
 Status KuduSchemaBuilder::Build(KuduSchema* schema) {
-  vector<KuduColumnSchema> cols;
-  cols.resize(data_->specs.size(), KuduColumnSchema());
-  for (int i = 0; i < cols.size(); i++) {
+  vector<KuduColumnSchema> cols(data_->specs.size(), KuduColumnSchema());
+  for (auto i = 0; i < cols.size(); ++i) {
     RETURN_NOT_OK(data_->specs[i]->ToColumnSchema(&cols[i]));
   }
 
@@ -672,7 +674,7 @@ Status KuduSchemaBuilder::Build(KuduSchema* schema) {
 
     num_key_cols = 1;
     if (!data_->specs[single_key_col_idx]->data_->primary_key_unique) {
-      data_->AddAutoIncrementingColumn(&num_key_cols, &cols);
+      RETURN_NOT_OK(data_->AddAutoIncrementingColumn(&num_key_cols, &cols));
     }
   } else {
     // Build a map from name to index of all of the columns.
@@ -721,7 +723,7 @@ Status KuduSchemaBuilder::Build(KuduSchema* schema) {
     num_key_cols = key_col_indexes.size();
 
     if (!data_->key_cols_unique) {
-      data_->AddAutoIncrementingColumn(&num_key_cols, &cols);
+      RETURN_NOT_OK(data_->AddAutoIncrementingColumn(&num_key_cols, &cols));
     }
   }
 
@@ -900,19 +902,20 @@ KuduColumnTypeAttributes KuduColumnSchema::type_attributes() const {
 }
 
 KuduColumnStorageAttributes KuduColumnSchema::storage_attributes() const {
-  ColumnStorageAttributes storage_attributes = DCHECK_NOTNULL(col_)->attributes();
+  const ColumnStorageAttributes sa = DCHECK_NOTNULL(col_)->attributes();
+  // The code below is to convert from the values of PB-based EncodingType and
+  // and CompressionType enums to the corresponding values of enums publicly
+  // exposed via Kudu client API.
   KuduColumnStorageAttributes::EncodingType encoding_type;
-  KuduColumnStorageAttributes::StringToEncodingType(
-      kudu::EncodingType_Name(storage_attributes.encoding),
-      &encoding_type);
+  DCHECK_OK(KuduColumnStorageAttributes::StringToEncodingType(
+      kudu::EncodingType_Name(sa.encoding), &encoding_type));
   KuduColumnStorageAttributes::CompressionType compression_type;
-  KuduColumnStorageAttributes::StringToCompressionType(
-      kudu::CompressionType_Name(storage_attributes.compression),
-      &compression_type);
+  DCHECK_OK(KuduColumnStorageAttributes::StringToCompressionType(
+      kudu::CompressionType_Name(sa.compression), &compression_type));
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-  return KuduColumnStorageAttributes(encoding_type, compression_type,
-                                     storage_attributes.cfile_block_size);
+  return KuduColumnStorageAttributes(
+      encoding_type, compression_type, sa.cfile_block_size);
 #pragma GCC diagnostic pop
 }
 
@@ -1047,6 +1050,9 @@ string KuduSchema::ToString() const {
   if (FLAGS_show_column_comment) {
     mode |= Schema::ToStringMode::WITH_COLUMN_COMMENTS;
   }
+  if (FLAGS_show_column_id) {
+    mode |= Schema::ToStringMode::WITH_COLUMN_IDS;
+  }
   return schema_->ToString(mode);
 }
 
