@@ -34,6 +34,7 @@ KuduTable::Data::Data(shared_ptr<KuduClient> client,
                       string id,
                       int num_replicas,
                       string owner,
+                      string comment,
                       const KuduSchema& schema,
                       PartitionSchema partition_schema,
                       map<string, string> extra_configs)
@@ -42,6 +43,7 @@ KuduTable::Data::Data(shared_ptr<KuduClient> client,
       id_(std::move(id)),
       num_replicas_(num_replicas),
       owner_(std::move(owner)),
+      comment_(std::move(comment)),
       schema_(schema),
       partition_schema_(std::move(partition_schema)),
       extra_configs_(std::move(extra_configs)) {
