@@ -31,7 +31,9 @@
 #include <memory>
 #include <mutex>
 #include <ostream>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -150,6 +152,7 @@ using kudu::clock::internal::kIntervalNone;
 using kudu::clock::internal::RecordedResponse;
 using std::deque;
 using std::lock_guard;
+using std::shared_lock;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -402,7 +405,7 @@ class BuiltInNtp::ServerState {
   }
 
   void UpdateNextPoll(MonoTime next) {
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
 
     // Increment the counter of NTP packets sent.
     ++o_pkt_total_num_;
@@ -417,20 +420,20 @@ class BuiltInNtp::ServerState {
   }
 
   void TimeoutAndSwitchNextServer() {
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
     ++o_pkt_timedout_num_;
     ++addr_idx_;
   }
 
   void InvalidatePacket() {
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
     ++i_pkt_total_num_;
   }
 
   void RecordPacket(const RecordedResponse& rr) {
     VLOG(1) << Substitute("NTP from $0 ($1):  $2 +/- $3us", host_.ToString(),
                           rr.addr.ToString(), rr.offset_us, rr.error_us);
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
     ++i_pkt_total_num_;
     ++i_pkt_valid_num_;
     responses_.emplace_back(rr);
@@ -440,7 +443,7 @@ class BuiltInNtp::ServerState {
   }
 
   bool IsReplayedPacket(const Sockaddr& from, const NtpPacket& packet) const {
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
     if (responses_.empty()) {
       return false;
     }
@@ -485,7 +488,7 @@ class BuiltInNtp::ServerState {
     vector<Sockaddr> addrs;
     auto s = host_.ResolveAddresses(&addrs);
     {
-      lock_guard<rw_spinlock> l(lock_);
+      lock_guard l(lock_);
       addrs_ = std::move(addrs);
     }
     return s;
@@ -545,7 +548,7 @@ BuiltInNtp::~BuiltInNtp() {
 }
 
 Status BuiltInNtp::Init() {
-  MutexLock l(state_lock_);
+  std::lock_guard l(state_lock_);
   CHECK_EQ(kUninitialized, state_);
 
   RETURN_NOT_OK(InitImpl());
@@ -666,13 +669,13 @@ Status BuiltInNtp::PopulateServers(std::vector<HostPort> servers) {
 }
 
 bool BuiltInNtp::is_shutdown() const {
-  MutexLock l(state_lock_);
+  std::lock_guard l(state_lock_);
   return state_ == kShutdown;
 }
 
 void BuiltInNtp::Shutdown() {
   {
-    MutexLock l(state_lock_);
+    std::lock_guard l(state_lock_);
     if (state_ == kShutdown) {
       return;
     }
@@ -683,7 +686,7 @@ void BuiltInNtp::Shutdown() {
   if (socket_.GetFd() >= 0) {
     // Shutting down the socket without closing it ensures that any attempt
     // to call sendmsg() will get EPIPE.
-    socket_.Shutdown(true, true);
+    WARN_NOT_OK(socket_.Shutdown(true, true), "coult not shutdown socket");
   }
   if (thread_) {
     thread_->Join();
@@ -1091,7 +1094,7 @@ Status BuiltInNtp::CombineClocks() {
   const int64_t compute_error = (best_interval.second - best_interval.first) / 2;
   {
     // Extra sanity check to make sure walltime doesn't go back.
-    std::lock_guard<rw_spinlock> l(last_computed_lock_);
+    std::lock_guard l(last_computed_lock_);
     if (last_computed_.wall > compute_wall) {
       return Status::IllegalState(Substitute(
           "walltime would move into past: "
@@ -1113,7 +1116,7 @@ Status BuiltInNtp::CombineClocks() {
 
   // We got a valid clock result, so wake up Init() that we are ready to be used.
   {
-    MutexLock l(state_lock_);
+    std::lock_guard l(state_lock_);
     if (state_ == kStarting) {
       state_ = kStarted;
     }
