@@ -22,20 +22,27 @@
 
 #include "kudu/clock/time_service.h"
 #include "kudu/gutil/macros.h"
+#include "kudu/gutil/ref_counted.h"
+#include "kudu/util/metrics.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
 namespace clock {
 
-// TimeService implementation which uses the 'ntp_adjtime' call (corresponding to the
-// 'adjtimex' syscall) to consult the Linux kernel for the current time
-// and error bound.
+// The SystemNtp is a TimeService implementation which uses the ntp_adjtime()
+// call [1] (similar to ntp_gettime() call [2] from the NTP kernel API, both
+// based on the adjtimex() syscall on Linux) to consult the Linux kernel
+// for the current time and error bound.
 //
-// This implementation relies on the ntpd service running on the local host
-// to keep the kernel's timekeeping up to date and in sync.
+// This implementation relies on the NTP service running on the local host
+// (implemented, for example, by chronyd, ntpd, etc.) to keep the system clock
+// synchronized with reference NTP time servers.
+//
+// [1] https://man7.org/linux/man-pages/man3/ntp_adjtime.3.html
+// [2] https://man7.org/linux/man-pages/man3/ntp_gettime.3.html
 class SystemNtp : public TimeService {
  public:
-  SystemNtp();
+  explicit SystemNtp(const scoped_refptr<MetricEntity>& metric_entity);
 
   Status Init() override;
 
@@ -48,10 +55,23 @@ class SystemNtp : public TimeService {
   void DumpDiagnostics(std::vector<std::string>* log) const override;
 
  private:
+  // Returns current timestamp, maxerror, and status as returned by the
+  // invocation of ntp_gettime()/ntp_adjtime() NTP kernel API call.
+  static std::string ClockNtpStatusForMetrics();
+
   // The maximum possible clock frequency skew rate reported by the kernel,
   // parts-per-million (PPM).
   int64_t skew_ppm_;
 
+  // Metric entity. Used to fetch information on NTP-related metrics upon
+  // calling DumpDiagnostics().
+  scoped_refptr<MetricEntity> metric_entity_;
+
+  // Metrics are set to detach to their last value. This means that, during
+  // running the destructor, there might be a need to access other class members
+  // declared above. Hence, this member must be declared last.
+  FunctionGaugeDetacher metric_detacher_;
+
   DISALLOW_COPY_AND_ASSIGN(SystemNtp);
 };
 
