@@ -75,7 +75,7 @@ class RowOperationsPBEncoder {
 struct DecodedRowOperation {
   RowOperationsPB::Type type;
 
-  // For INSERT, INSERT_IGNORE, or UPSERT, the whole projected row.
+  // For INSERT, INSERT_IGNORE, UPSERT, or UPSERT_IGNORE, the whole projected row.
   // For UPDATE, UPDATE_IGNORE, DELETE, or DELETE_IGNORE, the row key.
   const uint8_t* row_data;
 
@@ -93,6 +93,11 @@ struct DecodedRowOperation {
   // Per-row result status.
   Status result;
 
+  // True if an ignore op was ignored due to an error.
+  // As of now, the error could be one of the following:
+  // - UPDATE_IGNORE op on a row to update an immutable column.
+  bool error_ignored = false;
+
   // Stringifies, including redaction when appropriate.
   std::string ToString(const Schema& schema) const;
 
@@ -117,7 +122,8 @@ class RowOperationsPBDecoder {
   ~RowOperationsPBDecoder();
 
   template <DecoderMode mode>
-  Status DecodeOperations(std::vector<DecodedRowOperation>* ops);
+  Status DecodeOperations(std::vector<DecodedRowOperation>* ops,
+                          int64_t* auto_incrementing_counter = nullptr);
 
  private:
   Status ReadOpType(RowOperationsPB::Type* type);
@@ -143,7 +149,8 @@ class RowOperationsPBDecoder {
 
   Status DecodeInsertOrUpsert(const uint8_t* prototype_row_storage,
                               const ClientServerMapping& mapping,
-                              DecodedRowOperation* op);
+                              DecodedRowOperation* op,
+                              int64_t* auto_incrementing_counter);
   //------------------------------------------------------------
   // Serialization/deserialization support
   //------------------------------------------------------------
@@ -160,7 +167,8 @@ class RowOperationsPBDecoder {
   // Returns an error if the type isn't allowed by the decoder mode.
   template <DecoderMode mode>
   Status DecodeOp(RowOperationsPB::Type type, const uint8_t* prototype_row_storage,
-                  const ClientServerMapping& mapping, DecodedRowOperation* op);
+                  const ClientServerMapping& mapping, DecodedRowOperation* op,
+                  int64_t* auto_incrementing_counter);
 
   const RowOperationsPB* const pb_;
   // If 'client_schema_' and 'tablet_schema_' are the same object, the mapping
