@@ -46,7 +46,6 @@
 
 using std::any_of;
 using std::max;
-using std::move;
 using std::pair;
 using std::string;
 using std::unordered_set;
@@ -61,7 +60,7 @@ void ScanSpec::AddPredicate(ColumnPredicate pred) {
     predicate->Merge(pred);
   } else {
     string column_name = pred.column().name();
-    predicates_.emplace(move(column_name), move(pred));
+    predicates_.emplace(std::move(column_name), std::move(pred));
   }
 }
 
@@ -234,8 +233,8 @@ void ScanSpec::PushPredicatesIntoPrimaryKeyBounds(const Schema& schema,
   ContiguousRow lower_key(&schema, lower_buf);
   ContiguousRow upper_key(&schema, upper_buf);
 
-  int lower_bound_predicates_pushed = key_util::PushLowerBoundPrimaryKeyPredicates(
-      predicates_, &lower_key, arena);
+  int lower_bound_predicates_pushed =
+      key_util::PushLowerBoundPrimaryKeyPredicates(predicates_, &lower_key);
   int upper_bound_predicates_pushed = key_util::PushUpperBoundPrimaryKeyPredicates(
       predicates_, &upper_key, arena);
 
@@ -341,7 +340,7 @@ void ScanSpec::LiftPrimaryKeyBounds(const Schema& schema, Arena* arena) {
                                               lower_bound_key_->raw_keys()[col_idx + 1],
                                               nullptr));
         }
-        AddPredicate(move(predicate));
+        AddPredicate(std::move(predicate));
       } else {
         auto pred = ColumnPredicate::InclusiveRange(column, lower, upper, arena);
         if (pred) AddPredicate(*pred);
