@@ -173,9 +173,10 @@ class PeerMessageQueue {
     // The peer's latest overall health status.
     HealthReportPB::HealthStatus last_overall_health_status;
 
-    // Throttler for how often we will log status messages pertaining to this
-    // peer (eg when it is lagging, etc).
-    std::shared_ptr<logging::LogThrottler> status_log_throttler;
+    // Throttlers for how often we will log status messages pertaining to this
+    // peer: when it is lagging, WAL are GC-ed and it cannot catch up.
+    std::shared_ptr<logging::LogThrottler> status_log_throttler_lag;
+    std::shared_ptr<logging::LogThrottler> status_log_throttler_wal_gc;
 
    private:
     // The last term we saw from a given peer.
@@ -267,7 +268,7 @@ class PeerMessageQueue {
   // WARNING: In order to avoid copying the same messages to every peer,
   // entries are added to 'request' via AddAllocated() methods.
   // The owner of 'request' is expected not to delete the request prior
-  // to removing the entries through ExtractSubRange() or any other method
+  // to removing the entries through UnsafeArenaExtractSubRange() or any other method
   // that does not delete the entries. The simplest way is to pass the same
   // instance of ConsensusRequestPB to RequestForPeer(): the buffer will
   // replace the old entries with new ones without de-allocating the old
@@ -345,7 +346,7 @@ class PeerMessageQueue {
 
   void RegisterObserver(PeerMessageQueueObserver* observer);
 
-  Status UnRegisterObserver(PeerMessageQueueObserver* observer);
+  void UnRegisterObserver(PeerMessageQueueObserver* observer);
 
   struct Metrics {
     // Keeps track of the number of ops. that are completed by a majority but still need
@@ -464,13 +465,16 @@ class PeerMessageQueue {
   // response. Sets 'lmp_mismatch' to true if the given response indicates
   // there was a log-matching property mismatch on the remote, otherwise sets
   // it to false.
-  void UpdateExchangeStatus(TrackedPeer* peer, const TrackedPeer& prev_peer_state,
-                            const ConsensusResponsePB& response, bool* lmp_mismatch);
+  void UpdateExchangeStatus(TrackedPeer* peer,
+                            PeerStatus prev_last_exchange_status,
+                            const ConsensusResponsePB& response,
+                            bool* lmp_mismatch);
 
   // Check if the peer is a NON_VOTER candidate ready for promotion. If so,
   // trigger promotion.
-  void PromoteIfNeeded(TrackedPeer* peer, const TrackedPeer& prev_peer_state,
-                       const ConsensusStatusPB& status);
+  void PromoteIfNeeded(const TrackedPeer& peer,
+                       const ConsensusStatusPB& status,
+                       int64_t prev_last_received_index);
 
   // If there is a graceful leadership change underway, notify queue observers
   // to initiate leadership transfer to the specified peer under the following
@@ -590,6 +594,8 @@ class PeerMessageQueue {
 // The interface between RaftConsensus and the PeerMessageQueue.
 class PeerMessageQueueObserver {
  public:
+  virtual ~PeerMessageQueueObserver() = default;
+
   // Notify the observer that the commit index has advanced to 'committed_index'.
   virtual void NotifyCommitIndex(int64_t committed_index) = 0;
 
@@ -613,8 +619,6 @@ class PeerMessageQueueObserver {
 
   // Notify the observer that the health of one of the peers has changed.
   virtual void NotifyPeerHealthChange() = 0;
-
-  virtual ~PeerMessageQueueObserver() {}
 };
 
 }  // namespace consensus
