@@ -21,11 +21,13 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <ostream>
+#include <shared_mutex>
 #include <type_traits>
 #include <utility>
 
+#include <boost/iterator/iterator_facade.hpp>
+#include <boost/iterator/reverse_iterator.hpp>
 #include <boost/range/adaptor/reversed.hpp>
 #include <gflags/gflags.h>
 
@@ -177,6 +179,7 @@ DEFINE_validator(log_min_segments_to_retain, &ValidateLogsToRetain);
 using kudu::consensus::CommitMsg;
 using kudu::consensus::OpId;
 using kudu::consensus::ReplicateRefPtr;
+using std::shared_lock;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -298,7 +301,7 @@ void Log::AppendThread::Wake() {
 }
 
 void Log::SetActiveSegmentIdle() {
-  std::lock_guard<rw_spinlock> l(segment_idle_lock_);
+  std::lock_guard l(segment_idle_lock_);
   segment_allocator_.active_segment_->GoIdle();
 }
 
@@ -467,7 +470,7 @@ Status SegmentAllocator::AllocateOrRollOverIfNecessary(
   bool should_rollover = false;
   // if the size of this entry overflows the current segment, get a new one
   {
-    std::lock_guard<RWMutex> l(allocation_lock_);
+    std::lock_guard l(allocation_lock_);
     if (allocation_state_ == kAllocationNotStarted) {
       if ((active_segment_->written_offset() + write_size_bytes + 4) > max_segment_size_) {
         VLOG_WITH_PREFIX(1) << "Max segment size reached. Starting new segment allocation";
@@ -594,7 +597,7 @@ Status SegmentAllocator::AllocateSegmentAndRollOver(
     scoped_refptr<ReadableLogSegment>* finished_segment,
     scoped_refptr<ReadableLogSegment>* new_readable_segment) {
   {
-    std::lock_guard<RWMutex> l(allocation_lock_);
+    std::lock_guard l(allocation_lock_);
     RETURN_NOT_OK(AsyncAllocateSegmentUnlocked());
   }
   return RollOver(finished_segment, new_readable_segment);
@@ -604,7 +607,7 @@ void SegmentAllocator::SetSchemaForNextSegment(Schema schema,
                                                uint32_t version) {
   VLOG_WITH_PREFIX(2) << Substitute("Setting schema version $0 for next log segment $1",
                                     version, schema.ToString());
-  std::lock_guard<rw_spinlock> l(schema_lock_);
+  std::lock_guard l(schema_lock_);
   schema_ = std::move(schema);
   schema_version_ = version;
 }
@@ -627,7 +630,7 @@ Status SegmentAllocator::AllocateNewSegment() {
 
   // We must mark allocation as finished when returning from this method.
   auto alloc_finished = MakeScopedCleanup([&] () {
-    std::lock_guard<RWMutex> l(allocation_lock_);
+    std::lock_guard l(allocation_lock_);
     allocation_state_ = kAllocationFinished;
   });
 
@@ -638,7 +641,7 @@ Status SegmentAllocator::AllocateNewSegment() {
   string path_tmpl = JoinPathSegments(ctx_->log_dir, tmp_suffix);
   VLOG_WITH_PREFIX(2) << "Creating temp. file for place holder segment, template: " << path_tmpl;
   unique_ptr<RWFile> segment_file;
-  Env* env = ctx_->fs_manager->env();
+  Env* env = ctx_->fs_manager->GetEnv();
   RWFileOptions opts;
   opts.is_sensitive = true;
   RETURN_NOT_OK_PREPEND(env->NewTempRWFile(
@@ -671,7 +674,7 @@ Status SegmentAllocator::SwitchToAllocatedSegment(
   const auto& tablet_id = ctx_->tablet_id;
   string new_segment_path = ctx_->fs_manager->GetWalSegmentFileName(
       tablet_id, active_segment_sequence_number_);
-  Env* env = ctx_->fs_manager->env();
+  Env* env = ctx_->fs_manager->GetEnv();
   RETURN_NOT_OK_PREPEND(env->RenameFile(next_segment_path_, new_segment_path),
                         "could not rename next WAL segment");
   if (opts_->force_fsync_all) {
@@ -720,7 +723,7 @@ Status SegmentAllocator::SwitchToAllocatedSegment(
   // Now set 'active_segment_' to the new segment.
   active_segment_ = std::move(new_segment);
 
-  std::lock_guard<RWMutex> l(allocation_lock_);
+  std::lock_guard l(allocation_lock_);
   allocation_state_ = kAllocationNotStarted;
   return Status::OK();
 }
@@ -761,7 +764,7 @@ Status Log::Open(LogOptions options,
                  scoped_refptr<Log>* log) {
 
   string tablet_wal_path = fs_manager->GetTabletWalDir(tablet_id);
-  RETURN_NOT_OK(env_util::CreateDirIfMissing(fs_manager->env(), tablet_wal_path));
+  RETURN_NOT_OK(env_util::CreateDirIfMissing(fs_manager->GetEnv(), tablet_wal_path));
 
   LogContext ctx({ tablet_id, std::move(tablet_wal_path) });
   ctx.metric_entity = metric_entity;
@@ -789,7 +792,7 @@ Status Log::Init() {
   CHECK_EQ(kLogInitialized, log_state_);
 
   // Init the index.
-  log_index_.reset(new LogIndex(ctx_.fs_manager->env(),
+  log_index_.reset(new LogIndex(ctx_.fs_manager->GetEnv(),
                                 ctx_.file_cache,
                                 ctx_.log_dir));
 
@@ -866,7 +869,9 @@ Status Log::AsyncAppendReplicates(vector<ReplicateRefPtr> replicates,
       CreateBatchFromPB(REPLICATE, batch_pb, std::move(callback));
 
   for (LogEntryPB& entry : *batch_pb.mutable_entry()) {
-    entry.release_replicate();
+    // Release the ownership of ReplicateMsg in every entry since it's managed
+    // at the upper level where the 'replicates' parameter is passed from.
+    entry.unsafe_arena_release_replicate();
   }
 
   return AsyncAppend(std::move(batch));
@@ -884,8 +889,7 @@ Status Log::AsyncAppendCommit(const consensus::CommitMsg& commit_msg,
   unique_ptr<LogEntryBatch> entry_batch = CreateBatchFromPB(
       COMMIT, batch_pb, std::move(callback));
   entry->unsafe_arena_release_commit();
-  AsyncAppend(std::move(entry_batch));
-  return Status::OK();
+  return AsyncAppend(std::move(entry_batch));
 }
 
 Status Log::WriteBatch(LogEntryBatch* entry_batch) {
@@ -958,7 +962,7 @@ Status Log::UpdateIndexForBatch(const LogEntryBatch& batch,
 }
 
 Status Log::AllocateSegmentAndRollOverForTests() {
-  std::lock_guard<rw_spinlock> l(segment_idle_lock_);
+  std::lock_guard l(segment_idle_lock_);
   scoped_refptr<ReadableLogSegment> finished_segment;
   scoped_refptr<ReadableLogSegment> new_readable_segment;
   RETURN_NOT_OK(segment_allocator_.AllocateSegmentAndRollOver(
@@ -1013,7 +1017,7 @@ Status Log::Append(LogEntryPB* entry) {
   LogEntryBatchPB entry_batch_pb;
   entry_batch_pb.mutable_entry()->UnsafeArenaAddAllocated(entry);
   LogEntryBatch entry_batch(entry->type(), entry_batch_pb, &DoNothingStatusCB);
-  entry_batch_pb.mutable_entry()->ExtractSubrange(0, 1, nullptr);
+  entry_batch_pb.mutable_entry()->UnsafeArenaExtractSubrange(0, 1, nullptr);
   Status s = WriteBatch(&entry_batch);
   if (s.ok()) {
     s = Sync();
@@ -1029,7 +1033,7 @@ Status Log::WaitUntilAllFlushed() {
   Synchronizer s;
   unique_ptr<LogEntryBatch> reserved_entry_batch =
       CreateBatchFromPB(FLUSH_MARKER, entry_batch, s.AsStatusCallback());
-  AsyncAppend(std::move(reserved_entry_batch));
+  RETURN_NOT_OK(AsyncAppend(std::move(reserved_entry_batch)));
   return s.Wait();
 }
 
@@ -1043,7 +1047,7 @@ Status Log::GC(RetentionIndexes retention_indexes, int32_t* num_gced) {
     SegmentSequence segments_to_delete;
 
     {
-      std::lock_guard<percpu_rwlock> l(state_lock_);
+      std::lock_guard l(state_lock_);
       CHECK_EQ(kLogWriting, log_state_);
 
       GetSegmentsToGCUnlocked(retention_indexes, &segments_to_delete);
@@ -1075,7 +1079,7 @@ Status Log::GC(RetentionIndexes retention_indexes, int32_t* num_gced) {
         // segments_to_delete goes out of scope.
         RETURN_NOT_OK(ctx_.file_cache->DeleteFile(segment->path()));
       } else {
-        RETURN_NOT_OK(ctx_.fs_manager->env()->DeleteFile(segment->path()));
+        RETURN_NOT_OK(ctx_.fs_manager->GetEnv()->DeleteFile(segment->path()));
       }
       (*num_gced)++;
     }
@@ -1153,7 +1157,7 @@ Status Log::Close() {
   append_thread_->Shutdown();
 
   {
-    std::lock_guard<percpu_rwlock> l(state_lock_);
+    std::lock_guard l(state_lock_);
     switch (log_state_) {
       case kLogWriting:
         log_state_ = kLogClosed;
@@ -1180,12 +1184,12 @@ Status Log::Close() {
 
 bool Log::HasOnDiskData(FsManager* fs_manager, const string& tablet_id) {
   const string wal_dir = fs_manager->GetTabletWalDir(tablet_id);
-  return fs_manager->env()->FileExists(wal_dir);
+  return fs_manager->GetEnv()->FileExists(wal_dir);
 }
 
 Status Log::DeleteOnDiskData(FsManager* fs_manager, const string& tablet_id) {
   string wal_dir = fs_manager->GetTabletWalDir(tablet_id);
-  Env* env = fs_manager->env();
+  Env* env = fs_manager->GetEnv();
   if (!env->FileExists(wal_dir)) {
     return Status::OK();
   }
@@ -1213,7 +1217,7 @@ Status Log::RemoveRecoveryDirIfExists(FsManager* fs_manager, const string& table
   string tmp_path = Substitute("$0-$1", recovery_path, GetCurrentTimeMicros());
   VLOG(1) << kLogPrefix << "Renaming log recovery dir from "  << recovery_path
           << " to " << tmp_path;
-  RETURN_NOT_OK_PREPEND(fs_manager->env()->RenameFile(recovery_path, tmp_path),
+  RETURN_NOT_OK_PREPEND(fs_manager->GetEnv()->RenameFile(recovery_path, tmp_path),
                         Substitute("Could not rename old recovery dir from: $0 to: $1",
                                    recovery_path, tmp_path));
 
@@ -1225,7 +1229,7 @@ Status Log::RemoveRecoveryDirIfExists(FsManager* fs_manager, const string& table
   // We don't need to delete through the file cache; we're guaranteed that
   // the log has been closed (though this invariant isn't verifiable here
   // without additional plumbing).
-  RETURN_NOT_OK_PREPEND(fs_manager->env()->DeleteRecursively(tmp_path),
+  RETURN_NOT_OK_PREPEND(fs_manager->GetEnv()->DeleteRecursively(tmp_path),
                         "Could not remove renamed recovery dir " + tmp_path);
   VLOG(1) << kLogPrefix << "Completed deletion of old log recovery files and directory "
           << tmp_path;
