@@ -24,7 +24,9 @@
 #include <limits>
 #include <map>
 #include <memory>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 #include <glog/logging.h>
@@ -162,7 +164,7 @@ class SegmentAllocator {
   FRIEND_TEST(LogTest, TestAutoStopIdleAppendThread);
   FRIEND_TEST(LogTest, TestWriteAndReadToAndFromInProgressSegment);
   SegmentAllocationState allocation_state() {
-    shared_lock<RWMutex> l(allocation_lock_);
+    std::shared_lock<RWMutex> l(allocation_lock_);
     return allocation_state_;
   }
 
@@ -631,6 +633,7 @@ struct LogEntryBatchLogicalSize {
 
 class LogFaultHooks {
  public:
+  virtual ~LogFaultHooks() = default;
 
   // Executed immediately before returning from Log::Sync() at *ALL*
   // times.
@@ -645,8 +648,6 @@ class LogFaultHooks {
 
   virtual Status PreClose() { return Status::OK(); }
   virtual Status PostClose() { return Status::OK(); }
-
-  virtual ~LogFaultHooks() {}
 };
 
 }  // namespace log
