@@ -17,7 +17,6 @@
 
 #include "kudu/consensus/log_anchor_registry.h"
 
-#include <mutex>
 #include <ostream>
 #include <string>
 #include <utility>
@@ -46,14 +45,14 @@ LogAnchorRegistry::~LogAnchorRegistry() {
 void LogAnchorRegistry::Register(int64_t log_index,
                                  const string& owner,
                                  LogAnchor* anchor) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   RegisterUnlocked(log_index, owner, anchor);
 }
 
 Status LogAnchorRegistry::RegisterOrUpdate(int64_t log_index,
                                            const std::string& owner,
                                            LogAnchor* anchor) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (anchor->is_registered) {
     RETURN_NOT_OK(UnregisterUnlocked(anchor));
   }
@@ -62,18 +61,18 @@ Status LogAnchorRegistry::RegisterOrUpdate(int64_t log_index,
 }
 
 Status LogAnchorRegistry::Unregister(LogAnchor* anchor) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return UnregisterUnlocked(anchor);
 }
 
 Status LogAnchorRegistry::UnregisterIfAnchored(LogAnchor* anchor) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (!anchor->is_registered) return Status::OK();
   return UnregisterUnlocked(anchor);
 }
 
 Status LogAnchorRegistry::GetEarliestRegisteredLogIndex(int64_t* log_index) {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   auto iter = anchors_.begin();
   if (iter == anchors_.end()) {
     return Status::NotFound("No anchors in registry");
@@ -85,13 +84,13 @@ Status LogAnchorRegistry::GetEarliestRegisteredLogIndex(int64_t* log_index) {
 }
 
 size_t LogAnchorRegistry::GetAnchorCountForTests() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return anchors_.size();
 }
 
 std::string LogAnchorRegistry::DumpAnchorInfo() const {
   string buf;
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   MonoTime now = MonoTime::Now();
   for (const AnchorMultiMap::value_type& entry : anchors_) {
     const LogAnchor* anchor = entry.second;
@@ -156,17 +155,18 @@ MinLogIndexAnchorer::~MinLogIndexAnchorer() {
   CHECK_OK(ReleaseAnchor());
 }
 
-void MinLogIndexAnchorer::AnchorIfMinimum(int64_t log_index) {
-  std::lock_guard<simple_spinlock> l(lock_);
+Status MinLogIndexAnchorer::AnchorIfMinimum(int64_t log_index) {
+  std::lock_guard l(lock_);
   if (log_index < minimum_log_index_ ||
       PREDICT_FALSE(minimum_log_index_ == kInvalidOpIdIndex)) {
     minimum_log_index_ = log_index;
-    registry_->RegisterOrUpdate(minimum_log_index_, owner_, &anchor_);
+    return registry_->RegisterOrUpdate(minimum_log_index_, owner_, &anchor_);
   }
+  return Status::OK();
 }
 
 Status MinLogIndexAnchorer::ReleaseAnchor() {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   if (PREDICT_TRUE(minimum_log_index_ != kInvalidOpIdIndex)) {
     return registry_->Unregister(&anchor_);
   }
@@ -174,7 +174,7 @@ Status MinLogIndexAnchorer::ReleaseAnchor() {
 }
 
 int64_t MinLogIndexAnchorer::minimum_log_index() const {
-  std::lock_guard<simple_spinlock> l(lock_);
+  std::lock_guard l(lock_);
   return minimum_log_index_;
 }
 
