@@ -947,6 +947,7 @@ class RaftConsensus : public std::enable_shared_from_this<RaftConsensus>,
   scoped_refptr<Counter> follower_memory_pressure_rejections_;
   scoped_refptr<AtomicGauge<int64_t>> term_metric_;
   scoped_refptr<AtomicGauge<int64_t>> num_failed_elections_metric_;
+  scoped_refptr<Histogram> election_duration_metric_;
 
   // NOTE: it's important that this is the first member to be destructed. This
   // ensures we do not attempt to collect metrics while calling the destructor.
@@ -1003,7 +1004,7 @@ struct ConsensusBootstrapInfo {
 //   on a restart.
 class ConsensusRoundHandler {
  public:
-  virtual ~ConsensusRoundHandler() {}
+  virtual ~ConsensusRoundHandler() = default;
 
   virtual Status StartFollowerOp(const scoped_refptr<ConsensusRound>& context) = 0;
 
@@ -1043,7 +1044,7 @@ class ConsensusRound : public RefCountedThreadSafe<ConsensusRound> {
 
   // Returns the id of the (replicate) operation this context
   // refers to. This is only set _after_ RaftConsensus::Replicate(context).
-  OpId id() const {
+  const OpId& id() const {
     return replicate_msg_->get()->id();
   }
 
