@@ -37,9 +37,11 @@
 
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/data_dirs.h"
+#include "kudu/fs/dir_manager.h"
 #include "kudu/fs/error_manager.h"
 #include "kudu/fs/file_block_manager.h"
 #include "kudu/fs/fs.pb.h"
+#include "kudu/fs/fs_manager.h"
 #include "kudu/fs/fs_report.h"
 #include "kudu/fs/log_block_manager.h"
 #include "kudu/gutil/basictypes.h"
@@ -87,8 +89,10 @@ METRIC_DECLARE_gauge_uint64(block_manager_blocks_open_reading);
 METRIC_DECLARE_gauge_uint64(block_manager_blocks_open_writing);
 METRIC_DECLARE_counter(block_manager_total_writable_blocks);
 METRIC_DECLARE_counter(block_manager_total_readable_blocks);
+METRIC_DECLARE_counter(block_manager_total_blocks_deleted);
 METRIC_DECLARE_counter(block_manager_total_bytes_written);
 METRIC_DECLARE_counter(block_manager_total_bytes_read);
+METRIC_DECLARE_counter(log_block_manager_holes_punched);
 
 // Data directory metrics.
 METRIC_DECLARE_gauge_uint64(data_dirs_full);
@@ -96,9 +100,9 @@ METRIC_DECLARE_gauge_uint64(data_dirs_full);
 // The LogBlockManager is only supported on Linux, since it requires hole punching.
 #define RETURN_NOT_LOG_BLOCK_MANAGER() \
   do { \
-    if (FLAGS_block_manager != "log") { \
-      LOG(INFO) << "This platform does not use the log block manager by default. Skipping test."; \
-      return; \
+    if (!FsManager::IsLogType(FLAGS_block_manager)) { \
+      GTEST_SKIP() << "This platform does not use the log block manager by default. " \
+                      "Skipping test."; \
     } \
   } while (false)
 
@@ -108,13 +112,7 @@ namespace fs {
 static const char* kTestData = "test data";
 
 template<typename T>
-string block_manager_type();
-
-template<>
-string block_manager_type<FileBlockManager>() { return "file"; }
-
-template<>
-string block_manager_type<LogBlockManager>() { return "log"; }
+string block_manager_type() { return T::name(); }
 
 template <typename T>
 class BlockManagerTest : public KuduTest {
@@ -122,16 +120,16 @@ class BlockManagerTest : public KuduTest {
   BlockManagerTest() :
       test_tablet_name_("test_tablet"),
       test_block_opts_(CreateBlockOptions({ test_tablet_name_ })),
-      file_cache_("test_cache", env_, 1, scoped_refptr<MetricEntity>()),
-      bm_(CreateBlockManager(scoped_refptr<MetricEntity>(),
-                             shared_ptr<MemTracker>())) {
+      file_cache_("test_cache", env_, 1, scoped_refptr<MetricEntity>()) {
+    FLAGS_block_manager = T::name();
+    bm_.reset(CreateBlockManager(scoped_refptr<MetricEntity>(), shared_ptr<MemTracker>()));
     CHECK_OK(file_cache_.Init());
   }
 
   void SetUp() override {
     // Pass in a report to prevent the block manager from logging unnecessarily.
     FsReport report;
-    ASSERT_OK(bm_->Open(&report, nullptr, nullptr));
+    ASSERT_OK(bm_->Open(&report, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
     ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
     ASSERT_OK(dd_manager_->GetDataDirGroupPB(test_tablet_name_, &test_group_pb_));
   }
@@ -181,8 +179,9 @@ class BlockManagerTest : public KuduTest {
     BlockManagerOptions opts;
     opts.metric_entity = metric_entity;
     opts.parent_mem_tracker = parent_mem_tracker;
-    return new T(env_, this->dd_manager_.get(), &error_manager_,
-                 &file_cache_, std::move(opts));
+    error_manager_ = new FsErrorManager();
+    return new T(env_, dd_manager_, error_manager_,
+                 &file_cache_, std::move(opts), fs::kDefaultTenantName);
   }
 
   Status ReopenBlockManager(const scoped_refptr<MetricEntity>& metric_entity,
@@ -204,7 +203,7 @@ class BlockManagerTest : public KuduTest {
           env_, paths, opts, &dd_manager_));
     }
     bm_.reset(CreateBlockManager(metric_entity, parent_mem_tracker));
-    RETURN_NOT_OK(bm_->Open(nullptr, nullptr, nullptr));
+    RETURN_NOT_OK(bm_->Open(nullptr, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
 
     // Certain tests may maintain their own directory groups, in which case
     // the default test group should not be used.
@@ -221,11 +220,18 @@ class BlockManagerTest : public KuduTest {
   void RunBlockDistributionTest(const vector<string>& paths);
 
   static Status CountFilesCb(int* num_files, Env::FileType type,
-                      const string& /*dirname*/,
-                      const string& basename) {
+                             const string& dirname,
+                             const string& basename) {
     if (basename == kInstanceMetadataFileName) {
       return Status::OK();
     }
+#if !defined(NO_ROCKSDB)
+    // Ignore the 'kRocksDBDirName' directory which contains the RocksDB related files.
+    string parent_dir = *SplitPath(dirname).rbegin();
+    if (parent_dir == kRocksDBDirName) {
+      return Status::OK();
+    }
+#endif
     if (type == Env::FILE_TYPE) {
       *num_files += 1;
     }
@@ -233,7 +239,7 @@ class BlockManagerTest : public KuduTest {
   }
 
   // Utility function that counts the number of files within a directory
-  // hierarchy, ignoring '.', '..', and file 'kInstanceMetadataFileName'.
+  // hierarchy, ignoring '.', '..', 'kRocksDBDirName', and file 'kInstanceMetadataFileName'.
   Status CountFiles(const string& root, int* num_files) {
     *num_files = 0;
     return env_->Walk(
@@ -243,22 +249,26 @@ class BlockManagerTest : public KuduTest {
         });
   }
 
+  ~BlockManagerTest() override {
+    dd_manager_->WaitOnClosures();
+  }
+
   // Keep an internal copy of the data dir group to act as metadata.
   DataDirGroupPB test_group_pb_;
   string test_tablet_name_;
   CreateBlockOptions test_block_opts_;
-  FsErrorManager error_manager_;
-  unique_ptr<DataDirManager> dd_manager_;
+  scoped_refptr<FsErrorManager> error_manager_;
+  scoped_refptr<DataDirManager> dd_manager_;
   FileCache file_cache_;
-  unique_ptr<T> bm_;
+  scoped_refptr<T> bm_;
 };
 
 template <>
-void BlockManagerTest<LogBlockManager>::SetUp() {
+void BlockManagerTest<LogBlockManagerNativeMeta>::SetUp() {
   RETURN_NOT_LOG_BLOCK_MANAGER();
   // Pass in a report to prevent the block manager from logging unnecessarily.
   FsReport report;
-  ASSERT_OK(bm_->Open(&report, nullptr, nullptr));
+  ASSERT_OK(bm_->Open(&report, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
   ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
 
   // Store the DataDirGroupPB for tests that reopen the block manager.
@@ -297,8 +307,8 @@ void BlockManagerTest<FileBlockManager>::RunBlockDistributionTest(const vector<s
   }
 }
 
-template <>
-void BlockManagerTest<LogBlockManager>::RunBlockDistributionTest(const vector<string>& paths) {
+template<typename T>
+void BlockManagerTest<T>::RunBlockDistributionTest(const vector<string>& paths) {
   vector<int> files_in_each_path(paths.size());
   int num_blocks_per_dir = 30;
   // Spread across 1, then 3, then 5 data directories.
@@ -376,8 +386,8 @@ void BlockManagerTest<FileBlockManager>::RunMultipathTest(const vector<string>&
   ASSERT_EQ(20, num_blocks);
 }
 
-template <>
-void BlockManagerTest<LogBlockManager>::RunMultipathTest(const vector<string>& paths) {
+template<typename T>
+void BlockManagerTest<T>::RunMultipathTest(const vector<string>& paths) {
   // Write (3 * numPaths * 2) blocks, in groups of (numPaths * 2). That should
   // yield two containers per path.
   CreateBlockOptions opts({ "multipath_test" });
@@ -395,9 +405,7 @@ void BlockManagerTest<LogBlockManager>::RunMultipathTest(const vector<string>& p
   }
   ASSERT_OK(transaction->CommitCreatedBlocks());
 
-  // Verify the results. (numPaths * 2) containers were created, each
-  // consisting of 2 files. Thus, there should be a total of
-  // (numPaths * 4) files, ignoring '.', '..', and instance files.
+  // Verify the results.
   int sum = 0;
   for (const string& path : paths) {
     vector<string> children;
@@ -406,7 +414,22 @@ void BlockManagerTest<LogBlockManager>::RunMultipathTest(const vector<string>& p
     ASSERT_OK(CountFiles(path, &files_in_path));
     sum += files_in_path;
   }
-  ASSERT_EQ(paths.size() * 4, sum);
+
+  if (std::is_same<T, LogBlockManagerNativeMeta>::value) {
+    // (numPaths * 2) containers were created, each consisting of 2 files.
+    // Thus, there should be a total of (numPaths * 4) files, ignoring '.',
+    // '..', and instance files.
+    ASSERT_EQ(paths.size() * 4, sum);
+  } else {
+#if !defined(NO_ROCKSDB)
+    // (numPaths * 2) containers were created, each consisting of 1 file.
+    // Thus, there should be a total of (numPaths * 2) files, ignoring '.',
+    // '..', 'kRocksDBDirName', and instance files.
+    bool is_logr = std::is_same<T, LogBlockManagerRdbMeta>::value;
+    ASSERT_TRUE(is_logr);
+    ASSERT_EQ(paths.size() * 2, sum);
+#endif
+  }
 }
 
 template <>
@@ -426,8 +449,8 @@ void BlockManagerTest<FileBlockManager>::RunMemTrackerTest() {
   ASSERT_EQ(tracker->consumption(), initial_mem);
 }
 
-template <>
-void BlockManagerTest<LogBlockManager>::RunMemTrackerTest() {
+template<typename T>
+void BlockManagerTest<T>::RunMemTrackerTest() {
   shared_ptr<MemTracker> tracker = MemTracker::CreateTracker(-1, "test tracker");
   ASSERT_OK(ReopenBlockManager(scoped_refptr<MetricEntity>(),
                                tracker,
@@ -447,7 +470,12 @@ void BlockManagerTest<LogBlockManager>::RunMemTrackerTest() {
 
 // What kinds of BlockManagers are supported?
 #if defined(__linux__)
-typedef ::testing::Types<FileBlockManager, LogBlockManager> BlockManagers;
+#if defined(NO_ROCKSDB)
+typedef ::testing::Types<FileBlockManager, LogBlockManagerNativeMeta> BlockManagers;
+#else
+typedef ::testing::Types<FileBlockManager, LogBlockManagerNativeMeta, LogBlockManagerRdbMeta>
+    BlockManagers;
+#endif
 #else
 typedef ::testing::Types<FileBlockManager> BlockManagers;
 #endif
@@ -629,8 +657,8 @@ TYPED_TEST(BlockManagerTest, CloseManyBlocksTest) {
       for (int i = 0; i < sizeof(data); i += sizeof(uint32_t)) {
         data[i] = rand.Next();
       }
-      written_block->Append(Slice(data, sizeof(data)));
-      written_block->Finalize();
+      ASSERT_OK(written_block->Append(Slice(data, sizeof(data))));
+      ASSERT_OK(written_block->Finalize());
       creation_transaction->AddCreatedBlock(std::move(written_block));
     }
   }
@@ -756,10 +784,10 @@ TYPED_TEST(BlockManagerTest, PersistenceTest) {
   // The existing block manager is left open, which proxies for the process
   // having crashed without cleanly shutting down the block manager. The
   // on-disk metadata should still be clean.
-  unique_ptr<BlockManager> new_bm(this->CreateBlockManager(
+  scoped_refptr<BlockManager> new_bm(this->CreateBlockManager(
       scoped_refptr<MetricEntity>(),
       MemTracker::CreateTracker(-1, "other tracker")));
-  ASSERT_OK(new_bm->Open(nullptr, nullptr, nullptr));
+  ASSERT_OK(new_bm->Open(nullptr, BlockManager::MergeReport::NOT_REQUIRED, nullptr, nullptr));
 
   // Test that the state of all three blocks is properly reflected.
   unique_ptr<ReadableBlock> read_block;
@@ -1209,6 +1237,13 @@ TYPED_TEST(BlockManagerTest, ConcurrentCloseFinalizedWritableBlockTest) {
 }
 
 TYPED_TEST(BlockManagerTest, TestBlockTransaction) {
+  MetricRegistry registry;
+  scoped_refptr<MetricEntity> entity = METRIC_ENTITY_server.Instantiate(&registry, "test");
+  ASSERT_OK(this->ReopenBlockManager(entity,
+                                     shared_ptr<MemTracker>(),
+                                     { this->test_dir_ },
+                                     false /* create */));
+
   // Create a BlockCreationTransaction. In this transaction,
   // create some blocks and commit the writes all together.
   const string kTestData = "test data";
@@ -1249,6 +1284,26 @@ TYPED_TEST(BlockManagerTest, TestBlockTransaction) {
   }
   vector<BlockId> deleted_blocks;
   ASSERT_OK(deletion_transaction->CommitDeletedBlocks(&deleted_blocks));
+  deletion_transaction.reset();
+  this->dd_manager_->WaitOnClosures();
+
+  // Metric 'log_block_manager_holes_punched' is only updated when --block_manager
+  // is "log" or "logr".
+  int64_t holes_punched = 0;
+  if (FsManager::IsLogType(FLAGS_block_manager)) {
+    // Continuous blocks will be hole punched together, so 'log_block_manager_holes_punched' is
+    // less or equal to the size of 'deleted_blocks', but it must be greater than 0.
+    holes_punched = down_cast<Counter *>(
+        entity->FindOrNull(METRIC_log_block_manager_holes_punched).get())->value();
+    ASSERT_GE(deleted_blocks.size(), holes_punched);
+    ASSERT_GT(holes_punched, 0);
+  }
+
+  // Metric 'block_manager_total_blocks_deleted' is equal to the size of 'deleted_blocks'.
+  int64_t total_blocks_deleted = down_cast<Counter*>(
+      entity->FindOrNull(METRIC_block_manager_total_blocks_deleted).get())->value();
+  ASSERT_EQ(deleted_blocks.size(), total_blocks_deleted);
+
   for (const auto& block : deleted_blocks) {
     created_blocks.erase(std::remove(created_blocks.begin(), created_blocks.end(), block),
                          created_blocks.end());
@@ -1259,15 +1314,42 @@ TYPED_TEST(BlockManagerTest, TestBlockTransaction) {
   // in order to test that the first failure properly propagates.
   FLAGS_crash_on_eio = false;
   FLAGS_env_inject_eio = 1.0;
+  deletion_transaction = this->bm_->NewDeletionTransaction();
   for (const auto& block : created_blocks) {
     deletion_transaction->AddDeletedBlock(block);
   }
   deleted_blocks.clear();
   Status s = deletion_transaction->CommitDeletedBlocks(&deleted_blocks);
-  ASSERT_TRUE(s.IsIOError());
-  ASSERT_STR_CONTAINS(s.ToString(), Substitute("only deleted $0 blocks, "
-                                               "first failure",
-                                               deleted_blocks.size()));
+  deletion_transaction.reset();
+  this->dd_manager_->WaitOnClosures();
+
+  if (FsManager::IsLogType(FLAGS_block_manager)) {
+    // Metric 'log_block_manager_holes_punched' is not changed because IO error injected.
+    ASSERT_EQ(holes_punched, down_cast<Counter*>(
+        entity->FindOrNull(METRIC_log_block_manager_holes_punched).get())->value());
+  }
+
+  // Because the "logr" block_manager uses RocksDB to store metadata, and the IO error injected
+  // into util/env_posix.cc does not affect RocksDB, updating the metadata stored in the
+  // logr-based block manager succeeds without any errors.
+  if (FLAGS_block_manager == "logr") {
+#if defined(NO_ROCKSDB)
+    ASSERT_TRUE(false);
+#else
+    ASSERT_OK(s);
+    ASSERT_EQ(created_blocks.size(), deleted_blocks.size());
+    ASSERT_EQ(total_blocks_deleted + deleted_blocks.size(), down_cast<Counter*>(
+        entity->FindOrNull(METRIC_block_manager_total_blocks_deleted).get())->value());
+#endif
+  } else {
+    ASSERT_TRUE(s.IsIOError());
+    ASSERT_TRUE(deleted_blocks.empty());
+    ASSERT_EQ(total_blocks_deleted, down_cast<Counter*>(
+        entity->FindOrNull(METRIC_block_manager_total_blocks_deleted).get())->value());
+    ASSERT_STR_MATCHES(s.ToString(),
+                       Substitute("only 0/$0 blocks deleted, first failure",
+                                  created_blocks.size()));
+  }
 }
 
 } // namespace fs
