@@ -85,7 +85,7 @@ class DataDirsTest : public KuduTest {
       test_block_opts_(CreateBlockOptions({ test_tablet_name_ })),
       entity_(METRIC_ENTITY_server.Instantiate(&registry_, "test")) {}
 
-  virtual void SetUp() override {
+  void SetUp() override {
     KuduTest::SetUp();
     FLAGS_fs_target_data_dirs_per_tablet = kNumDirs / 2 + 1;
     DataDirManagerOptions opts;
@@ -110,7 +110,7 @@ class DataDirsTest : public KuduTest {
   const CreateBlockOptions test_block_opts_;
   MetricRegistry registry_;
   scoped_refptr<MetricEntity> entity_;
-  std::unique_ptr<DataDirManager> dd_manager_;
+  scoped_refptr<DataDirManager> dd_manager_;
 };
 
 TEST_F(DataDirsTest, TestCreateGroup) {
