@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 #include "kudu/fs/dir_util.h"
 
 #include <cstdint>
@@ -25,6 +26,7 @@
 #include <glog/logging.h>
 
 #include "kudu/fs/fs.pb.h"
+#include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -135,13 +137,14 @@ Status CheckHolePunch(Env* env, const string& path) {
 } while (0)
 
 DirInstanceMetadataFile::DirInstanceMetadataFile(Env* env,
-                                                   string uuid,
-                                                   string dir_type,
-                                                   string filename)
+                                                 string uuid,
+                                                 string dir_type,
+                                                 string filename)
     : env_(env),
       uuid_(std::move(uuid)),
       dir_type_(std::move(dir_type)),
-      filename_(std::move(filename)) {}
+      filename_(std::move(filename)) {
+}
 
 DirInstanceMetadataFile::~DirInstanceMetadataFile() {
   if (lock_) {
@@ -168,7 +171,7 @@ Status DirInstanceMetadataFile::Create(const set<string>& all_uuids,
 
   // If we're initializing the log block manager, check that we support
   // hole-punching.
-  if (dir_type_ == "log") {
+  if (FsManager::IsLogType(dir_type_)) {
     RETURN_NOT_OK_FAIL_INSTANCE_PREPEND(CheckHolePunch(env_, dir_name),
                                         kHolePunchErrorMsg);
   }
@@ -247,7 +250,7 @@ Status DirInstanceMetadataFile::LoadFromDisk() {
   uint64_t block_size;
   RETURN_NOT_OK_FAIL_INSTANCE_PREPEND(env_->GetBlockSize(filename_, &block_size),
       Substitute("Failed to load metadata file. Could not get block size of $0", filename_));
-  if (pb->filesystem_block_size_bytes() != block_size) {
+  if (FsManager::IsLogType(dir_type_) && pb->filesystem_block_size_bytes() != block_size) {
     return Status::IOError("Wrong filesystem block size", Substitute(
         "Expected $0 but was $1", pb->filesystem_block_size_bytes(), block_size));
   }
