@@ -18,6 +18,7 @@
 #pragma once
 
 #include <atomic>
+#include <cstddef>
 #include <cstdint>
 #include <deque>
 #include <map>
@@ -34,10 +35,11 @@
 
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
+#include "kudu/fs/data_dirs.h"
+#include "kudu/fs/error_manager.h"
 #include "kudu/fs/fs.pb.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/util/atomic.h"
 #include "kudu/util/locks.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/oid_generator.h"
@@ -49,14 +51,16 @@ class Env;
 class FileCache;
 
 namespace fs {
-class DataDirManager;
 class Dir;
-class FsErrorManager;
 struct FsReport;
 
 namespace internal {
 class LogBlock;
 class LogBlockContainer;
+class LogBlockContainerNativeMeta;
+#if !defined(NO_ROCKSDB)
+class LogBlockContainerRdbMeta;
+#endif
 class LogBlockDeletionTransaction;
 class LogWritableBlock;
 struct LogBlockContainerLoadResult;
@@ -65,6 +69,12 @@ struct LogBlockManagerMetrics;
 
 typedef scoped_refptr<internal::LogBlock> LogBlockRefPtr;
 typedef scoped_refptr<internal::LogBlockContainer> LogBlockContainerRefPtr;
+typedef scoped_refptr<internal::LogBlockContainerNativeMeta> LogBlockContainerNativeMetaRefPtr;
+#if !defined(NO_ROCKSDB)
+typedef scoped_refptr<internal::LogBlockContainerRdbMeta> LogBlockContainerRdbMetaRefPtr;
+#endif
+typedef std::unordered_map<std::string, std::vector<BlockRecordPB>> ContainerBlocksByName;
+typedef std::unordered_map<std::string, LogBlockContainerRefPtr> ContainersByName;
 
 // A log-backed (i.e. sequentially allocated file) block storage
 // implementation.
@@ -78,20 +88,18 @@ typedef scoped_refptr<internal::LogBlockContainer> LogBlockContainerRefPtr;
 // Implementation details
 // ----------------------
 //
-// A container is comprised of two files, one for metadata and one for
-// data. Both are written to sequentially. During a write, the block's data
-// is written as-is to the data file. After the block has been
-// synchronized, a small record is written to the metadata file containing
-// the block's ID and its location within the data file.
+// A container is comprised of two parts, one for metadata and one for
+// data. During a write, the block's data is written as-is to the data file.
+// After the block has been synchronized, a small record is written to the
+// metadata part containing the block's ID and its location within the data
+// file.
 //
 // Block deletions are handled similarly. When a block is deleted, a record
 // is written describing the deletion, orphaning the old block data. The
 // orphaned data can be reclaimed instantaneously via hole punching, or
 // later via garbage collection. The latter is used when hole punching is
 // not supported on the filesystem, or on next boot if there's a crash
-// after deletion but before hole punching. The metadata file itself is not
-// compacted, as it is expected to remain quite small even after a great
-// many create/delete cycles.
+// after deletion but before hole punching.
 //
 // Data and metadata operations are carefully ordered to ensure the
 // correctness of the persistent representation at all times. During the
@@ -150,15 +158,7 @@ typedef scoped_refptr<internal::LogBlockContainer> LogBlockContainerRefPtr;
 // collected every now and then, though newer systems can take advantage of
 // filesystem hole punching (as described above) to reclaim space.
 //
-// The on-disk container metadata design favors simplicity and contiguous
-// access over space consumption and scalability to a very large number of
-// blocks. To be more specific, the separation of metadata from data allows
-// for high performance sustained reads at block manager open time at a
-// manageability cost: a container is not a single file, and needs multiple
-// open fds to be of use. Moreover, the log-structured nature of the
-// metadata is simple and performant at open time.
-//
-// Likewise, the default container placement policy favors simplicity over
+// The default container placement policy favors simplicity over
 // performance. In the future, locality hints will ensure that blocks
 // pertaining to similar data are colocated, improving scan performance.
 //
@@ -185,17 +185,10 @@ class LogBlockManager : public BlockManager {
   static const char* kContainerMetadataFileSuffix;
   static const char* kContainerDataFileSuffix;
 
-  // Note: all objects passed as pointers should remain alive for the lifetime
-  // of the block manager.
-  LogBlockManager(Env* env,
-                  DataDirManager* dd_manager,
-                  FsErrorManager* error_manager,
-                  FileCache* file_cache,
-                  BlockManagerOptions opts);
-
   ~LogBlockManager() override;
 
-  Status Open(FsReport* report, std::atomic<int>* containers_processed,
+  Status Open(FsReport* report, MergeReport need_merage,
+              std::atomic<int>* containers_processed,
               std::atomic<int>* containers_total) override;
 
   Status CreateBlock(const CreateBlockOptions& opts,
@@ -204,6 +197,8 @@ class LogBlockManager : public BlockManager {
   Status OpenBlock(const BlockId& block_id,
                    std::unique_ptr<ReadableBlock>* block) override;
 
+  bool FindBlockPath(const BlockId& block_id, std::string* path) const override;
+
   std::unique_ptr<BlockCreationTransaction> NewCreationTransaction() override;
 
   std::shared_ptr<BlockDeletionTransaction> NewDeletionTransaction() override;
@@ -212,22 +207,40 @@ class LogBlockManager : public BlockManager {
 
   void NotifyBlockId(BlockId block_id) override;
 
-  FsErrorManager* error_manager() override { return error_manager_; }
+  scoped_refptr<FsErrorManager> error_manager() override { return error_manager_; }
 
- private:
+  std::string tenant_id() const override { return tenant_id_; }
+
+ protected:
+  // Note: all objects passed as pointers should remain alive for the lifetime
+  // of the block manager.
+  LogBlockManager(Env* env,
+                  scoped_refptr<DataDirManager> dd_manager,
+                  scoped_refptr<FsErrorManager> error_manager,
+                  FileCache* file_cache,
+                  BlockManagerOptions opts,
+                  std::string tenant_id);
+
+  FRIEND_TEST(LogBlockManagerNativeMetaTest,
+              TestContainerBlockLimitingByMetadataSizeWithCompaction);
+  FRIEND_TEST(LogBlockManagerNativeMetaTest, TestMetadataTruncation);
   FRIEND_TEST(LogBlockManagerTest, TestAbortBlock);
   FRIEND_TEST(LogBlockManagerTest, TestCloseFinalizedBlock);
   FRIEND_TEST(LogBlockManagerTest, TestCompactFullContainerMetadataAtStartup);
   FRIEND_TEST(LogBlockManagerTest, TestFinalizeBlock);
   FRIEND_TEST(LogBlockManagerTest, TestLIFOContainerSelection);
   FRIEND_TEST(LogBlockManagerTest, TestLookupBlockLimit);
-  FRIEND_TEST(LogBlockManagerTest, TestMetadataTruncation);
+  FRIEND_TEST(LogBlockManagerTest, TestMisalignedBlocksFuzz);
   FRIEND_TEST(LogBlockManagerTest, TestParseKernelRelease);
   FRIEND_TEST(LogBlockManagerTest, TestBumpBlockIds);
   FRIEND_TEST(LogBlockManagerTest, TestReuseBlockIds);
   FRIEND_TEST(LogBlockManagerTest, TestFailMultipleTransactionsPerContainer);
 
   friend class internal::LogBlockContainer;
+  friend class internal::LogBlockContainerNativeMeta;
+#if !defined(NO_ROCKSDB)
+  friend class internal::LogBlockContainerRdbMeta;
+#endif
   friend class internal::LogBlockDeletionTransaction;
   friend class internal::LogWritableBlock;
 
@@ -260,6 +273,22 @@ class LogBlockManager : public BlockManager {
       BlockIdHash,
       BlockIdEqual> BlockRecordMap;
 
+  // Creates a new LogBlockContainer in 'dir', 'container' will be set as the newly created
+  // container only if created successfully.
+  //
+  // Returns Status::OK() if created successfully, otherwise returns an error.
+  virtual Status CreateContainer(Dir* dir, LogBlockContainerRefPtr* container) = 0;
+
+  // Opens an existing LogBlockContainer in 'dir' identified by 'id', 'container' will be set as the
+  // opened container only if created successfully.
+  //
+  // Returns Status::OK() if created successfully, otherwise returns an error, the error is recorded
+  // in 'report'.
+  virtual Status OpenContainer(Dir* dir,
+                               FsReport* report,
+                               const std::string& id,
+                               LogBlockContainerRefPtr* container) = 0;
+
   // Adds an as of yet unseen container to this block manager.
   //
   // Must be called with 'lock_' held.
@@ -314,13 +343,13 @@ class LogBlockManager : public BlockManager {
   bool AddLogBlock(LogBlockRefPtr lb);
 
   // Removes the given set of LogBlocks from in-memory data structures, and
-  // appends the block deletion metadata to record the on-disk deletion.
+  // adds the block deletion metadata to record the on-disk deletion.
   // The 'log_blocks' out parameter will be set with the LogBlocks that were
   // successfully removed. The 'deleted' out parameter will be set with the
   // blocks were already deleted, e.g encountered 'NotFound' error during removal.
   //
   // Returns the first deletion failure that was seen, if any.
-  Status RemoveLogBlocks(std::vector<BlockId> block_ids,
+  Status RemoveLogBlocks(const std::vector<BlockId>& block_ids,
                          std::vector<LogBlockRefPtr>* log_blocks,
                          std::vector<BlockId>* deleted);
 
@@ -347,11 +376,28 @@ class LogBlockManager : public BlockManager {
                 FsReport* report,
                 std::vector<LogBlockRefPtr> need_repunching,
                 std::vector<LogBlockContainerRefPtr> dead_containers,
-                std::unordered_map<
-                    std::string,
-                    std::vector<BlockRecordPB>> low_live_block_containers);
+                const ContainerBlocksByName& low_live_block_containers);
 
-  // Rewrites a container metadata file, appending all entries in 'records'.
+  // Fetch all the containers we're going to repair.
+  //
+  // The issues to repair are described in 'report' and 'low_live_block_containers', the containers
+  // are returned by 'containers_by_name'.
+  virtual void FindContainersToRepair(FsReport* report,
+                                      const ContainerBlocksByName& low_live_block_containers,
+                                      ContainersByName* containers_by_name);
+
+  // Do the repair work.
+  //
+  // The following repairs will be performed:
+  // 1. Container data files in 'report->full_container_space_check' will be truncated.
+  //
+  // Returns an error if repairing a fatal inconsistency failed.
+  virtual Status DoRepair(Dir* dir,
+                          FsReport* report,
+                          const ContainerBlocksByName& low_live_block_containers,
+                          const ContainersByName& containers_by_name);
+
+  // Rewrites a container metadata file, adding all entries in 'records'.
   // The new metadata file is created as a temporary file and renamed over the
   // existing file after it is fully written.
   //
@@ -380,6 +426,9 @@ class LogBlockManager : public BlockManager {
                    std::atomic<int>* containers_processed = nullptr,
                    std::atomic<int>* containers_total = nullptr);
 
+  // Estimate the count of containers according to the count of children directories.
+  virtual size_t EstimateContainerCount(size_t children_count) const = 0;
+
   // Reads records from one log block container in the data directory.
   // The result details will be collected into 'result'.
   void LoadContainer(Dir* dir,
@@ -411,10 +460,10 @@ class LogBlockManager : public BlockManager {
 
   // Manages and owns the data directories in which the block manager will
   // place its blocks.
-  DataDirManager* dd_manager_;
+  scoped_refptr<DataDirManager> dd_manager_;
 
   // Manages callbacks used to handle disk failure.
-  FsErrorManager* error_manager_;
+  scoped_refptr<FsErrorManager> error_manager_;
 
   // The options that the LogBlockManager was created with.
   const BlockManagerOptions opts_;
@@ -475,16 +524,164 @@ class LogBlockManager : public BlockManager {
   ObjectIdGenerator oid_generator_;
 
   // For generating block IDs.
-  AtomicInt<uint64_t> next_block_id_;
+  std::atomic<uint64_t> next_block_id_;
 
   // Metrics for the block manager.
   //
   // May be null if instantiated without metrics.
   std::unique_ptr<internal::LogBlockManagerMetrics> metrics_;
 
+  // Which tenant this log block manager belongs to.
+  std::string tenant_id_;
+
   DISALLOW_COPY_AND_ASSIGN(LogBlockManager);
 };
 
+// The metadata part of a container is written to a native metadata file.
+// The metadata file itself is not compacted, as it is expected to remain quite
+// small even after many create/delete cycles.
+//
+// The on-disk container metadata design favors simplicity and contiguous
+// access over space consumption and scalability to a very large number of
+// blocks. To be more specific, the separation of metadata from data allows
+// for high performance sustained reads at block manager open time at a
+// manageability cost: a container is not a single file, and needs multiple
+// open fds to be of use. Moreover, the log-structured nature of the
+// metadata is simple and performant at open time.
+class LogBlockManagerNativeMeta : public LogBlockManager {
+ public:
+  static constexpr const char* const name() { return "log"; }
+
+  LogBlockManagerNativeMeta(Env* env,
+                            scoped_refptr<DataDirManager> dd_manager,
+                            scoped_refptr<FsErrorManager> error_manager,
+                            FileCache* file_cache,
+                            BlockManagerOptions opts,
+                            std::string tenant_id)
+      : LogBlockManager(env, std::move(dd_manager), std::move(error_manager),
+                        file_cache, std::move(opts), std::move(tenant_id)) {
+  }
+
+ private:
+  size_t EstimateContainerCount(size_t children_count) const override {
+    return children_count / 2;
+  }
+
+  Status CreateContainer(Dir* dir, LogBlockContainerRefPtr* container) override;
+
+  // Returns Status::Aborted() in the case that the metadata and data files
+  // both appear to have no data (e.g. due to a crash just after creating
+  // one of them but before writing any records).
+  Status OpenContainer(Dir* dir,
+                       FsReport* report,
+                       const std::string& id,
+                       LogBlockContainerRefPtr* container) override;
+
+  void FindContainersToRepair(FsReport* report,
+                              const ContainerBlocksByName& low_live_block_containers,
+                              ContainersByName* containers_by_name) override;
+
+  // Do the repair work.
+  //
+  // The following repairs will be performed:
+  // 1. Repairs in LogBlockManager::DoRepair().
+  // 2. Container metadata files in 'report->partial_record_check' will be truncated.
+  // 3. Container data and metadata files in 'report->incomplete_container_check' will be deleted.
+  // 4. Container metadata files in 'low_live_block_containers' will be compacted.
+  //
+  // Returns an error if repairing a fatal inconsistency failed.
+  Status DoRepair(Dir* dir,
+                  FsReport* report,
+                  const ContainerBlocksByName& low_live_block_containers,
+                  const ContainersByName& containers_by_name) override;
+
+private:
+  FRIEND_TEST(LogBlockManagerNativeMetaTest,
+              TestContainerBlockLimitingByMetadataSizeWithCompaction);
+
+  static bool ContainerShouldCompactForTests(internal::LogBlockContainer* container);
+};
+
+#if !defined(NO_ROCKSDB)
+// All the container's metadata is written into a RocksDB instance which is
+// shared by all containers in the same data directory.
+// The metadata records in RocksDB are all of CREATE type, the records are
+// deleted from RocksDB when the corresponding blocks are being removed from
+// the block manager. The data in the RocksDB instance is compacted in
+// background automatically, see https://github.com/facebook/rocksdb/wiki/Compaction.
+//
+// Comparing with the LogBlockManagerNativeMeta, the LogBlockManagerRdbMeta has a
+// better performance on opening containers, the latter only contains live block
+// records, while the former needs to scan through all the records in the metadata
+// which may adversely affect the performance when the live block ratio is very low.
+// RocksDB provides many configuration options, we can tune the performance, minimize
+// the read/write and space amplification.
+class LogBlockManagerRdbMeta : public LogBlockManager {
+ public:
+  static constexpr const char* const name() { return "logr"; }
+
+  LogBlockManagerRdbMeta(Env* env,
+                         scoped_refptr<DataDirManager> dd_manager,
+                         scoped_refptr<FsErrorManager> error_manager,
+                         FileCache* file_cache,
+                         BlockManagerOptions opts,
+                         std::string tenant_id)
+      : LogBlockManager(env, std::move(dd_manager), std::move(error_manager),
+                        file_cache, std::move(opts), std::move(tenant_id)) {
+  }
+
+ private:
+  friend class internal::LogBlockContainerRdbMeta;
+  friend class RdbMetadataLBMCorruptor;
+  FRIEND_TEST(LogBlockManagerRdbMetaTest, TestHalfPresentContainer);
+
+  size_t EstimateContainerCount(size_t children_count) const override {
+    // TODO(yingchun): exclude the kRocksDBDirName directory when get the children count.
+    return children_count;
+  }
+
+  Status CreateContainer(Dir* dir, LogBlockContainerRefPtr* container) override;
+
+  // Returns Status::Aborted() in the case that the data part appears to have no
+  // data (e.g. due to a crash just after creating it but before writing any
+  // records).
+  Status OpenContainer(Dir* dir,
+                       FsReport* report,
+                       const std::string& id,
+                       LogBlockContainerRefPtr* container) override;
+
+  void FindContainersToRepair(FsReport* report,
+                              const ContainerBlocksByName& low_live_block_containers,
+                              ContainersByName* containers_by_name) override;
+
+  // Do the repair work.
+  //
+  // The following repairs will be performed:
+  // 1. Repairs in LogBlockManager::DoRepair().
+  // 2. Container data files in 'report->incomplete_container_check' will be deleted, and the
+  //    records of these containers in RocksDB will be deleted as well.
+  // 3. Container metadata records in 'report->corrupted_rdb_record_check' will be deleted from
+  //    RocksDB.
+  //
+  // Returns an error if repairing a fatal inconsistency failed.
+  Status DoRepair(Dir* dir,
+                  FsReport* report,
+                  const ContainerBlocksByName& low_live_block_containers,
+                  const ContainersByName& containers_by_name) override;
+
+  // Delete all the metadata part of a container.
+  //
+  // The metadata records are prefixed by the container's 'id' of the RocksDB instance in 'dir'.
+  static Status DeleteContainerMetadata(Dir* dir, const std::string& id);
+
+  // Construct the key used in RocksDB.
+  //
+  // The key is constructed by the container's 'container_id' and the block's 'block_id'.
+  static std::string ConstructRocksDBKey(const std::string& container_id,
+                                         const BlockId& block_id);
+};
+#endif
+
 } // namespace fs
 } // namespace kudu
 
