@@ -101,6 +101,7 @@ void SleepForMilliseconds(int64_t milliseconds) {
   SleepForNanoseconds(milliseconds * 1000 * 1000);
 }
 
+#if !(defined(__MACH__) && defined(__APPLE__))
 // Helper function estimates cycles/sec by observing cycles elapsed during
 // sleep(). Using small sleep time decreases accuracy significantly.
 static int64 EstimateCyclesPerSecond(const int estimate_time_ms) {
@@ -114,6 +115,7 @@ static int64 EstimateCyclesPerSecond(const int estimate_time_ms) {
   const int64 guess = int64(multiplier * (CycleClock::Now() - start_ticks));
   return guess;
 }
+#endif
 
 // ReadIntFromFile is only called on linux and cygwin platforms.
 #if defined(__linux__) || defined(__CYGWIN__) || defined(__CYGWIN32__)
@@ -128,7 +130,9 @@ static bool SlurpSmallTextFile(const char* file, char* buf, int buflen) {
   bool ret = false;
   int fd;
   RETRY_ON_EINTR(fd, open(file, O_RDONLY));
-  if (fd == -1) return ret;
+  if (fd == -1) {
+    return ret;
+  }
 
   memset(buf, '\0', buflen);
   int n;
@@ -139,10 +143,8 @@ static bool SlurpSmallTextFile(const char* file, char* buf, int buflen) {
     ret = true;
   }
 
-  int close_ret;
-  RETRY_ON_EINTR(close_ret, close(fd));
-  if (PREDICT_FALSE(close_ret != 0)) {
-    PLOG(WARNING) << "Failed to close fd " << fd;
+  if (PREDICT_FALSE(close(fd) != 0)) {
+    PLOG(WARNING) << "error closing " << file;
   }
 
   return ret;
@@ -236,8 +238,8 @@ static void InitializeSystemInfo() {
   if (already_called)  return;
   already_called = true;
 
+#if defined(__linux__) || defined(__CYGWIN__) || defined(__CYGWIN32__)
   bool saw_mhz = false;
-
   if (RunningOnValgrind()) {
     // Valgrind may slow the progress of time artificially (--scale-time=N
     // option). We thus can't rely on CPU Mhz info stored in /sys or /proc
@@ -246,7 +248,6 @@ static void InitializeSystemInfo() {
     saw_mhz = true;
   }
 
-#if defined(__linux__) || defined(__CYGWIN__) || defined(__CYGWIN32__)
   char line[1024];
   char* err;
   int freq;
@@ -347,10 +348,9 @@ static void InitializeSystemInfo() {
       num_cpus++;  // count up every time we see an "processor :" entry
     }
   } while (chars_read > 0);
-  int ret;
-  RETRY_ON_EINTR(ret, close(fd));
-  if (PREDICT_FALSE(ret != 0)) {
-    PLOG(WARNING) << "Failed to close fd " << fd;
+
+  if (PREDICT_FALSE(close(fd) != 0)) {
+    PLOG(WARNING) << "error closing " << pname;
   }
 
   if (!saw_mhz) {
