@@ -33,6 +33,7 @@
 
 #include <glog/logging.h>
 #include <gtest/gtest.h>
+#include <rapidjson/document.h>
 
 #include "kudu/client/client.h"
 #include "kudu/client/schema.h"
@@ -43,6 +44,7 @@
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol-test-util.h"
 #include "kudu/gutil/mathlimits.h"
+#include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/cluster_itest_util.h"
 #include "kudu/integration-tests/external_mini_cluster-itest-base.h"
@@ -65,12 +67,18 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+using kudu::client::KuduClient;
+using kudu::client::KuduSchema;
+using kudu::client::sp::shared_ptr;
+using kudu::cluster::ClusterNodes;
+using kudu::tools::RunKuduTool;
 using std::multimap;
 using std::set;
 using std::string;
 using std::thread;
 using std::unique_ptr;
 using std::vector;
+using strings::Split;
 using strings::Substitute;
 
 METRIC_DECLARE_entity(server);
@@ -78,10 +86,6 @@ METRIC_DECLARE_histogram(handler_latency_kudu_tserver_TabletServerAdminService_C
 
 namespace kudu {
 
-using client::KuduClient;
-using client::KuduSchema;
-using cluster::ClusterNodes;
-
 const char* const kTableName = "test-table";
 
 class CreateTableITest : public ExternalMiniClusterITestBase {
@@ -458,7 +462,7 @@ TEST_F(CreateTableITest, TestSpreadReplicas) {
     rpc::RpcController rpc;
     tserver::ListTabletsRequestPB req;
     tserver::ListTabletsResponsePB resp;
-    cluster_->tserver_proxy(ts_idx)->ListTablets(req, &resp, &rpc);
+    ASSERT_OK(cluster_->tserver_proxy(ts_idx)->ListTablets(req, &resp, &rpc));
     for (auto i = 0; i < resp.status_and_schema_size(); ++i) {
       auto tablet_status = resp.status_and_schema(i).tablet_status();
       if (tablet_status.has_partition()) {
@@ -692,7 +696,7 @@ TEST_F(CreateTableITest, ProxyAdvertisedAddresses) {
   // Get information on table's locations via endpoint for proxied RPCs.
   {
     Sockaddr ma;
-    ma.ParseFromNumericHostPort(m_proxied_addr);
+    ASSERT_OK(ma.ParseFromNumericHostPort(m_proxied_addr));
     auto mp = std::make_shared<master::MasterServiceProxy>(
         cluster_->messenger(), ma, ma.host());
 
@@ -725,4 +729,251 @@ TEST_F(CreateTableITest, ProxyAdvertisedAddresses) {
   ASSERT_OK(client_->DeleteTable(kTableName));
 }
 
+class NotEnoughHealthyTServersTest :
+    public CreateTableITest,
+    public ::testing::WithParamInterface<bool> {
+};
+
+INSTANTIATE_TEST_SUITE_P(AddNewTS, NotEnoughHealthyTServersTest, ::testing::Bool());
+
+TEST_P(NotEnoughHealthyTServersTest, TestNotEnoughHealthyTServers) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+  const auto add_new_ts = GetParam();
+  constexpr const char* const kNotEnoughReplicasTableName = "kudu.not_enough_replicas";
+  constexpr const char* const kOverRegistedTSTableName = "kudu.over.registed.ts";
+  constexpr const char* const kFiveReplicaTableName = "kudu.five.replica";
+  constexpr const char* const kOneReplicaTableName = "kudu.one.replica";
+  constexpr int kNumTabletServers = 5;
+  constexpr int kTSUnresponsiveTimeoutMs = 4000;
+  constexpr int kHeartbeatIntervalMs = 3000;
+
+  // Do not validate the number of replicas.
+  vector<string> master_flags = {
+      "--catalog_manager_check_ts_count_for_create_table=false",
+      "--catalog_manager_check_ts_count_for_alter_table=false",
+      Substitute("--tserver_unresponsive_timeout_ms=$0", kTSUnresponsiveTimeoutMs),
+      Substitute("--heartbeat_interval_ms=$0", kHeartbeatIntervalMs),
+      "--allow_unsafe_replication_factor=true",
+      "--allow_creating_under_replicated_tables=true"
+  };
+  NO_FATALS(StartCluster({}, master_flags, kNumTabletServers));
+
+  const auto& master_address = cluster_->master()->bound_rpc_addr().ToString();
+
+  auto client_schema = KuduSchema::FromSchema(GetSimpleTestSchema());
+  auto create_table_func = [&](const string& table_name, int replica_num) -> Status {
+    unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
+    return table_creator->table_name(table_name)
+                  .schema(&client_schema)
+                  .set_range_partition_columns({ "key" })
+                  .num_replicas(replica_num)
+                  .Create();
+  };
+
+  // The number of replicas can't be over the number of registered tablet servers.
+  // RF = 6.
+  {
+    Status s = create_table_func(kOverRegistedTSTableName, 6);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "not enough registered tablet servers to");
+    shared_ptr<client::KuduTable> table;
+    s = client_->OpenTable(kOverRegistedTSTableName, &table);
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  }
+
+  {
+    // Shutdown 3 tablet servers.
+    for (int i = 0; i < 3; i++) {
+      cluster_->tablet_server(i)->Shutdown();
+    }
+    // Wait the 3 tablet servers heartbeat timeout and unresponsive timeout. Then catalog
+    // manager will take them as unavailable tablet servers. KSCK gets the status of tablet
+    // server from tablet serve interface. Here must wait the catalog manager to take the
+    // as unavailable.
+    SleepFor(MonoDelta::FromMilliseconds(3*(kTSUnresponsiveTimeoutMs + kHeartbeatIntervalMs)));
+  }
+
+  // RF = 5. Creating table will fail because there are not enough live tablet servers.
+  {
+    Status s = create_table_func(kOverRegistedTSTableName, 5);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "not enough live tablet servers to");
+  }
+
+  {
+    // Restart the first tablet server.
+    ASSERT_OK(cluster_->tablet_server(0)->Restart());
+    // Wait the restarted tablet server to send a heartbeat and be registered in catalog manager.
+    SleepFor(MonoDelta::FromMilliseconds(kHeartbeatIntervalMs));
+  }
+
+  // Create a table with RF=5. It should succeed.
+  ASSERT_OK(create_table_func(kFiveReplicaTableName, 5));
+
+  {
+    // Restart the second tablet server.
+    ASSERT_OK(cluster_->tablet_server(1)->Restart());
+    // Wait the restarted tablet server to send a heartbeat and be registered in catalog manager.
+    SleepFor(MonoDelta::FromMilliseconds(kHeartbeatIntervalMs));
+  }
+
+  // RF = 1.
+  ASSERT_OK(create_table_func(kOneReplicaTableName, 1));
+
+  // Create a five-replicas table.
+  ASSERT_OK(create_table_func(kNotEnoughReplicasTableName, 5));
+
+  // Add another column. Test alter table logic.
+  {
+    shared_ptr<KuduClient> client;
+    ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+    unique_ptr<client::KuduTableAlterer> table_alterer(
+        client_->NewTableAlterer(kNotEnoughReplicasTableName));
+    table_alterer->AddColumn("new_column")->Type(client::KuduColumnSchema::INT32);
+    ASSERT_OK(table_alterer->Alter());
+  }
+
+  {
+    const string cmd = Substitute(
+      "cluster ksck $0 --sections=TABLE_SUMMARIES --ksck_format=json_compact", master_address);
+    string out;
+    const auto s = RunKuduTool(Split(cmd, " ", strings::SkipEmpty()), &out);
+    // Since there are under-replicated tables, the ksck tool exits
+    // with non-zero code.
+    ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+    rapidjson::Document doc;
+    doc.Parse<0>(out.c_str());
+    ASSERT_EQ(3, doc["table_summaries"].Size());
+    const rapidjson::Value& items = doc["table_summaries"];
+    for (int i = 0; i < items.Size(); i++) {
+      if (string(kOneReplicaTableName) == items[i]["name"].GetString()) {
+        ASSERT_EQ(string("HEALTHY"), items[i]["health"].GetString());
+      } else {
+        ASSERT_EQ(string("UNDER_REPLICATED"), items[i]["health"].GetString());
+      }
+    }
+  }
+
+  if (add_new_ts) {
+    // Add one new tablet server.
+    ASSERT_OK(cluster_->AddTabletServer());
+  } else {
+    // Restart the stopped tablet server.
+    ASSERT_OK(cluster_->tablet_server(2)->Restart());
+  }
+
+  // All tables will become healthy.
+  {
+    ASSERT_EVENTUALLY([&] {
+      static const string kTableStatusHealthy = "HEALTHY";
+      const string cmd = Substitute(
+          "cluster ksck $0 --sections=TABLE_SUMMARIES --ksck_format=json_compact", master_address);
+      string out;
+      string in;
+      const auto s = (RunKuduTool(Split(cmd, " ", strings::SkipEmpty()), &out, nullptr, in));
+      if (add_new_ts) {
+        // Not all tablet servers are running, so the ksck tool exits
+        // with non-zero code.
+        ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+      } else {
+        ASSERT_OK(s);
+      }
+      rapidjson::Document doc;
+      doc.Parse<0>(out.c_str());
+      ASSERT_EQ(3, doc["table_summaries"].Size());
+      const rapidjson::Value& items = doc["table_summaries"];
+      for (int i = 0; i < items.Size(); i++) {
+        ASSERT_EQ(kTableStatusHealthy, items[i]["health"].GetString());
+      }
+    });
+  }
+}
+
+TEST_F(CreateTableITest, TestNotAffectedCreatingTables) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  constexpr int kTimeout = 4000;
+  constexpr int kNumTServers = 3;
+  constexpr int kHeartbeatIntervalMs = 2000;
+  constexpr const char* const kNotEnoughTServersTableId = "NotEnoughTServersTable";
+  constexpr const char* const kTwoReplicasTableId = "TwoReplicasTable";
+  constexpr const char* const kOneReplicasTableId = "OneReplicasTable";
+
+  vector<string> master_flags = { "--allow_unsafe_replication_factor=true",
+                                  Substitute("--tserver_unresponsive_timeout_ms=$0", kTimeout),
+                                  Substitute("--heartbeat_interval_ms=$0", kHeartbeatIntervalMs),
+                                  "--catalog_manager_check_ts_count_for_create_table=false" };
+  NO_FATALS(StartCluster({}, master_flags, kNumTServers));
+
+  string master_address = cluster_->master()->bound_rpc_addr().ToString();
+
+  // Shutdown one tablet server.
+  cluster_->tablet_server(0)->Shutdown();
+
+  // Wait until the tablet server become unavailable for replicas placement.
+  SleepFor(MonoDelta::FromMilliseconds(kTimeout + kHeartbeatIntervalMs));
+
+  auto client_schema = KuduSchema::FromSchema(GetSimpleTestSchema());
+
+  auto create_table_func = [&](const string& table_name, int replica_num) -> Status {
+    unique_ptr<client::KuduTableCreator> table_creator(
+        client_->NewTableCreator());
+    return table_creator->table_name(table_name)
+                        .schema(&client_schema)
+                        .set_range_partition_columns({ "key" })
+                        .num_replicas(replica_num)
+                        .timeout(MonoDelta::FromMilliseconds(kTimeout))
+                        .Create();
+  };
+
+  // Create a table without enough healthy tablet servers.
+  // It will timeout.
+  {
+    Status s = create_table_func(kNotEnoughTServersTableId, cluster_->num_tablet_servers());
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "Timed out waiting for Table Creation");
+  }
+  // Create a two-replicas table should succeed.
+  ASSERT_OK(create_table_func(kTwoReplicasTableId, 2));
+
+  // Create an one-replica table should succeed.
+  ASSERT_OK(create_table_func(kOneReplicasTableId, 1));
+
+  {
+    const string cmd = Substitute(
+      "cluster ksck $0 --sections=TABLE_SUMMARIES --ksck_format=json_compact", master_address);
+    string out;
+    const auto s = RunKuduTool(Split(cmd, " ", strings::SkipEmpty()), &out);
+    // Since not all registered tablet servers are up and running, the ksck tool
+    // exits with non-zero code.
+    ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+    rapidjson::Document doc;
+    doc.Parse<0>(out.c_str());
+    ASSERT_FALSE(doc.HasParseError());
+    ASSERT_TRUE(doc.HasMember("errors"));
+    ASSERT_FALSE(doc.HasMember("table_summaries"));
+  }
+  {
+    // Restart the tablet server and check the health of the cluster.
+    ASSERT_OK(cluster_->tablet_server(0)->Restart());
+    ASSERT_EVENTUALLY([&] {
+      string out;
+      string cmd = Substitute(
+          "cluster ksck $0 --sections=TABLE_SUMMARIES --ksck_format=json_compact",
+          master_address);
+      // KSCK will be OK.
+      ASSERT_OK(RunKuduTool(Split(cmd, " ", strings::SkipEmpty()), &out));
+      rapidjson::Document doc;
+      doc.Parse<0>(out.c_str());
+      ASSERT_FALSE(doc.HasParseError());
+      // Contains 3 tables kTwoReplicasTableId,
+      // kOneReplicasTableId and kNotEnoughTServersTableId.
+      ASSERT_TRUE(doc.HasMember("table_summaries"));
+      ASSERT_EQ(3, doc["table_summaries"].Size());
+      ASSERT_STR_CONTAINS(out, kTwoReplicasTableId);
+      ASSERT_STR_CONTAINS(out, kOneReplicasTableId);
+      ASSERT_STR_CONTAINS(out, kNotEnoughTServersTableId);
+    });
+  }
+}
 } // namespace kudu
