@@ -19,6 +19,7 @@
 #include <functional>
 #include <memory>
 #include <ostream>
+#include <set>
 #include <string>
 #include <tuple>
 #include <type_traits>
@@ -260,7 +261,7 @@ class TabletServerDiskErrorITest : public DiskErrorITestBase {
     NO_FATALS(writes.StopAndJoin());
 
     // Now add the last server.
-    cluster_->AddTabletServer();
+    ASSERT_OK(cluster_->AddTabletServer());
     for (int i = 0; i < cluster_->num_tablet_servers(); i++) {
       // Prevent attempts to copy over replicas, e.g. ones that don't get to a
       // running state due to an error.
@@ -332,38 +333,50 @@ TEST_P(TabletServerDiskErrorITest, TestSpaceAvailableMetrics) {
   // flags --fs_wal_dir_available_space_cache_seconds and
   // --fs_data_dirs_available_space_cache_seconds.
   const auto get_metrics = [&] (int64_t* wal_dir_space, int64_t* data_dir_space) {
-    RETURN_NOT_OK(itest::GetInt64Metric(cluster_->tablet_server(0)->bound_http_hostport(),
+    const auto& addr = cluster_->tablet_server(0)->bound_http_hostport();
+    RETURN_NOT_OK(itest::GetInt64Metric(addr,
                                         &METRIC_ENTITY_server, nullptr,
                                         &METRIC_wal_dir_space_available_bytes, "value",
                                         wal_dir_space));
-    return itest::GetInt64Metric(cluster_->tablet_server(0)->bound_http_hostport(),
+    return itest::GetInt64Metric(addr,
                                  &METRIC_ENTITY_server, nullptr,
                                  &METRIC_data_dirs_space_available_bytes, "value",
                                  data_dir_space);
-    };
-  int64_t wal_dir_space;
-  int64_t data_dir_space;
-  int64_t wal_dir_space_refetch;
-  int64_t data_dir_space_refetch;
-  ASSERT_OK(get_metrics(&wal_dir_space, &data_dir_space));
-  ASSERT_GT(wal_dir_space, 0);
-  ASSERT_GT(data_dir_space, 0);
-  ASSERT_OK(get_metrics(&wal_dir_space_refetch, &data_dir_space_refetch));
-  ASSERT_EQ(wal_dir_space, wal_dir_space_refetch);
-  ASSERT_EQ(data_dir_space, data_dir_space_refetch);
+  };
+
+  // Even with 10 seconds caching interval, it can happen that two metric
+  // snapshots are captured at different caching epochs, i.e. the metrics are
+  // refreshed in the cache between the two snapshots.  To avoid flakiness
+  // on busy test nodes due to scheduling anomalies or when running TSAN/ASAN
+  // binaries, the assertions below are wrapped into ASSERT_EVENTUALLY block.
+  ASSERT_EVENTUALLY([&] {
+    int64_t wal_dir_space = 0;
+    int64_t data_dir_space = 0;
+    ASSERT_OK(get_metrics(&wal_dir_space, &data_dir_space));
+    ASSERT_GT(wal_dir_space, 0);
+    ASSERT_GT(data_dir_space, 0);
+
+    int64_t wal_dir_space_refetch = 0;
+    int64_t data_dir_space_refetch = 0;
+    ASSERT_OK(get_metrics(&wal_dir_space_refetch, &data_dir_space_refetch));
+    ASSERT_EQ(wal_dir_space, wal_dir_space_refetch);
+    ASSERT_EQ(data_dir_space, data_dir_space_refetch);
+  });
 
   ExternalTabletServer* error_ts = cluster_->tablet_server(0);
 
-  // Inject EIO into one of the data directories and check if data_dirs_space_available_bytes
-  // now equals to -1
+  // Inject EIO into one of the data directories and check if
+  // data_dirs_space_available_bytes now equals to -1.
   error_ts->mutable_flags()->emplace_back(
-            Substitute("--env_inject_eio_globs=$0", JoinPathSegments(error_ts->data_dirs()[1],
-                                                                               "**")));
+      Substitute("--env_inject_eio_globs=$0",
+                 JoinPathSegments(error_ts->data_dirs()[1], "**")));
   error_ts->mutable_flags()->emplace_back("--env_inject_eio=1.0");
 
   error_ts->Shutdown();
   ASSERT_OK(error_ts->Restart());
 
+  int64_t wal_dir_space = 0;
+  int64_t data_dir_space = 0;
   ASSERT_OK(get_metrics(&wal_dir_space, &data_dir_space));
   ASSERT_NE(wal_dir_space, -1);
   ASSERT_EQ(data_dir_space, -1);
