@@ -20,6 +20,7 @@
 #include <map>
 #include <memory>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -50,6 +51,7 @@ using kudu::client::KuduTableCreator;
 using kudu::client::sp::shared_ptr;
 using kudu::cluster::ExternalMiniCluster;
 using kudu::hms::HmsClient;
+using std::optional;
 using std::string;
 using std::unique_ptr;
 using strings::Substitute;
@@ -118,7 +120,7 @@ Status HmsITestHarness::CreateKuduTable(const string& database_name,
 Status HmsITestHarness::CreateHmsTable(const string& database_name,
                                        const string& table_name,
                                        const string& table_type,
-                                       const boost::optional<const string&>& kudu_table_name) {
+                                       const optional<string>& kudu_table_name) {
   hive::Table hms_table;
   hms_table.dbName = database_name;
   hms_table.tableName = table_name;
@@ -201,7 +203,7 @@ Status HmsITestHarness::AlterHmsTableExternalPurge(const string& database_name,
 
 void HmsITestHarness::CheckTable(const string& database_name,
                                  const string& table_name,
-                                 const boost::optional<const string&>& user,
+                                 const optional<string>& user,
                                  const unique_ptr<ExternalMiniCluster>& cluster,
                                  const shared_ptr<KuduClient>& client,
                                  const string& table_type) {
@@ -238,6 +240,9 @@ void HmsITestHarness::CheckTable(const string& database_name,
             hms_table.parameters[hms::HmsClient::kKuduMasterAddrsKey]);
   ASSERT_EQ(hms::HmsClient::kKuduStorageHandler,
             hms_table.parameters[hms::HmsClient::kStorageHandlerKey]);
+  ASSERT_EQ(hms::HmsClient::kKuduInputFormat, hms_table.sd.inputFormat);
+  ASSERT_EQ(hms::HmsClient::kKuduOutputFormat, hms_table.sd.outputFormat);
+  ASSERT_EQ(hms::HmsClient::kKuduSerDeLib, hms_table.sd.serdeInfo.serializationLib);
 }
 
 void HmsITestHarness::CheckTableDoesNotExist(const string& database_name,
