@@ -67,6 +67,7 @@
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/tserver/tserver_service.pb.h"
 #include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/tools/tool_test_util.h"
 #include "kudu/util/curl_util.h"
 #include "kudu/util/env.h"
 #include "kudu/util/faststring.h"
@@ -100,6 +101,7 @@ using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::rpc::Messenger;
 using kudu::security::CreateTestSSLCertWithChainSignedByRoot;
 using kudu::security::CreateTestSSLExpiredCertWithChainSignedByRoot;
+using kudu::tools::RunKuduTool;
 using std::get;
 using std::string;
 using std::tuple;
@@ -457,6 +459,25 @@ TEST_F(SecurityITest, TestNoKerberosCredentials) {
                      "but client does not have Kerberos credentials available");
 }
 
+// Test trying to access the rebalance report without Kerberos credentials.
+TEST_F(SecurityITest, TestRebalanceReportUnauthorized) {
+  ASSERT_OK(StartCluster());
+  ASSERT_OK(cluster_->kdc()->Kdestroy());
+
+  const auto& master_addr = cluster_->master(0)->bound_rpc_addr().ToString();
+  string out;
+  string err;
+  Status s = RunKuduTool({
+    "cluster",
+    "rebalance",
+    master_addr,
+    "--report_only",
+  }, &out, &err);
+  ASSERT_TRUE(s.IsRuntimeError());
+  ASSERT_STR_CONTAINS(err,
+      "Not authorized: re-run ksck with administrator privileges");
+}
+
 // Regression test for KUDU-2121. Set up a Kerberized cluster with optional
 // authentication. An un-Kerberized client should be able to connect with SASL
 // PLAIN authentication.
@@ -806,9 +827,16 @@ TEST_F(SecurityITest, TestWorldReadableKeytab) {
   NO_FATALS(CreateWorldReadableFile(credentials_name));
   string binary = "kudu";
   NO_FATALS(GetFullBinaryPath(&binary));
+
+  // By default, KuduTest-based scenarios have their FLAGS_log_dir pointing to
+  // the TEST_TMPDIR directory, and the generated log files aren't cleaned up.
+  // So, customize the --log_dir flag to keep all the logs under the test's
+  // directory which is automatically removed once the scenario successfully
+  // completes.
   const vector<string> argv = { binary,
                                 "master",
                                 "run",
+                                Substitute("--log_dir=$0", GetTestDataDirectory()),
                                 Substitute("--keytab_file=$0", credentials_name) };
   string stderr;
   Status s = Subprocess::Call(argv, "", nullptr, &stderr);
@@ -822,9 +850,16 @@ TEST_F(SecurityITest, TestWorldReadablePrivateKey) {
   NO_FATALS(CreateWorldReadableFile(credentials_name));
   string binary = "kudu";
   NO_FATALS(GetFullBinaryPath(&binary));
+
+  // By default, KuduTest-based scenarios have their FLAGS_log_dir pointing to
+  // the TEST_TMPDIR directory, and the generated log files aren't cleaned up.
+  // So, customize the --log_dir flag to keep all the logs under the test's
+  // directory which is automatically removed once the scenario successfully
+  // completes.
   const vector<string> argv = { binary,
                                 "master",
                                 "run",
+                                Substitute("--log_dir=$0", GetTestDataDirectory()),
                                 "--unlock_experimental_flags",
                                 Substitute("--rpc_private_key_file=$0", credentials_name),
                                 "--rpc_certificate_file=fake_file",
@@ -873,6 +908,41 @@ TEST_F(SecurityITest, TestCorruptKerberosCC) {
   }
 }
 
+TEST_F(SecurityITest, TestNonDefaultPrincipalMultipleMaster) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+  cluster_opts_.principal = "oryx";
+
+  // Start with a single master setup and add two masters
+  ASSERT_OK(StartCluster());
+  ASSERT_OK(cluster_->master(0)->WaitForCatalogManager());
+  ASSERT_OK(cluster_->AddMaster());
+  ASSERT_OK(cluster_->master(1)->WaitForCatalogManager());
+  ASSERT_OK(cluster_->AddMaster());
+  ASSERT_OK(cluster_->master(2)->WaitForCatalogManager());
+
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(cluster_->VerifyVotersOnAllMasters(3));
+  });
+
+  // Tablet servers need to be restarted in order to update their master list.
+  for (int i = 0; i < cluster_->num_tablet_servers(); i++) {
+    cluster_->tablet_server(i)->Shutdown();
+    ASSERT_OK(cluster_->tablet_server(i)->Restart());
+  }
+
+  // Run a smoke test to verify that the cluster is working properly.
+  shared_ptr<KuduClient> client;
+  ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+  SmokeTestCluster(client);
+
+  // Shut down the leader master to simulate recoverable error.
+  cluster_->leader_master()->Shutdown();
+
+  // Run a smoke test to verify that the cluster is still working properly.
+  ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+  SmokeTestCluster(client);
+}
+
 TEST_F(SecurityITest, TestNonDefaultPrincipal) {
   const string kPrincipal = "oryx";
   cluster_opts_.principal = kPrincipal;
@@ -1157,6 +1227,64 @@ TEST_F(SecurityITest, IPKICACertWebServerEndPoints) {
   ASSERT_EQ(ca_cert_str_1, ca_cert_str_2);
 }
 
+TEST_F(SecurityITest, TestEnableAndDisableEncryptedIPKICert) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+  ASSERT_OK(StartCluster());
+
+  shared_ptr<KuduClient> client;
+  KuduClientBuilder b;
+  ASSERT_OK(cluster_->CreateClient(&b, &client));
+  NO_FATALS(SmokeTestCluster(client, /*transactional=*/false));
+
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    cluster_->master(i)->mutable_flags()->emplace_back("--ipki_private_key_password_cmd=echo foo");
+  }
+  cluster_->Shutdown();
+  ASSERT_OK(cluster_->Restart());
+
+  NO_FATALS(SmokeTestCluster(client, /*transactional=*/false));
+
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    cluster_->master(i)->mutable_flags()->pop_back();
+  }
+  cluster_->Shutdown();
+  // the master should fail to start, which means the tablet servers won't be
+  // able to connect to it and the mini cluster will time out waiting for the
+  // cluster to start up.
+  ASSERT_TRUE(cluster_->Restart().IsTimedOut());
+}
+
+TEST_F(SecurityITest, TestEnableAndDisableEncryptedTSK) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+  cluster_opts_.extra_master_flags.emplace_back("--authn_token_validity_seconds=20");
+  cluster_opts_.extra_master_flags.emplace_back("--authz_token_validity_seconds=20");
+  cluster_opts_.extra_master_flags.emplace_back("--tsk_rotation_seconds=20");
+  ASSERT_OK(StartCluster());
+
+  shared_ptr<KuduClient> client;
+  KuduClientBuilder b;
+  ASSERT_OK(cluster_->CreateClient(&b, &client));
+  NO_FATALS(SmokeTestCluster(client, /*transactional=*/false));
+
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    cluster_->master(i)->mutable_flags()->emplace_back("--tsk_private_key_password_cmd=echo foo");
+  }
+  cluster_->Shutdown();
+  SleepFor(MonoDelta::FromSeconds(20));
+  ASSERT_OK(cluster_->Restart());
+  NO_FATALS(SmokeTestCluster(client, /*transactional=*/false));
+
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    cluster_->master(i)->mutable_flags()->pop_back();
+  }
+  cluster_->Shutdown();
+  SleepFor(MonoDelta::FromSeconds(20));
+  // the master should fail to start, which means the tablet servers won't be
+  // able to connect to it and the mini cluster will time out waiting for the
+  // cluster to start up.
+  ASSERT_TRUE(cluster_->Restart().IsTimedOut());
+}
+
 class EncryptionPolicyTest :
     public SecurityITest,
     public ::testing::WithParamInterface<tuple<
@@ -1398,6 +1526,9 @@ TEST_P(ConnectToFollowerMasterTest, AuthnTokenVerifierHaveKeys) {
       Substitute("--rpc_encryption=$0", params.rpc_encryption));
   cluster_opts_.extra_tserver_flags.emplace_back(
       Substitute("--rpc_encryption=$0", params.rpc_encryption));
+  // Since leader falure detection is disabled, there is no leader at this point, so tablet servers
+  // can not be started.
+  cluster_opts_.num_tablet_servers = 0;
   ASSERT_OK(StartCluster());
 
   const auto& master_host = cluster_->master(0)->bound_rpc_addr().host();
