@@ -21,6 +21,7 @@
 #include <iterator>
 #include <memory>
 #include <ostream>
+#include <set>
 #include <string>
 #include <thread>
 #include <type_traits>
@@ -28,6 +29,7 @@
 #include <unordered_set>
 #include <vector>
 
+#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <glog/stl_logging.h>
 #include <gtest/gtest.h>
@@ -51,6 +53,7 @@
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/data_dirs.h"
+#include "kudu/fs/dir_manager.h"
 #include "kudu/fs/fs.pb.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/basictypes.h"
@@ -78,6 +81,8 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+DECLARE_string(block_manager);
+
 METRIC_DECLARE_gauge_uint64(tablets_num_failed);
 
 using kudu::client::KuduClient;
@@ -123,6 +128,9 @@ int IntToKey(int i) {
 class TsRecoveryITest : public ExternalMiniClusterITestBase,
                         public ::testing::WithParamInterface<string> {
  public:
+  TsRecoveryITest() {
+    FLAGS_block_manager = GetParam();
+  }
 
  protected:
   void StartClusterOneTs(vector<string> extra_tserver_flags = {},
@@ -134,7 +142,7 @@ void TsRecoveryITest::StartClusterOneTs(vector<string> extra_tserver_flags,
                                         vector<string> extra_master_flags) {
   ExternalMiniClusterOptions opts;
   opts.num_tablet_servers = 1;
-  opts.block_manager_type = GetParam();
+  opts.block_manager_type = FLAGS_block_manager;
   opts.extra_tserver_flags = std::move(extra_tserver_flags);
   opts.extra_master_flags = std::move(extra_master_flags);
   StartClusterWithOpts(opts);
@@ -146,7 +154,7 @@ void TsRecoveryITest::StartClusterOneTs(vector<string> extra_tserver_flags,
 // being unable to complete replay of the WAL and leaves the WAL recovery directory in place.
 // 3. The master should tombstone the FAILED replica, causing its recovery directory to be deleted.
 // A subsequent tablet copy and tablet bootstrap should cause the replica to become healthy again.
-TEST_F(TsRecoveryITest, TestTabletRecoveryAfterSegmentDelete) {
+TEST_P(TsRecoveryITest, TestTabletRecoveryAfterSegmentDelete) {
   // Start a cluster with 3 tablet servers consisting of 1 tablet with 3 replicas.
   // Configure a small log segment size to quickly create new log segments.
   // Since we want to write as quickly as possible, we disable WAL compression.
@@ -213,7 +221,7 @@ TEST_F(TsRecoveryITest, TestTabletRecoveryAfterSegmentDelete) {
     string segment = fs_manager->GetWalSegmentFileName(tablet_id, 2);
 
     LOG(INFO) << "Deleting WAL segment: " << segment;
-    ASSERT_OK(fs_manager->env()->DeleteFile(segment));
+    ASSERT_OK(fs_manager->GetEnv()->DeleteFile(segment));
   }
 
   ASSERT_OK(cluster_->Restart());
@@ -236,10 +244,9 @@ TEST_F(TsRecoveryITest, TestTabletRecoveryAfterSegmentDelete) {
 // Test for KUDU-2202 that ensures that blocks not found in the FS layer but
 // that are referenced by a tablet will not be reused.
 TEST_P(TsRecoveryITest, TestNoBlockIDReuseIfMissingBlocks) {
-  if (GetParam() != "log") {
-    LOG(INFO) << "Missing blocks is currently only supported by the log "
-                 "block manager. Exiting early!";
-    return;
+  if (!FsManager::IsLogType(FLAGS_block_manager)) {
+    GTEST_SKIP() << "Missing blocks is currently only supported by the log "
+                    "block manager. Exiting early!";
   }
   // Set up a basic server that flushes often so we create blocks quickly.
   NO_FATALS(StartClusterOneTs({
@@ -309,7 +316,8 @@ TEST_P(TsRecoveryITest, TestNoBlockIDReuseIfMissingBlocks) {
     vector<string> children;
     ASSERT_OK(env_->GetChildren(data_dir, &children));
     for (const string& child : children) {
-      if (child != "." && child != ".." && child != fs::kInstanceMetadataFileName) {
+      if (child != "." && child != ".." &&
+          child != fs::kInstanceMetadataFileName && child != fs::kRocksDBDirName) {
         ASSERT_OK(env_->DeleteFile(JoinPathSegments(data_dir, child)));
       }
     }
@@ -563,7 +571,13 @@ INSTANTIATE_TEST_SUITE_P(BlockManagerType, TsRecoveryITestDeathTest,
 TEST_P(TsRecoveryITestDeathTest, RecoverFromOpIdOverflow) {
   // Create the initial tablet files on disk, then shut down the cluster so we
   // can meddle with the WAL.
-  NO_FATALS(StartClusterOneTs());
+  NO_FATALS(StartClusterOneTs(
+      // To simplify test scaffolding, this scenario relies on lenient handling
+      // of operation indices read from the tablet's WAL upon bootstrapping.
+      // With regular guards in place, an inconsistency would be detected when
+      // setting committed OpId for PendingRounds by RaftConsensus::Start().
+      { "--raft_allow_committed_pending_index_gap" }
+  ));
   TestWorkload workload(cluster_.get());
   workload.set_num_replicas(1);
   workload.Setup();
@@ -587,7 +601,7 @@ TEST_P(TsRecoveryITestDeathTest, RecoverFromOpIdOverflow) {
     FsManagerOpts opts;
     opts.wal_root = ets->wal_dir();
     opts.data_roots = ets->data_dirs();
-    opts.block_manager_type = GetParam();
+    opts.block_manager_type = FLAGS_block_manager;
     unique_ptr<FsManager> fs_manager(new FsManager(env_, opts));
     ASSERT_OK(fs_manager->Open());
     scoped_refptr<ConsensusMetadataManager> cmeta_manager(
@@ -628,7 +642,7 @@ TEST_P(TsRecoveryITestDeathTest, RecoverFromOpIdOverflow) {
     // been written, thus avoiding the check.
     string wal_dir = fs_manager->GetTabletWalDir(tablet_id);
     vector<string> wal_children;
-    ASSERT_OK(fs_manager->env()->GetChildren(wal_dir, &wal_children));
+    ASSERT_OK(fs_manager->GetEnv()->GetChildren(wal_dir, &wal_children));
     // Skip '.', '..', and index files.
     std::unordered_set<string> wal_segments;
     for (const auto& filename : wal_children) {
@@ -640,9 +654,9 @@ TEST_P(TsRecoveryITestDeathTest, RecoverFromOpIdOverflow) {
                                       << wal_children;
     // If WAL segment index 1 exists, delete it.
     string first_segment = fs_manager->GetWalSegmentFileName(tablet_id, 1);
-    if (fs_manager->env()->FileExists(first_segment)) {
+    if (fs_manager->GetEnv()->FileExists(first_segment)) {
       LOG(INFO) << "Deleting first WAL segment: " << first_segment;
-      ASSERT_OK(fs_manager->env()->DeleteFile(first_segment));
+      ASSERT_OK(fs_manager->GetEnv()->DeleteFile(first_segment));
     }
 
     // We also need to update the ConsensusMetadata to match with the term we
@@ -781,7 +795,8 @@ class UpdaterThreads {
 // these updates would be mistakenly considered as "already flushed", despite
 // the fact that they were only written to the input rowset's memory stores, and
 // never hit disk.
-class Kudu969Test : public TsRecoveryITest {
+class Kudu969Test : public ExternalMiniClusterITestBase,
+                    public ::testing::WithParamInterface<string> {
 };
 INSTANTIATE_TEST_SUITE_P(DifferentFaultPoints,
                          Kudu969Test,
