@@ -26,7 +26,6 @@
 #include <iterator>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <numeric>
 #include <optional>
 #include <ostream>
@@ -35,6 +34,7 @@
 #include <string>
 #include <thread>
 #include <tuple>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -54,6 +54,7 @@
 #include "kudu/common/schema.h"
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/consensus.proxy.h"
+#include "kudu/gutil/basictypes.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
@@ -74,7 +75,7 @@
 #include "kudu/tserver/tablet_server.h"
 #include "kudu/tserver/ts_tablet_manager.h"
 #include "kudu/tserver/tserver.pb.h"
-#include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/barrier.h"
 #include "kudu/util/locks.h"
 #include "kudu/util/monotime.h"
@@ -618,7 +619,7 @@ TEST_F(TxnWriteOpsITest, FrequentElections) {
           CHECK_OK(txn->Rollback());
         }
         {
-          std::lock_guard<simple_spinlock> guard(transactions_lock);
+          std::lock_guard guard(transactions_lock);
           transactions.emplace_back(std::move(txn));
         }
       }
@@ -649,7 +650,7 @@ TEST_F(TxnWriteOpsITest, FrequentElections) {
         rpc.set_timeout(MonoDelta::FromSeconds(5));
         // A best effort call: the replica might already be a leader or electing
         // a new leader might fail, etc.
-        proxy.RunLeaderElection(req, &resp, &rpc);
+        ignore_result(proxy.RunLeaderElection(req, &resp, &rpc));
       }
       int sleep_time = rand() % static_cast<int>(max_sleep_ms);
       if (MonoTime::Now() > run_until) {
@@ -660,8 +661,7 @@ TEST_F(TxnWriteOpsITest, FrequentElections) {
       max_sleep_ms = std::min(max_sleep_ms * 1.1, 1000.0);
     }
   }
-  std::for_each(writers.begin(), writers.end(), [](thread& t) { t.join(); });
-  cleanup.cancel();
+  cleanup.run();
 
   NO_FATALS(cluster_->AssertNoCrashes());
   for (auto& txn : transactions) {
@@ -1067,7 +1067,7 @@ class TxnOpDispatcherITest : public KuduTest {
     }
     size_t elem_count = 0;
     for (auto& r : replicas) {
-      std::lock_guard<simple_spinlock> guard(r->txn_op_dispatchers_lock_);
+      std::lock_guard guard(r->txn_op_dispatchers_lock_);
       elem_count += r->txn_op_dispatchers_.size();
     }
     return elem_count;
@@ -1079,7 +1079,7 @@ class TxnOpDispatcherITest : public KuduTest {
     std::shared_ptr<typename TabletReplica::TxnOpDispatcher> d;
     size_t count = 0;
     for (auto& r : replicas) {
-      std::lock_guard<simple_spinlock> guard(r->txn_op_dispatchers_lock_);
+      std::lock_guard guard(r->txn_op_dispatchers_lock_);
       auto& dispatchers = r->txn_op_dispatchers_;
       if (!dispatchers.empty()) {
         d = dispatchers.begin()->second;
@@ -1097,7 +1097,7 @@ class TxnOpDispatcherITest : public KuduTest {
     auto replicas = GetAllReplicas(table_name);
     OpDispatchersPerTxnId result;
     for (auto& r : replicas) {
-      std::lock_guard<simple_spinlock> guard(r->txn_op_dispatchers_lock_);
+      std::lock_guard guard(r->txn_op_dispatchers_lock_);
       auto& dispatchers = r->txn_op_dispatchers_;
       for (auto& [txn_id, d] : dispatchers) {
         auto& dispatchers = LookupOrEmplace(&result, txn_id, OpDispatchers());
@@ -1184,7 +1184,7 @@ TEST_F(TxnOpDispatcherITest, LifecycleBasic) {
       for (const auto& [txn_id, dispatcher] : r->txn_op_dispatchers_) {
         ASSERT_EQ(ref_txn_id, txn_id);
         {
-          std::lock_guard<simple_spinlock> guard(dispatcher->lock_);
+          std::lock_guard guard(dispatcher->lock_);
           ASSERT_TRUE(dispatcher->preliminary_tasks_completed_);
           ASSERT_TRUE(dispatcher->ops_queue_.empty());
           ASSERT_FALSE(dispatcher->unregistered_);
@@ -1551,7 +1551,7 @@ TEST_F(TxnOpDispatcherITest, ErrorInProcessingWriteOp) {
   // Find and examine the TxnOpDispatcher instance.
   auto dispatcher = GetSingleTxnOpDispatcher();
   ASSERT_EVENTUALLY([&] {
-    std::lock_guard<simple_spinlock> guard(dispatcher->lock_);
+    std::lock_guard guard(dispatcher->lock_);
     // Due to scheduling anomalies, the operation above might be responded,
     // but TxnOpDispatcher::preliminary_tasks_completed_ field is not updated
     // yet. So, using ASSERT_EVENTUALLY here.
@@ -1594,7 +1594,7 @@ TEST_F(TxnOpDispatcherITest, ErrorInProcessingWriteOp) {
   ASSERT_EQ(2, dispatchers.size());
 
   for (auto& d : dispatchers) {
-    std::lock_guard<simple_spinlock> guard(d->lock_);
+    std::lock_guard guard(d->lock_);
     ASSERT_TRUE(d->preliminary_tasks_completed_);
     ASSERT_FALSE(d->unregistered_);
     ASSERT_OK(d->inflight_status_);
@@ -1684,7 +1684,7 @@ TEST_F(TxnOpDispatcherITest, NoPendingWriteOps) {
     ASSERT_EVENTUALLY([d] {
       // Eventually, every involved tablet should be registered as a transaction
       // participant and BEGIN_TXN should be issued.
-      std::lock_guard<simple_spinlock> guard(d->lock_);
+      std::lock_guard guard(d->lock_);
       ASSERT_TRUE(d->ops_queue_.empty());
       ASSERT_FALSE(d->unregistered_);
       ASSERT_OK(d->inflight_status_);
@@ -1891,7 +1891,7 @@ TEST_F(TxnOpDispatcherITest, DuplicateTxnParticipantRegistration) {
     auto& [dmap_txn_id, dispatchers] = *(dmap.begin());
     ASSERT_EQ(GetTxnId(txn), dmap_txn_id);
     for (auto& d : dispatchers) {
-      std::lock_guard<simple_spinlock> guard(d->lock_);
+      std::lock_guard guard(d->lock_);
       ASSERT_TRUE(d->ops_queue_.empty());
       ASSERT_FALSE(d->unregistered_);
       ASSERT_OK(d->inflight_status_);
@@ -1939,8 +1939,7 @@ TEST_F(TxnOpDispatcherITest, RollbackWriteOpPendingParticipantNotYetRegistered)
   const auto row_status = GetSingleRowError(session.get());
   ASSERT_TRUE(row_status.IsIllegalState()) << s.ToString();
 
-  rollback.join();
-  cleanup.cancel();
+  cleanup.run();
   ASSERT_OK(rollback_status);
 
   bool is_complete = false;
@@ -1993,8 +1992,7 @@ TEST_F(TxnOpDispatcherITest, RollbackWriteOpPendingParticipantRegistered) {
   Status s = InsertRows(txn.get(), 1, &key, &session);
   ASSERT_TRUE(s.IsIOError()) << s.ToString();
 
-  rollback.join();
-  cleanup.cancel();
+  cleanup.run();
   ASSERT_OK(rollback_status);
 
   bool is_complete = false;
@@ -2055,8 +2053,7 @@ TEST_F(TxnOpDispatcherITest, TxnWriteWhileReplicaDeleted) {
   // registered.
   ASSERT_GE(1, GetTxnOpDispatchersTotalCount(replicas));
 
-  cleanup.cancel();
-  tablet_deleter.join();
+  cleanup.run();
   ASSERT_OK(tablet_delete_status);
 }
 
