@@ -52,6 +52,7 @@
 #include "kudu/rebalance/rebalancer.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/rpc_controller.h"
+#include "kudu/security/init.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/cow_object.h"
 #include "kudu/util/monotime.h"
@@ -175,7 +176,11 @@ AutoRebalancerTask::~AutoRebalancerTask() {
 
 Status AutoRebalancerTask::Init() {
   DCHECK(!thread_) << "AutoRebalancerTask is already initialized";
-  RETURN_NOT_OK(MessengerBuilder("auto-rebalancer").Build(&messenger_));
+  MessengerBuilder builder("auto-rebalancer");
+  if (auto username = kudu::security::GetLoggedInUsernameFromKeytab()) {
+    builder.set_sasl_proto_name(username.value());
+  }
+  RETURN_NOT_OK(std::move(builder).Build(&messenger_));
   return Thread::Create("catalog manager", "auto-rebalancer",
                         [this]() { this->RunLoop(); }, &thread_);
 }
@@ -312,7 +317,7 @@ Status AutoRebalancerTask::GetMoves(
     for (const auto& elem : ts_id_by_location) {
       const auto& location = elem.first;
       ClusterRawInfo location_raw_info;
-      BuildClusterRawInfo(location, &location_raw_info);
+      RETURN_NOT_OK(BuildClusterRawInfo(location, &location_raw_info));
       RETURN_NOT_OK(GetMovesUsingRebalancingAlgo(
           location_raw_info, &algo, CrossLocations::NO, &rep_moves));
     }
