@@ -25,10 +25,11 @@
 #include <iosfwd>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <set>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
@@ -143,6 +144,11 @@ struct PersistentTabletInfo {
            pb.state() == SysTabletsEntryPB::DELETED;
   }
 
+  bool is_creating() const {
+    return pb.state() == SysTabletsEntryPB::CREATING ||
+           pb.state() == SysTabletsEntryPB::PREPARING;
+  }
+
   bool is_soft_deleted() const {
     return pb.state() == SysTabletsEntryPB::SOFT_DELETED;
   }
@@ -256,8 +262,18 @@ struct PersistentTableInfo {
            pb.state() == SysTablesEntryPB::SOFT_DELETED;
   }
 
+  // To improve the efficiency of the GC scheduler during the soft
+  // deletion process, we need to disable the corresponding table's
+  // compaction through 'AlterTable'. This is why we introduced a new
+  // approach to determine the soft deletion status based on the table's
+  // status and reserved time, as the existing approach using state to
+  // determine the soft deletion status may return an incorrect result
+  // during the Alter table process.
   bool is_soft_deleted() const {
-    return pb.state() == SysTablesEntryPB::SOFT_DELETED;
+    return pb.state() == SysTablesEntryPB::SOFT_DELETED ||
+        (pb.state() != SysTablesEntryPB::REMOVED &&
+         pb.has_soft_deleted_reserved_seconds() &&
+         pb.soft_deleted_reserved_seconds() != UINT32_MAX);
   }
 
   // Expired table must in SOFT_DELETED state.
@@ -377,7 +393,7 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
 
   // Returns a snapshot copy of the table info's tablet map.
   TabletInfoMap tablet_map() const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     TabletInfoMap ret;
     for (const auto& e : tablet_map_) {
       ret.emplace(e.first, make_scoped_refptr(e.second));
@@ -387,7 +403,7 @@ class TableInfo : public RefCountedThreadSafe<TableInfo> {
 
   // Returns the number of tablets.
   int num_tablets() const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     return tablet_map_.size();
   }
 
@@ -600,7 +616,7 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
 
    private:
     CatalogManager* catalog_;
-    shared_lock<RWMutex> leader_shared_lock_;
+    std::shared_lock<RWMutex> leader_shared_lock_;
     Status catalog_status_;
     Status leader_status_;
     int64_t initial_term_;
@@ -611,16 +627,15 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // Temporarily forces the catalog manager to be a follower. Only for tests!
   class ScopedLeaderDisablerForTests {
    public:
-
     explicit ScopedLeaderDisablerForTests(CatalogManager* catalog)
         : catalog_(catalog),
         old_leader_ready_term_(catalog->leader_ready_term_) {
-      std::lock_guard<simple_spinlock> l(catalog_->state_lock_);
+      std::lock_guard l(catalog_->state_lock_);
       catalog_->leader_ready_term_ = -1;
     }
 
-    ~ScopedLeaderDisablerForTests() {
-      std::lock_guard<simple_spinlock> l(catalog_->state_lock_);
+    virtual ~ScopedLeaderDisablerForTests() {
+      std::lock_guard l(catalog_->state_lock_);
       catalog_->leader_ready_term_ = old_leader_ready_term_;
     }
 
@@ -686,13 +701,23 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                                RecallDeletedTableResponsePB* resp,
                                rpc::RpcContext* rpc) WARN_UNUSED_RESULT;
 
+  enum class AlterType {
+    // Only normal type tables are allowed to be altered, not soft-deleted tables.
+    kNormal = 0,
+    // Alterations are allowed for both normal type tables and soft-deleted tables.
+    kForceToSoftDeleted
+  };
+
   // Alter the specified table in response to an AlterTableRequest RPC.
   //
   // The RPC context is provided for logging/tracing purposes,
   // but this function does not itself respond to the RPC.
+  // The 'alter_type' used by soft-delete function, which means the table
+  // state will not change to 'ALTERING' for disable the compaction.
   Status AlterTableRpc(const AlterTableRequestPB& req,
                        AlterTableResponsePB* resp,
-                       rpc::RpcContext* rpc);
+                       rpc::RpcContext* rpc,
+                       AlterType alter_type = AlterType::kNormal);
 
   // Alter the specified table in response to an 'ALTER TABLE' HMS
   // notification log listener event.
@@ -719,6 +744,10 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                         const security::TokenSigner* token_signer,
                         TableInfoMapType map_type = kAllTableType);
 
+  // List all the tables in the process of being created or altered.
+  Status ListInFlightTables(const ListInFlightTablesRequestPB* req,
+                            ListInFlightTablesResponsePB* resp);
+
   // Lists all the running tables. If 'user' is provided, only lists those that
   // the given user is authorized to see.
   Status ListTables(const ListTablesRequestPB* req,
@@ -842,6 +871,14 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // NOTE: This should only be used by tests or web-ui
   void GetAllTables(std::vector<scoped_refptr<TableInfo>>* tables);
 
+  // Retrieve all tables from the normalized_table_names_map_.
+  // The normalized_table_names_map_ inherently does not contain soft-deleted
+  // or not running tables, so no manual exclusion is needed. May fail if
+  // the catalog manager is not yet running. Caller must hold leader_lock_.
+  //
+  // NOTE: This should only be used by tests or web-ui
+  void GetNormalizedTables(std::vector<scoped_refptr<TableInfo>>* tables);
+
   // Check if a table exists by name, setting 'exist' appropriately. May fail
   // if the catalog manager is not yet running. Caller must hold leader_lock_.
   //
@@ -852,13 +889,13 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // table specified by 'tablet_id'.
   //
   // See also: TabletReplicaLookupIf, ConsensusServiceImpl.
-  virtual Status GetTabletReplica(const std::string& tablet_id,
-                                  scoped_refptr<tablet::TabletReplica>* replica) const override;
+  Status GetTabletReplica(const std::string& tablet_id,
+                          scoped_refptr<tablet::TabletReplica>* replica) const override;
 
-  virtual void GetTabletReplicas(
+  void GetTabletReplicas(
       std::vector<scoped_refptr<tablet::TabletReplica>>* replicas) const override;
 
-  virtual const NodeInstancePB& NodeInstance() const override;
+  const NodeInstancePB& NodeInstance() const override;
 
   bool IsInitialized() const;
 
@@ -929,6 +966,14 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   scoped_refptr<TableInfo> FindTableWithNameUnlocked(const std::string& table_name,
                                                      TableInfoMapType map_type = kAllTableType);
 
+  void set_ipki_private_key_password(std::string ipki_private_key_password) {
+    ipki_private_key_password_ = std::move(ipki_private_key_password);
+  }
+
+  void set_tsk_private_key_password(std::string tsk_private_key_password) {
+    tsk_private_key_password_ = std::move(tsk_private_key_password);
+  }
+
  private:
   // These tests calls ElectedAsLeaderCb() directly.
   FRIEND_TEST(kudu::client::ClientTest, TestSoftDeleteAndReserveTable);
@@ -969,6 +1014,15 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
                             RecallDeletedTableResponsePB* resp,
                             rpc::RpcContext* rpc);
 
+  // Disable compaction for soft deleted table to improve the scheduling performance
+  // of the compaction.
+  Status DisableCompactionForSoftDeletedTable(const DeleteTableRequestPB& req,
+                                              rpc::RpcContext* rpc);
+
+  // Enable compaction for recalled table to make the recalled table turn to normal.
+  Status EnableCompactionforRecalledTable(const RecallDeletedTableRequestPB& req,
+                                          rpc::RpcContext* rpc);
+
   // Check whether the table's write limit is reached,
   // if true, the write permission should be disabled.
   static bool IsTableWriteDisabled(const scoped_refptr<TableInfo>& table,
@@ -1084,6 +1138,12 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   Status LoadCertAuthorityInfo(std::unique_ptr<security::PrivateKey>* key,
                                std::unique_ptr<security::Cert>* cert);
 
+  // Same as above, but it doesn't try to decrypt it even if
+  // ipki_private_key_password_ is set, and if it's able to load it, then it
+  // encrypts it using the above password and rewrites the system table entry.
+  Status LoadAndEncryptCertAuthorityInfo(std::unique_ptr<security::PrivateKey>* key,
+                                         std::unique_ptr<security::Cert>* cert);
+
   // Store cluster ID into the system table.
   Status StoreClusterId(const std::string& cluster_id);
 
@@ -1390,6 +1450,11 @@ class CatalogManager : public tserver::TabletReplicaLookupIf {
   // Always acquire this lock before state_lock_.
   RWMutex leader_lock_;
 
+  // Password for the encrypted IPKI and TSK private keys stored in the
+  // sys-catalog table.
+  std::string ipki_private_key_password_;
+  std::string tsk_private_key_password_;
+
   // Async operations are accessing some private methods
   // (TODO: this stuff should be deferred and done in the background thread)
   friend class AsyncAlterTable;
