@@ -24,7 +24,6 @@
 #include <iterator>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <string>
@@ -204,37 +203,6 @@ Status RunListMasters(ListMastersResponsePB* resp, ExternalMiniCluster* cluster)
   return RunLeaderMasterRPC(list_masters, cluster);
 }
 
-// Verify the 'cluster' contains 'num_masters' on all masters, and that they
-// are all voters. Returns an error if the expected state is not present.
-//
-// This should be used instead of VerifyVoterMastersForCluster() if it is
-// required that all masters have accepted the config change. E.g. tests that
-// restart a cluster after adding a master should verify that all masters agree
-// before restarting, in case lagging masters start up with stale configs.
-//
-// TODO(awong): we should be more robust to starting up with mismatched on-disk
-// configs, if we can help it.
-Status VerifyVotersOnAllMasters(int num_masters, ExternalMiniCluster* cluster) {
-  for (int i = 0; i < cluster->num_masters(); i++) {
-    ListMastersResponsePB resp;
-    ListMastersRequestPB req;
-    RpcController rpc;
-    RETURN_NOT_OK(cluster->master_proxy(i)->ListMasters(req, &resp, &rpc));
-    if (num_masters != resp.masters_size()) {
-      return Status::IllegalState(Substitute("expected $0 masters but got $1",
-                                             num_masters, resp.masters_size()));
-    }
-    for (const auto& master : resp.masters()) {
-      if ((master.role() != RaftPeerPB::LEADER && master.role() != RaftPeerPB::FOLLOWER) ||
-          master.member_type() != RaftPeerPB::VOTER ||
-          master.registration().rpc_addresses_size() != 1) {
-        return Status::IllegalState(Substitute("bad master: $0", SecureShortDebugString(master)));
-      }
-    }
-  }
-  return Status::OK();
-}
-
 // Verify the ExternalMiniCluster 'cluster' contains 'num_masters' overall and
 // are all VOTERS. Populates the new master addresses in 'master_hps', if not
 // nullptr. Returns an error if the expected state is not present.
@@ -465,7 +433,7 @@ class DynamicMultiMasterTest : public KuduTest {
     if (cluster == nullptr) {
       cluster = cluster_.get();
     }
-    NO_FATALS(VerifyVoterMastersForCluster(num_masters, master_hps, cluster));
+    ASSERT_OK(VerifyVoterMastersForCluster(num_masters, master_hps, cluster));
   }
 
   // Fetch a follower (non-leader) master index from the cluster.
@@ -948,6 +916,8 @@ INSTANTIATE_TEST_SUITE_P(, ParameterizedRemoveMasterTest,
 
 // Tests removing a non-leader master from the cluster.
 TEST_P(ParameterizedRemoveMasterTest, TestRemoveMaster) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   NO_FATALS(StartCluster({// Keeping RPC timeouts short to quickly detect downed servers.
                           // This will put the health status into an UNKNOWN state until the point
                           // where they are considered FAILED.
@@ -1218,6 +1188,8 @@ TEST_F(DynamicMultiMasterTest, TestRemoveMasterFeatureFlagNotSpecified) {
 
 // Test that attempts to request a non-leader master to add a new master.
 TEST_F(DynamicMultiMasterTest, TestAddMasterToNonLeader) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   NO_FATALS(SetUpWithNumMasters(2));
   NO_FATALS(StartCluster());
 
@@ -1257,6 +1229,8 @@ TEST_F(DynamicMultiMasterTest, TestAddMasterToNonLeader) {
 
 // Test that attempts to request a non-leader master to remove a master.
 TEST_F(DynamicMultiMasterTest, TestRemoveMasterToNonLeader) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   NO_FATALS(SetUpWithNumMasters(2));
   NO_FATALS(StartCluster());
 
@@ -1327,6 +1301,8 @@ TEST_F(DynamicMultiMasterTest, TestAddMasterMissingAndIncorrectAddress) {
 // Test that attempts to remove a master with missing master address and a non-existent
 // hostname.
 TEST_F(DynamicMultiMasterTest, TestRemoveMasterMissingAndIncorrectHostname) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   NO_FATALS(SetUpWithNumMasters(2));
   NO_FATALS(StartCluster());
 
@@ -1565,6 +1541,8 @@ TEST_F(AutoAddMasterTest, TestSomeMastersUnreachable) {
 
 // Test if we fail to replicate the AddMaster request.
 TEST_F(AutoAddMasterTest, TestFailWithoutReplicatingAddMaster) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   // Make master followers unable to accept updates, including config changes.
   // We'll set this for all masters including leaders for simplicity.
   for (int i = 0; i < cluster_->num_masters(); i++) {
@@ -1655,7 +1633,7 @@ TEST_F(AutoAddMasterTest, TestAddWithOnGoingDdl) {
   const auto generate_client = [&] (shared_ptr<KuduClient>* c) {
     vector<string> master_addrs;
     {
-      std::lock_guard<simple_spinlock> l(master_addrs_lock);
+      std::lock_guard l(master_addrs_lock);
       master_addrs = master_addrs_unlocked;
     }
     shared_ptr<KuduClient> client;
@@ -1700,10 +1678,10 @@ TEST_F(AutoAddMasterTest, TestAddWithOnGoingDdl) {
     ASSERT_OK(new_master->WaitForCatalogManager());
     num_masters++;
     ASSERT_EVENTUALLY([&] {
-      ASSERT_OK(VerifyVotersOnAllMasters(num_masters, cluster_.get()));
+      ASSERT_OK(cluster_->VerifyVotersOnAllMasters(num_masters));
     });
     {
-      std::lock_guard<simple_spinlock> l(master_addrs_lock);
+      std::lock_guard l(master_addrs_lock);
       master_addrs_unlocked.emplace_back(new_master->bound_rpc_hostport().ToString());
     }
     cluster_->Shutdown();
@@ -1711,11 +1689,7 @@ TEST_F(AutoAddMasterTest, TestAddWithOnGoingDdl) {
     ASSERT_OK(cluster_->WaitForTabletServerCount(cluster_->num_tablet_servers(),
                                                  MonoDelta::FromSeconds(5)));
   }
-  proceed = false;
-  thread_joiner.cancel();
-  for (auto& t : threads) {
-    t.join();
-  }
+  thread_joiner.run();
   for (const auto& e : errors) {
     // NOTE: the table may exist if the CreateTable call is retried.
     if (e.ok() || e.IsTimedOut() || e.IsAlreadyPresent()) {
@@ -1785,7 +1759,7 @@ TEST_F(AutoAddMasterTest, TestAddNewMaster) {
   ASSERT_OK(peer->WaitForCatalogManager());
   auto expected_num_masters = ++idx;
   ASSERT_EVENTUALLY([&] {
-    ASSERT_OK(VerifyVotersOnAllMasters(expected_num_masters, cluster_.get()));
+    ASSERT_OK(cluster_->VerifyVotersOnAllMasters(expected_num_masters));
   });
   NO_FATALS(cluster_->AssertNoCrashes());
 }
@@ -1812,7 +1786,7 @@ TEST_F(MinidumpTest, TestAddNewMasterMinidumpsEnabled) {
   ASSERT_OK(peer->WaitForCatalogManager());
   auto expected_num_masters = ++idx;
   ASSERT_EVENTUALLY([&] {
-    ASSERT_OK(VerifyVotersOnAllMasters(expected_num_masters, cluster_.get()));
+    ASSERT_OK(cluster_->VerifyVotersOnAllMasters(expected_num_masters));
   });
   NO_FATALS(cluster_->AssertNoCrashes());
 }
@@ -1827,8 +1801,16 @@ class ParameterizedAutoAddMasterTest : public AutoAddMasterTest,
 };
 
 TEST_P(ParameterizedAutoAddMasterTest, TestBasicAddition) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
   TestWorkload w(cluster_.get());
   w.set_num_replicas(1);
+  // Using one thread and injecting delays between write batches to reduce
+  // the amount of data accumulated under the tablet server's WAL and data
+  // directories since this scenario might run for quite a long time,
+  // especially on slow machines.
+  w.set_num_write_threads(1);
+  w.set_write_interval_millis(5);
   w.Setup();
   w.Start();
   int num_masters = args_.orig_num_masters;
