@@ -67,6 +67,7 @@
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/net/sockaddr.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/status.h"
 #include "kudu/util/thread.h"
 #include "kudu/util/threadpool.h"
@@ -122,6 +123,14 @@ DEFINE_string(master_rpc_proxy_advertised_addresses, "",
               "flag for a single master.");
 TAG_FLAG(master_rpc_proxy_advertised_addresses, experimental);
 
+DEFINE_string(ipki_private_key_password_cmd, "",
+              "A Unix command whose output returns the password to encrypt "
+              "and decrypt the IPKI root CA private key.");
+
+DEFINE_string(tsk_private_key_password_cmd, "",
+              "A Unix command whose output returns the password to encrypt "
+              "and decrypt the token signing key");
+
 DECLARE_bool(txn_manager_lazily_initialized);
 DECLARE_bool(txn_manager_enabled);
 DECLARE_string(master_addresses);
@@ -189,15 +198,17 @@ GROUP_FLAG_VALIDATOR(multi_master_pp, &ValidateMultiMasterProxiedRpcFlags);
 
 constexpr const char* kReplaceMasterMessage =
     "this master may return incorrect results and should be replaced";
-void CrashMasterOnDiskError(const string& uuid) {
+void CrashMasterOnDiskError(const string& uuid, const string& /* tenant_id */) {
   LOG(FATAL) << Substitute("Disk error detected on data directory $0: $1",
                            uuid, kReplaceMasterMessage);
 }
-void CrashMasterOnCFileCorruption(const string& tablet_id) {
+void CrashMasterOnCFileCorruption(const string& tablet_id,
+                                  const string& /* tenant_id */) {
   LOG(FATAL) << Substitute("CFile corruption detected on system catalog $0: $1",
                            tablet_id, kReplaceMasterMessage);
 }
-void CrashMasterOnKudu2233Corruption(const string& tablet_id) {
+void CrashMasterOnKudu2233Corruption(const string& tablet_id,
+                                     const string& /* tenant_id */) {
   LOG(FATAL) << Substitute("KUDU-2233 corruption detected on system catalog $0: $1 ",
                            tablet_id, kReplaceMasterMessage);
 }
@@ -254,6 +265,26 @@ Master::~Master() {
 Status Master::Init() {
   CHECK_EQ(kStopped, state_);
 
+  // As getting a password from a shell command relies on forking the process,
+  // it's best to do this in the very beginning of Init() to avoid having to
+  // fork unnecessary threads.
+  string ipki_private_key_password;
+  if (!FLAGS_ipki_private_key_password_cmd.empty()) {
+    RETURN_NOT_OK_PREPEND(security::GetPasswordFromShellCommand(
+          FLAGS_ipki_private_key_password_cmd, &ipki_private_key_password),
+        "could not get IPKI private key password from configured command");
+  }
+
+  string tsk_private_key_password;
+  if (!FLAGS_tsk_private_key_password_cmd.empty()) {
+    RETURN_NOT_OK_PREPEND(security::GetPasswordFromShellCommand(
+          FLAGS_tsk_private_key_password_cmd, &tsk_private_key_password),
+        "could not get TSK private key password from configured command");
+  }
+
+  catalog_manager_->set_ipki_private_key_password(std::move(ipki_private_key_password));
+  catalog_manager_->set_tsk_private_key_password(tsk_private_key_password);
+
   cfile::BlockCache::GetSingleton()->StartInstrumentation(
       metric_entity(), opts_.block_cache_metrics_policy());
 
@@ -280,7 +311,8 @@ Status Master::Init() {
       FLAGS_authn_token_validity_seconds,
       FLAGS_authz_token_validity_seconds,
       FLAGS_tsk_rotation_seconds,
-      messenger_->shared_token_verifier()));
+      messenger_->shared_token_verifier(),
+      std::move(tsk_private_key_password)));
 
   if (!FLAGS_master_rpc_proxy_advertised_addresses.empty()) {
     vector<HostPort> host_ports;
@@ -359,8 +391,8 @@ void Master::InitCatalogManagerTask() {
   if (!s.ok()) {
     LOG(ERROR) << "Unable to init master catalog manager: " << s.ToString();
   }
-  catalog_manager_init_status_.Set(s);
   startup_path_handler_->initialize_master_catalog_progress()->Stop();
+  catalog_manager_init_status_.Set(s);
 }
 
 Status Master::InitCatalogManager() {
@@ -512,8 +544,15 @@ void Master::ShutdownImpl() {
     init_pool_->Shutdown();
     maintenance_manager_->Shutdown();
     catalog_manager_->Shutdown();
-    fs_manager_->UnsetErrorNotificationCb(ErrorHandlerType::DISK_ERROR);
+    fs_manager_->UnsetErrorNotificationCb(ErrorHandlerType::KUDU_2233_CORRUPTION);
     fs_manager_->UnsetErrorNotificationCb(ErrorHandlerType::CFILE_CORRUPTION);
+    fs_manager_->UnsetErrorNotificationCb(ErrorHandlerType::DISK_ERROR);
+    if (expired_reserved_tables_deleter_thread_) {
+      // Set the countdown latch to zero to trigger expired_reserved_tables_deleter_thread_
+      // to stop. Then wait for the thread to actually stop.
+      stop_background_threads_latch_.Reset(0);
+      expired_reserved_tables_deleter_thread_->Join();
+    }
 
     // 3. Shut down generic subsystems.
     KuduServer::Shutdown();
