@@ -747,7 +747,9 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
                                             Webserver::PrerenderedWebResponse* resp) {
   ostringstream* output = &resp->output;
   if (!master_->catalog_manager()->IsInitialized()) {
-    JsonError(Status::ServiceUnavailable("CatalogManager is not running"), output);
+    Status s = Status::ServiceUnavailable("CatalogManager is not running");
+    JsonError(s, output);
+    LOG(WARNING) << s.ToString();
     return;
   }
 
@@ -761,6 +763,7 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
   auto s = master_->catalog_manager()->sys_catalog()->VisitTables(&d);
   if (!s.ok()) {
     JsonError(s, output);
+    LOG(WARNING) << s.ToString();
     return;
   }
   jw.EndArray();
@@ -770,10 +773,54 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
   s = master_->catalog_manager()->sys_catalog()->VisitTablets(&d);
   if (!s.ok()) {
     JsonError(s, output);
+    LOG(WARNING) << s.ToString();
     return;
   }
   jw.EndArray();
 
+  jw.String("masters");
+  jw.StartArray();
+  vector<ServerEntryPB> masters;
+  s = master_->ListMasters(&masters, /*use_external_addr=*/false);
+  if (!s.ok()) {
+    JsonError(s, output);
+    LOG(WARNING) << s.ToString();
+    return;
+  }
+
+  for (const auto& master : masters) {
+    jw.StartObject();
+    jw.String("uuid");
+    jw.String(master.instance_id().permanent_uuid());
+
+    jw.String("rpc_addrs");
+    jw.StartArray();
+    for (const HostPortPB& host_port : master.registration().rpc_addresses()) {
+      jw.String(Substitute("$0:$1", host_port.host(), host_port.port()));
+    }
+    jw.EndArray();
+
+    jw.String("http_addrs");
+    jw.StartArray();
+    for (const HostPortPB& host_port : master.registration().http_addresses()) {
+      jw.String(Substitute("$0://$1:$2", master.registration().https_enabled() ? "https" : "http",
+                                         host_port.host(), host_port.port()));
+    }
+    jw.EndArray();
+
+    jw.String("role");
+    jw.String(RaftPeerPB_Role_Name(master.role()));
+
+    jw.String("version");
+    jw.String(master.registration().software_version());
+
+    jw.String("start_time");
+    jw.String(StartTimeToString(master.registration()));
+
+    jw.EndObject();
+  }
+  jw.EndArray();
+
   jw.String("tablet_servers");
   jw.StartArray();
   vector<shared_ptr<TSDescriptor> > descs;
@@ -826,55 +873,54 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
 }
 
 Status MasterPathHandlers::Register(Webserver* server) {
-  constexpr const bool is_styled = true;
   constexpr const bool is_on_nav_bar = true;
   server->RegisterPathHandler(
       "/tablet-servers", "Tablet Servers",
       [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
         this->HandleTabletServers(req, resp);
       },
-      is_styled, is_on_nav_bar);
+      StyleMode::STYLED, is_on_nav_bar);
   server->RegisterPathHandler(
       "/tables", "Tables",
       [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
         this->HandleCatalogManager(req, resp);
       },
-      is_styled, is_on_nav_bar);
+      StyleMode::STYLED, is_on_nav_bar);
   server->RegisterPathHandler(
       "/table", "",
       [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
         this->HandleTablePage(req, resp);
       },
-      is_styled, false);
+      StyleMode::STYLED, false);
   server->RegisterPathHandler(
       "/masters", "Masters",
       [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
         this->HandleMasters(req, resp);
       },
-      is_styled, is_on_nav_bar);
+      StyleMode::STYLED, is_on_nav_bar);
   server->RegisterPrerenderedPathHandler(
       "/ipki-ca-cert", "IPKI CA certificate",
       [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
         this->HandleIpkiCaCert(DataFormat::PEM, req, resp);
       },
-      false /*is_styled*/, true /*is_on_nav_bar*/);
+      StyleMode::UNSTYLED , true /*is_on_nav_bar*/);
   server->RegisterPrerenderedPathHandler(
       "/ipki-ca-cert-pem", "IPKI CA certificate (PEM format)",
       [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
         this->HandleIpkiCaCert(DataFormat::PEM, req, resp);
       },
-      false /*is_styled*/, false /*is_on_nav_bar*/);
+      StyleMode::UNSTYLED, false /*is_on_nav_bar*/);
   server->RegisterBinaryDataPathHandler(
       "/ipki-ca-cert-der", "IPKI CA certificate (DER format)",
       [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
         this->HandleIpkiCaCert(DataFormat::DER, req, resp);
       });
-  server->RegisterPrerenderedPathHandler(
+  server->RegisterJsonPathHandler(
       "/dump-entities", "Dump Entities",
       [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
         this->HandleDumpEntities(req, resp);
       },
-      false, false);
+      false /*is_on_nav_bar*/);
   return Status::OK();
 }
 
@@ -898,26 +944,25 @@ pair<string, string> MasterPathHandlers::TSDescToLinkPair(const TSDescriptor& de
 
 string MasterPathHandlers::MasterAddrsToCsv() const {
   vector<HostPort> master_addresses;
-  Status s = master_->GetMasterHostPorts(&master_addresses);
-  LOG(WARNING) << "Unable to fetch master addresses: " << s.ToString();
-  if (!s.ok()) {
+  if (auto s = master_->GetMasterHostPorts(&master_addresses); !s.ok()) {
+    LOG(WARNING) << "unable to fetch master addresses: " << s.ToString();
     return string();
   }
   if (!master_addresses.empty()) {
-    vector<string> all_addresses;
-    all_addresses.reserve(master_addresses.size());
-    for (const HostPort& hp : master_addresses) {
-      all_addresses.push_back(hp.ToString());
-    }
-    return JoinElements(all_addresses, ",");
+    return JoinMapped(master_addresses,
+                      [] (const HostPort& hp) { return hp.ToString(); },
+                      ",");
   }
   vector<HostPort> hps;
-  s = master_->rpc_server()->GetBoundHostPorts(&hps);
-  if (PREDICT_FALSE(!s.ok())) {
-    LOG(WARNING) << "Unable to determine proper local hostname: " << s.ToString();
+  if (auto s = master_->rpc_server()->GetBoundHostPorts(&hps); !s.ok()) {
+    LOG(WARNING) << "unable to get bound host:ports pairs: " << s.ToString();
     return string();
   }
-  return hps[0].ToString();
+  if (hps.empty()) {
+    LOG(WARNING) << "empty list of bound host:port pairs";
+    return string();
+  }
+  return hps.front().ToString();
 }
 
 Status MasterPathHandlers::GetLeaderMasterHttpAddr(string* leader_http_addr) const {
