@@ -653,6 +653,19 @@ void MasterServiceImpl::IsAlterTableDone(const IsAlterTableDoneRequestPB* req,
   rpc->RespondSuccess();
 }
 
+void MasterServiceImpl::ListInFlightTables(const ListInFlightTablesRequestPB* req,
+                                           ListInFlightTablesResponsePB* resp,
+                                           rpc::RpcContext* rpc) {
+  CatalogManager::ScopedLeaderSharedLock l(server_->catalog_manager());
+  if (!l.CheckIsInitializedAndIsLeaderOrRespond(resp, rpc)) {
+    return;
+  }
+
+  auto s = server_->catalog_manager()->ListInFlightTables(req, resp);
+  CheckRespErrorOrSetUnknown(s, resp);
+  rpc->RespondSuccess();
+}
+
 void MasterServiceImpl::ListTables(const ListTablesRequestPB* req,
                                    ListTablesResponsePB* resp,
                                    rpc::RpcContext* rpc) {
@@ -868,8 +881,10 @@ void MasterServiceImpl::ConnectToMaster(const ConnectToMasterRequestPB* /*req*/,
                    resp->mutable_error()->mutable_status());
         resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
         rpc->RespondSuccess();
-        KLOG_EVERY_N_SECS(WARNING, 60) << Substitute("invalid request from $0: $1",
-                                                     rpc->requestor_string(), kErrMsg);
+        KLOG_EVERY_N_SECS(WARNING, 60)
+            << Substitute("invalid request from $0: $1",
+                          rpc->requestor_string(), kErrMsg)
+            << THROTTLE_MSG;
         return;
       }
 
