@@ -25,6 +25,7 @@
 #include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -50,6 +51,7 @@
 #include "kudu/consensus/consensus_peers.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
+#include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/opid_util.h"
 #include "kudu/consensus/quorum_util.h"
 #include "kudu/consensus/raft_consensus.h"
@@ -183,7 +185,6 @@ bool ArePBsEqual(const google::protobuf::Message& prev_pb,
 
 } // anonymous namespace
 
-
 SysCatalogTable::SysCatalogTable(Master* master,
                                  ElectedLeaderCallback leader_cb)
     : metric_registry_(master->metric_registry()),
@@ -260,7 +261,7 @@ Status SysCatalogTable::Load(FsManager *fs_manager) {
   }
 
   LOG(INFO) << "Verifying existing consensus state";
-  const string tablet_id = metadata->tablet_id();
+  const auto& tablet_id = metadata->tablet_id();
   scoped_refptr<ConsensusMetadata> cmeta;
   RETURN_NOT_OK_PREPEND(cmeta_manager_->Load(tablet_id, &cmeta),
                         "Unable to load consensus metadata for tablet " + tablet_id);
@@ -377,7 +378,7 @@ Status SysCatalogTable::CreateNew(FsManager *fs_manager) {
     }
   }
 
-  string tablet_id = metadata->tablet_id();
+  const auto& tablet_id = metadata->tablet_id();
   RETURN_NOT_OK_PREPEND(cmeta_manager_->Create(tablet_id, config, consensus::kMinimumTerm),
                         "Unable to persist consensus metadata for tablet " + tablet_id);
 
@@ -910,6 +911,17 @@ Status SysCatalogTable::AddClusterIdEntry(
 
 Status SysCatalogTable::AddCertAuthorityEntry(
     const SysCertAuthorityEntryPB& entry) {
+  return AddOrUpdateCertAuthorityEntry(entry, RowOperationsPB::INSERT);
+}
+
+Status SysCatalogTable::UpdateCertAuthorityEntry(
+    const SysCertAuthorityEntryPB& entry) {
+  return AddOrUpdateCertAuthorityEntry(entry, RowOperationsPB::UPDATE);
+}
+
+Status SysCatalogTable::AddOrUpdateCertAuthorityEntry(
+    const SysCertAuthorityEntryPB& entry,
+    RowOperationsPB::Type op) {
   WriteRequestPB req;
   req.set_tablet_id(kSysCatalogTabletId);
   RETURN_NOT_OK(SchemaToPB(schema_, req.mutable_schema()));
@@ -922,7 +934,7 @@ Status SysCatalogTable::AddCertAuthorityEntry(
   CHECK_OK(row.SetStringNoCopy(kSysCatalogTableColId, kSysCertAuthorityEntryId));
   CHECK_OK(row.SetStringNoCopy(kSysCatalogTableColMetadata, metadata_buf));
   RowOperationsPBEncoder enc(req.mutable_row_operations());
-  enc.Add(RowOperationsPB::INSERT, row);
+  enc.Add(op, row);
 
   return SyncWrite(req);
 }
