@@ -18,8 +18,8 @@
 
 #include <cstdint>
 #include <memory>
-#include <mutex>
 #include <optional>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>  // IWYU pragma: keep
 #include <unordered_map>
@@ -184,13 +184,13 @@ class TSDescriptor : public enable_make_shared<TSDescriptor> {
   // Set the number of live replicas (i.e. running or bootstrapping).
   void set_num_live_replicas(int n) {
     DCHECK_GE(n, 0);
-    std::lock_guard<rw_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     num_live_replicas_ = n;
   }
 
   // Set the number of live replicas in each dimension.
   void set_num_live_replicas_by_dimension(TabletNumByDimensionMap num_live_tablets_by_dimension) {
-    std::lock_guard<rw_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     num_live_tablets_by_dimension_ = std::move(num_live_tablets_by_dimension);
   }
 
@@ -198,7 +198,7 @@ class TSDescriptor : public enable_make_shared<TSDescriptor> {
   void set_num_live_replicas_by_range_per_table(
       std::string table_id,
       TabletNumByRangeMap num_live_tablets_by_table) {
-    std::lock_guard<rw_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     num_live_tablets_by_range_per_table_.emplace(table_id, num_live_tablets_by_table);
   }
 
@@ -206,7 +206,7 @@ class TSDescriptor : public enable_make_shared<TSDescriptor> {
   // If dimension is none, return the total number of replicas in the tablet server.
   // Otherwise, return the number of replicas in the dimension.
   int num_live_replicas(const std::optional<std::string>& dimension = std::nullopt) const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     if (dimension) {
       int32_t num_live_tablets = 0;
       if (num_live_tablets_by_dimension_) {
@@ -220,18 +220,18 @@ class TSDescriptor : public enable_make_shared<TSDescriptor> {
   // Return the number of live replicas (i.e. running or bootstrapping)
   // in the given range for the given table.
   int num_live_replicas_by_range(const std::string& range_key, const std::string& table_id) const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     int32_t num_live_tablets_by_range = 0;
-    if (ContainsKey(num_live_tablets_by_range_per_table_, table_id)) {
-      auto ranges = FindOrDie(num_live_tablets_by_range_per_table_, table_id);
-      ignore_result(FindCopy(ranges, range_key, &num_live_tablets_by_range));
+    if (const auto* ranges = FindOrNull(
+          num_live_tablets_by_range_per_table_, table_id); ranges != nullptr) {
+      ignore_result(FindCopy(*ranges, range_key, &num_live_tablets_by_range));
     }
     return num_live_tablets_by_range;
   }
 
   // Return the number of live replicas (i.e. running or bootstrapping) in the given table.
   int num_live_replicas_by_table(const std::string& table_id) const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     int32_t num_live_tablets_by_table = 0;
     if (ContainsKey(num_live_tablets_by_range_per_table_, table_id)) {
       auto ranges = FindOrDie(num_live_tablets_by_range_per_table_, table_id);
@@ -246,7 +246,7 @@ class TSDescriptor : public enable_make_shared<TSDescriptor> {
   // since the location could change at any time if the tablet server
   // re-registers.
   std::optional<std::string> location() const {
-    shared_lock<rw_spinlock> l(lock_);
+    std::shared_lock<rw_spinlock> l(lock_);
     return location_;
   }
 
