@@ -25,6 +25,8 @@
 #include <mutex>
 #include <optional>
 #include <ostream>
+#include <shared_mutex>
+#include <type_traits>
 #include <utility>
 
 #include <gflags/gflags.h>
@@ -62,6 +64,7 @@ METRIC_DEFINE_gauge_int32(server, cluster_replica_skew,
 using kudu::pb_util::SecureShortDebugString;
 using std::lock_guard;
 using std::unordered_set;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using strings::Substitute;
@@ -155,7 +158,7 @@ Status TSManager::RegisterTS(const NodeInstancePB& instance,
         ? uuid : registration.rpc_addresses(0).host();
     TRACE(Substitute("tablet server $0: assigning location", uuid));
     string location_str;
-    const auto s = location_cache_->GetLocation(cmd_arg, &location_str);
+    auto s = location_cache_->GetLocation(cmd_arg, &location_str);
     TRACE(Substitute(
         "tablet server $0: assigned location '$1'", uuid, location_str));
 
@@ -166,7 +169,7 @@ Status TSManager::RegisterTS(const NodeInstancePB& instance,
       KLOG_EVERY_N_SECS(ERROR, 60) << Substitute(
           "Unable to assign location to tablet server $0: $1",
           Substitute("$0 ($1:$2)", uuid, addr.host(), addr.port()),
-          s.ToString());
+          s.ToString()) << THROTTLE_MSG;
       return s;
     }
     location.emplace(std::move(location_str));
@@ -175,7 +178,7 @@ Status TSManager::RegisterTS(const NodeInstancePB& instance,
   shared_ptr<TSDescriptor> descriptor;
   bool new_tserver = false;
   {
-    lock_guard<rw_spinlock> l(lock_);
+    lock_guard l(lock_);
     auto* descriptor_ptr = FindOrNull(servers_by_id_, uuid);
     if (descriptor_ptr) {
       descriptor = *descriptor_ptr;
@@ -253,7 +256,7 @@ Status TSManager::SetTServerState(const string& ts_uuid,
                                   TServerStatePB ts_state,
                                   ChangeTServerStateRequestPB::HandleMissingTS handle_missing_ts,
                                   SysCatalogTable* sys_catalog) {
-  lock_guard<RWMutex> l(ts_state_lock_);
+  lock_guard l(ts_state_lock_);
   auto existing_state = FindWithDefault(
       ts_state_by_uuid_, ts_uuid, { TServerStatePB::NONE, -1 }).first;
   if (existing_state == ts_state) {
@@ -303,14 +306,14 @@ TServerStatePB TSManager::GetTServerState(const string& ts_uuid) const {
 }
 
 Status TSManager::ReloadTServerStates(SysCatalogTable* sys_catalog) {
-  lock_guard<RWMutex> l(ts_state_lock_);
+  lock_guard l(ts_state_lock_);
   ts_state_by_uuid_ = {};
   TServerStateLoader loader(this);
   return sys_catalog->VisitTServerStates(&loader);
 }
 
 void TSManager::SetAllTServersNeedFullTabletReports() {
-  lock_guard<rw_spinlock> l(lock_);
+  lock_guard l(lock_);
   for (auto& id_and_desc : servers_by_id_) {
     id_and_desc.second->UpdateNeedsFullTabletReport(true);
   }
@@ -318,7 +321,7 @@ void TSManager::SetAllTServersNeedFullTabletReports() {
 
 Status TSManager::UnregisterTServer(const std::string& ts_uuid,
                                     bool force_unregister_live_tserver) {
-  lock_guard<rw_spinlock> l(lock_);
+  lock_guard l(lock_);
   shared_ptr<TSDescriptor> ts_desc;
   if (!FindCopy(servers_by_id_, ts_uuid, &ts_desc)) {
     return Status::NotFound(Substitute("Requested tserver $0 has not been registered", ts_uuid));
