@@ -35,17 +35,20 @@
 
 #include "kudu/client/client.h"
 #include "kudu/client/master_rpc.h"
+#include "kudu/consensus/metadata.pb.h"
 #include "kudu/fs/default_key_provider.h"
 #include "kudu/fs/fs.pb.h"
 #include "kudu/fs/key_provider.h"
 #include "kudu/postgres/mini_postgres.h"
 #include "kudu/ranger-kms/mini_ranger_kms.h"
 #include "kudu/rpc/rpc_header.pb.h"
+#include "kudu/tablet/metadata.pb.h"
 #if !defined(NO_CHRONY)
 #include "kudu/clock/test/mini_chronyd.h"
 #endif
 #include "kudu/common/wire_protocol.h"
 #include "kudu/common/wire_protocol.pb.h"
+#include "kudu/consensus/consensus.proxy.h"
 #include "kudu/gutil/basictypes.h"
 #include "kudu/gutil/strings/escaping.h"
 #include "kudu/gutil/strings/join.h"
@@ -63,7 +66,6 @@
 #include "kudu/security/test/mini_kdc.h"
 #include "kudu/server/server_base.pb.h"
 #include "kudu/server/server_base.proxy.h"
-#include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/tserver/tserver_admin.proxy.h"
@@ -75,6 +77,7 @@
 #include "kudu/util/jwt-util.h"
 #include "kudu/util/mini_oidc.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/dns_resolver.h"
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/net/socket.h"
 #include "kudu/util/path_util.h"
@@ -92,8 +95,11 @@ using kudu::client::internal::ConnectToClusterRpc;
 #if !defined(NO_CHRONY)
 using kudu::clock::MiniChronyd;
 #endif
+using kudu::master::ListMastersRequestPB;
+using kudu::master::ListMastersResponsePB;
 using kudu::master::ListTablesRequestPB;
 using kudu::master::ListTablesResponsePB;
+using kudu::consensus::RaftPeerPB;
 using kudu::master::MasterServiceProxy;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
@@ -104,6 +110,7 @@ using kudu::tserver::ListTabletsRequestPB;
 using kudu::tserver::ListTabletsResponsePB;
 using kudu::tserver::TabletServerAdminServiceProxy;
 using kudu::tserver::TabletServerServiceProxy;
+using kudu::consensus::ConsensusServiceProxy;
 using std::back_inserter;
 using std::copy;
 using std::map;
@@ -606,7 +613,7 @@ Status ExternalMiniCluster::StartMasters() {
     scoped_refptr<ExternalMaster> peer;
     RETURN_NOT_OK(CreateMaster(master_rpc_addrs, i, &peer));
     RETURN_NOT_OK_PREPEND(peer->Start(), Substitute("Unable to start Master at index $0", i));
-    RETURN_NOT_OK(peer->SetServerKey());
+    RETURN_NOT_OK(peer->SetEncryptionKey());
     masters_.emplace_back(std::move(peer));
   }
   return Status::OK();
@@ -683,7 +690,7 @@ Status ExternalMiniCluster::AddTabletServer() {
   }
 
   RETURN_NOT_OK(ts->Start());
-  RETURN_NOT_OK(ts->SetServerKey());
+  RETURN_NOT_OK(ts->SetEncryptionKey());
   tablet_servers_.push_back(ts);
   return Status::OK();
 }
@@ -993,6 +1000,36 @@ Status ExternalMiniCluster::WaitForTabletsRunning(
   return Status::TimedOut(SecureDebugString(resp));
 }
 
+Status ExternalMiniCluster::VerifyVotersOnAllMasters(int num_masters) {
+  for (int i = 0; i < this->num_masters(); i++) {
+    ListMastersResponsePB resp;
+    ListMastersRequestPB req;
+    RpcController rpc;
+    RETURN_NOT_OK(master_proxy(i)->ListMasters(req, &resp, &rpc));
+    if (num_masters != resp.masters_size()) {
+      return Status::IllegalState(Substitute("expected $0 masters but got $1",
+                                             num_masters, resp.masters_size()));
+    }
+    for (const auto& master : resp.masters()) {
+      if ((master.role() != RaftPeerPB::LEADER && master.role() != RaftPeerPB::FOLLOWER) ||
+          master.member_type() != RaftPeerPB::VOTER ||
+          master.registration().rpc_addresses_size() != 1) {
+        return Status::IllegalState(Substitute("bad master: $0", SecureShortDebugString(master)));
+      }
+    }
+  }
+  return Status::OK();
+}
+
+ExternalMaster* ExternalMiniCluster::leader_master() {
+  int idx = 0;
+  Status s = GetLeaderMasterIndex(&idx);
+  if (!s.ok()) {
+    return nullptr;
+  }
+  return master(idx);
+}
+
 Status ExternalMiniCluster::GetLeaderMasterIndex(int* idx) {
   scoped_refptr<ConnectToClusterRpc> rpc;
   Synchronizer sync;
@@ -1024,18 +1061,22 @@ Status ExternalMiniCluster::GetLeaderMasterIndex(int* idx) {
   rpc->SendRpc();
   RETURN_NOT_OK(sync.Wait());
   bool found = false;
-  for (int i = 0; i < masters_.size(); i++) {
-    const auto& bound_hp = masters_[i]->bound_rpc_hostport();
+  DnsResolver dns_resolver;
+  for (int i = 0; i < masters_.size() && !found; i++) {
+    vector<Sockaddr> addresses;
+    RETURN_NOT_OK(dns_resolver.ResolveAddresses(masters_[i]->bound_rpc_hostport(), &addresses));
+    for (const auto& adress : addresses) {
+      if (adress == leader_master_addr) {
+        found = true;
+        *idx = i;
+        break;
+      }
+    }
+
     // If using BindMode::UNIQUE_LOOPBACK mode, in rare cases different masters
     // might bind to different local IP addresses but use same port numbers.
     // So, it's necessary to check both the returned hostnames and IP addresses
     // to point to leader master.
-    if (bound_hp.port() == leader_master_addr.port() &&
-        bound_hp.host() == leader_master_hostname) {
-      found = true;
-      *idx = i;
-      break;
-    }
   }
   if (!found) {
     // There is never a situation where this should happen, so it's
@@ -1123,6 +1164,13 @@ std::shared_ptr<TabletServerAdminServiceProxy> ExternalMiniCluster::tserver_admi
   return std::make_shared<TabletServerAdminServiceProxy>(messenger_, addr, addr.host());
 }
 
+std::shared_ptr<ConsensusServiceProxy> ExternalMiniCluster::tserver_consensus_proxy(
+    int idx) const {
+  CHECK_LT(idx, tablet_servers_.size());
+  const auto& addr = CHECK_NOTNULL(tablet_server(idx))->bound_rpc_addr();
+  return std::make_shared<consensus::ConsensusServiceProxy>(messenger_, addr, addr.host());
+}
+
 Status ExternalMiniCluster::CreateClient(client::KuduClientBuilder* builder,
                                          client::sp::shared_ptr<client::KuduClient>* client) const {
   client::KuduClientBuilder defaults;
@@ -1374,19 +1422,37 @@ Env* ExternalDaemon::env() const {
   return Env::Default();
 }
 
-Status ExternalDaemon::SetServerKey() {
+Status ExternalDaemon::SetEncryptionKey(const string& tenant_id) {
   string path = JoinPathSegments(this->wal_dir(), "instance");;
   LOG(INFO) << "Reading " << path;
   InstanceMetadataPB instance;
   RETURN_NOT_OK(pb_util::ReadPBContainerFromPath(env(), path, &instance, pb_util::NOT_SENSITIVE));
-  if (!instance.server_key().empty()) {
-    string key;
-    RETURN_NOT_OK(key_provider_->DecryptServerKey(instance.server_key(),
-                                                  instance.server_key_iv(),
-                                                  instance.server_key_version(),
-                                                  &key));
-    LOG(INFO) << "Setting key " << key;
-    env()->SetEncryptionKey(reinterpret_cast<const uint8_t*>(a2b_hex(key).c_str()), key.size() * 4);
+  string decrypted_key;
+  if (!instance.tenants().empty()) {
+    const InstanceMetadataPB::TenantMetadataPB* tenant = nullptr;
+    for (const auto& tdata : instance.tenants()) {
+      if (tdata.tenant_id() == tenant_id) {
+        tenant = &tdata;
+        break;
+      }
+    }
+    if (tenant && !tenant->tenant_key().empty()) {
+      RETURN_NOT_OK(key_provider_->DecryptEncryptionKey(tenant->tenant_key(),
+                                                        tenant->tenant_key_iv(),
+                                                        tenant->tenant_key_version(),
+                                                        &decrypted_key));
+    }
+  } else if (!instance.server_key().empty()) {
+    RETURN_NOT_OK(key_provider_->DecryptEncryptionKey(instance.server_key(),
+                                                      instance.server_key_iv(),
+                                                      instance.server_key_version(),
+                                                      &decrypted_key));
+  }
+
+  if (!decrypted_key.empty()) {
+    LOG(INFO) << "Setting key " << decrypted_key;
+    env()->SetEncryptionKey(reinterpret_cast<const uint8_t*>(a2b_hex(decrypted_key).c_str()),
+                            decrypted_key.size() * 4);
   }
   return Status::OK();
 }
