@@ -99,6 +99,10 @@ class TabletServerAdminServiceProxy;
 class TabletServerServiceProxy;
 } // namespace tserver
 
+namespace consensus {
+class ConsensusServiceProxy;
+} // namespace consensus
+
 namespace cluster {
 
 class ExternalDaemon;
@@ -346,7 +350,7 @@ class ExternalMiniCluster : public MiniCluster {
   explicit ExternalMiniCluster(ExternalMiniClusterOptions opts);
 
   // Destroys a cluster.
-  virtual ~ExternalMiniCluster();
+  ~ExternalMiniCluster() override;
 
   // Start the cluster.
   Status Start() override;
@@ -383,11 +387,7 @@ class ExternalMiniCluster : public MiniCluster {
 
   // Return a pointer to the running leader master. This may be NULL
   // if the cluster is not started.
-  //
-  // TODO(unknown): Use the appropriate RPC here to return the leader master,
-  // to allow some of the existing tests (e.g., raft_consensus-itest)
-  // to use multiple masters.
-  ExternalMaster* leader_master() { return master(0); }
+  ExternalMaster* leader_master();
 
   // Perform an RPC to determine the leader of the external mini
   // cluster.  Set 'index' to the leader master's index (for calls to
@@ -479,21 +479,21 @@ class ExternalMiniCluster : public MiniCluster {
   }
 
   // Returns the WALs root directory for the tablet server 'ts_idx'.
-  virtual std::string WalRootForTS(int ts_idx) const override;
+  std::string WalRootForTS(int ts_idx) const override;
 
   // Returns the UUID for the tablet server 'ts_idx'.
-  virtual std::string UuidForTS(int ts_idx) const override;
+  std::string UuidForTS(int ts_idx) const override;
 
   // Returns the Env on which the cluster operates. If encryption is enabled,
   // the encryption key is incorrect. For reading/writing files, ts_env() and
   // master_env() should be used instead.
-  virtual Env* env() const override;
+  Env* env() const override;
 
   // Returns the Env on which a specific tablet server operates.
-  virtual Env* ts_env(int ts_idx) const override;
+  Env* ts_env(int ts_idx) const override;
 
   // Returns the Env on which a specific master operates.
-  virtual Env* master_env(int master_idx) const override;
+  Env* master_env(int master_idx) const override;
 
   BindMode bind_mode() const override {
     return opts_.bind_mode;
@@ -507,6 +507,8 @@ class ExternalMiniCluster : public MiniCluster {
   std::shared_ptr<tserver::TabletServerServiceProxy> tserver_proxy(int idx) const override;
   std::shared_ptr<tserver::TabletServerAdminServiceProxy> tserver_admin_proxy(
       int idx) const override;
+  std::shared_ptr<consensus::ConsensusServiceProxy> tserver_consensus_proxy(
+      int idx) const override;
 
   std::string block_manager_type() const {
     return opts_.block_manager_type;
@@ -611,6 +613,18 @@ class ExternalMiniCluster : public MiniCluster {
                       size_t idx,
                       scoped_refptr<ExternalMaster>* master);
 
+  // Verify the cluster contains 'num_masters' on all masters, and that they
+  // are all voters. Returns an error if the expected state is not present.
+  //
+  // This should be used if it is required that all masters have accepted the config
+  // change. E.g. tests that restart a cluster after adding a master should verify
+  // that all masters agree before restarting, in case lagging masters start up with
+  // stale configs.
+  //
+  // TODO(awong): we should be more robust to starting up with mismatched on-disk
+  // configs, if we can help it.
+  Status VerifyVotersOnAllMasters(int num_masters);
+
  private:
   Status StartMasters();
 
@@ -667,6 +681,8 @@ struct ExternalDaemonOptions {
   std::string ranger_cluster_key;
 };
 
+static const std::string kDefaultTenantID = "00000000000000000000000000000000";
+
 class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
  public:
   explicit ExternalDaemon(ExternalDaemonOptions opts);
@@ -722,7 +738,7 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
                         const std::string& principal_base,
                         const std::string& bind_host);
 
-  Status SetServerKey();
+  Status SetEncryptionKey(const std::string& tenant_id = kDefaultTenantID);
 
   // Sends a SIGSTOP signal to the daemon.
   Status Pause() WARN_UNUSED_RESULT;
@@ -866,11 +882,11 @@ class ExternalMaster : public ExternalDaemon {
  public:
   explicit ExternalMaster(ExternalDaemonOptions opts);
 
-  virtual Status Start() override;
+  Status Start() override;
 
   // Restarts the daemon.
   // Requires that it has previously been shutdown.
-  virtual Status Restart() override WARN_UNUSED_RESULT;
+  Status Restart() override WARN_UNUSED_RESULT;
 
   Env* env() const override { return env_.get(); }
 
@@ -893,13 +909,15 @@ class ExternalMaster : public ExternalDaemon {
   static std::vector<std::string> GetMasterFlags(const ExternalDaemonOptions& opts);
  private:
   friend class RefCountedThreadSafe<ExternalMaster>;
+
   // Get flags specific to ExternalMaster where 'rpc_bind_addr' and 'http_addr' are
   // the RPC and HTTP addresses to bind in case of start or the corresponding bound
   // addresses in case of restart.
   static std::vector<std::string> GetCommonFlags(const HostPort& rpc_bind_addr,
                                                  const HostPort& http_addr = HostPort());
+  ~ExternalMaster() override;
+
   const std::unique_ptr<Env> env_;
-  virtual ~ExternalMaster();
 };
 
 class ExternalTabletServer : public ExternalDaemon {
@@ -907,11 +925,11 @@ class ExternalTabletServer : public ExternalDaemon {
   ExternalTabletServer(ExternalDaemonOptions opts,
                        std::vector<HostPort> master_addrs);
 
-  virtual Status Start() override;
+  Status Start() override;
 
   // Restarts the daemon.
   // Requires that it has previously been shutdown.
-  virtual Status Restart() override WARN_UNUSED_RESULT;
+  Status Restart() override WARN_UNUSED_RESULT;
 
   Env* env() const override { return env_.get(); }
  private:
@@ -919,7 +937,7 @@ class ExternalTabletServer : public ExternalDaemon {
   const std::unique_ptr<Env> env_;
 
   friend class RefCountedThreadSafe<ExternalTabletServer>;
-  virtual ~ExternalTabletServer();
+  ~ExternalTabletServer() override;
 };
 
 } // namespace cluster
