@@ -29,17 +29,24 @@
 
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/postgres/mini_postgres.h"
 #include "kudu/ranger/ranger.pb.h"
-#include "kudu/util/curl_util.h"
 #include "kudu/util/env.h"
 #include "kudu/util/path_util.h"
 #include "kudu/util/status.h"
+#include "kudu/util/subprocess.h"
 #include "kudu/util/test_util.h"
 
 namespace kudu {
 class EasyJson;
-class Subprocess;
+
+namespace postgres {
+class MiniPostgres;
+}  // namespace postgres
+}  // namespace kudu
+
+using kudu::postgres::MiniPostgres;
+
+namespace kudu {
 
 namespace ranger {
 
@@ -70,19 +77,19 @@ struct AuthorizationPolicy {
 // Wrapper around Apache Ranger to be used in integration tests.
 class MiniRanger {
  public:
-  explicit MiniRanger(std::string host)
-    : MiniRanger(GetTestDataDirectory(), std::move(host)) {}
+  explicit MiniRanger(std::string host, std::shared_ptr<postgres::MiniPostgres> mini_pg) :
+  MiniRanger(GetTestDataDirectory(), std::move(host), std::move(mini_pg)) {}
 
   ~MiniRanger();
 
-  MiniRanger(std::string data_root, std::string host)
+  MiniRanger(std::string data_root,
+             std::string host,
+             std::shared_ptr<postgres::MiniPostgres> mini_pg)
     : data_root_(std::move(data_root)),
       host_(std::move(host)),
-      mini_pg_(data_root_, host_),
+      mini_pg_(std::move(mini_pg)),
       kerberos_(false),
-      env_(Env::Default()) {
-        curl_.set_auth(CurlAuthType::BASIC, "admin", "admin");
-      }
+      env_(Env::Default()) {}
 
   // Starts Ranger and its dependencies.
   Status Start() WARN_UNUSED_RESULT;
@@ -115,14 +122,26 @@ class MiniRanger {
     return ranger_admin_url_;
   }
 
- private:
+
+  // Returns Ranger admin's home directory.
+  std::string ranger_admin_home() const {
+    return JoinPathSegments(data_root_, "ranger-admin");
+  }
+
+  bool IsRunning() const { return process_ && process_->IsStarted(); }
+
+  // Sends a POST request to Ranger with 'payload'.
+  Status PostToRanger(const std::string& url, const EasyJson& payload, bool secure = false)
+    WARN_UNUSED_RESULT;
+
+private:
   // Starts the Ranger service.
   Status StartRanger() WARN_UNUSED_RESULT;
 
   // Initializes Ranger within 'admin_home' (home directory of the Ranger
   // admin). Sets 'fresh_install' to true if 'admin_home' didn't exist before
   // calling InitRanger().
-  Status InitRanger(std::string admin_home, bool* fresh_install)
+  Status InitRanger(const std::string& admin_home, bool* fresh_install)
     WARN_UNUSED_RESULT;
 
   // Creates configuration files.
@@ -135,14 +154,6 @@ class MiniRanger {
   // Creates a Kudu service in Ranger.
   Status CreateKuduService() WARN_UNUSED_RESULT;
 
-  // Sends a POST request to Ranger with 'payload'.
-  Status PostToRanger(std::string url, EasyJson payload) WARN_UNUSED_RESULT;
-
-  // Returns Ranger admin's home directory.
-  std::string ranger_admin_home() const {
-    return JoinPathSegments(data_root_, "ranger-admin");
-  }
-
   std::string bin_dir() const {
     std::string exe;
     CHECK_OK(env_->GetExecutablePath(&exe));
@@ -162,8 +173,7 @@ class MiniRanger {
   // Directory in which to put all our stuff.
   const std::string data_root_;
   const std::string host_;
-
-  postgres::MiniPostgres mini_pg_;
+  std::shared_ptr<kudu::postgres::MiniPostgres> mini_pg_;
   std::unique_ptr<Subprocess> process_;
 
   // URL of the Ranger admin REST API.
@@ -183,7 +193,6 @@ class MiniRanger {
   std::string krb5_config_;
 
   Env* env_;
-  EasyCurl curl_;
 
   uint16_t port_ = 0;
 
