@@ -25,7 +25,7 @@
 #include <utility>
 #include <vector>
 
-#include <boost/container_hash/extensions.hpp>
+#include <boost/container_hash/hash.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <google/protobuf/any.pb.h>
@@ -50,6 +50,7 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+DECLARE_string(keytab_file);
 DECLARE_string(log_dir);
 DECLARE_string(ranger_config_path);
 DECLARE_string(ranger_log_config_dir);
@@ -65,7 +66,6 @@ using kudu::subprocess::SubprocessMetrics;
 using kudu::subprocess::SubprocessRequestPB;
 using kudu::subprocess::SubprocessResponsePB;
 using kudu::subprocess::SubprocessServer;
-using std::move;
 using std::string;
 using std::unordered_map;
 using std::unordered_set;
@@ -161,11 +161,11 @@ class RangerClientTest : public KuduTest {
   void Allow(string user_name, ActionPB action, string database_name,
              string table_name, string column_name = "") {
     AuthorizedAction authorized_action;
-    authorized_action.user_name = move(user_name);
+    authorized_action.user_name = std::move(user_name);
     authorized_action.action = action;
-    authorized_action.database_name = move(database_name);
-    authorized_action.table_name = move(table_name);
-    authorized_action.column_name = move(column_name);
+    authorized_action.database_name = std::move(database_name);
+    authorized_action.table_name = std::move(table_name);
+    authorized_action.column_name = std::move(column_name);
 
     next_response_->emplace(authorized_action);
   }
@@ -346,14 +346,21 @@ TEST_F(RangerClientTest, TestInvalidJARFails) {
   ASSERT_FALSE(ValidateRangerConfiguration());
 }
 
+TEST_F(RangerClientTest, TestEmptyKeytabFile) {
+  FLAGS_ranger_config_path = test_dir_;
+  ASSERT_FALSE(ValidateRangerConfiguration());
+}
+
 TEST_F(RangerClientTest, TestMultipleInvalidJARsLeftUnchecked) {
   FLAGS_ranger_config_path = test_dir_;
   FLAGS_ranger_jar_path = "/this/is/not/a/real/location/hopefully.jar:/another/invalid/path.jar";
+  FLAGS_keytab_file = "/this/is/a/dummy/keytab_file";
   ASSERT_TRUE(ValidateRangerConfiguration());
 }
 
 TEST_F(RangerClientTest, TestDefaultJARPath) {
   FLAGS_ranger_config_path = test_dir_;
+  FLAGS_keytab_file = "/this/is/a/dummy/keytab_file";
   ASSERT_TRUE(ValidateRangerConfiguration());
 }
 
@@ -429,7 +436,8 @@ TEST_F(RangerClientTestExtraJVMArgs, SmokeTest) {
 
 TEST_F(RangerClientTestExtraJVMArgs, TestCrashOnInvalidArguments) {
   FLAGS_ranger_java_extra_args = "-XX:+InvalidArgumentHopeTheyWontChooseThisNameForAFLag";
-  ASSERT_DEATH(InitializeRanger(), "The subprocess has exited with status 1");
+  ASSERT_DEATH({ ASSERT_OK(InitializeRanger()); },
+               "The subprocess has exited with status 1");
 }
 
 namespace {
