@@ -16,7 +16,6 @@
 // under the License.
 #pragma once
 
-#include <cstddef>
 #include <cstdint>
 #include <limits>
 #include <memory>
@@ -53,8 +52,6 @@ class OutboundCall;
 class ReactorThread;
 class RpcConnectionPB;
 class RpczStore;
-class SocketStatsPB;
-class TransportDetailsPB;
 
 enum class CredentialsPolicy;
 
@@ -89,8 +86,8 @@ class Connection : public RefCountedThreadSafe<Connection> {
   // remote: the address of the remote end
   // socket: the socket to take ownership of.
   // direction: whether we are the client or server side
-  Connection(ReactorThread *reactor_thread,
-             Sockaddr remote,
+  Connection(ReactorThread* reactor_thread,
+             const Sockaddr& remote,
              std::unique_ptr<Socket> socket,
              Direction direction,
              CredentialsPolicy policy = CredentialsPolicy::ANY_CREDENTIALS);
@@ -138,7 +135,7 @@ class Connection : public RefCountedThreadSafe<Connection> {
 
   // Cancel an outbound call by removing any reference to it by CallAwaitingResponse
   // in 'awaiting_responses_'.
-  void CancelOutboundCall(const std::shared_ptr<OutboundCall> &call);
+  void CancelOutboundCall(const std::shared_ptr<OutboundCall>& call);
 
   // The address of the remote end of the connection.
   const Sockaddr& remote() const { return remote_; }
@@ -184,10 +181,10 @@ class Connection : public RefCountedThreadSafe<Connection> {
   RpczStore* rpcz_store();
 
   // libev callback when data is available to read.
-  void ReadHandler(ev::io &watcher, int revents);
+  void ReadHandler(ev::io& watcher, int revents); // NOLINT(google-runtime-references)
 
   // libev callback when we may write to the socket.
-  void WriteHandler(ev::io &watcher, int revents);
+  void WriteHandler(ev::io& watcher, int revents);// NOLINT(google-runtime-references)
 
   enum ProcessOutboundTransfersResult {
     // All of the transfers in the queue have been sent successfully.
@@ -224,7 +221,7 @@ class Connection : public RefCountedThreadSafe<Connection> {
   void MarkNegotiationComplete();
 
   Status DumpPB(const DumpConnectionsRequestPB& req,
-                RpcConnectionPB* resp);
+                RpcConnectionPB* resp) const;
 
   ReactorThread* reactor_thread() const { return reactor_thread_; }
 
@@ -263,10 +260,6 @@ class Connection : public RefCountedThreadSafe<Connection> {
     scheduled_for_shutdown_ = true;
   }
 
-  size_t num_queued_outbound_transfers() const {
-    return outbound_transfers_.size();
-  }
-
  private:
   friend struct CallAwaitingResponse;
   friend class QueueTransferTask;
@@ -279,9 +272,9 @@ class Connection : public RefCountedThreadSafe<Connection> {
     ~CallAwaitingResponse();
 
     // Notification from libev that the call has timed out.
-    void HandleTimeout(ev::timer &watcher, int revents);
+    void HandleTimeout(ev::timer& watcher, int revents);  // NOLINT(google-runtime-references)
 
-    Connection *conn;
+    Connection* conn;
     std::shared_ptr<OutboundCall> call;
     ev::timer timeout_timer;
 
@@ -312,7 +305,7 @@ class Connection : public RefCountedThreadSafe<Connection> {
 
   // The given CallAwaitingResponse has elapsed its user-defined timeout.
   // Set it to Failed.
-  void HandleOutboundCallTimeout(CallAwaitingResponse *car);
+  void HandleOutboundCallTimeout(CallAwaitingResponse* car);
 
   // Queue a transfer for sending on this connection.
   // We will take ownership of the transfer.
@@ -321,11 +314,7 @@ class Connection : public RefCountedThreadSafe<Connection> {
 
   // Internal test function for injecting cancellation request when 'call'
   // reaches state specified in 'FLAGS_rpc_inject_cancellation_state'.
-  void MaybeInjectCancellation(const std::shared_ptr<OutboundCall> &call);
-
-  Status GetSocketStatsPB(SocketStatsPB* pb) const;
-
-  Status GetTransportDetailsPB(TransportDetailsPB* pb) const;
+  void MaybeInjectCancellation(const std::shared_ptr<OutboundCall>& call);
 
   // The reactor thread that created this connection.
   ReactorThread* const reactor_thread_;
@@ -344,7 +333,7 @@ class Connection : public RefCountedThreadSafe<Connection> {
   RemoteUser remote_user_;
 
   // whether we are client or server
-  Direction direction_;
+  const Direction direction_;
 
   // The last time we read or wrote from the socket.
   MonoTime last_activity_time_;
