@@ -19,9 +19,10 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <optional>
+#include <shared_mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -97,6 +98,11 @@ class MessengerBuilder {
     return *this;
   }
 
+  MessengerBuilder& set_acceptor_listen_backlog(int max_queue_len) {
+    acceptor_listen_backlog_ = max_queue_len;
+    return *this;
+  }
+
   // Set the number of reactor threads that will be used for sending and
   // receiving.
   MessengerBuilder& set_num_reactors(int num_reactors) {
@@ -104,6 +110,12 @@ class MessengerBuilder {
     return *this;
   }
 
+  // Set the maximum size of RPC message for sending and receiving.
+  MessengerBuilder& set_rpc_max_message_size(int64_t rpc_max_message_size) {
+    rpc_max_message_size_ = rpc_max_message_size;
+    return *this;
+  }
+
   // Set the minimum number of connection-negotiation threads that will be used
   // to handle the blocking connection-negotiation step.
   MessengerBuilder& set_min_negotiation_threads(int min_negotiation_threads) {
@@ -144,6 +156,12 @@ class MessengerBuilder {
     return *this;
   }
 
+  // Set the name of the node where the result messenger will be running.
+  MessengerBuilder& set_hostname(const std::string& hostname) {
+    hostname_ = hostname;
+    return *this;
+  }
+
   // Set the SASL protocol name that is used for the SASL negotiation.
   MessengerBuilder& set_sasl_proto_name(const std::string& sasl_proto_name) {
     sasl_proto_name_ = sasl_proto_name;
@@ -192,7 +210,7 @@ class MessengerBuilder {
   // list and
   // https://www.openssl.org/docs/man1.1.1/man3/SSL_CTX_set_ciphersuites.html
   // for SSL_CTX_set_ciphersuites() API details.
-  MessengerBuilder &set_rpc_tls_ciphersuites(
+  MessengerBuilder& set_rpc_tls_ciphersuites(
       const std::string& rpc_tls_ciphersuites) {
     rpc_tls_ciphersuites_ = rpc_tls_ciphersuites;
     return *this;
@@ -200,7 +218,7 @@ class MessengerBuilder {
 
   // Set the minimum protocol version to allow when for securing RPC connections
   // with TLS. May be one of 'TLSv1', 'TLSv1.1', 'TLSv1.2', 'TLSv1.3'.
-  MessengerBuilder &set_rpc_tls_min_protocol(
+  MessengerBuilder& set_rpc_tls_min_protocol(
       const std::string& rpc_tls_min_protocol) {
     rpc_tls_min_protocol_ = rpc_tls_min_protocol;
     return *this;
@@ -264,12 +282,15 @@ class MessengerBuilder {
  private:
   const std::string name_;
   MonoDelta connection_keepalive_time_;
+  int acceptor_listen_backlog_;
   int num_reactors_;
+  int64_t rpc_max_message_size_;
   int min_negotiation_threads_;
   int max_negotiation_threads_;
   MonoDelta coarse_timer_granularity_;
   scoped_refptr<MetricEntity> metric_entity_;
   int64_t rpc_negotiation_timeout_ms_;
+  std::string hostname_;
   std::string sasl_proto_name_;
   std::string rpc_authentication_;
   std::string rpc_encryption_;
@@ -387,20 +408,20 @@ class Messenger {
   JwtVerifier* mutable_jwt_verifier() { return jwt_verifier_.get(); }
 
   std::optional<security::SignedTokenPB> authn_token() const {
-    std::lock_guard<simple_spinlock> l(authn_token_lock_);
+    std::lock_guard l(authn_token_lock_);
     return authn_token_;
   }
   void set_authn_token(const security::SignedTokenPB& token) {
-    std::lock_guard<simple_spinlock> l(authn_token_lock_);
+    std::lock_guard l(authn_token_lock_);
     authn_token_ = token;
   }
 
   std::optional<security::JwtRawPB> jwt() const {
-    std::lock_guard<simple_spinlock> l(authn_token_lock_);
+    std::lock_guard l(authn_token_lock_);
     return jwt_;
   }
   void set_jwt(const security::JwtRawPB& token) {
-    std::lock_guard<simple_spinlock> l(authn_token_lock_);
+    std::lock_guard l(authn_token_lock_);
     jwt_ = token;
   }
 
@@ -419,12 +440,12 @@ class Messenger {
   }
 
   void SetServicesRegistered() {
-    std::lock_guard<percpu_rwlock> guard(lock_);
+    std::lock_guard guard(lock_);
     state_ = kServicesRegistered;
   }
 
   bool closing() const {
-    shared_lock<rw_spinlock> l(lock_.get_lock());
+    std::shared_lock<rw_spinlock> l(lock_.get_lock());
     return state_ == kClosing;
   }
 
@@ -434,6 +455,16 @@ class Messenger {
     return rpc_negotiation_timeout_ms_;
   }
 
+  int64_t rpc_max_message_size() const { return rpc_max_message_size_; }
+
+  // The name of the node where this Messenger is running. The best case is
+  // FQDN retrieved using getaddrinfo(), but it might be just local hostname
+  // retrived by gethostname(). It can also be empty if Messenger has been
+  // created without setting the hostname.
+  const std::string& hostname() const {
+    return hostname_;
+  }
+
   const std::string& sasl_proto_name() const {
     return sasl_proto_name_;
   }
@@ -452,7 +483,7 @@ class Messenger {
 
   explicit Messenger(const MessengerBuilder& bld);
 
-  Reactor* RemoteToReactor(const Sockaddr& remote);
+  Reactor* RemoteToReactor(const Sockaddr& remote) const;
   Status Init();
   void RunTimeoutThread();
   void UpdateCurTime();
@@ -470,6 +501,12 @@ class Messenger {
   // any references. See 'retain_self_' for more info.
   void AllExternalReferencesDropped();
 
+  // Get the total number of currently pending connections across all the RPC
+  // endpoints this messenger is bound to. This utility method returns -1
+  // if the information on the listened socket's backlog cannot be retrieved
+  // from all of the RPC endpoints.
+  int32_t GetPendingConnectionsNum();
+
   const std::string name_;
 
   // Protects closing_, acceptor_pools_, rpc_services_.
@@ -535,6 +572,12 @@ class Messenger {
   // Timeout in milliseconds after which an incomplete connection negotiation will timeout.
   const int64_t rpc_negotiation_timeout_ms_;
 
+  // Maximum RPC message size (in bytes) set by MessengerBuilder.
+  int64_t rpc_max_message_size_;
+
+  // The name of the node where this messenger is running.
+  const std::string hostname_;
+
   // The SASL protocol name that is used for the SASL negotiation.
   const std::string sasl_proto_name_;
 
@@ -542,7 +585,12 @@ class Messenger {
   const std::string keytab_file_;
 
   // Whether to set SO_REUSEPORT on the listening sockets.
-  bool reuseport_;
+  const bool reuseport_;
+
+  // Acceptor's listened socket backlog: the capacity of the queue to
+  // accommodate incoming (but not accepted yet) connection requests to the
+  // messenger's listening sockets.
+  const int acceptor_listen_backlog_;
 
   // The ownership of the Messenger object is somewhat subtle. The pointer graph
   // looks like this:
@@ -589,6 +637,8 @@ class Messenger {
   // within a Reactor thread itself.
   std::shared_ptr<Messenger> retain_self_;
 
+  FunctionGaugeDetacher metric_detacher_;
+
   DISALLOW_COPY_AND_ASSIGN(Messenger);
 };
 
