@@ -72,6 +72,12 @@ DEFINE_bool(rpc_encrypt_loopback_connections, false,
             "an attacker.");
 TAG_FLAG(rpc_encrypt_loopback_connections, advanced);
 
+DEFINE_bool(rpc_suppress_negotiation_trace, false,
+            "Whether to suppress all negotiation traces: do not dump trace "
+            "of a connection negotiation into the log, even for a failed one. "
+            "For testing only!");
+TAG_FLAG(rpc_suppress_negotiation_trace, unsafe);
+
 using kudu::security::RpcAuthentication;
 using kudu::security::RpcEncryption;
 using std::string;
@@ -251,13 +257,10 @@ static Status DoServerNegotiation(Connection* conn,
                                   bool encrypt_loopback,
                                   const MonoTime& deadline) {
   auto* messenger = conn->reactor_thread()->reactor()->messenger();
-  if (authentication == RpcAuthentication::REQUIRED &&
-      messenger->keytab_file().empty() &&
-      !messenger->tls_context().is_external_cert()) {
-    return Status::InvalidArgument("RPC authentication (--rpc_authentication) may not be "
-                                   "required unless Kerberos (--keytab_file) or external PKI "
-                                   "(--rpc_certificate_file et al) are configured");
-  }
+  DCHECK(authentication != RpcAuthentication::REQUIRED ||
+      !messenger->keytab_file().empty() ||
+      messenger->tls_context().is_external_cert())
+      << "misconfiguration: check ValidateRpcAuthnFlags() for details";
 
   if (FLAGS_rpc_negotiation_inject_delay_ms > 0) {
     LOG(WARNING) << "Injecting " << FLAGS_rpc_negotiation_inject_delay_ms
@@ -273,7 +276,9 @@ static Status DoServerNegotiation(Connection* conn,
                                        encryption,
                                        encrypt_loopback,
                                        messenger->sasl_proto_name());
-
+  if (!messenger->hostname().empty()) {
+    server_negotiation.set_server_fqdn(messenger->hostname());
+  }
   if (authentication != RpcAuthentication::DISABLED && !messenger->keytab_file().empty()) {
     RETURN_NOT_OK(server_negotiation.EnableGSSAPI());
   }
@@ -325,7 +330,8 @@ void Negotiation::RunNegotiation(const scoped_refptr<Connection>& conn,
       (s.IsNetworkError() && s.posix_code() == ECONNREFUSED) ||
       s.IsNotAuthorized());
 
-  if (is_bad || FLAGS_rpc_trace_negotiation) {
+  if ((is_bad || FLAGS_rpc_trace_negotiation) &&
+      PREDICT_TRUE(!FLAGS_rpc_suppress_negotiation_trace)) {
     string msg = Trace::CurrentTrace()->DumpToString();
     if (is_bad) {
       LOG(WARNING) << "Failed RPC negotiation. Trace:\n" << msg;
