@@ -27,6 +27,7 @@
 #include <mutex>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 
 #include <boost/intrusive/list.hpp>
@@ -50,6 +51,7 @@
 #include "kudu/util/countdown_latch.h"
 #include "kudu/util/debug/sanitizer_scopes.h"
 #include "kudu/util/flag_tags.h"
+#include "kudu/util/flags.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/sockaddr.h"
@@ -70,6 +72,7 @@ static const int kDefaultLibEvFlags = ev::KQUEUE;
 static const int kDefaultLibEvFlags = ev::AUTO;
 #endif
 
+using std::function;
 using std::string;
 using std::shared_ptr;
 using std::unique_ptr;
@@ -231,7 +234,7 @@ void ReactorThread::PollCompleteCb(struct ev_loop* loop) noexcept {
 }
 
 void ReactorThread::Shutdown(Messenger::ShutdownMode mode) {
-  CHECK(reactor_->closing()) << "Should be called after setting closing_ flag";
+  DCHECK(reactor_->closing()) << "Should be called after setting closing_ flag";
 
   VLOG(1) << name() << ": shutting down Reactor thread.";
   WakeThread();
@@ -239,7 +242,11 @@ void ReactorThread::Shutdown(Messenger::ShutdownMode mode) {
   if (mode == Messenger::ShutdownMode::SYNC) {
     // Join() will return a bad status if asked to join on the currently
     // running thread.
-    CHECK_OK(ThreadJoiner(thread_.get()).Join());
+    const auto s = ThreadJoiner(thread_.get()).Join();
+    if (PREDICT_FALSE(!s.ok())) {
+      LOG(DFATAL) << Substitute(
+          "$0: failed to join $1", s.ToString(), thread_->ToString());
+    }
   }
 }
 
@@ -606,7 +613,16 @@ Status ReactorThread::StartConnectionNegotiation(const scoped_refptr<Connection>
 
   scoped_refptr<Trace> trace(new Trace());
   ADOPT_TRACE(trace.get());
-  TRACE("Submitting negotiation task for $0", conn->ToString());
+  string local_addr_str;
+  {
+    Sockaddr local_addr;
+    if (auto s = conn->GetLocalAddress(&local_addr); PREDICT_FALSE(!s.ok())) {
+      WARN_NOT_OK(s, "failed to retrieve information on local address");
+    } else {
+      local_addr_str = Substitute(" (local address $0)", local_addr.ToString());
+    }
+  }
+  TRACE("Submitting negotiation task for $0$1", conn->ToString(), local_addr_str);
   auto authentication = reactor()->messenger()->authentication();
   auto encryption = reactor()->messenger()->encryption();
   auto loopback_encryption = reactor()->messenger()->loopback_encryption();
@@ -696,7 +712,8 @@ void ReactorThread::DestroyConnection(Connection* conn,
   // Unlink connection from lists.
   if (conn->direction() == Connection::CLIENT) {
     const auto range = client_conns_.equal_range(conn->outbound_connection_id());
-    CHECK(range.first != range.second) << "Couldn't find connection " << conn->ToString();
+    DCHECK(range.first != range.second)
+        << "couldn't find connection " << conn->ToString();
     // The client_conns_ container is a multi-map.
     for (auto it = range.first; it != range.second;) {
       if (it->second.get() == conn) {
@@ -717,8 +734,7 @@ void ReactorThread::DestroyConnection(Connection* conn,
   }
 }
 
-DelayedTask::DelayedTask(std::function<void(const Status&)> func,
-                         MonoDelta when)
+DelayedTask::DelayedTask(function<void(const Status&)> func, MonoDelta when)
     : func_(std::move(func)),
       when_(when),
       thread_(nullptr) {
@@ -775,7 +791,7 @@ Status Reactor::Init() {
 
 void Reactor::Shutdown(Messenger::ShutdownMode mode) {
   {
-    std::lock_guard<LockType> l(lock_);
+    std::lock_guard l(lock_);
     if (closing_) {
       return;
     }
@@ -803,14 +819,14 @@ const string& Reactor::name() const {
 }
 
 bool Reactor::closing() const {
-  std::lock_guard<LockType> l(lock_);
+  std::lock_guard l(lock_);
   return closing_;
 }
 
 // Task to call an arbitrary function within the reactor thread.
 class RunFunctionTask : public ReactorTask {
  public:
-  explicit RunFunctionTask(std::function<Status()> f)
+  explicit RunFunctionTask(function<Status()> f)
       : function_(std::move(f)), latch_(1) {}
 
   void Run(ReactorThread* /*reactor*/) override {
@@ -830,7 +846,7 @@ class RunFunctionTask : public ReactorTask {
   }
 
  private:
-  const std::function<Status()> function_;
+  const function<Status()> function_;
   Status status_;
   CountDownLatch latch_;
 };
@@ -839,7 +855,7 @@ Status Reactor::GetMetrics(ReactorMetrics* metrics) {
   return RunOnReactorThread([&]() { return this->thread_.GetMetrics(metrics); });
 }
 
-Status Reactor::RunOnReactorThread(std::function<Status()> f) {
+Status Reactor::RunOnReactorThread(function<Status()> f) {
   RunFunctionTask task(std::move(f));
   ScheduleReactorTask(&task);
   return task.Wait();
@@ -939,7 +955,7 @@ void Reactor::ScheduleReactorTask(ReactorTask* task) {
   bool was_empty;
   {
     std::unique_lock<LockType> l(lock_);
-    if (closing_) {
+    if (PREDICT_FALSE(closing_)) {
       // We guarantee the reactor lock is not taken when calling Abort().
       l.unlock();
       task->Abort(ShutdownError(false));
@@ -954,7 +970,7 @@ void Reactor::ScheduleReactorTask(ReactorTask* task) {
 }
 
 bool Reactor::DrainTaskQueue(boost::intrusive::list<ReactorTask>* tasks) { // NOLINT(*)
-  std::lock_guard<LockType> l(lock_);
+  std::lock_guard l(lock_);
   if (closing_) {
     return false;
   }
