@@ -45,6 +45,7 @@
 #include "kudu/util/jwt.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/path_util.h"
 #include "kudu/util/pb_util.h"
@@ -419,16 +420,22 @@ class RpcTestBase : public KuduTest {
  public:
   RpcTestBase()
       : n_acceptor_pool_threads_(2),
+        n_negotiation_threads_(4),
         n_server_reactor_threads_(3),
         n_worker_threads_(3),
         keepalive_time_ms_(1000),
+        rpc_negotiation_timeout_ms_(3000),
         service_queue_length_(200),
         metric_entity_(METRIC_ENTITY_server.Instantiate(&metric_registry_, "test.rpc_test")) {
   }
 
   void TearDown() override {
     if (service_pool_) {
-      server_messenger_->UnregisterService(service_name_);
+      // NOTE: by design, the service might not be registered during this phase
+      //       in a few scenarios, but for the majority of them such outcome
+      //       isn't expected; at least, let's log about error, if any
+      WARN_NOT_OK(server_messenger_->UnregisterService(service_name_),
+                  "error unregistering service");
       service_pool_->Shutdown();
     }
     if (server_messenger_) {
@@ -445,7 +452,8 @@ class RpcTestBase : public KuduTest {
                          const std::string& rpc_certificate_file = "",
                          const std::string& rpc_private_key_file = "",
                          const std::string& rpc_ca_certificate_file = "",
-                         const std::string& rpc_private_key_password_cmd = "") {
+                         const std::string& rpc_private_key_password_cmd = "",
+                         int64_t rpc_max_message_size = FLAGS_rpc_max_message_size) {
     MessengerBuilder bld(name);
 
     if (enable_ssl) {
@@ -458,6 +466,7 @@ class RpcTestBase : public KuduTest {
     }
 
     bld.set_num_reactors(n_reactors);
+    bld.set_rpc_max_message_size(rpc_max_message_size);
     bld.set_connection_keepalive_time(MonoDelta::FromMilliseconds(keepalive_time_ms_));
     if (keepalive_time_ms_ >= 0) {
       // In order for the keepalive timing to be accurate, we need to scan connections
@@ -467,6 +476,14 @@ class RpcTestBase : public KuduTest {
           MonoDelta::FromMilliseconds(std::min(keepalive_time_ms_ / 5, 100)));
     }
     bld.set_metric_entity(metric_entity_);
+    bld.set_rpc_negotiation_timeout_ms(rpc_negotiation_timeout_ms_);
+    bld.set_min_negotiation_threads(n_negotiation_threads_);
+    bld.set_max_negotiation_threads(n_negotiation_threads_);
+
+    std::string hostname;
+    RETURN_NOT_OK(GetFQDN(&hostname));
+    bld.set_hostname(hostname);
+
     return bld.Build(messenger);
   }
 
@@ -513,6 +530,37 @@ static void DoTestSidecar(Proxy* p, int size1, int size2) {
     CHECK_EQ(Slice(expected), second);
   }
 
+static Status DoTestSidecarWithSizeLimits(Proxy* p, int size1, int size2) {
+    const uint32_t kSeed = 12345;
+
+    SendTwoStringsRequestPB req;
+    req.set_size1(size1);
+    req.set_size2(size2);
+    req.set_random_seed(kSeed);
+
+    SendTwoStringsResponsePB resp;
+    RpcController controller;
+    controller.set_timeout(MonoDelta::FromMilliseconds(10000));
+    Status status = p->SyncRequest(GenericCalculatorService::kSendTwoStringsMethodName,
+                                   req, &resp, &controller);
+    if (status.ok()) {
+      Slice first = GetSidecarPointer(controller, resp.sidecar1(), size1);
+      Slice second = GetSidecarPointer(controller, resp.sidecar2(), size2);
+      Random rng(kSeed);
+      faststring expected;
+
+      expected.resize(size1);
+      RandomString(expected.data(), size1, &rng);
+      CHECK_EQ(Slice(expected), first);
+
+      expected.resize(size2);
+      RandomString(expected.data(), size2, &rng);
+      CHECK_EQ(Slice(expected), second);
+    }
+
+    return status;
+  }
+
   static Status DoTestOutgoingSidecar(Proxy* p, int size1, int size2) {
     return DoTestOutgoingSidecar(p, {std::string(size1, 'a'), std::string(size2, 'b')});
   }
@@ -607,13 +655,15 @@ static void DoTestSidecar(Proxy* p, int size1, int size2) {
 
   // Start a simple socket listening on a local port, returning the address.
   // This isn't an RPC server -- just a plain socket which can be helpful for testing.
-  static Status StartFakeServer(Socket *listen_sock, Sockaddr *listen_addr) {
+  static Status StartFakeServer(Socket* listen_sock,
+                                Sockaddr* listen_addr,
+                                int listen_backlog = 1) {
     Sockaddr bind_addr = Sockaddr::Wildcard();
     bind_addr.set_port(0);
     RETURN_NOT_OK(listen_sock->Init(bind_addr.family(), 0));
-    RETURN_NOT_OK(listen_sock->BindAndListen(bind_addr, 1));
+    RETURN_NOT_OK(listen_sock->BindAndListen(bind_addr, listen_backlog));
     RETURN_NOT_OK(listen_sock->GetSocketAddress(listen_addr));
-    LOG(INFO) << "Bound to: " << listen_addr->ToString();
+    VLOG(1) << "Bound to: " << listen_addr->ToString();
     return Status::OK();
   }
 
@@ -664,9 +714,11 @@ static void DoTestSidecar(Proxy* p, int size1, int size2) {
 
  protected:
   int n_acceptor_pool_threads_;
+  int n_negotiation_threads_;
   int n_server_reactor_threads_;
   int n_worker_threads_;
   int keepalive_time_ms_;
+  int rpc_negotiation_timeout_ms_;
   int service_queue_length_;
 
   std::string service_name_;
