@@ -19,12 +19,13 @@
 #include <cstdint>
 #include <memory>
 #include <string>
+#include <type_traits>
 
 #include <glog/logging.h>
 
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
-#include "kudu/rpc/messenger.h"
+#include "kudu/rpc/messenger.h" // IWYU pragma: keep
 #include "kudu/security/simple_acl.h"
 #include "kudu/server/server_base_options.h"
 #include "kudu/util/countdown_latch.h"
@@ -51,6 +52,10 @@ namespace clock {
 class Clock;
 } // namespace clock
 
+namespace cloud {
+class InstanceMetadata;
+} // namespace cloud
+
 namespace rpc {
 class ResultTracker;
 class RpcContext;
@@ -222,7 +227,19 @@ class ServerBase {
   // The ACL of users who may act as part of the Kudu service.
   security::SimpleAcl service_acl_;
 
+  CountDownLatch stop_background_threads_latch_;
+
  private:
+  // Whether to enable sanity check on wall clock jumps for certain environments
+  // that are prone to such an issue. The logic is controlled by
+  // --wall_clock_jump_detection and --wall_clock_jump_threshold_sec flags.
+  // If with OK status the *threshold_usec > 0, the clock jump detection
+  // should be enabled with the specified threshold, while *threshold_usec == 0
+  // means clock jump detection isn't needed.
+  static Status WallClockJumpDetectionNeeded(
+      uint64_t* threshold_usec,
+      std::unique_ptr<cloud::InstanceMetadata>* im);
+
   Status InitAcls();
   void GenerateInstanceID();
   Status DumpServerInfo(const std::string& path,
@@ -273,7 +290,6 @@ class ServerBase {
 #ifdef TCMALLOC_ENABLED
   scoped_refptr<Thread> tcmalloc_memory_gc_thread_;
 #endif
-  CountDownLatch stop_background_threads_latch_;
 
   std::unique_ptr<ScopedGLogMetrics> glog_metrics_;
 
