@@ -60,17 +60,60 @@ DEFINE_string(test_server_key_version, "",
               "It is only used when creating the file system, it's disregarded on "
               "consecutive startups. It should only be used in tests.");
 TAG_FLAG(test_server_key_version, hidden);
+
+DEFINE_string(test_tenant_name, "",
+              "Tenant name in plain-text to be persisted into the instance file. "
+              "It is only used when creating the file system, it's disregarded on "
+              "consecutive startups. It should only be used in tests.");
+TAG_FLAG(test_tenant_name, hidden);
+
+DEFINE_string(test_tenant_id, "",
+              "Tenant id in plain-text to be persisted into the instance file. "
+              "It is only used when creating the file system, it's disregarded on "
+              "consecutive startups. It should only be used in tests.");
+TAG_FLAG(test_tenant_id, hidden);
+
+DEFINE_string(test_tenant_key, "",
+              "Tenant key in plain-text to be persisted into the instance file. "
+              "It is only used when creating the file system, it's disregarded on "
+              "consecutive startups. It should only be used in tests.");
+TAG_FLAG(test_tenant_key, hidden);
+
+DEFINE_string(test_tenant_key_iv, "",
+              "Tenant key IV in plain-text to be persisted into the instance file. "
+              "It is only used when creating the file system, it's disregarded on "
+              "consecutive startups. It should only be used in tests.");
+TAG_FLAG(test_tenant_key_iv, hidden);
+
+
+DEFINE_string(test_tenant_key_version, "",
+              "Tenant key version in plain-text to be persisted into the instance file. "
+              "It is only used when creating the file system, it's disregarded on "
+              "consecutive startups. It should only be used in tests.");
+TAG_FLAG(test_tenant_key_version, hidden);
+
 namespace kudu {
 namespace server {
 
+ServerKeyInfo::ServerKeyInfo()
+  : server_key(FLAGS_test_server_key),
+    server_key_iv(FLAGS_test_server_key_iv),
+    server_key_version(FLAGS_test_server_key_version) {
+}
+
+TenantKeyInfo::TenantKeyInfo()
+  : tenant_name(FLAGS_test_tenant_name),
+    tenant_id(FLAGS_test_tenant_id),
+    tenant_key(FLAGS_test_tenant_key),
+    tenant_key_iv(FLAGS_test_tenant_key_iv),
+    tenant_key_version(FLAGS_test_tenant_key_version) {
+}
+
 ServerBaseOptions::ServerBaseOptions()
   : env(Env::Default()),
     dump_info_path(FLAGS_server_dump_info_path),
     dump_info_format(FLAGS_server_dump_info_format),
-    metrics_log_interval_ms(FLAGS_metrics_log_interval_ms),
-    server_key(FLAGS_test_server_key),
-    server_key_iv(FLAGS_test_server_key_iv),
-    server_key_version(FLAGS_test_server_key_version) {
+    metrics_log_interval_ms(FLAGS_metrics_log_interval_ms) {
 }
 
 } // namespace server
