@@ -22,6 +22,7 @@
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
 #include <openssl/ssl.h>
 #endif
+#include <gssapi/gssapi_krb5.h>
 #include <sys/socket.h>
 
 #include <algorithm>
@@ -31,13 +32,15 @@
 #include <cstring>
 #include <functional>
 #include <map>
-#include <mutex>
+#include <shared_mutex>
 #include <sstream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
 #include <vector>
+// IWYU pragma: no_include <__string>
 
 #include <boost/algorithm/string/case_conv.hpp>
 #include <gflags/gflags.h>
@@ -76,6 +79,7 @@ typedef sig_t sighandler_t;
 using mustache::RenderTemplate;
 using std::ostringstream;
 using std::pair;
+using std::shared_lock;
 using std::stringstream;
 using std::string;
 using std::unordered_set;
@@ -88,16 +92,54 @@ DEFINE_int32(webserver_max_post_length_bytes, 1024 * 1024,
 TAG_FLAG(webserver_max_post_length_bytes, advanced);
 TAG_FLAG(webserver_max_post_length_bytes, runtime);
 
+DEFINE_string(webserver_cache_control_options, "no-store",
+              "The webserver adds 'Cache-Control' HTTP header with this value "
+              "to all responses.");
+TAG_FLAG(webserver_cache_control_options, advanced);
+TAG_FLAG(webserver_cache_control_options, runtime);
+
 DEFINE_string(webserver_x_frame_options, "DENY",
               "The webserver will add an 'X-Frame-Options' HTTP header with this value "
               "to all responses. This can help prevent clickjacking attacks.");
 TAG_FLAG(webserver_x_frame_options, advanced);
+TAG_FLAG(webserver_x_frame_options, runtime);
 
 DEFINE_bool(webserver_enable_csp, true,
             "The webserver adds the Content-Security-Policy header to response when enabled.");
 TAG_FLAG(webserver_enable_csp, advanced);
 TAG_FLAG(webserver_enable_csp, runtime);
 
+DEFINE_int32(webserver_hsts_max_age_seconds, -1,
+             "The time, in seconds, that a browser should remember that the "
+             "webserver must only be accessed using HTTPS. The HTTP Strict "
+             "Transport Security (HSTS) policy is implemented by adding a "
+             "'Strict-Transport-Security' header. If greater or equal to zero, "
+             "a TLS-enabled webserver adds the HSTS header with corresponding "
+             "'max-age' setting to the response. If the setting is negative, "
+             "the HSTS header is not present in the response. As an option, it "
+             "is possible to add 'includeSubDomains' directive as specified "
+             "by the --webserver_hsts_include_sub_domains flag. "
+             "WARNING: once enabled, the HSTS policy affects *everything* at "
+             "the node/domain, so any HTTP endpoint at the node/domain "
+             "effectively becomes unreachable to the browser which has reached "
+             "the embedded webserver at this node via HTTPS");
+TAG_FLAG(webserver_hsts_max_age_seconds, advanced);
+TAG_FLAG(webserver_hsts_max_age_seconds, runtime);
+
+DEFINE_bool(webserver_hsts_include_sub_domains, false,
+            "Whether to add 'includeSubDomains' directive into the "
+            "'Strict-Transport-Security' header when the latter is enabled. "
+            "See --webserver_hsts_max_age_seconds's description for details.");
+TAG_FLAG(webserver_hsts_include_sub_domains, advanced);
+TAG_FLAG(webserver_hsts_include_sub_domains, runtime);
+
+DEFINE_string(webserver_x_content_type_options, "nosniff",
+              "The webserver adds 'X-Content-Type-Options' HTTP header with "
+              "this value to all responses.");
+TAG_FLAG(webserver_x_content_type_options, advanced);
+TAG_FLAG(webserver_x_content_type_options, runtime);
+
+DECLARE_string(spnego_keytab_file);
 
 namespace kudu {
 
@@ -315,14 +357,24 @@ Status Webserver::Start() {
   }
 
   if (opts_.require_spnego) {
-    // We assume that security::InitKerberosForServer() has already been called, which
-    // ensures that the keytab path has been propagated into this environment variable
-    // where the GSSAPI calls will pick it up.
-    const char* kt_file = getenv("KRB5_KTNAME");
+    // If the spnego_keytab_file flag is empty, GSSAPI will find the keytab path from
+    // KRB5_KTNAME environment variable which is set by InitKerberosForServer().
+    // Setting spnego_keytab_file flag will make GSSAPI to use spnego dedicated keytab
+    // instead of KRB5_KTNAME.
+    const char* kt_file = FLAGS_spnego_keytab_file.empty() ?
+      getenv("KRB5_KTNAME") :
+      FLAGS_spnego_keytab_file.c_str();
     if (!kt_file || !Env::Default()->FileExists(kt_file)) {
       return Status::InvalidArgument("Unable to configure web server for SPNEGO authentication: "
                                      "must configure a keytab file for the server");
     }
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+    // NOTE: this call is wrapped into 'ignored' pragma to suppress compilation
+    //       warnings on macOS with Xcode where many gssapi_krb5 functions are
+    //       deprecated in favor of GSS.framework.
+    krb5_gss_register_acceptor_identity(kt_file);
+#pragma GCC diagnostic pop
   }
 
   options.emplace_back("listening_ports");
@@ -391,7 +443,7 @@ Status Webserver::Start() {
                       [this](const WebRequest& req, WebResponse* resp) {
                         this->RootHandler(req, resp);
                       },
-                      /*is_styled=*/true, /*is_on_nav_bar=*/true);
+                      StyleMode::STYLED, /*is_on_nav_bar=*/true);
 
   vector<Sockaddr> addrs;
   RETURN_NOT_OK(GetBoundAddresses(&addrs));
@@ -475,7 +527,7 @@ int Webserver::LogMessageCallbackStatic(const struct sq_connection* /*connection
     // the squeasel server uses the log callback to report on errors.
     {
       static simple_spinlock kErrMsgLock_;
-      std::unique_lock<simple_spinlock> l(kErrMsgLock_);
+      std::lock_guard l(kErrMsgLock_);
       kWebserverLastErrMsg = message;
     }
     LOG(ERROR) << "Webserver: " << message;
@@ -631,8 +683,8 @@ sq_callback_result_t Webserver::RunPathHandler(
   }
 
   // Should we render with css styles?
-  StyleMode use_style = handler.is_styled() && !ContainsKey(req.parsed_args, "raw") ?
-                        StyleMode::STYLED : StyleMode::UNSTYLED;
+  StyleMode use_style = ContainsKey(req.parsed_args, "raw") ?
+                        StyleMode::UNSTYLED : handler.style_mode();
   SendResponse(connection, resp, &req, use_style);
   return SQ_HANDLED_OK;
 }
@@ -646,27 +698,28 @@ void Webserver::SendResponse(struct sq_connection* connection,
     DCHECK(req);
     stringstream ss;
     RenderMainTemplate(*req, resp->output.str(), &ss);
-    resp->output.str(ss.str());
+    resp->output.rdbuf()->swap(*ss.rdbuf());
   }
 
   // Check if gzip compression is accepted by the caller. If so, compress the
   // content and replace the prerendered output.
-  const char* accept_encoding_str = sq_get_header(connection, "Accept-Encoding");
+  auto& output = resp->output;
   bool is_compressed = false;
+  const char* accept_encoding_str = sq_get_header(connection, "Accept-Encoding");
   vector<string> encodings = strings::Split(accept_encoding_str, ",");
   for (string& encoding : encodings) {
     StripWhiteSpace(&encoding);
     if (encoding == "gzip") {
       // Don't bother compressing empty content.
-      string uncompressed = resp->output.str();
-      if (uncompressed.empty()) {
+      DCHECK(output.good());
+      if (output.tellp() == 0) {
         break;
       }
 
       ostringstream oss;
-      Status s = zlib::CompressLevel(uncompressed, 1, &oss);
+      const auto s = zlib::Compress(output.str(), &oss);
       if (PREDICT_TRUE(s.ok())) {
-        resp->output.str(oss.str());
+        output.swap(oss);
         is_compressed = true;
       } else {
         LOG(WARNING) << "Could not compress output: " << s.ToString();
@@ -677,7 +730,7 @@ void Webserver::SendResponse(struct sq_connection* connection,
 
   // We've deferred constructing the content for as long as possible; we must
   // do so now so that we can determine the content length.
-  string body = resp->output.str();
+  string body = output.str();
 
   // Buffers up the headers and content as follows:
   //
@@ -692,7 +745,7 @@ void Webserver::SendResponse(struct sq_connection* connection,
   oss << Substitute("HTTP/1.1 $0\r\n", HttpStatusCodeToString(resp->status_code));
 
   // The "Content-Type" is defined by the value of the 'mode' parameter.
-  const char* content_type = nullptr;
+  const char* content_type = "text/plain";
   switch (mode) {
     case StyleMode::STYLED:
       content_type = "text/html";
@@ -703,9 +756,12 @@ void Webserver::SendResponse(struct sq_connection* connection,
     case StyleMode::BINARY:
       content_type = "application/octet-stream";
       break;
+    case StyleMode::JSON:
+      content_type = "application/json";
+      break;
     default:
-      LOG(FATAL) << "unexpected style mode: " << static_cast<uint32_t>(mode);
-      break;  // unreachable
+      LOG(DFATAL) << "unexpected style mode: " << static_cast<uint32_t>(mode);
+      break;
   }
   oss << Substitute("Content-Type: $0\r\n", content_type);
 
@@ -713,6 +769,11 @@ void Webserver::SendResponse(struct sq_connection* connection,
   if (is_compressed) {
     oss << "Content-Encoding: gzip\r\n";
   }
+  const auto& cache_control_options = FLAGS_webserver_cache_control_options;
+  if (!cache_control_options.empty()) {
+    oss << Substitute("Cache-Control: $0\r\n", cache_control_options);
+  }
+
   if (PREDICT_TRUE(FLAGS_webserver_enable_csp)) {
     // TODO(aserbin): add information on when to update the SHA hash and
     //                how to do so (ideally, the exact command line)
@@ -720,19 +781,37 @@ void Webserver::SendResponse(struct sq_connection* connection,
         << "style-src 'self' 'unsafe-hashes' 'sha256-47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=';"
         << "img-src 'self' data:;\r\n";
   }
-  oss << Substitute("X-Frame-Options: $0\r\n", FLAGS_webserver_x_frame_options);
+  const auto& x_frame_options = FLAGS_webserver_x_frame_options;
+  if (!x_frame_options.empty()) {
+    oss << Substitute("X-Frame-Options: $0\r\n", x_frame_options);
+  }
+  if (IsSecure() && FLAGS_webserver_hsts_max_age_seconds >= 0) {
+    oss << Substitute("Strict-Transport-Security: max-age=$0",
+                      FLAGS_webserver_hsts_max_age_seconds);
+    if (FLAGS_webserver_hsts_include_sub_domains) {
+      oss << "; includeSubDomains";
+    }
+    oss << "\r\n";
+  }
+  const auto& x_content_type_options = FLAGS_webserver_x_content_type_options;
+  if (!x_content_type_options.empty()) {
+    oss << Substitute("X-Content-Type-Options: $0\r\n", x_content_type_options);
+  }
   static const unordered_set<string> kInvalidHeaders = {
+    "Cache-Control",
     "Content-Encoding",
     "Content-Length",
-    "Content-Type",
     "Content-Security-Policy",
+    "Content-Type",
+    "Strict-Transport-Security",
+    "X-Content-Type-Options",
     "X-Frame-Options",
   };
   for (const auto& entry : resp->response_headers) {
     // It's forbidden to override the above headers.
     if (PREDICT_FALSE(ContainsKey(kInvalidHeaders, entry.first))) {
-      LOG(FATAL) << Substitute("Reserved header $0 was overridden by handler",
-                               entry.first);
+      LOG(DFATAL) << Substitute("reserved header $0 was overridden by handler",
+                                entry.first);
     }
     oss << Substitute("$0: $1\r\n", entry.first, entry.second);
   }
@@ -760,7 +839,7 @@ string Webserver::MustachePartialTag(const string& path) {
 }
 
 void Webserver::RegisterPathHandler(const string& path, const string& alias,
-    const PathHandlerCallback& callback, bool is_styled, bool is_on_nav_bar) {
+    const PathHandlerCallback& callback, StyleMode style_mode, bool is_on_nav_bar) {
   string render_path = (path == "/") ? "/home" : path;
   auto wrapped_cb = [=](const WebRequest& req, PrerenderedWebResponse* rendered_resp) {
     WebResponse resp;
@@ -772,30 +851,42 @@ void Webserver::RegisterPathHandler(const string& path, const string& alias,
     // do not render the page
     if (render_path != "/home" || is_started_) {
       stringstream out;
-      Render(render_path, resp.output, is_styled, &out);
-      rendered_resp->output << out.rdbuf();
+      Render(render_path, resp.output, style_mode, &out);
+      rendered_resp->output.rdbuf()->swap(*out.rdbuf());
     }
   };
-  RegisterPrerenderedPathHandler(path, alias, wrapped_cb, is_styled, is_on_nav_bar);
+  RegisterPrerenderedPathHandler(path, alias, wrapped_cb, style_mode, is_on_nav_bar);
 }
 
 void Webserver::RegisterPrerenderedPathHandler(const string& path, const string& alias,
-    const PrerenderedPathHandlerCallback& callback, bool is_styled, bool is_on_nav_bar) {
-  std::lock_guard<RWMutex> l(lock_);
-  InsertOrDie(&path_handlers_, path, new PathHandler(is_styled, is_on_nav_bar, alias, callback));
+    const PrerenderedPathHandlerCallback& callback, StyleMode style_mode, bool is_on_nav_bar) {
+  std::lock_guard l(lock_);
+  InsertOrDie(&path_handlers_, path, new PathHandler(style_mode, is_on_nav_bar, alias, callback));
 }
 
 void Webserver::RegisterBinaryDataPathHandler(
     const string& path,
     const string& alias,
     const PrerenderedPathHandlerCallback& callback) {
-  std::lock_guard<RWMutex> l(lock_);
-  InsertOrDie(&path_handlers_, path, new PathHandler(false /*is_styled*/,
+  std::lock_guard l(lock_);
+  InsertOrDie(&path_handlers_, path, new PathHandler(StyleMode::BINARY,
                                                      false /*is_on_nav_bar*/,
                                                      alias,
                                                      callback));
 }
 
+void Webserver::RegisterJsonPathHandler(
+    const string& path,
+    const string& alias,
+    const PrerenderedPathHandlerCallback& callback,
+    bool is_on_nav_bar) {
+  std::lock_guard l(lock_);
+  InsertOrDie(&path_handlers_, path, new PathHandler(StyleMode::JSON,
+                                                     is_on_nav_bar,
+                                                     alias,
+                                                     callback));
+}
+
 bool Webserver::MustacheTemplateAvailable(const string& path) const {
   if (!static_pages_available()) {
     return false;
@@ -880,11 +971,11 @@ void Webserver::RenderMainTemplate(
   RenderTemplate(kMainTemplate, opts_.doc_root, ej.value(), output);
 }
 
-void Webserver::Render(const string& path, const EasyJson& ej, bool use_style,
+void Webserver::Render(const string& path, const EasyJson& ej, StyleMode style_mode,
                        stringstream* output) {
   if (MustacheTemplateAvailable(path)) {
     RenderTemplate(MustachePartialTag(path), opts_.doc_root, ej.value(), output);
-  } else if (use_style) {
+  } else if (style_mode == StyleMode::STYLED) {
     (*output) << "<pre>" << ej.ToString() << "</pre>";
   } else {
     (*output) << ej.ToString();
@@ -896,7 +987,7 @@ bool Webserver::static_pages_available() const {
 }
 
 void Webserver::set_footer_html(const std::string& html) {
-  std::lock_guard<RWMutex> l(lock_);
+  std::lock_guard l(lock_);
   footer_html_ = html;
 }
 
