@@ -137,18 +137,16 @@ Status CFileSet::DoOpen(const IOContext* io_context) {
 
   // Lazily open the column data cfiles. Each one will be fully opened
   // later, when the first iterator seeks for the first time.
-  RowSetMetadata::ColumnIdToBlockIdMap block_map = rowset_metadata_->GetColumnBlocksById();
-  for (const RowSetMetadata::ColumnIdToBlockIdMap::value_type& e : block_map) {
-    ColumnId col_id = e.first;
-    DCHECK(!ContainsKey(readers_by_col_id_, col_id)) << "already open";
-
+  const RowSetMetadata::ColumnIdToBlockIdMap block_map = rowset_metadata_->GetColumnBlocksById();
+  for (const auto& [col_id, _] : block_map) {
     unique_ptr<CFileReader> reader;
     RETURN_NOT_OK(OpenReader(rowset_metadata_->fs_manager(),
                              cfile_reader_tracker_,
                              rowset_metadata_->column_data_block_for_col_id(col_id),
                              io_context,
                              &reader));
-    readers_by_col_id_[col_id] = std::move(reader);
+    const auto& ins_info = readers_by_col_id_.emplace(col_id, std::move(reader));
+    DCHECK(ins_info.second) << "already open";
     VLOG(1) << "Successfully opened cfile for column id " << col_id
             << " in " << rowset_metadata_->ToString();
   }
@@ -170,6 +168,7 @@ Status CFileSet::DoOpen(const IOContext* io_context) {
   } else {
     RETURN_NOT_OK(LoadMinMaxKeys(io_context));
   }
+
   // Verify the loaded keys are valid.
   if (Slice(min_encoded_key_) > max_encoded_key_) {
     return Status::Corruption(Substitute("Min key $0 > max key $1",
@@ -178,6 +177,12 @@ Status CFileSet::DoOpen(const IOContext* io_context) {
                               ToString());
   }
 
+  // Update the min/max key in metadata explicitly if user specified.
+  if (FLAGS_rowset_metadata_store_keys && !rowset_metadata_->has_encoded_keys()) {
+    rowset_metadata_->set_min_encoded_key(min_encoded_key_);
+    rowset_metadata_->set_max_encoded_key(max_encoded_key_);
+  }
+
   return Status::OK();
 }
 
@@ -203,7 +208,7 @@ Status CFileSet::OpenBloomReader(const IOContext* io_context) {
 
 Status CFileSet::LoadMinMaxKeys(const IOContext* io_context) {
   CFileReader* key_reader = key_index_reader();
-  RETURN_NOT_OK(key_index_reader()->Init(io_context));
+  RETURN_NOT_OK(key_reader->Init(io_context));
   if (!key_reader->GetMetadataEntry(DiskRowSet::kMinKeyMetaEntryName, &min_encoded_key_)) {
     return Status::Corruption("No min key found", ToString());
   }
@@ -240,8 +245,9 @@ unique_ptr<CFileSet::Iterator> CFileSet::NewIterator(
 }
 
 Status CFileSet::CountRows(const IOContext* io_context, rowid_t *count) const {
-  RETURN_NOT_OK(key_index_reader()->Init(io_context));
-  return key_index_reader()->CountRows(count);
+  CFileReader* key_reader = key_index_reader();
+  RETURN_NOT_OK(key_reader->Init(io_context));
+  return key_reader->CountRows(count);
 }
 
 Status CFileSet::GetBounds(string* min_encoded_key,
@@ -336,8 +342,9 @@ Status CFileSet::CheckRowPresent(const RowSetKeyProbe& probe, const IOContext* i
 
 Status CFileSet::NewKeyIterator(const IOContext* io_context,
                                 unique_ptr<CFileIterator>* key_iter) const {
-  RETURN_NOT_OK(key_index_reader()->Init(io_context));
-  return key_index_reader()->NewIterator(key_iter, CFileReader::CACHE_BLOCK, io_context);
+  CFileReader* key_reader = key_index_reader();
+  RETURN_NOT_OK(key_reader->Init(io_context));
+  return key_reader->NewIterator(key_iter, CFileReader::CACHE_BLOCK, io_context);
 }
 
 ////////////////////////////////////////////////////////////
@@ -586,9 +593,7 @@ Status CFileSet::Iterator::MaterializeColumn(ColumnMaterializationContext *ctx)
   RETURN_NOT_OK(PrepareColumn(ctx));
   ColumnIterator* iter = col_iters_[ctx->col_idx()].get();
 
-  RETURN_NOT_OK(iter->Scan(ctx));
-
-  return Status::OK();
+  return iter->Scan(ctx);
 }
 
 Status CFileSet::Iterator::FinishBatch() {
