@@ -21,6 +21,7 @@
 #include <memory>
 #include <optional>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -35,7 +36,7 @@
 #include "kudu/common/schema.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
-#include "kudu/tablet/rowset_metadata.h"
+#include "kudu/tablet/rowset_metadata.h" // IWYU pragma: keep
 #include "kudu/util/make_shared.h"
 #include "kudu/util/memory/arena.h"
 #include "kudu/util/status.h"
@@ -56,6 +57,10 @@ namespace fs {
 struct IOContext;
 }  // namespace fs
 
+namespace tserver {
+class TabletServerTest_SetEncodedKeysWhenStartingUp_Test;
+} // namespace tserver
+
 namespace tablet {
 
 class RowSetKeyProbe;
@@ -133,6 +138,7 @@ class CFileSet :
 
  private:
   friend class Iterator;
+  FRIEND_TEST(tserver::TabletServerTest, SetEncodedKeysWhenStartingUp);
 
   DISALLOW_COPY_AND_ASSIGN(CFileSet);
 
@@ -182,27 +188,28 @@ class CFileSet :
 // together, and iterated in parallel.
 class CFileSet::Iterator : public ColumnwiseIterator {
  public:
+  ~Iterator() override;
 
-  virtual Status Init(ScanSpec *spec) OVERRIDE;
+  Status Init(ScanSpec *spec) override;
 
-  virtual Status PrepareBatch(size_t *nrows) OVERRIDE;
+  Status PrepareBatch(size_t *nrows) override;
 
-  virtual Status InitializeSelectionVector(SelectionVector *sel_vec) OVERRIDE;
+  Status InitializeSelectionVector(SelectionVector *sel_vec) override;
 
   Status MaterializeColumn(ColumnMaterializationContext *ctx) override;
 
-  virtual Status FinishBatch() OVERRIDE;
+  Status FinishBatch() override;
 
-  virtual bool HasNext() const OVERRIDE {
+  bool HasNext() const override {
     DCHECK(initted_);
     return cur_idx_ < upper_bound_idx_;
   }
 
-  virtual std::string ToString() const OVERRIDE {
+  std::string ToString() const override {
     return std::string("rowset iterator for ") + base_data_->ToString();
   }
 
-  const Schema &schema() const OVERRIDE {
+  const Schema &schema() const override {
     return *projection_;
   }
 
@@ -213,9 +220,8 @@ class CFileSet::Iterator : public ColumnwiseIterator {
   }
 
   // Collect the IO statistics for each of the underlying columns.
-  virtual void GetIteratorStats(std::vector<IteratorStats> *stats) const OVERRIDE;
+  void GetIteratorStats(std::vector<IteratorStats> *stats) const override;
 
-  virtual ~Iterator();
  private:
   DISALLOW_COPY_AND_ASSIGN(Iterator);
   FRIEND_TEST(TestCFileSet, TestRangeScan);
