@@ -66,6 +66,7 @@ using kudu::fs::IOContext;
 using kudu::fs::FsErrorManager;
 using kudu::fs::KUDU_2233_CORRUPTION;
 using std::deque;
+using std::make_shared;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
@@ -113,8 +114,8 @@ void AdvanceToLastInList(const Mutation** m) {
   }
 }
 
-// CompactionInput yielding rows and mutations from a MemRowSet.
-class MemRowSetCompactionInput : public CompactionInput {
+// CompactionOrFlushInput yielding rows and mutations from a MemRowSet.
+class MemRowSetCompactionInput : public CompactionOrFlushInput {
  public:
   MemRowSetCompactionInput(const MemRowSet& memrowset,
                            const MvccSnapshot& snap,
@@ -138,7 +139,7 @@ class MemRowSetCompactionInput : public CompactionInput {
   }
 
   Status PrepareBlock(vector<CompactionInputRow>* block) override {
-    int num_in_block = iter_->remaining_in_leaf();
+    const auto num_in_block = iter_->remaining_in_leaf();
     block->resize(num_in_block);
 
     // Realloc the internal block storage if we don't have enough space to
@@ -163,7 +164,7 @@ class MemRowSetCompactionInput : public CompactionInput {
 
       // Handle the rare case where a row was inserted and deleted in the same operation.
       // This row can never be observed and should not be compacted/flushed. This saves
-      // us some trouble later on on compactions.
+      // us some trouble later during compaction.
       //
       // See CompareDuplicatedRows().
       if (PREDICT_FALSE(input_row.redo_head != nullptr &&
@@ -228,8 +229,8 @@ class MemRowSetCompactionInput : public CompactionInput {
 
 ////////////////////////////////////////////////////////////
 
-// CompactionInput yielding rows and mutations from an on-disk DiskRowSet.
-class DiskRowSetCompactionInput : public CompactionInput {
+// CompactionOrFlushInput yielding rows and mutations from an on-disk DiskRowSet.
+class DiskRowSetCompactionInput : public CompactionOrFlushInput {
  public:
   DiskRowSetCompactionInput(unique_ptr<RowwiseIterator> base_iter,
                             unique_ptr<DeltaIterator> redo_delta_iter,
@@ -476,7 +477,12 @@ int CompareDuplicatedRows(const CompactionInputRow& left,
     // the row again. If that's the case, the delete with the higher timestamp
     // defines the newer input, or
     int ret = left_last->timestamp().CompareTo(right_last->timestamp());
-    CHECK_NE(0, ret);
+    // Log timestamp and mutation history, invalid ts value could mean potential corruption.
+    CHECK_NE(0, ret)
+        << Substitute("different DELETE REDO mutations must not have same timestamp, "
+                      "Left Redo Mutations: $0, Right Redo Mutations: $1",
+                      Mutation::StringifyMutationList(*left.row.schema(), left.redo_head),
+                      Mutation::StringifyMutationList(*right.row.schema(), right.redo_head));
     return ret;
   }
 
@@ -534,7 +540,7 @@ void TransferRedoHistory(CompactionInputRow* newer, CompactionInputRow* older) {
 }
 
 
-class MergeCompactionInput : public CompactionInput {
+class MergeCompactionInput : public CompactionOrFlushInput {
  private:
   // State kept for each of the inputs.
   struct MergeState {
@@ -579,7 +585,7 @@ class MergeCompactionInput : public CompactionInput {
       return schema.Compare(pending.back().row, (*other.next()).row) < 0;
     }
 
-    shared_ptr<CompactionInput> input;
+    shared_ptr<CompactionOrFlushInput> input;
     vector<CompactionInputRow> pending;
     int pending_idx;
 
@@ -587,7 +593,7 @@ class MergeCompactionInput : public CompactionInput {
   };
 
  public:
-  MergeCompactionInput(const vector<shared_ptr<CompactionInput>>& inputs,
+  MergeCompactionInput(const vector<shared_ptr<CompactionOrFlushInput>>& inputs,
                        const Schema* schema)
       : schema_(schema),
         num_dup_rows_(0),
@@ -599,7 +605,7 @@ class MergeCompactionInput : public CompactionInput {
     }
   }
 
-  virtual ~MergeCompactionInput() {
+  ~MergeCompactionInput() override {
     STLDeleteElements(&states_);
   }
 
@@ -984,11 +990,12 @@ Mutation* MergeUndoHistories(Mutation* left, Mutation* right) {
 // If 'old_row' has previous versions, this transforms prior version in undos
 // and adds them to 'new_undo_head'.
 Status MergeDuplicatedRowHistory(const string& tablet_id,
-                                 const FsErrorManager* error_manager,
-                                 CompactionInputRow* old_row,
-                                 Mutation** new_undo_head,
-                                 Arena* arena) {
-  if (PREDICT_TRUE(old_row->previous_ghost == nullptr)) return Status::OK();
+                                 const scoped_refptr<FsErrorManager>& error_manager,
+                                 const CompactionInputRow& old_row,
+                                 Arena* arena,
+                                 const HistoryGcOpts& history_gc_opts,
+                                 Mutation** new_undo_head) {
+  if (PREDICT_TRUE(old_row.previous_ghost == nullptr)) return Status::OK();
 
   // Use an all inclusive snapshot as all of the previous version's undos and redos
   // are guaranteed to be committed, otherwise the compaction wouldn't be able to
@@ -997,7 +1004,7 @@ Status MergeDuplicatedRowHistory(const string& tablet_id,
 
   faststring dst;
 
-  CompactionInputRow* previous_ghost = old_row->previous_ghost;
+  CompactionInputRow* previous_ghost = old_row.previous_ghost;
   while (previous_ghost != nullptr) {
 
     // First step is to transform the old rows REDO's into UNDOs, if there are any.
@@ -1010,10 +1017,11 @@ Status MergeDuplicatedRowHistory(const string& tablet_id,
 
     RETURN_NOT_OK(ApplyMutationsAndGenerateUndos(all_snap,
                                                  *previous_ghost,
-                                                 &pv_new_undos_head,
-                                                 &pv_delete_redo,
                                                  arena,
-                                                 &previous_ghost->row));
+                                                 history_gc_opts,
+                                                 &previous_ghost->row,
+                                                 &pv_new_undos_head,
+                                                 &pv_delete_redo));
 
     // We should be left with only one redo, the delete.
 #ifndef NDEBUG
@@ -1089,11 +1097,11 @@ string CompactionInputRowToString(const CompactionInputRow& input_row) {
 
 ////////////////////////////////////////////////////////////
 
-Status CompactionInput::Create(const DiskRowSet& rowset,
-                               const Schema* projection,
-                               const MvccSnapshot& snap,
-                               const IOContext* io_context,
-                               unique_ptr<CompactionInput>* out) {
+Status CompactionOrFlushInput::Create(const DiskRowSet& rowset,
+                                      const Schema* projection,
+                                      const MvccSnapshot& snap,
+                                      const IOContext* io_context,
+                                      shared_ptr<CompactionOrFlushInput>* out) {
   CHECK(projection->has_column_ids());
 
   unique_ptr<ColumnwiseIterator> base_cwise(rowset.base_data_->NewIterator(projection, io_context));
@@ -1117,51 +1125,53 @@ Status CompactionInput::Create(const DiskRowSet& rowset,
   RETURN_NOT_OK_PREPEND(rowset.delta_tracker_->NewDeltaIterator(
       undo_opts, DeltaTracker::UNDOS_ONLY, &undo_deltas), "Could not open UNDOs");
 
-  out->reset(new DiskRowSetCompactionInput(std::move(base_iter),
-                                           std::move(redo_deltas),
-                                           std::move(undo_deltas)));
+  *out = make_shared<DiskRowSetCompactionInput>(
+      std::move(base_iter), std::move(redo_deltas), std::move(undo_deltas));
   return Status::OK();
 }
 
-CompactionInput* CompactionInput::Create(const MemRowSet& memrowset,
-                                         const Schema* projection,
-                                         const MvccSnapshot& snap) {
+shared_ptr<CompactionOrFlushInput> CompactionOrFlushInput::Create(
+    const MemRowSet& memrowset,
+    const Schema* projection,
+    const MvccSnapshot& snap) {
   CHECK(projection->has_column_ids());
-  return new MemRowSetCompactionInput(memrowset, snap, projection);
+  return make_shared<MemRowSetCompactionInput>(memrowset, snap, projection);
 }
 
-CompactionInput* CompactionInput::Merge(const vector<shared_ptr<CompactionInput>>& inputs,
-                                        const Schema* schema) {
+shared_ptr<CompactionOrFlushInput> CompactionOrFlushInput::Merge(
+    const vector<shared_ptr<CompactionOrFlushInput>>& inputs,
+    const Schema* schema) {
   CHECK(schema->has_column_ids());
-  return new MergeCompactionInput(inputs, schema);
+  return make_shared<MergeCompactionInput>(inputs, schema);
 }
 
 
-Status RowSetsInCompaction::CreateCompactionInput(const MvccSnapshot& snap,
-                                                  const Schema* schema,
-                                                  const IOContext* io_context,
-                                                  shared_ptr<CompactionInput>* out) const {
+Status RowSetsInCompactionOrFlush::CreateCompactionOrFlushInput(
+    const MvccSnapshot& snap,
+    const Schema* schema,
+    const IOContext* io_context,
+    shared_ptr<CompactionOrFlushInput>* out) const {
   CHECK(schema->has_column_ids());
 
-  vector<shared_ptr<CompactionInput>> inputs;
+  vector<shared_ptr<CompactionOrFlushInput>> inputs;
   for (const auto& rs : rowsets_) {
-    unique_ptr<CompactionInput> input;
+    shared_ptr<CompactionOrFlushInput> input;
     RETURN_NOT_OK_PREPEND(rs->NewCompactionInput(schema, snap, io_context, &input),
                           Substitute("Could not create compaction input for rowset $0",
                                      rs->ToString()));
-    inputs.push_back(shared_ptr<CompactionInput>(input.release()));
+    inputs.emplace_back(std::move(input));
   }
 
   if (inputs.size() == 1) {
     *out = std::move(inputs[0]);
   } else {
-    out->reset(CompactionInput::Merge(inputs, schema));
+    *out = CompactionOrFlushInput::Merge(inputs, schema);
   }
 
   return Status::OK();
 }
 
-void RowSetsInCompaction::DumpToLog() const {
+void RowSetsInCompactionOrFlush::DumpToLog() const {
   VLOG(1) << "Selected " << rowsets_.size() << " rowsets to compact:";
   // Dump the selected rowsets to the log, and collect corresponding iterators.
   for (const auto& rs : rowsets_) {
@@ -1170,13 +1180,11 @@ void RowSetsInCompaction::DumpToLog() const {
   }
 }
 
-void RemoveAncientUndos(const HistoryGcOpts& history_gc_opts,
-                        Mutation** undo_head,
+bool RemoveAncientUndos(const HistoryGcOpts& history_gc_opts,
                         const Mutation* redo_head,
-                        bool* is_garbage_collected) {
-  *is_garbage_collected = false;
+                        Mutation** undo_head) {
   if (!history_gc_opts.gc_enabled()) {
-    return;
+    return false;
   }
 
   // Make sure there is at most one REDO in the redo_head and that, if present, it's a DELETE.
@@ -1186,8 +1194,7 @@ void RemoveAncientUndos(const HistoryGcOpts& history_gc_opts,
 
     // Garbage collect rows that are deleted before the AHM.
     if (history_gc_opts.IsAncientHistory(redo_head->timestamp())) {
-      *is_garbage_collected = true;
-      return;
+      return true;
     }
   }
 
@@ -1209,6 +1216,7 @@ void RemoveAncientUndos(const HistoryGcOpts& history_gc_opts,
     prev_undo = undo_mut;
     undo_mut = undo_mut->next();
   }
+  return false;
 }
 
 // Applies the REDOs of 'src_row' in accordance with the input snapshot,
@@ -1219,10 +1227,11 @@ void RemoveAncientUndos(const HistoryGcOpts& history_gc_opts,
 // NOTE: input REDOs are expected to be in increasing timestamp order.
 Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
                                       const CompactionInputRow& src_row,
-                                      Mutation** new_undo_head,
-                                      Mutation** new_redo_head,
                                       Arena* arena,
-                                      RowBlockRow* dst_row) {
+                                      const HistoryGcOpts& history_gc_opts,
+                                      RowBlockRow* dst_row,
+                                      Mutation** new_undo_head,
+                                      Mutation** new_redo_head) {
   bool is_deleted = false;
 
   #define ERROR_LOG_CONTEXT \
@@ -1236,6 +1245,10 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
   // Const cast this away here since we're ever only going to point to it
   // which doesn't actually mutate it and having Mutation::set_next()
   // take a non-const value is required in other places.
+  // If there are any existing ancient undo mutations in the list,
+  // those will be removed in the caller i.e. FlushCompactionInput
+  // post processing of 'applying the mutations' and 'merging duplicate
+  // history'.
   Mutation* undo_head = const_cast<Mutation*>(src_row.undo_head);
   Mutation* redo_delete = nullptr;
 
@@ -1274,6 +1287,14 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
           continue;
         }
 
+        // If the timestamp of current redo mutation is ancient, don't consider
+        // it when converting to undo delta. Any ancient redo mutation is
+        // not taken into consideration when maintaining the past undo deltas
+        // as all these ancient updates are expected to be GC'ed anyway.
+        if (history_gc_opts.IsAncientHistory(redo_mut->timestamp())) {
+          continue;
+        }
+
         // create the UNDO mutation in the provided arena.
         current_undo = Mutation::CreateInArena(arena, redo_mut->timestamp(),
                                                undo_encoder.as_changelist());
@@ -1286,6 +1307,22 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
         redo_decoder.TwiddleDeleteStatus(&is_deleted);
         // Delete mutations are left as redos. Encode the DELETE as a redo.
         undo_encoder.SetToDelete();
+
+        // No need to check whether current redo mutation is ancient or not.
+        // There are two cases hereafter:
+        // 1. If this is the last mutation (of type DELETE) for this row and
+        //    if this mutation turns out to be ancient, it will be detected
+        //    in caller while processing ancient undo deltas. Essentially,
+        //    all the mutations for this row will be GC'ed in such a case with
+        //    just one allocation for redo_delete which is acceptable.
+        //
+        // 2. If there is at least a subsequent mutation for this row, it
+        //    would be a re-insert case as noted below. For such scenario,
+        //    redo_delete is anyway going to get deleted.
+        // As both cases are only causing one time object allocation for short
+        // duration, we can let this allocation go through without checking for
+        // ancient mark in order to avoid too many changes to current logic of
+        // handling DELETE and REINSERT mutations.
         redo_delete = Mutation::CreateInArena(arena,
                                               redo_mut->timestamp(),
                                               undo_encoder.as_changelist());
@@ -1294,7 +1331,7 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
       // When we see a reinsert REDO we do the following:
       // 1 - Reset the REDO head, which contained a DELETE REDO.
       // 2 - Apply the REINSERT to the row, passing an undo_encoder that encodes the state of
-      //     the row prior to to the REINSERT.
+      //     the row prior to the REINSERT.
       // 3 - Create a mutation for the REINSERT above and add it to the UNDOs, this mutation
       //     will have the timestamp of the DELETE REDO.
       // 4 - Create a new delete UNDO. This mutation will have the timestamp of the REINSERT REDO.
@@ -1315,12 +1352,22 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
             dst_row, static_cast<Arena*>(nullptr), &undo_encoder), ERROR,
                           "Unable to apply reinsert undo. \n" + ERROR_LOG_CONTEXT);
 
-        // 3 - Create a mutation for the REINSERT above and add it to the UNDOs.
-        current_undo = Mutation::CreateInArena(arena,
-                                               delete_ts,
-                                               undo_encoder.as_changelist());
-        SetHead(&undo_head, current_undo);
+        // Check if redo mutation of type DELETE from previous iteration is
+        // ancient. Skip creating undo object for the same if it is ancient.
+        // We still want to process redo mutation from current iteration though.
+        if (!history_gc_opts.IsAncientHistory(delete_ts)) {
+          // 3 - Create a mutation for the REINSERT above and add it to the UNDOs.
+          current_undo = Mutation::CreateInArena(arena,
+                                                 delete_ts,
+                                                 undo_encoder.as_changelist());
+          SetHead(&undo_head, current_undo);
+        }
 
+        // Check if redo mutation from current iteration is ancient. Skip
+        // creating corresponding undo object for the same if it is ancient.
+        if (history_gc_opts.IsAncientHistory(redo_mut->timestamp())) {
+          continue;
+        }
         // 4 - Create a DELETE mutation and add it to the UNDOs.
         undo_encoder.Reset();
         undo_encoder.SetToDelete();
@@ -1342,91 +1389,36 @@ Status ApplyMutationsAndGenerateUndos(const MvccSnapshot& snap,
   #undef ERROR_LOG_CONTEXT
 }
 
-Status FlushCompactionInput(const string& tablet_id,
-                            const FsErrorManager* error_manager,
-                            CompactionInput* input,
-                            const MvccSnapshot& snap,
-                            const HistoryGcOpts& history_gc_opts,
-                            RollingDiskRowSetWriter* out) {
-  RETURN_NOT_OK(input->Init());
-  vector<CompactionInputRow> rows;
+// Append REDO and UNDO deltas to DiskRowSetWriter output.
+static Status AppendDeltasToDRS(RollingDiskRowSetWriter* out,
+                              Mutation* new_undos_head,
+                              Mutation* new_redos_head,
+                              RowBlockRow* dst_row) {
+  rowid_t index_in_current_drs;
 
-  DCHECK(out->schema().has_column_ids());
+  if (new_undos_head != nullptr) {
+    // Append UNDO deltas to DiskRowSetWriter output.
+    RETURN_NOT_OK(out->AppendUndoDeltas(dst_row->row_index(),
+                                        new_undos_head,
+                                        &index_in_current_drs));
+  }
 
-  RowBlock block(&out->schema(), kCompactionOutputBlockNumRows, nullptr);
+  if (new_redos_head != nullptr) {
+    // Append REDO deltas to DiskRowSetWriter output.
+    RETURN_NOT_OK(out->AppendRedoDeltas(dst_row->row_index(),
+                                        new_redos_head,
+                                        &index_in_current_drs));
+  }
 
-  while (input->HasMoreBlocks()) {
-    RETURN_NOT_OK(input->PrepareBlock(&rows));
+  DVLOG(4) << "Output Row: " << dst_row->schema()->DebugRow(*dst_row)
+           << "; RowId: " << index_in_current_drs;
 
-    int n = 0;
-    int live_row_count = 0;
-    for (int i = 0; i < rows.size(); i++) {
-      CompactionInputRow* input_row = &rows[i];
-      RETURN_NOT_OK(out->RollIfNecessary());
+  return Status::OK();
+}
 
-      const Schema* schema = input_row->row.schema();
-      DCHECK_SCHEMA_EQ(*schema, out->schema());
-      DCHECK(schema->has_column_ids());
-
-      RowBlockRow dst_row = block.row(n);
-      RETURN_NOT_OK(CopyRow(input_row->row, &dst_row, static_cast<Arena*>(nullptr)));
-
-      DVLOG(4) << "Input Row: " << CompactionInputRowToString(*input_row);
-
-      // Collect the new UNDO/REDO mutations.
-      Mutation* new_undos_head = nullptr;
-      Mutation* new_redos_head = nullptr;
-
-      // Apply all REDOs to the base row, generating UNDOs for it. This does
-      // not take into account any 'previous_ghost' members.
-      RETURN_NOT_OK(ApplyMutationsAndGenerateUndos(snap,
-                                                   *input_row,
-                                                   &new_undos_head,
-                                                   &new_redos_head,
-                                                   input->PreparedBlockArena(),
-                                                   &dst_row));
-
-      // Merge the histories of 'input_row' with previous ghosts, if there are any.
-      RETURN_NOT_OK(MergeDuplicatedRowHistory(tablet_id,
-                                              error_manager,
-                                              input_row,
-                                              &new_undos_head,
-                                              input->PreparedBlockArena()));
-
-      // Remove ancient UNDOS and check whether the row should be garbage collected.
-      bool is_garbage_collected;
-      RemoveAncientUndos(history_gc_opts,
-                         &new_undos_head,
-                         new_redos_head,
-                         &is_garbage_collected);
-
-      DVLOG(4) << "Output Row: " << RowToString(dst_row, new_redos_head, new_undos_head) <<
-          "; Was garbage collected? " << is_garbage_collected;
-
-      // Whether this row was garbage collected
-      if (is_garbage_collected) {
-        // Don't flush the row.
-        continue;
-      }
-
-      rowid_t index_in_current_drs;
-
-      if (new_undos_head != nullptr) {
-        out->AppendUndoDeltas(dst_row.row_index(), new_undos_head, &index_in_current_drs);
-      }
-
-      if (new_redos_head != nullptr) {
-        out->AppendRedoDeltas(dst_row.row_index(), new_redos_head, &index_in_current_drs);
-      }
-
-      // If the REDO is empty, it should not be a DELETE.
-      if (new_redos_head == nullptr) {
-        live_row_count++;
-      }
-
-      DVLOG(4) << "Output Row: " << dst_row.schema()->DebugRow(dst_row)
-               << "; RowId: " << index_in_current_drs;
 #ifndef NDEBUG
+// Sanity check for UNDO list.
+static void UndoListSanityCheck(Mutation* new_undos_head) {
       auto* u = new_undos_head;
       bool is_deleted = false;
       // The resulting list should have the following invariants:
@@ -1445,18 +1437,144 @@ Status FlushCompactionInput(const string& tablet_id,
         CHECK_GE(u->timestamp(), u->next()->timestamp());
         u = u->next();
       }
+}
 #endif // NDEBUG
 
-      n++;
-      if (n == block.nrows()) {
+// For each input row, go through all the REDO mutations and apply those to base row.
+// Generate corresponding UNDO deltas for applied mutations.
+// For a row with 'ghost' entries, merge their histories of mutations.
+// Remove ancient UNDO mutations and check if row is required to be garbage collected.
+// Append REDO and UNDO deltas to DRS output.
+// Do sanity check for final UNDO list.
+static Status ApplyMutationsAndMergeDuplicateHistory(const MvccSnapshot& snap,
+                                                     const CompactionInputRow& input_row,
+                                                     size_t cur_row_idx,
+                                                     RowBlock* block,
+                                                     Arena* arena,
+                                                     const string& tablet_id,
+                                                     const scoped_refptr<FsErrorManager>& err_mgr,
+                                                     const HistoryGcOpts& history_gc_opts,
+                                                     RollingDiskRowSetWriter* out,
+                                                     int* live_row_count,
+                                                     bool* is_garbage_collected) {
+  RETURN_NOT_OK(out->RollIfNecessary());
+
+  const Schema* schema = input_row.row.schema();
+  DCHECK_SCHEMA_EQ(*schema, out->schema());
+  DCHECK(schema->has_column_ids());
+
+  DVLOG(4) << "Input Row: " << CompactionInputRowToString(input_row);
+
+  RowBlockRow dst_row = block->row(cur_row_idx);
+  RETURN_NOT_OK(CopyRow(input_row.row, &dst_row, static_cast<Arena*>(nullptr)));
+
+  // Collect the new UNDO/REDO mutations.
+  Mutation* new_undos_head = nullptr;
+  Mutation* new_redos_head = nullptr;
+
+  // Apply all REDOs to the base row, generating UNDOs for it. This does
+  // not take into account any 'previous_ghost' members.
+  RETURN_NOT_OK(ApplyMutationsAndGenerateUndos(snap,
+                                               input_row,
+                                               arena,
+                                               history_gc_opts,
+                                               &dst_row,
+                                               &new_undos_head,
+                                               &new_redos_head));
+
+  // Merge the histories of 'input_row' with previous ghosts, if there are any.
+  RETURN_NOT_OK(MergeDuplicatedRowHistory(tablet_id,
+                                          err_mgr,
+                                          input_row,
+                                          arena,
+                                          history_gc_opts,
+                                          &new_undos_head));
+
+  // Remove ancient UNDOS and check whether the row should be garbage collected.
+  *is_garbage_collected = RemoveAncientUndos(history_gc_opts,
+                                             new_redos_head,
+                                             &new_undos_head);
+
+  DVLOG(4) << "Output Row: " << RowToString(dst_row, new_redos_head, new_undos_head) <<
+      "; Was garbage collected? " << *is_garbage_collected;
+
+  // Skip further processing if this row was garbage collected
+  if (!*is_garbage_collected) {
+    RETURN_NOT_OK(AppendDeltasToDRS(out,
+                                    new_undos_head,
+                                    new_redos_head,
+                                    &dst_row));
+
+    // If the REDO is empty, it should not be a DELETE.
+    if (new_redos_head == nullptr) {
+      (*live_row_count)++;
+    }
+
+#ifndef NDEBUG
+    UndoListSanityCheck(new_undos_head);
+#endif // NDEBUG
+  }
+
+  return Status::OK();
+}
+
+// Following method processes the compaction input by reading input rows in
+// blocks and for each row inside the block:
+// - Apply all REDO mutations collected for the row at hand.
+// - Generate corresponding UNDO deltas for applied mutations.
+// - For a row with 'ghost' entries, merge their histories of mutations.
+// - Remove any ancient UNDO mutations, as those are not applicable anymore.
+// - Append UNDO and REDO deltas to DiskRowSetWriter output.
+// - Append fully or partially (resized) processed rowblock to DRS writer output.
+Status FlushCompactionInput(const string& tablet_id,
+                            const scoped_refptr<FsErrorManager>& error_manager,
+                            CompactionOrFlushInput* input,
+                            const MvccSnapshot& snap,
+                            const HistoryGcOpts& history_gc_opts,
+                            RollingDiskRowSetWriter* out) {
+  RETURN_NOT_OK(input->Init());
+  vector<CompactionInputRow> rows;
+
+  DCHECK(out->schema().has_column_ids());
+
+  RowBlock block(&out->schema(), kCompactionOutputBlockNumRows, nullptr);
+
+  while (input->HasMoreBlocks()) {
+    RETURN_NOT_OK(input->PrepareBlock(&rows));
+
+    size_t cur_row_idx = 0;
+    int live_row_count = 0;
+    for (const auto& row : rows) {
+      bool is_garbage_collected = false;
+
+      RETURN_NOT_OK(ApplyMutationsAndMergeDuplicateHistory(snap,
+                                                           row,
+                                                           cur_row_idx,
+                                                           &block,
+                                                           input->PreparedBlockArena(),
+                                                           tablet_id,
+                                                           error_manager,
+                                                           history_gc_opts,
+                                                           out,
+                                                           &live_row_count,
+                                                           &is_garbage_collected));
+      // Whether this row was garbage collected
+      if (is_garbage_collected) {
+        // Don't flush the row.
+        continue;
+      }
+
+      if (++cur_row_idx == block.nrows()) {
+        // Append fully processed rowblock to DRS writer output.
         RETURN_NOT_OK(out->AppendBlock(block, live_row_count));
         live_row_count = 0;
-        n = 0;
+        cur_row_idx = 0;
       }
     }
 
-    if (n > 0) {
-      block.Resize(n);
+    if (cur_row_idx > 0) {
+      block.Resize(cur_row_idx);
+      // Append partially (resized) processed rowblock to DRS writer output.
       RETURN_NOT_OK(out->AppendBlock(block, live_row_count));
       block.Resize(block.row_capacity());
     }
@@ -1467,7 +1585,7 @@ Status FlushCompactionInput(const string& tablet_id,
 }
 
 Status ReupdateMissedDeltas(const IOContext* io_context,
-                            CompactionInput* input,
+                            CompactionOrFlushInput* input,
                             const HistoryGcOpts& history_gc_opts,
                             const MvccSnapshot& snap_to_exclude,
                             const MvccSnapshot& snap_to_include,
@@ -1639,7 +1757,9 @@ Status ReupdateMissedDeltas(const IOContext* io_context,
 }
 
 
-Status DebugDumpCompactionInput(CompactionInput* input, int64_t* rows_left, vector<string>* lines) {
+Status DebugDumpCompactionInput(CompactionOrFlushInput* input,
+                                int64_t* rows_left,
+                                vector<string>* lines) {
   RETURN_NOT_OK(input->Init());
   vector<CompactionInputRow> rows;
 
