@@ -26,6 +26,7 @@
 #include <memory>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_set>
 #include <utility>
 #include <vector>
@@ -35,8 +36,8 @@
 #include <gtest/gtest.h>
 
 #include "kudu/clock/logical_clock.h"
-#include "kudu/common/columnblock.h"
 #include "kudu/common/columnblock-test-util.h"
+#include "kudu/common/columnblock.h"
 #include "kudu/common/common.pb.h"
 #include "kudu/common/row_changelist.h"
 #include "kudu/common/rowblock.h"
@@ -58,6 +59,7 @@
 #include "kudu/tablet/mvcc.h"
 #include "kudu/tablet/rowset.h"
 #include "kudu/tablet/tablet-test-util.h"
+#include "kudu/util/bitmap.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/memory/arena.h"
@@ -212,7 +214,7 @@ TEST_F(TestDeltaMemStore, TestUpdateCount) {
   ASSERT_OK(fs.CreateNewBlock({}, &block));
   DeltaFileWriter dfw(std::move(block));
   ASSERT_OK(dfw.Start());
-  dms_->FlushToFile(&dfw);
+  ASSERT_OK(dms_->FlushToFile(&dfw));
   unique_ptr<DeltaStats> stats = dfw.release_delta_stats();
 
   ASSERT_EQ(n_rows / 2, stats->update_count_for_col_id(schema_.column_id(kIntColumn)));
@@ -589,9 +591,8 @@ TEST_F(TestDeltaMemStore, TestCollectMutations) {
 
   ASSERT_EQ(2, dms_->Count());
 
-  const int kBatchSize = 10;
-  vector<Mutation *> mutations;
-  mutations.resize(kBatchSize);
+  constexpr size_t kBatchSize = 10;
+  vector<Mutation*> mutations(kBatchSize);
 
   RowIteratorOptions opts;
   opts.projection = &schema_;
