@@ -21,6 +21,7 @@
 #include <map>
 #include <optional>
 #include <ostream>
+#include <shared_mutex>
 #include <vector>
 
 #include <gflags/gflags.h>
@@ -41,6 +42,7 @@
 #include "kudu/fs/fs_manager.h"
 #include "kudu/fs/io_context.h"
 #include "kudu/gutil/port.h"
+#include "kudu/gutil/ref_counted.h"
 #include "kudu/tablet/cfile_set.h"
 #include "kudu/tablet/compaction.h"
 #include "kudu/tablet/delta_compaction.h"
@@ -55,7 +57,6 @@
 #include "kudu/util/compression/compression.pb.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/flag_tags.h"
-#include "kudu/util/locks.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/scoped_cleanup.h"
@@ -89,6 +90,7 @@ using kudu::fs::IOContext;
 using kudu::fs::WritableBlock;
 using kudu::log::LogAnchorRegistry;
 using std::optional;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
@@ -96,8 +98,6 @@ using std::vector;
 
 namespace kudu {
 
-class Mutex;
-
 namespace consensus {
 class OpId;
 }
@@ -242,7 +242,7 @@ Status DiskRowSetWriter::AppendBlock(const RowBlock &block, int live_row_count)
 
 Status DiskRowSetWriter::Finish() {
   TRACE_EVENT0("tablet", "DiskRowSetWriter::Finish");
-  BlockManager* bm = rowset_metadata_->fs_manager()->block_manager();
+  auto bm = rowset_metadata_->fs_manager()->block_manager();
   unique_ptr<BlockCreationTransaction> transaction = bm->NewCreationTransaction();
   RETURN_NOT_OK(FinishAndReleaseBlocks(transaction.get()));
   return transaction->CommitCreatedBlocks();
@@ -336,7 +336,7 @@ RollingDiskRowSetWriter::RollingDiskRowSetWriter(
       can_roll_(false),
       written_count_(0),
       written_size_(0) {
-  BlockManager* bm = tablet_metadata->fs_manager()->block_manager();
+  auto bm = tablet_metadata->fs_manager()->block_manager();
   block_transaction_ = bm->NewCreationTransaction();
   CHECK(schema.has_column_ids());
 }
@@ -430,7 +430,7 @@ Status RollingDiskRowSetWriter::AppendDeltas(rowid_t row_idx_in_block,
   for (const Mutation *mut = delta_head; mut != nullptr; mut = mut->next()) {
     DeltaKey undo_key(*row_idx, mut->timestamp());
     RETURN_NOT_OK(writer->AppendDelta<Type>(undo_key, mut->changelist()));
-    delta_stats->UpdateStats(mut->timestamp(), mut->changelist());
+    RETURN_NOT_OK(delta_stats->UpdateStats(mut->timestamp(), mut->changelist()));
   }
   return Status::OK();
 }
@@ -575,7 +575,7 @@ Status DiskRowSet::MajorCompactDeltaStoresWithColumnIds(const vector<ColumnId>&
                                                         HistoryGcOpts history_gc_opts) {
   VLOG_WITH_PREFIX(1) << "Major compacting REDO delta stores (cols: " << col_ids << ")";
   TRACE_EVENT0("tablet", "DiskRowSet::MajorCompactDeltaStoresWithColumnIds");
-  std::lock_guard<Mutex> l(*mutable_delta_tracker()->compact_flush_lock());
+  std::lock_guard l(*mutable_delta_tracker()->compact_flush_lock());
   RETURN_NOT_OK(mutable_delta_tracker()->CheckWritableUnlocked());
 
   // Keep a reference to the tablet's schema. This is to prevent race condition
@@ -619,7 +619,7 @@ Status DiskRowSet::MajorCompactDeltaStoresWithColumnIds(const vector<ColumnId>&
                                &new_base));
   {
     // Update the delta tracker and the base data with the changes.
-    std::lock_guard<rw_spinlock> lock(component_lock_);
+    std::lock_guard lock(component_lock_);
     RETURN_NOT_OK(compaction->UpdateDeltaTracker(delta_tracker_.get(), io_context));
     base_data_.swap(new_base);
   }
@@ -675,8 +675,8 @@ Status DiskRowSet::NewRowIterator(const RowIteratorOptions& opts,
 Status DiskRowSet::NewCompactionInput(const Schema* projection,
                                       const MvccSnapshot &snap,
                                       const IOContext* io_context,
-                                      unique_ptr<CompactionInput>* out) const {
-  return CompactionInput::Create(*this, projection, snap, io_context, out);
+                                      shared_ptr<CompactionOrFlushInput>* out) const {
+  return CompactionOrFlushInput::Create(*this, projection, snap, io_context, out);
 }
 
 Status DiskRowSet::MutateRow(Timestamp timestamp,
@@ -862,9 +862,8 @@ double DiskRowSet::DeltaStoresCompactionPerfImprovementScore(DeltaCompactionType
   }
 
   if (type == RowSet::MAJOR_DELTA_COMPACTION) {
-    vector<ColumnId> col_ids_for_compact;
-    delta_tracker_->GetColumnIdsToCompact(&col_ids_for_compact);
-    if (!col_ids_for_compact.empty()) {
+    // No need to get all the delta files which need to be compacted.
+    if (delta_tracker_->DeltaStoreNeedToBeCompacted()) {
       DiskRowSetSpace drss;
       GetDiskRowSetSpaceUsage(&drss);
       double ratio = static_cast<double>(drss.redo_deltas_size) / drss.base_data_size;
@@ -930,9 +929,9 @@ Status DiskRowSet::DeleteAncientUndoDeltas(Timestamp ancient_history_mark,
 }
 
 Status DiskRowSet::DebugDumpImpl(int64_t* rows_left, vector<string>* lines) {
-  // Using CompactionInput to dump our data is an easy way of seeing all the
+  // Using CompactionOrFlushInput to dump our data is an easy way of seeing all the
   // rows and deltas.
-  unique_ptr<CompactionInput> input;
+  shared_ptr<CompactionOrFlushInput> input;
   RETURN_NOT_OK(NewCompactionInput(rowset_metadata_->tablet_schema().get(),
                                    MvccSnapshot::CreateSnapshotIncludingAllOps(),
                                    nullptr, &input));
