@@ -79,6 +79,10 @@ namespace log {
 class LogAnchorRegistry;
 }
 
+namespace tserver {
+class TabletServerTest_SetEncodedKeysWhenStartingUp_Test;
+} // namespace tserver
+
 namespace tools {
 Status DumpRowSetInternal(const fs::IOContext& ctx,
                           const std::shared_ptr<tablet::RowSetMetadata>& rs_meta,
@@ -89,7 +93,7 @@ Status DumpRowSetInternal(const fs::IOContext& ctx,
 namespace tablet {
 
 class CFileSet;
-class CompactionInput;
+class CompactionOrFlushInput;
 class DeltaFileWriter;
 class DeltaStats;
 class HistoryGcOpts;
@@ -364,19 +368,20 @@ class DiskRowSet :
                    ProbeStats* stats,
                    OperationResultPB* result) override;
 
-  Status CheckRowPresent(const RowSetKeyProbe &probe, const fs::IOContext* io_context,
+  Status CheckRowPresent(const RowSetKeyProbe &probe,
+                         const fs::IOContext* io_context,
                          bool *present, ProbeStats* stats) const override;
 
   ////////////////////
   // Read functions.
   ////////////////////
-  virtual Status NewRowIterator(const RowIteratorOptions& opts,
-                                std::unique_ptr<RowwiseIterator>* out) const override;
+  Status NewRowIterator(const RowIteratorOptions& opts,
+                        std::unique_ptr<RowwiseIterator>* out) const override;
 
-  virtual Status NewCompactionInput(const Schema* projection,
-                                    const MvccSnapshot &snap,
-                                    const fs::IOContext* io_context,
-                                    std::unique_ptr<CompactionInput>* out) const override;
+  Status NewCompactionInput(const Schema* projection,
+                            const MvccSnapshot &snap,
+                            const fs::IOContext* io_context,
+                            std::shared_ptr<CompactionOrFlushInput>* out) const override;
 
   // Gets the number of rows in this rowset, checking 'num_rows_' first. If not
   // yet set, consults the base data and stores the result in 'num_rows_'.
@@ -482,8 +487,9 @@ class DiskRowSet :
   FRIEND_TEST(TestCompaction, TestOneToOne);
   FRIEND_TEST(TestRowSet, TestRowSetUpdate);
   FRIEND_TEST(TestRowSet, TestDMSFlush);
+  FRIEND_TEST(tserver::TabletServerTest, SetEncodedKeysWhenStartingUp);
 
-  friend class CompactionInput;
+  friend class CompactionOrFlushInput;
   friend class Tablet;
   friend Status kudu::tools::DumpRowSetInternal(
       const fs::IOContext& ctx,
