@@ -217,7 +217,7 @@ Status MemRowSet::Insert(Timestamp timestamp,
         << "succeeded!";
   }
 
-  anchorer_.AnchorIfMinimum(op_id.index());
+  RETURN_NOT_OK(anchorer_.AnchorIfMinimum(op_id.index()));
 
   debug_insert_count_++;
   live_row_count_.Increment();
@@ -286,7 +286,7 @@ Status MemRowSet::MutateRow(Timestamp timestamp,
 
   stats->mrs_consulted++;
 
-  anchorer_.AnchorIfMinimum(op_id.index());
+  RETURN_NOT_OK(anchorer_.AnchorIfMinimum(op_id.index()));
   debug_update_count_++;
   if (delta.is_delete()) {
     live_row_count_.IncrementBy(-1);
@@ -343,8 +343,8 @@ Status MemRowSet::NewRowIterator(const RowIteratorOptions& opts,
 Status MemRowSet::NewCompactionInput(const Schema* projection,
                                      const MvccSnapshot& snap,
                                      const IOContext* /*io_context*/,
-                                     unique_ptr<CompactionInput>* out) const  {
-  out->reset(CompactionInput::Create(*this, projection, snap));
+                                     shared_ptr<CompactionOrFlushInput>* out) const  {
+  *out = CompactionOrFlushInput::Create(*this, projection, snap);
   return Status::OK();
 }
 
@@ -357,7 +357,7 @@ Status MemRowSet::GetBounds(string *min_encoded_key,
 class MemRowSet::Iterator::MRSRowProjector {
  public:
   typedef RowProjector::ProjectionIdxMapping ProjectionIdxMapping;
-  virtual ~MRSRowProjector() {}
+  virtual ~MRSRowProjector() = default;
   virtual Status ProjectRowForRead(const MRSRow& src_row,
                                    RowBlockRow* dst_row,
                                    Arena* arena) = 0;
