@@ -40,6 +40,7 @@
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/countdown_latch.h"
 #include "kudu/util/memory/arena.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/status.h"
 
 namespace google {
@@ -94,6 +95,7 @@ class Op {
   };
 
   Op(consensus::DriverType type, OpType op_type);
+  virtual ~Op() = default;
 
   // Returns the OpState for this op.
   virtual OpState* state() = 0;
@@ -143,8 +145,6 @@ class Op {
   // Each implementation should have its own ToString() method.
   virtual std::string ToString() const = 0;
 
-  virtual ~Op() {}
-
  private:
   const consensus::DriverType type_;
   const OpType op_type_;
@@ -268,6 +268,11 @@ class OpState {
     return request_id_;
   }
 
+  // Get the startup time of this op.
+  MonoTime start_time() const { return start_time_; }
+  // Set the startup time of this op.
+  void set_start_time(MonoTime start_time) { start_time_ = start_time; }
+
  protected:
   explicit OpState(TabletReplica* tablet_replica);
   virtual ~OpState();
@@ -307,6 +312,10 @@ class OpState {
 
   // The defined consistency mode for this op.
   ExternalConsistencyMode external_consistency_mode_;
+
+  // Use to record the op's start time.
+  // 'set_start_time()' needs to be called at the beginning of the op to initialize it.
+  std::atomic<MonoTime> start_time_;
 };
 
 // A parent class for the callback that gets called when ops
@@ -321,6 +330,7 @@ class OpState {
 class OpCompletionCallback {
  public:
   OpCompletionCallback();
+  virtual ~OpCompletionCallback();
 
   // Allows to set an error for this op and a mapping to a server level code.
   // Calling this method does not mean the op is completed.
@@ -337,8 +347,6 @@ class OpCompletionCallback {
   // Subclasses should override this.
   virtual void OpCompleted();
 
-  virtual ~OpCompletionCallback();
-
  protected:
   Status status_;
   tserver::TabletServerErrorPB::Code code_;
