@@ -81,7 +81,7 @@ class FollowerOpCompletionCallback : public OpCompletionCallback {
         response_(response),
         result_tracker_(std::move(result_tracker)) {}
 
-  virtual void OpCompleted() {
+  void OpCompleted() override {
     if (status_.ok()) {
       result_tracker_->RecordCompletionAndRespond(request_id_, response_);
     } else {
@@ -94,7 +94,7 @@ class FollowerOpCompletionCallback : public OpCompletionCallback {
     }
   }
 
-  virtual ~FollowerOpCompletionCallback() {}
+ ~FollowerOpCompletionCallback() override = default;
 
  private:
   const RequestIdPB& request_id_;
@@ -145,7 +145,7 @@ Status OpDriver::Init(unique_ptr<Op> op,
 
   auto self = shared_from_this();
   if (type == consensus::FOLLOWER) {
-    std::lock_guard<simple_spinlock> lock(opid_lock_);
+    std::lock_guard lock(opid_lock_);
     op_id_copy_ = op_->state()->op_id();
     DCHECK(op_id_copy_.IsInitialized());
     replication_state_ = REPLICATING;
@@ -153,7 +153,7 @@ Status OpDriver::Init(unique_ptr<Op> op,
     // Start the replica op in the thread that is updating consensus, for
     // non-leader ops.
     // Replica ops were already assigned a timestamp so we don't need to
-    // acquire locks before calling Start(). Starting the the op here gives a
+    // acquire locks before calling Start(). Starting the op here gives a
     // strong guarantee to consensus that the op is on mvcc when it moves
     // "safe" time so that we don't risk marking a timestamp "safe" before all
     // ops before it are in-flight are on mvcc.
@@ -197,7 +197,7 @@ Status OpDriver::Init(unique_ptr<Op> op,
 }
 
 consensus::OpId OpDriver::GetOpId() {
-  std::lock_guard<simple_spinlock> lock(opid_lock_);
+  std::lock_guard lock(opid_lock_);
   return op_id_copy_;
 }
 
@@ -214,7 +214,7 @@ Op::OpType OpDriver::op_type() const {
 }
 
 string OpDriver::ToString() const {
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   return ToStringUnlocked();
 }
 
@@ -312,7 +312,7 @@ Status OpDriver::Prepare() {
   // phase.
   ReplicationState repl_state_copy;
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     CHECK_EQ(prepare_state_, NOT_PREPARED);
     prepare_state_ = PREPARED;
     repl_state_copy = replication_state_;
@@ -348,14 +348,14 @@ Status OpDriver::Prepare() {
       TRACE("REPLICATION: starting");
       // Trigger consensus replication.
       {
-        std::lock_guard<simple_spinlock> lock(lock_);
+        std::lock_guard lock(lock_);
         replication_state_ = REPLICATING;
         replication_start_time_ = MonoTime::Now();
       }
 
       Status s = consensus_->Replicate(mutable_state()->consensus_round());
       if (PREDICT_FALSE(!s.ok())) {
-        std::lock_guard<simple_spinlock> lock(lock_);
+        std::lock_guard lock(lock_);
         CHECK_EQ(replication_state_, REPLICATING);
         op_status_ = s;
         replication_state_ = REPLICATION_FAILED;
@@ -391,7 +391,7 @@ void OpDriver::HandleFailure(const Status& s) {
   ReplicationState repl_state_copy;
 
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     op_status_ = s;
     repl_state_copy = replication_state_;
   }
@@ -426,7 +426,7 @@ void OpDriver::ReplicationFinished(const Status& status) {
 
   ADOPT_TRACE(trace());
   {
-    std::lock_guard<simple_spinlock> op_id_lock(opid_lock_);
+    std::lock_guard op_id_lock(opid_lock_);
     // TODO: it's a bit silly that we have three copies of the opid:
     // one here, one in ConsensusRound, and one in OpState.
 
@@ -445,7 +445,7 @@ void OpDriver::ReplicationFinished(const Status& status) {
   MonoDelta replication_duration;
   PrepareState prepare_state_copy;
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     CHECK_EQ(replication_state_, REPLICATING);
     if (status.ok()) {
       replication_state_ = REPLICATED;
@@ -457,6 +457,9 @@ void OpDriver::ReplicationFinished(const Status& status) {
     replication_duration = replication_finished_time - replication_start_time_;
   }
 
+  if (auto* metrics = op_->state()->tablet_replica()->tablet()->metrics()) {
+    metrics->replication_duration->Increment(replication_duration.ToMicroseconds());
+  }
   TRACE_COUNTER_INCREMENT("replication_time_us", replication_duration.ToMicroseconds());
   TRACE("REPLICATION: finished");
 
@@ -477,7 +480,7 @@ void OpDriver::Abort(const Status& status) {
 
   ReplicationState repl_state_copy;
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     repl_state_copy = replication_state_;
     op_status_ = status;
   }
@@ -532,7 +535,7 @@ void OpDriver::ApplyTask() {
 
 #if DCHECK_IS_ON()
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     DCHECK_EQ(replication_state_, REPLICATED);
     DCHECK_EQ(prepare_state_, PREPARED);
   }
@@ -597,7 +600,7 @@ Status OpDriver::CommitWait() {
 
 void OpDriver::Finalize() {
   ADOPT_TRACE(trace());
-  std::lock_guard<simple_spinlock> lock(lock_);
+  std::lock_guard lock(lock_);
   op_->Finish(Op::APPLIED);
   mutable_state()->completion_callback()->OpCompleted();
   op_tracker_->Release(this);
@@ -642,7 +645,7 @@ std::string OpDriver::LogPrefix() const {
   string ts_string;
 
   {
-    std::lock_guard<simple_spinlock> lock(lock_);
+    std::lock_guard lock(lock_);
     repl_state_copy = replication_state_;
     prep_state_copy = prepare_state_;
     ts_string = state()->has_timestamp() ? state()->timestamp().ToString() : "No timestamp";
