@@ -20,8 +20,8 @@
 #include <cstddef>
 #include <cstdint>
 #include <memory>
-#include <mutex>
 #include <optional>
+#include <shared_mutex>
 #include <string>
 #include <type_traits>
 #include <vector>
@@ -40,7 +40,6 @@
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/bitset.h"
 #include "kudu/util/locks.h"
-#include "kudu/util/monotime.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
@@ -195,13 +194,13 @@ class WriteOpState : public OpState {
   void ReleaseMvccTxn(Op::OpResult result);
 
   void set_schema_at_decode_time(const SchemaPtr& schema) {
-    std::lock_guard<simple_spinlock> l(op_state_lock_);
+    std::lock_guard l(op_state_lock_);
     schema_ptr_at_decode_time_ = schema;
     schema_at_decode_time_ = schema.get();
   }
 
   const Schema* schema_at_decode_time() const {
-    std::lock_guard<simple_spinlock> l(op_state_lock_);
+    std::lock_guard l(op_state_lock_);
     return schema_at_decode_time_;
   }
 
@@ -320,7 +319,7 @@ class WriteOpState : public OpState {
 
   // A lock held on the tablet's schema. Prevents concurrent schema change
   // from racing with a write.
-  shared_lock<rw_semaphore> schema_lock_;
+  std::shared_lock<rw_semaphore> schema_lock_;
 
   // The Schema of the tablet when the op was first decoded. This is verified
   // at APPLY time to ensure we don't have races against schema change.
@@ -337,7 +336,7 @@ class WriteOpState : public OpState {
 
   // Lock protecting the transaction's state, ensuring it remains open for the
   // duration of this write.
-  shared_lock<rw_semaphore> txn_lock_;
+  std::shared_lock<rw_semaphore> txn_lock_;
 
   DISALLOW_COPY_AND_ASSIGN(WriteOpState);
 };
@@ -399,9 +398,6 @@ class WriteOp : public Op {
   // successfully written rows, the latter is for failed ones.
   void UpdatePerRowMetricsAndErrors();
 
-  // this op's start time
-  MonoTime start_time_;
-
   std::unique_ptr<WriteOpState> state_;
 
  private:
