@@ -46,9 +46,9 @@
 #include "kudu/common/timestamp.h"
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/block_manager.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/stl_util.h"
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/tablet/delta_key.h"
 #include "kudu/tablet/delta_stats.h"
@@ -241,8 +241,9 @@ TYPED_TEST(TestTablet, TestGhostRowsOnDiskRowSets) {
   LocalTabletWriter writer(this->tablet().get(), &this->client_schema_);
 
   for (int i = 0; i < 3; i++) {
-    CHECK_OK(this->InsertTestRow(&writer, 0, 0));
-    this->DeleteTestRow(&writer, 0);
+    SCOPED_TRACE(strings::Substitute("iteration $0", i));
+    ASSERT_OK(this->InsertTestRow(&writer, 0, 0));
+    ASSERT_OK(this->DeleteTestRow(&writer, 0));
     ASSERT_OK(this->tablet()->Flush());
   }
 
@@ -453,18 +454,17 @@ TYPED_TEST(TestTablet, TestReinsertDuringFlush) {
    public:
     explicit MyCommonHooks(TestFixture *test) : test_(test) {}
 
-    Status PostWriteSnapshot() OVERRIDE {
+    Status PostWriteSnapshot() override {
       LocalTabletWriter writer(test_->tablet().get(), &test_->client_schema());
-      test_->InsertTestRow(&writer, 0, 1);
-      CHECK_OK(test_->DeleteTestRow(&writer, 0));
+      RETURN_NOT_OK(test_->InsertTestRow(&writer, 0, 1));
+      RETURN_NOT_OK(test_->DeleteTestRow(&writer, 0));
       CHECK_EQ(1, writer.last_op_result().mutated_stores_size());
       CHECK_EQ(1L, writer.last_op_result().mutated_stores(0).mrs_id());
-      test_->InsertTestRow(&writer, 0, 2);
-      CHECK_OK(test_->DeleteTestRow(&writer, 0));
+      RETURN_NOT_OK(test_->InsertTestRow(&writer, 0, 2));
+      RETURN_NOT_OK(test_->DeleteTestRow(&writer, 0));
       CHECK_EQ(1, writer.last_op_result().mutated_stores_size());
       CHECK_EQ(1L, writer.last_op_result().mutated_stores(0).mrs_id());
-      test_->InsertTestRow(&writer, 0, 3);
-      return Status::OK();
+      return test_->InsertTestRow(&writer, 0, 3);
     }
 
    private:
@@ -672,7 +672,7 @@ TYPED_TEST(TestTablet, TestRowIteratorComplex) {
 
     bool set_to_null = TestSetupExpectsNulls<TypeParam>(i);
     if (set_to_null) {
-      this->UpdateTestRowToNull(&writer, i);
+      ASSERT_OK(this->UpdateTestRowToNull(&writer, i));
     } else {
       ASSERT_OK_FAST(this->UpdateTestRow(&writer, i, i));
     }
@@ -1163,29 +1163,27 @@ class MyCommonHooks : public Tablet::FlushCompactCommonHooks {
     return Status::OK();
   }
 
-  virtual Status PostTakeMvccSnapshot() OVERRIDE {
+  Status PostTakeMvccSnapshot() override {
     // before we flush we update the MemRowSet afterwards we update the
     // DeltaMemStore
     if (!flushed_) {
       return DoHook(MRS_MUTATION);
-    } else {
-      return DoHook(DELTA_MUTATION);
     }
+    return DoHook(DELTA_MUTATION);
   }
-  virtual Status PostWriteSnapshot() OVERRIDE {
+  Status PostWriteSnapshot() override {
     if (!flushed_) {
       return DoHook(MRS_MUTATION);
-    } else {
-      return DoHook(DELTA_MUTATION);
     }
+    return DoHook(DELTA_MUTATION);
   }
-  virtual Status PostSwapInDuplicatingRowSet() OVERRIDE {
+  Status PostSwapInDuplicatingRowSet() override {
     return DoHook(DUPLICATED_MUTATION);
   }
-  virtual Status PostReupdateMissedDeltas() OVERRIDE {
+  Status PostReupdateMissedDeltas() override {
     return DoHook(DUPLICATED_MUTATION);
   }
-  virtual Status PostSwapNewRowSet() OVERRIDE {
+  Status PostSwapNewRowSet() override {
     return DoHook(DELTA_MUTATION);
   }
  protected:
@@ -1349,7 +1347,7 @@ TYPED_TEST(TestTablet, TestMetricsInit) {
     ASSERT_OK(registry->WriteAsJson(&writer, MetricJsonOptions()));
   }
   // Open tablet, should still work
-  this->harness()->Open();
+  ASSERT_OK(this->harness()->Open());
   {
     std::ostringstream out;
     JsonWriter writer(&out, JsonWriter::PRETTY);
