@@ -89,7 +89,7 @@ METRIC_DEFINE_counter(tablet, scanner_rows_returned, "Scanner Rows Returned",
                       "Number of rows returned by scanners to clients. This count "
                       "is measured after predicates are applied, and thus is not "
                       "a reflection of the amount of work being done by scanners.",
-                      kudu::MetricLevel::kDebug);
+                      kudu::MetricLevel::kInfo);
 METRIC_DEFINE_counter(tablet, scanner_cells_returned, "Scanner Cells Returned",
                       kudu::MetricUnit::kCells,
                       "Number of table cells returned by scanners to clients. This count "
@@ -109,7 +109,7 @@ METRIC_DEFINE_counter(tablet, scanner_predicates_disabled, "Scanner Column Predi
                       "This count measures the number of disableable column predicates like "
                       "Bloom filter predicate that are automatically disabled if determined to "
                       "be ineffective.",
-                      kudu::MetricLevel::kInfo);
+                      kudu::MetricLevel::kDebug);
 
 METRIC_DEFINE_counter(tablet, scanner_rows_scanned, "Scanner Rows Scanned",
                       kudu::MetricUnit::kRows,
@@ -118,7 +118,7 @@ METRIC_DEFINE_counter(tablet, scanner_rows_scanned, "Scanner Rows Scanned",
                       "or MVCC-based filtering. Thus, this is a better measure of actual "
                       "table rows that have been processed by scan operations compared "
                       "to the Scanner Rows Returned metric.",
-                      kudu::MetricLevel::kDebug);
+                      kudu::MetricLevel::kInfo);
 
 METRIC_DEFINE_counter(tablet, scanner_cells_scanned_from_disk, "Scanner Cells Scanned From Disk",
                       kudu::MetricUnit::kCells,
@@ -126,9 +126,9 @@ METRIC_DEFINE_counter(tablet, scanner_cells_scanned_from_disk, "Scanner Cells Sc
                       "as a raw count prior to application of predicates, deleted data,"
                       "or MVCC-based filtering. Thus, this is a better measure of actual "
                       "table cells that have been processed by scan operations compared "
-                      "to the Scanner Cells Returned metric.\n"
+                      "to the Scanner Cells Returned metric. "
                       "Note that this only counts data that has been flushed to disk, "
-                      "and does not include data read from in-memory stores. However, it"
+                      "and does not include data read from in-memory stores. However, it "
                       "includes both cache misses and cache hits.",
                       kudu::MetricLevel::kDebug);
 
@@ -138,20 +138,41 @@ METRIC_DEFINE_counter(tablet, scanner_bytes_scanned_from_disk, "Scanner Bytes Sc
                       "as a raw count prior to application of predicates, deleted data,"
                       "or MVCC-based filtering. Thus, this is a better measure of actual "
                       "IO that has been caused by scan operations compared "
-                      "to the Scanner Bytes Returned metric.\n"
+                      "to the Scanner Bytes Returned metric. "
                       "Note that this only counts data that has been flushed to disk, "
-                      "and does not include data read from in-memory stores. However, it"
+                      "and does not include data read from in-memory stores. However, it "
                       "includes both cache misses and cache hits.",
                       kudu::MetricLevel::kDebug);
 
 METRIC_DEFINE_counter(tablet, scans_started, "Scans Started",
                       kudu::MetricUnit::kScanners,
                       "Number of scanners which have been started on this tablet",
-                      kudu::MetricLevel::kDebug);
+                      kudu::MetricLevel::kInfo);
 METRIC_DEFINE_gauge_size(tablet, tablet_active_scanners, "Active Scanners",
                          kudu::MetricUnit::kScanners,
                          "Number of scanners that are currently active on this tablet",
-                         kudu::MetricLevel::kDebug);
+                         kudu::MetricLevel::kInfo);
+
+METRIC_DEFINE_histogram(tablet, scan_duration_wall_time,
+                        "Scan Requests Wall Time",
+                        kudu::MetricUnit::kMilliseconds,
+                        "Duration of scan requests, wall time.",
+                        kudu::MetricLevel::kDebug,
+                        60000LU, 1);
+
+METRIC_DEFINE_histogram(tablet, scan_duration_system_time,
+                        "Scan Requests System Time",
+                        kudu::MetricUnit::kMilliseconds,
+                        "Duration of scan requests, system time.",
+                        kudu::MetricLevel::kDebug,
+                        60000LU, 1);
+
+METRIC_DEFINE_histogram(tablet, scan_duration_user_time,
+                        "Scan Requests User Time",
+                        kudu::MetricUnit::kMilliseconds,
+                        "Duration of scan requests, user time.",
+                        kudu::MetricLevel::kDebug,
+                        60000LU, 1);
 
 METRIC_DEFINE_counter(tablet, bloom_lookups, "Bloom Filter Lookups",
                       kudu::MetricUnit::kProbes,
@@ -195,7 +216,7 @@ METRIC_DEFINE_counter(tablet, ops_timed_out_in_prepare_queue,
                       "corresponding operations were waiting in the tablet's "
                       "prepare queue, and thus were not started but "
                       "acknowledged with TimedOut error status.",
-                      kudu::MetricLevel::kInfo);
+                      kudu::MetricLevel::kWarn);
 
 METRIC_DEFINE_histogram(tablet, bloom_lookups_per_op, "Bloom Lookups per Operation",
                         kudu::MetricUnit::kProbes,
@@ -221,9 +242,23 @@ METRIC_DEFINE_histogram(tablet, delta_file_lookups_per_op, "Delta File Lookups p
                         "operation. A single operation may perform several delta file "
                         "lookups if the tablet is not fully compacted. High frequency of "
                         "high values may indicate that compaction is falling behind.",
-                        kudu::MetricLevel::kInfo,
+                        kudu::MetricLevel::kDebug,
                         20, 2);
 
+METRIC_DEFINE_histogram(tablet, alter_schema_duration,
+                        "Alter Schema Op Duration",
+                        kudu::MetricUnit::kMicroseconds,
+                        "Duration of alter schema ops to this tablet.",
+                        kudu::MetricLevel::kDebug,
+                        60000000LU, 2);
+
+METRIC_DEFINE_histogram(tablet, replication_duration,
+                        "Replica Replication Duration",
+                        kudu::MetricUnit::kMicroseconds,
+                        "Duration of replication between replicas on the leader.",
+                        kudu::MetricLevel::kDebug,
+                        60000000LU, 2);
+
 METRIC_DEFINE_histogram(tablet, write_op_duration_client_propagated_consistency,
   "Write Op Duration with Propagated Consistency",
   kudu::MetricUnit::kMicroseconds,
@@ -311,63 +346,63 @@ METRIC_DEFINE_histogram(tablet, flush_dms_duration,
   "DeltaMemStore Flush Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent flushing DeltaMemStores.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, flush_mrs_duration,
   "MemRowSet Flush Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent flushing MemRowSets.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, compact_rs_duration,
   "RowSet Compaction Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent compacting RowSets.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, delta_minor_compact_rs_duration,
   "Minor Delta Compaction Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent minor delta compacting.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, delta_major_compact_rs_duration,
   "Major Delta Compaction Duration",
   kudu::MetricUnit::kSeconds,
   "Seconds spent major delta compacting.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000000LU, 2);
 
 METRIC_DEFINE_histogram(tablet, undo_delta_block_gc_init_duration,
   "Undo Delta Block GC Init Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent initializing ancient UNDO delta blocks.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, undo_delta_block_gc_delete_duration,
   "Undo Delta Block GC Delete Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent deleting ancient UNDO delta blocks.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, undo_delta_block_gc_perform_duration,
   "Undo Delta Block GC Perform Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent running the maintenance operation to GC ancient UNDO delta blocks.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, compact_rs_mem_usage,
   "Peak Memory Usage for CompactRowSetsOp",
   kudu::MetricUnit::kBytes,
   "Peak memory usage of rowset merge compaction operations (CompactRowSetsOp)",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, compact_rs_mem_usage_to_deltas_size_ratio,
@@ -375,14 +410,14 @@ METRIC_DEFINE_histogram(tablet, compact_rs_mem_usage_to_deltas_size_ratio,
   kudu::MetricUnit::kUnits,
   "Ratio of the peak memory usage to the estimated on-disk size of all deltas "
   "for rowsets involved in rowset merge compaction (CompactRowSetsOp)",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_histogram(tablet, deleted_rowset_gc_duration,
   "Deleted Rowset GC Duration",
   kudu::MetricUnit::kMilliseconds,
   "Time spent running the maintenance operation to GC deleted rowsets.",
-  kudu::MetricLevel::kInfo,
+  kudu::MetricLevel::kDebug,
   60000LU, 1);
 
 METRIC_DEFINE_counter(tablet, leader_memory_pressure_rejections,
@@ -426,6 +461,9 @@ TabletMetrics::TabletMetrics(const scoped_refptr<MetricEntity>& entity)
     MINIT(scanner_predicates_disabled),
     MINIT(scans_started),
     GINIT(tablet_active_scanners),
+    MINIT(scan_duration_wall_time),
+    MINIT(scan_duration_system_time),
+    MINIT(scan_duration_user_time),
     MINIT(bloom_lookups),
     MINIT(key_file_lookups),
     MINIT(delta_file_lookups),
@@ -441,6 +479,8 @@ TabletMetrics::TabletMetrics(const scoped_refptr<MetricEntity>& entity)
     MINIT(snapshot_read_inflight_wait_duration),
     MINIT(write_op_duration_client_propagated_consistency),
     MINIT(write_op_duration_commit_wait_consistency),
+    MINIT(alter_schema_duration),
+    MINIT(replication_duration),
     GINIT(flush_dms_running),
     GINIT(flush_mrs_running),
     GINIT(compact_rs_running),
