@@ -65,6 +65,9 @@ struct TabletMetrics {
   scoped_refptr<Counter> scanner_predicates_disabled;
   scoped_refptr<Counter> scans_started;
   scoped_refptr<AtomicGauge<size_t>> tablet_active_scanners;
+  scoped_refptr<Histogram> scan_duration_wall_time;
+  scoped_refptr<Histogram> scan_duration_system_time;
+  scoped_refptr<Histogram> scan_duration_user_time;
 
   // Probe stats.
   scoped_refptr<Counter> bloom_lookups;
@@ -86,6 +89,8 @@ struct TabletMetrics {
   scoped_refptr<Histogram> snapshot_read_inflight_wait_duration;
   scoped_refptr<Histogram> write_op_duration_client_propagated_consistency;
   scoped_refptr<Histogram> write_op_duration_commit_wait_consistency;
+  scoped_refptr<Histogram> alter_schema_duration;
+  scoped_refptr<Histogram> replication_duration;
 
   scoped_refptr<AtomicGauge<uint32_t> > flush_dms_running;
   scoped_refptr<AtomicGauge<uint32_t> > flush_mrs_running;
