@@ -56,6 +56,7 @@
 #include "kudu/tablet/ops/write_op.h"
 #include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet_metadata.h"
+#include "kudu/tablet/tablet_metrics.h"
 #include "kudu/tablet/tablet_replica-test-base.h"
 #include "kudu/tablet/tablet_replica_mm_ops.h"
 #include "kudu/tserver/tserver.pb.h"
@@ -79,6 +80,7 @@ DECLARE_int32(tablet_history_max_age_sec);
 METRIC_DECLARE_entity(tablet);
 
 METRIC_DECLARE_gauge_uint64(live_row_count);
+METRIC_DECLARE_histogram(alter_schema_duration);
 
 using kudu::consensus::CommitMsg;
 using kudu::consensus::ConsensusBootstrapInfo;
@@ -245,7 +247,7 @@ class DelayedApplyOp : public WriteOp {
         apply_continue_(DCHECK_NOTNULL(apply_continue)) {
   }
 
-  virtual Status Apply(CommitMsg** commit_msg) override {
+  Status Apply(CommitMsg** commit_msg) override {
     apply_started_->CountDown();
     LOG(INFO) << "Delaying apply...";
     apply_continue_->Wait();
@@ -259,6 +261,26 @@ class DelayedApplyOp : public WriteOp {
   DISALLOW_COPY_AND_ASSIGN(DelayedApplyOp);
 };
 
+TEST_F(TabletReplicaTest, TestAlterSchemaMetric) {
+  ConsensusBootstrapInfo info;
+  ASSERT_OK(StartReplicaAndWaitUntilLeader(info));
+  const int orig_schema_version = tablet()->metadata()->schema_version();
+
+  // Get the metric.
+  auto alter_schema_duration =
+    tablet_replica_->tablet()->metrics()->alter_schema_duration;
+  const auto before_cnt = alter_schema_duration->TotalCount();
+
+  // Add a new column.
+  SchemaBuilder builder(*tablet()->metadata()->schema());
+  ASSERT_OK(builder.AddColumn("new_col", INT32));
+  SchemaPB new_schema;
+  ASSERT_OK(SchemaToPB(builder.Build(), &new_schema));
+  ASSERT_OK(UpdateSchema(new_schema, orig_schema_version + 1));
+
+  ASSERT_EQ(before_cnt + 1, alter_schema_duration->TotalCount());
+}
+
 // Ensure that Log::GC() doesn't delete logs when the MRS has an anchor.
 TEST_F(TabletReplicaTest, TestMRSAnchorPreventsLogGC) {
   ConsensusBootstrapInfo info;
@@ -286,7 +308,7 @@ TEST_F(TabletReplicaTest, TestMRSAnchorPreventsLogGC) {
   ASSERT_EQ(0, num_gced) << "earliest needed: " << retention.for_durability;
 
   // Flush MRS as needed to ensure that we don't have OpId anchors in the MRS.
-  tablet_replica_->tablet()->Flush();
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
 
   // The first two segments should be deleted.
@@ -399,7 +421,7 @@ TEST_F(TabletReplicaTest, TestActiveOpPreventsLogGC) {
 
   // Flush MRS as needed to ensure that we don't have OpId anchors in the MRS.
   ASSERT_EQ(1, tablet_replica_->log_anchor_registry()->GetAnchorCountForTests());
-  tablet_replica_->tablet()->Flush();
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
 
   // Verify no anchors after Flush().
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
@@ -497,7 +519,7 @@ TEST_F(TabletReplicaTest, TestGCEmptyLog) {
   ConsensusBootstrapInfo info;
   ASSERT_OK(StartReplica(info));
   // We don't wait on consensus on purpose.
-  ASSERT_OK(tablet_replica_->RunLogGC());
+  tablet_replica_->RunLogGC();
 }
 
 TEST_F(TabletReplicaTest, TestFlushOpsPerfImprovements) {
@@ -571,8 +593,8 @@ TEST_F(TabletReplicaTest, TestRollLogSegmentSchemaOnAlter) {
   NO_FATALS(write());
   ASSERT_OK(RollLog(tablet_replica_.get()));
   NO_FATALS(write());
-  tablet_replica_->tablet()->Flush();
-  ASSERT_OK(tablet_replica_->RunLogGC());
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
+  tablet_replica_->RunLogGC();
 
   // Now write some more and restart. If our segment header schema previously
   // didn't have "new_col", bootstrapping would fail, complaining about a
@@ -612,7 +634,7 @@ TEST_F(TabletReplicaTest, Kudu2690Test) {
     ASSERT_OK(GenerateSequentialInsertRequest(new_client_schema, req.get()));
     ASSERT_OK(ExecuteWrite(tablet_replica_.get(), *req));
   }
-  ASSERT_OK(tablet_replica_->RunLogGC());
+  tablet_replica_->RunLogGC();
 
   // Before KUDU-2960 was fixed, bootstrapping would fail, complaining that the
   // write requests contained a column that was not in the log segment header's
@@ -749,5 +771,23 @@ TEST_F(TabletReplicaTest, RowLocksLongWaitAndLogging) {
   t1.join();
 }
 
+// Test the replication duration metric works.
+TEST_F(TabletReplicaTest, TestReplicationDurationMetric) {
+  ConsensusBootstrapInfo info;
+  ASSERT_OK(StartReplicaAndWaitUntilLeader(info));
+
+  // The metric should be zero at the beginning.
+  ASSERT_EQ(0, tablet_replica_->tablet()->metrics()->replication_duration->TotalCount());
+
+  auto req = std::make_unique<WriteRequestPB>();
+  ASSERT_OK(GenerateSequentialInsertRequest(GetTestSchema(), req.get()));
+  ASSERT_OK(ExecuteWrite(tablet_replica_.get(), *req));
+
+  // The metric should be non-zero after the write completes.
+  ASSERT_EVENTUALLY([&]{
+    ASSERT_EQ(1, tablet_replica_->tablet()->metrics()->replication_duration->TotalCount());
+  });
+}
+
 } // namespace tablet
 } // namespace kudu
