@@ -21,6 +21,7 @@
 #include <memory>
 #include <mutex>
 #include <optional>
+#include <shared_mutex>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -107,7 +108,7 @@ class Txn : public RefCountedThreadSafe<Txn> {
   // driven via an op driver, lock acquisition is expected to be serialized in
   // a single thread.
   void AcquireWriteLock(std::unique_lock<rw_semaphore>* txn_lock);
-  void AcquireReadLock(shared_lock<rw_semaphore>* txn_lock);
+  void AcquireReadLock(std::shared_lock<rw_semaphore>* txn_lock);
 
   // Adopts the input partition lock, maintaining it until the transaction is
   // complete (aborted or finalized). Rather than maintaining multiple
@@ -234,14 +235,14 @@ class Txn : public RefCountedThreadSafe<Txn> {
         &begin_commit_anchor_));
     SetState(kCommitInProgress);
   }
-  void FinalizeCommit(int64_t finalized_commit_timestamp) {
+  Status FinalizeCommit(int64_t finalized_commit_timestamp) {
     commit_timestamp_ = finalized_commit_timestamp;
     SetState(kCommitted);
-    log_anchor_registry_->UnregisterIfAnchored(&begin_commit_anchor_);
+    return log_anchor_registry_->UnregisterIfAnchored(&begin_commit_anchor_);
   }
-  void AbortTransaction() {
+  Status AbortTransaction() {
     SetState(kAborted);
-    log_anchor_registry_->UnregisterIfAnchored(&begin_commit_anchor_);
+    return log_anchor_registry_->UnregisterIfAnchored(&begin_commit_anchor_);
   }
 
   // Simple accessors for state. No locks are required to call these.
@@ -265,7 +266,7 @@ class Txn : public RefCountedThreadSafe<Txn> {
   }
 
   void ReleasePartitionLock() {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     partition_lock_.Release();
   }
 
