@@ -36,7 +36,6 @@
 
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
-#include <google/protobuf/stubs/common.h>
 #include <gtest/gtest.h>
 #include <rapidjson/document.h>
 
@@ -49,7 +48,7 @@
 #include "kudu/server/server_base.pb.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/tablet.pb.h"
-#include "kudu/tools/ksck_checksum.h"
+#include "kudu/tools/ksck_checksum.h" // IWYU pragma: keep
 #include "kudu/tools/ksck_results.h"
 #include "kudu/transactions/txn_status_tablet.h"
 #include "kudu/util/jsonreader.h"
@@ -236,23 +235,23 @@ class MockKsckCluster : public KsckCluster {
       : fetch_info_status_(Status::OK()) {
   }
 
-  virtual Status Connect() override {
+  Status Connect() override {
     return fetch_info_status_;
   }
 
-  virtual Status RetrieveTabletServers() override {
+  Status RetrieveTabletServers() override {
     return Status::OK();
   }
 
-  virtual Status RetrieveTablesList() override {
+  Status RetrieveTablesList() override {
     return Status::OK();
   }
 
-  virtual Status RetrieveAllTablets() override {
+  Status RetrieveAllTablets() override {
     return Status::OK();
   }
 
-  virtual Status RetrieveTabletsList(const shared_ptr<KsckTable>& /* unused */) override {
+  Status RetrieveTabletsList(const shared_ptr<KsckTable>& /* unused */) override {
     return Status::OK();
   }
 
@@ -318,7 +317,7 @@ class KsckTest : public KuduTest {
     table_summary.consensus_mismatch_tablets = consensus_mismatch_tablets;
     table_summary.unavailable_tablets = unavailable_tablets;
     std::ostringstream oss;
-    PrintTableSummaries({ table_summary }, "table", oss);
+    CHECK_OK(PrintTableSummaries({ table_summary }, "table", oss));
     return oss.str();
   }
 
@@ -480,9 +479,8 @@ class KsckTest : public KuduTest {
 
   const string KsckResultsToJsonString(int sections = PrintSections::ALL_SECTIONS) {
     ostringstream json_stream;
-    ksck_->results().PrintJsonTo(PrintMode::JSON_COMPACT,
-                                 sections,
-                                 json_stream);
+    CHECK_OK(ksck_->results().PrintJsonTo(
+        PrintMode::JSON_COMPACT, sections, json_stream));
     return json_stream.str();
   }
 
