@@ -25,14 +25,12 @@
 #include <iostream>
 #include <iterator>
 #include <map>
-#include <mutex>
 #include <set>
 #include <type_traits>
 #include <vector>
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
-#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/partition.h"
 #include "kudu/consensus/quorum_util.h"
@@ -302,7 +300,7 @@ Status Ksck::CheckMasterHealth() {
       }
 
       {
-        std::lock_guard<simple_spinlock> lock(master_summaries_lock);
+        std::lock_guard lock(master_summaries_lock);
         master_summaries.emplace_back(std::move(sh));
       }
 
@@ -510,7 +508,7 @@ Status Ksck::FetchInfoFromTabletServers() {
       summary.health = health;
 
       {
-        std::lock_guard<simple_spinlock> lock(tablet_server_summaries_lock);
+        std::lock_guard lock(tablet_server_summaries_lock);
         tablet_server_summaries.push_back(std::move(summary));
       }
 
@@ -847,6 +845,8 @@ bool Ksck::VerifyTable(const shared_ptr<KsckTable>& table, vector<TableSummary>*
   ts.id = table->id();
   ts.name = table->name();
   ts.replication_factor = table->num_replicas();
+  const auto& tablet = table->tablets().front();
+  ts.is_range_partitioned = !tablet->partition().begin().range_key().empty();
   VLOG(1) << Substitute("Verifying $0 tablet(s) for table $1 configured with num_replicas = $2",
                         table->tablets().size(), table->name(), table->num_replicas());
   for (const auto& tablet : table->tablets()) {
