@@ -22,6 +22,7 @@
 #include <memory>
 #include <sstream>
 #include <string>
+#include <type_traits>
 #include <thread>
 #include <vector>
 
@@ -36,7 +37,6 @@
 #include "kudu/common/partial_row.h"
 #include "kudu/gutil/basictypes.h"
 #include "kudu/gutil/macros.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/data_gen_util.h"
@@ -109,7 +109,7 @@ class RemoteKsckTest : public KuduTest {
     CHECK_OK(b.Build(&schema_));
   }
 
-  virtual void SetUp() OVERRIDE {
+  void SetUp() override {
     KuduTest::SetUp();
 
     // Speed up testing, saves about 700ms per TEST_F.
@@ -150,7 +150,7 @@ class RemoteKsckTest : public KuduTest {
     ksck_.reset(new Ksck(cluster_, &err_stream_));
   }
 
-  virtual void TearDown() OVERRIDE {
+  void TearDown() override {
     if (mini_cluster_) {
       mini_cluster_->Shutdown();
       mini_cluster_.reset();
@@ -517,10 +517,9 @@ TEST_F(RemoteKsckTest, TestChecksumSnapshotLastingLongerThanAHM) {
   // happen during the wait to start the checksum, for each tablet. It's likely
   // this sometimes won't happen in builds that are slower, so we'll just
   // disable the test for those builds.
-  #if defined(THREAD_SANITIZER) || defined(ADDRESS_SANITIZER)
-    LOG(WARNING) << "test is skipped in TSAN and ASAN builds";
-    return;
-  #endif
+#if defined(THREAD_SANITIZER) || defined(ADDRESS_SANITIZER)
+  GTEST_SKIP() << "test is skipped in TSAN and ASAN builds";
+#endif
 
   // Write something so we have rows to checksum, and because we need a valid
   // timestamp from the client to use for a checksum scan.
@@ -655,7 +654,7 @@ TEST_F(RemoteKsckTest, TestClusterWithLocation) {
 
 // Test filtering on a cluster with no table.
 TEST_F(RemoteKsckTest, TestFilterOnNoTableCluster) {
-  client_->DeleteTable(kTableName);
+  ASSERT_OK(client_->DeleteTable(kTableName));
   cluster_->set_table_filters({ "ksck-test-table" });
   FLAGS_checksum_scan = true;
   FLAGS_consensus = false;
