@@ -43,6 +43,7 @@
 #include "kudu/client/value.h"
 #include "kudu/client/write_op.h"
 #include "kudu/common/column_predicate.h"
+#include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/partition.h"
 #include "kudu/common/row.h"
@@ -54,13 +55,17 @@
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/bitmap.h"
+#include "kudu/util/easy_json.h"
 #include "kudu/util/jsonreader.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/memory/arena.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/oid_generator.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/stopwatch.h"
 #include "kudu/util/string_case.h"
+#include "kudu/util/threadpool.h"
+#include "kudu/util/throttler.h"
 
 using kudu::client::KuduClient;
 using kudu::client::KuduColumnSchema;
@@ -139,6 +144,21 @@ DEFINE_string(replica_selection, "CLOSEST",
               "Replica selection for scan operations. Acceptable values are: "
               "CLOSEST, LEADER (maps into KuduClient::CLOSEST_REPLICA and "
               "KuduClient::LEADER_ONLY correspondingly).");
+DEFINE_bool(strict_column_id, false,
+            "Whether to compare column IDs when comparing schemas. It should be enabled when we "
+            "expect the destination table has the same server internal column IDs as the source "
+            "table. For example, before using 'kudu remote_replica copy' to copy data from "
+            "another table, we should enable this flag to make sure the table schemas are "
+            "completely the same.");
+DEFINE_int64(table_copy_throttler_bytes_per_sec, 0,
+             "Limit table copying speed. It limits the copying speed of all the tablets "
+             "in one table for one session. The default value is 0, which means not limiting "
+             "the speed. The unit is bytes/second");
+DEFINE_double(table_copy_throttler_burst_factor, 1.0F,
+             "Burst factor for table copy throttling. The maximum rate the throttler "
+             "allows within a token refill period (100ms) equals burst factor multiplied "
+             "base rate (--table_copy_throttler_bytes_per_sec). The default value is 1.0, "
+             "which means the maximum rate is equal to --table_copy_throttler_bytes_per_sec.");
 
 DECLARE_bool(row_count_only);
 DECLARE_int32(num_threads);
@@ -223,93 +243,149 @@ PredicateType ParsePredicateType(const string& predicate_type) {
     return PredicateType::IsNotNull;
   } else if (predicate_type_uc == "IN") {
     return PredicateType::InList;
-  } else {
-    LOG(FATAL) << Substitute("unhandled predicate type $0", predicate_type);
-    return PredicateType::None;
   }
+  return PredicateType::None;
 }
 
-KuduValue* ParseValue(KuduColumnSchema::DataType type,
-                      const rapidjson::Value* value) {
-  CHECK(value != nullptr);
+Status ParseValue(const rapidjson::Value& value,
+                  KuduColumnSchema::DataType type,
+                  unique_ptr<KuduValue>* out) {
+  DCHECK(out);
+
   switch (type) {
     case KuduColumnSchema::DataType::INT8:
     case KuduColumnSchema::DataType::INT16:
     case KuduColumnSchema::DataType::INT32:
-      CHECK(value->IsInt());
-      return KuduValue::FromInt(value->GetInt());
+      if (!value.IsInt()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'int'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::FromInt(value.GetInt()));
+      break;
     case KuduColumnSchema::DataType::INT64:
-      CHECK(value->IsInt64());
-      return KuduValue::FromInt(value->GetInt64());
+      if (!value.IsInt64()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'int64'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::FromInt(value.GetInt64()));
+      break;
     case KuduColumnSchema::DataType::STRING:
-      CHECK(value->IsString());
-      return KuduValue::CopyString(value->GetString());
+      if (!value.IsString()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'string'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::CopyString(value.GetString()));
+      break;
     case KuduColumnSchema::DataType::BOOL:
-      CHECK(value->IsBool());
-      return KuduValue::FromBool(value->GetBool());
+      if (!value.IsBool()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'bool'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::FromBool(value.GetBool()));
+      break;
     case KuduColumnSchema::DataType::FLOAT:
-      CHECK(value->IsDouble());
-      return KuduValue::FromFloat(static_cast<float>(value->GetDouble()));
+      if (!value.IsFloat()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'float'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::FromFloat(value.GetFloat()));
+      break;
     case KuduColumnSchema::DataType::DOUBLE:
-      CHECK(value->IsDouble());
-      return KuduValue::FromDouble(value->GetDouble());
-    default:
-      LOG(FATAL) << Substitute("unhandled data type $0", type);
+      if (!value.IsDouble()) {
+        return Status::InvalidArgument(Substitute(
+            "$0: expected value of type 'double'", EasyJson::ToString(value)));
+      }
+      out->reset(KuduValue::FromDouble(value.GetDouble()));
+      break;
+    default: {
+      auto s = Status::NotSupported(Substitute("unsupported column type $0", type));
+      LOG(DFATAL) << s.ToString();
+      return s;
+    }
   }
 
-  return nullptr;
+  return Status::OK();
 }
 
-KuduPredicate* NewComparisonPredicate(const client::sp::shared_ptr<KuduTable>& table,
-                                      KuduColumnSchema::DataType type,
-                                      const string& predicate_type,
-                                      const string& column_name,
-                                      const rapidjson::Value* value) {
-  KuduValue* kudu_value = ParseValue(type, value);
-  CHECK(kudu_value != nullptr);
+Status NewComparisonPredicate(const client::sp::shared_ptr<KuduTable>& table,
+                              const string& column_name,
+                              KuduColumnSchema::DataType column_type,
+                              const string& comparison_op_str,
+                              const rapidjson::Value& value,
+                              unique_ptr<KuduPredicate>* out) {
+  DCHECK(out);
+  unique_ptr<KuduValue> kudu_value;
+  RETURN_NOT_OK(ParseValue(value, column_type, &kudu_value));
   KuduPredicate::ComparisonOp cop;
-  if (predicate_type == "<") {
+  if (comparison_op_str == "<") {
     cop = KuduPredicate::ComparisonOp::LESS;
-  } else if (predicate_type == "<=") {
+  } else if (comparison_op_str == "<=") {
     cop = KuduPredicate::ComparisonOp::LESS_EQUAL;
-  } else if (predicate_type == "=") {
+  } else if (comparison_op_str == "=") {
     cop = KuduPredicate::ComparisonOp::EQUAL;
-  } else if (predicate_type == ">") {
+  } else if (comparison_op_str == ">") {
     cop = KuduPredicate::ComparisonOp::GREATER;
-  } else if (predicate_type == ">=") {
+  } else if (comparison_op_str == ">=") {
     cop = KuduPredicate::ComparisonOp::GREATER_EQUAL;
   } else {
-    return nullptr;
+    return Status::NotSupported(Substitute("'$0': unsupported comparison operator",
+                                           comparison_op_str));
   }
-  return table->NewComparisonPredicate(column_name, cop, kudu_value);
+
+  out->reset(table->NewComparisonPredicate(column_name, cop, kudu_value.release()));
+  return Status::OK();
 }
 
-KuduPredicate* NewIsNullPredicate(const client::sp::shared_ptr<KuduTable>& table,
-                                  const string& column_name,
-                                  PredicateType pt) {
+Status NewIsNullPredicate(const client::sp::shared_ptr<KuduTable>& table,
+                          const string& column_name,
+                          PredicateType pt,
+                          unique_ptr<KuduPredicate>* out) {
+  DCHECK(out);
   switch (pt) {
     case PredicateType::IsNotNull:
-      return table->NewIsNotNullPredicate(column_name);
+      out->reset(table->NewIsNotNullPredicate(column_name));
+      break;
     case PredicateType::IsNull:
-      return table->NewIsNullPredicate(column_name);
+      out->reset(table->NewIsNullPredicate(column_name));
+      break;
     default:
-      return nullptr;
+      DCHECK(false);
+      return Status::NotSupported(
+          Substitute("$0: unsupported nullability predicate", static_cast<uint16_t>(pt)));
   }
+  return Status::OK();
 }
 
-KuduPredicate* NewInListPredicate(const client::sp::shared_ptr<KuduTable>& table,
-                                  KuduColumnSchema::DataType type,
-                                  const string& name,
-                                  const JsonReader& reader,
-                                  const rapidjson::Value *object) {
-  CHECK(object->IsArray());
-  vector<const rapidjson::Value*> values;
-  reader.ExtractObjectArray(object, nullptr, &values);
-  vector<KuduValue *> kudu_values;
-  for (const auto& value : values) {
-    kudu_values.emplace_back(ParseValue(type, value));
+Status NewInListPredicate(const client::sp::shared_ptr<KuduTable>& table,
+                          const string& column_name,
+                          KuduColumnSchema::DataType column_type,
+                          const rapidjson::Value& object,
+                          const JsonReader& reader,
+                          unique_ptr<KuduPredicate>* out) {
+  if (!object.IsArray()) {
+    return Status::InvalidArgument(Substitute(
+        "$0: expecting an array for IN (in-list) predicate values",
+        EasyJson::ToString(object)));
   }
-  return table->NewInListPredicate(name, &kudu_values);
+  vector<const rapidjson::Value*> values;
+  RETURN_NOT_OK(reader.ExtractObjectArray(&object, nullptr, &values));
+  // Using vector of auto-pointers to avoid memory leakage if ParseValue()
+  // returns non-OK status.
+  vector<unique_ptr<KuduValue>> kudu_values;
+  for (const auto* v : values) {
+    unique_ptr<KuduValue> kudu_value;
+    RETURN_NOT_OK(ParseValue(*v, column_type, &kudu_value));
+    kudu_values.emplace_back(std::move(kudu_value));
+  }
+
+  vector<KuduValue*> kudu_values_raw;
+  kudu_values_raw.reserve(kudu_values.size());
+  for (auto& v : kudu_values) {
+    kudu_values_raw.emplace_back(v.release());
+  }
+  out->reset(table->NewInListPredicate(column_name, &kudu_values_raw));
+
+  return Status::OK();
 }
 
 Status AddPredicate(const client::sp::shared_ptr<KuduTable>& table,
@@ -322,35 +398,46 @@ Status AddPredicate(const client::sp::shared_ptr<KuduTable>& table,
     return Status::OK();
   }
 
-  Schema schema_internal = KuduSchema::ToSchema(table->schema());
-  int idx = schema_internal.find_column(column_name);
-  if (PREDICT_FALSE(idx == Schema::kColumnNotFound)) {
+  const Schema schema_internal = KuduSchema::ToSchema(table->schema());
+  const int idx = schema_internal.find_column(column_name);
+  if (idx == Schema::kColumnNotFound) {
     return Status::NotFound("no such column", column_name);
   }
-  auto type = table->schema().Column(static_cast<size_t>(idx)).type();
-  KuduPredicate* predicate = nullptr;
-  PredicateType pt = ParsePredicateType(predicate_type);
+  const auto column_type = table->schema().Column(static_cast<size_t>(idx)).type();
+  const PredicateType pt = ParsePredicateType(predicate_type);
+  unique_ptr<KuduPredicate> predicate;
   switch (pt) {
     case PredicateType::Equality:
     case PredicateType::Range:
-      CHECK(value);
-      predicate = NewComparisonPredicate(table, type, predicate_type, column_name, *value);
+      if (!value.has_value()) {
+        return Status::InvalidArgument("missing value for range/equality predicate");
+      }
+      RETURN_NOT_OK(NewComparisonPredicate(
+          table, column_name, column_type, predicate_type, *value.value(), &predicate));
       break;
     case PredicateType::IsNotNull:
     case PredicateType::IsNull:
-      CHECK(!value);
-      predicate = NewIsNullPredicate(table, column_name, pt);
+      if (value.has_value()) {
+        return Status::InvalidArgument(Substitute(
+            "'$0': unexpected value for NULL/NOT NULL predicate",
+            EasyJson::ToString(*value.value())));
+      }
+      RETURN_NOT_OK(NewIsNullPredicate(table, column_name, pt, &predicate));
       break;
     case PredicateType::InList: {
-      CHECK(value);
-      predicate = NewInListPredicate(table, type, column_name, reader, *value);
+      if (!value.has_value()) {
+        return Status::InvalidArgument("missing value for IN (in-list) predicate");
+      }
+      RETURN_NOT_OK(NewInListPredicate(
+          table, column_name, column_type, *value.value(), reader, &predicate));
       break;
     }
     default:
-      return Status::NotSupported(Substitute("not support predicate_type $0", predicate_type));
+      return Status::InvalidArgument(
+          Substitute("'$0': unsupported predicate", predicate_type));
   }
-  CHECK(predicate);
-  RETURN_NOT_OK(builder->AddConjunctPredicate(predicate));
+  DCHECK(predicate);
+  RETURN_NOT_OK(builder->AddConjunctPredicate(predicate.release()));
 
   return Status::OK();
 }
@@ -366,89 +453,163 @@ Status AddPredicates(const client::sp::shared_ptr<KuduTable>& table,
   RETURN_NOT_OK(reader.ExtractObjectArray(reader.root(),
                                           nullptr,
                                           &predicate_objects));
+  static constexpr const char* const kFmtErrPredicateName =
+      "$0: predicate name must be a string";
   vector<unique_ptr<KuduPredicate>> predicates;
-  for (int i = 0; i < predicate_objects.size(); ++i) {
+  for (size_t i = 0; i < predicate_objects.size(); ++i) {
+    const auto* obj = predicate_objects[i];
     if (i == 0) {
-      CHECK(predicate_objects[i]->IsString());
+      if (!obj->IsString()) {
+        return Status::InvalidArgument(Substitute(
+            kFmtErrPredicateName, EasyJson::ToString(*obj)));
+      }
       string op;
-      ToUpperCase(predicate_objects[i]->GetString(), &op);
+      ToUpperCase(obj->GetString(), &op);
       if (op != "AND") {
-        return Status::InvalidArgument(Substitute("only 'AND' operator is supported now"));
+        return Status::InvalidArgument(Substitute(
+            "$0: only 'AND' is supported as predicate operator", op));
       }
       continue;
     }
 
-    CHECK(predicate_objects[i]->IsArray());
-    vector<const rapidjson::Value*> elements;
-    reader.ExtractObjectArray(predicate_objects[i], nullptr, &elements);
-    if (elements.size() == 2 || elements.size() == 3) {
-      CHECK(elements[0]->IsString());
-      CHECK(elements[1]->IsString());
-      RETURN_NOT_OK(AddPredicate(table,
-          elements[0]->GetString(),
-          elements[1]->GetString(),
-          elements.size() == 2 ? nullopt
-                               : optional<const rapidjson::Value*>(elements[2]),
-          reader,
-          builder));
-    } else {
-      return Status::InvalidArgument(
-          Substitute("invalid predicate elements count $0", elements.size()));
+    if (!obj->IsArray()) {
+      return Status::InvalidArgument(Substitute(
+          "$0: expected JSON array for predicates", EasyJson::ToString(*obj)));
     }
+    vector<const rapidjson::Value*> elements;
+    RETURN_NOT_OK(reader.ExtractObjectArray(obj, nullptr, &elements));
+    if (elements.size() != 2 && elements.size() != 3) {
+      return Status::InvalidArgument(
+          Substitute("$0: malformed predicate", EasyJson::ToString(*obj)));
+    }
+    if (const auto* elem = elements[0]; !elem->IsString()) {
+      return Status::InvalidArgument(Substitute(
+          kFmtErrPredicateName, EasyJson::ToString(*elem)));
+    }
+    if (const auto* elem = elements[1]; !elem->IsString()) {
+      return Status::InvalidArgument(Substitute(
+          "$0: column name must be a string", EasyJson::ToString(*elem)));
+    }
+    RETURN_NOT_OK(AddPredicate(
+        table,
+        elements[0]->GetString(),
+        elements[1]->GetString(),
+        elements.size() == 2 ? nullopt
+                             : optional<const rapidjson::Value*>(elements[2]),
+        reader,
+        builder));
   }
 
   return Status::OK();
 }
 
+Status SchemasMatch(const Schema& src_table_schema,
+                    const Schema& dst_table_schema) {
+  bool same_schema = (src_table_schema == dst_table_schema);
+  if (FLAGS_strict_column_id) {
+      same_schema &= (src_table_schema.column_ids() == dst_table_schema.column_ids());
+  }
+  if (!same_schema) {
+    // The Schema's "==" operator uses the default COMPARE_ALL mode, so we show all the column
+    // information to help users to understand the difference.
+    static const auto kSchemaStringifyMode =
+        Schema::ToStringMode::BASE_INFO |
+        Schema::ToStringMode::WITH_COLUMN_ATTRIBUTES |
+        Schema::ToStringMode::WITH_COLUMN_COMMENTS |
+        Schema::ToStringMode::WITH_COLUMN_IDS;
+    return Status::NotSupported(Substitute(
+        "destination table's schema differs from the source one ($0 vs $1)",
+        dst_table_schema.ToString(kSchemaStringifyMode),
+        src_table_schema.ToString(kSchemaStringifyMode)));
+  }
+  return Status::OK();
+}
+
 Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table,
                               const client::sp::shared_ptr<KuduClient>& dst_client,
                               const string& dst_table_name) {
   client::sp::shared_ptr<KuduTable> dst_table;
-  Status s = dst_client->OpenTable(dst_table_name, &dst_table);
-  if (!s.IsNotFound()) {
+  auto s = dst_client->OpenTable(dst_table_name, &dst_table);
+  if (!s.IsNotFound() && !s.ok()) {
     return s;
   }
 
   // Destination table exists.
-  const KuduSchema& src_table_schema = src_table->schema();
+  const Schema src_schema_internal = KuduSchema::ToSchema(src_table->schema());
   if (s.ok()) {
     if (src_table->id() == dst_table->id()) {
       return Status::AlreadyPresent("Destination table is the same as the source table.");
     }
-
-    const KuduSchema& dst_table_schema = dst_table->schema();
-    if (src_table_schema != dst_table_schema) {
-      return Status::NotSupported(Substitute(
-          "destination table's schema differs from the source one ($0 vs $1)",
-          dst_table_schema.ToString(), src_table_schema.ToString()));
-    }
-
+    RETURN_NOT_OK(SchemasMatch(src_schema_internal,
+                               KuduSchema::ToSchema(dst_table->schema())));
     return Status::OK();
   }
 
-  // Destination table does NOT exist.
+  // The destination table does NOT exist.
   if (!FLAGS_create_table) {
     return Status::NotFound(Substitute("Table $0 does not exist in the destination cluster.",
                                        dst_table_name));
   }
 
-  Schema schema_internal = KuduSchema::ToSchema(src_table_schema);
-  // Convert Schema to KuduSchema will drop internal ColumnIds.
-  KuduSchema dst_table_schema = KuduSchema::FromSchema(schema_internal);
+  // Construct the destination table schema.
+  //
+  // 'to_delete_columns' is used to store the dummy columns that will be dropped after the table
+  // has been created if there are some column id holes in the source table schema.
+  vector<string> to_delete_columns;
+  static ObjectIdGenerator oid_generator;
+  SchemaBuilder builder;
+  int32_t expect_column_id = src_schema_internal.column_id(0);
+  for (size_t idx = 0; idx < src_schema_internal.num_columns();) {
+    const int32_t actual_column_id = src_schema_internal.column_id(idx);
+    if (expect_column_id == actual_column_id) {
+      // Construct the destination column schema according to the source column for continuous
+      // column id.
+      RETURN_NOT_OK(builder.AddColumn(src_schema_internal.column(idx),
+                                      src_schema_internal.is_key_column(idx)));
+      VLOG(1) << Substitute("Add a real column $0 for column id $1",
+                            src_schema_internal.column(idx).ToString(),
+                            actual_column_id);
+      // The expected column id is continuous.
+      ++expect_column_id;
+      ++idx;
+    } else {
+      // When there are column id holes, the expected column id must be less than the actual
+      // column id.
+      if (PREDICT_FALSE(expect_column_id >= actual_column_id)) {
+        return Status::Corruption(
+            Substitute("The internal column IDs must be monotonically increasing, but we got $0 "
+                       "while expecting $1.",
+                       actual_column_id, expect_column_id));
+      }
+      // Fill the hole with dummy columns.
+      while (expect_column_id < actual_column_id) {
+        auto dummy_column_name = "dummy_" + oid_generator.Next();
+        RETURN_NOT_OK(builder.AddColumn(dummy_column_name, DataType::INT8));
+        VLOG(1) << Substitute("Add a dummy column $0 for column id $1",
+                              dummy_column_name, expect_column_id);
+        // The dummy columns will be dropped after the table is created.
+        to_delete_columns.emplace_back(dummy_column_name);
+        ++expect_column_id;
+      }
+    }
+  }
+
+  const Schema dst_schema_internal = builder.Build();
   const auto& partition_schema = src_table->partition_schema();
 
-  auto convert_column_ids_to_names = [&schema_internal] (const vector<ColumnId>& column_ids) {
+  auto convert_column_ids_to_names = [&dst_schema_internal] (const vector<ColumnId>& column_ids) {
     vector<string> column_names;
     column_names.reserve(column_ids.size());
     for (const auto& column_id : column_ids) {
-      column_names.emplace_back(schema_internal.column_by_id(column_id).name());
+      column_names.emplace_back(dst_schema_internal.column_by_id(column_id).name());
     }
     return column_names;
   };
 
   // Table schema and replica number.
-  int num_replicas = FLAGS_create_table_replication_factor == -1 ?
+  const int num_replicas = FLAGS_create_table_replication_factor == -1 ?
       src_table->num_replicas() : FLAGS_create_table_replication_factor;
+  const KuduSchema dst_table_schema = KuduSchema::FromSchema(dst_schema_internal);
   unique_ptr<KuduTableCreator> table_creator(dst_client->NewTableCreator());
   table_creator->table_name(dst_table_name)
       .schema(&dst_table_schema)
@@ -473,7 +634,7 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
       }
       for (int i = 0; i < hash_bucket_nums_str.size(); i++) {
         int bucket_num = 0;
-        bool is_number = safe_strto32(hash_bucket_nums_str[i], &bucket_num);
+        const bool is_number = safe_strto32(hash_bucket_nums_str[i], &bucket_num);
         if (!is_number) {
           return Status::InvalidArgument(Substitute("'$0': cannot parse the number "
                                                     "of hash buckets.",
@@ -494,18 +655,18 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
 
   int i = 0;
   for (const auto& hash_dimension : partition_schema.hash_schema()) {
-    int num_buckets = hash_bucket_nums[i] != -1 ? hash_bucket_nums[i] :
-                                                  hash_dimension.num_buckets;
-    auto hash_columns = convert_column_ids_to_names(hash_dimension.column_ids);
+    const int num_buckets = hash_bucket_nums[i] != -1 ? hash_bucket_nums[i] :
+                                                        hash_dimension.num_buckets;
+    const auto hash_columns = convert_column_ids_to_names(hash_dimension.column_ids);
     table_creator->add_hash_partitions(hash_columns,
                                        num_buckets,
-                                       hash_dimension.seed);
+                                       static_cast<int32_t>(hash_dimension.seed));
     i++;
   }
 
   // Add range partition schema.
   if (!partition_schema.range_schema().column_ids.empty()) {
-    auto range_columns
+    const auto range_columns
       = convert_column_ids_to_names(partition_schema.range_schema().column_ids);
     table_creator->set_range_partition_columns(range_columns);
   }
@@ -523,11 +684,11 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
     }
 
     // Partitions are considered metadata, so don't redact them.
-    ScopedDisableRedaction no_redaction;
+    const ScopedDisableRedaction no_redaction;
 
     Arena arena(256);
-    std::unique_ptr<KuduPartialRow> lower(new KuduPartialRow(&schema_internal));
-    std::unique_ptr<KuduPartialRow> upper(new KuduPartialRow(&schema_internal));
+    std::unique_ptr<KuduPartialRow> lower(new KuduPartialRow(&dst_schema_internal));
+    std::unique_ptr<KuduPartialRow> upper(new KuduPartialRow(&dst_schema_internal));
     Slice range_key_start(partition.begin().range_key());
     Slice range_key_end(partition.end().range_key());
     RETURN_NOT_OK(partition_schema.DecodeRangeKey(&range_key_start, lower.get(), &arena));
@@ -536,14 +697,31 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
     table_creator->add_range_partition(lower.release(), upper.release());
   }
 
-  if (partition_schema.hash_schema().empty() &&
-      partition_schema.range_schema().column_ids.empty()) {
+  if (partition_schema.range_schema().column_ids.empty()) {
     // This src table is unpartitioned, just create a table range partitioned on no columns.
     table_creator->set_range_partition_columns({});
   }
 
+  table_creator->set_allow_empty_partition(true);
+
   // Create table.
   RETURN_NOT_OK(table_creator->Create());
+
+  // Drop the dummy columns.
+  if (!to_delete_columns.empty()) {
+    unique_ptr<client::KuduTableAlterer> alterer(dst_client->NewTableAlterer(dst_table_name));
+    for (const auto &to_delete_column: to_delete_columns) {
+      VLOG(1) << Substitute("Drop dummy column $0", to_delete_column);
+      alterer->DropColumn(to_delete_column);
+    }
+    RETURN_NOT_OK(alterer->Alter());
+  }
+
+  // Check that the schemas match.
+  RETURN_NOT_OK(dst_client->OpenTable(dst_table_name, &dst_table));
+  RETURN_NOT_OK(SchemasMatch(src_schema_internal,
+                             KuduSchema::ToSchema(dst_table->schema())));
+
   LOG(INFO) << "Table " << dst_table_name << " created successfully";
 
   return Status::OK();
@@ -571,8 +749,15 @@ TableScanner::TableScanner(
       scan_batch_size_(-1),
       out_(nullptr) {
   CHECK_OK(SetReplicaSelection(FLAGS_replica_selection));
+  if (FLAGS_table_copy_throttler_bytes_per_sec > 0) {
+    throttler_ = std::make_unique<Throttler>(Throttler::kNoLimit,
+                                             FLAGS_table_copy_throttler_bytes_per_sec,
+                                             FLAGS_table_copy_throttler_burst_factor);
+  }
 }
 
+TableScanner::~TableScanner() {}
+
 Status TableScanner::ScanData(const vector<KuduScanToken*>& tokens,
                               const function<Status(const KuduScanBatch& batch)>& cb) {
   for (const auto* token : tokens) {
@@ -593,13 +778,21 @@ Status TableScanner::ScanData(const vector<KuduScanToken*>& tokens,
       count += batch.NumRows();
       total_count_ += batch.NumRows();
       ++next_batch_calls;
+      // Limit table copying speed.
+      if (throttler_) {
+        SCOPED_LOG_SLOW_EXECUTION(INFO, 1000, "Table copy throttler");
+        while (!throttler_->Take(0,
+                                 batch.direct_data().size() + batch.indirect_data().size())) {
+          SleepFor(MonoDelta::FromMicroseconds(Throttler::kRefillPeriodMicros / 2));
+        }
+      }
       RETURN_NOT_OK(cb(batch));
     }
     sw.stop();
 
     if (FLAGS_report_scanner_stats && out_) {
       auto& out = *out_;
-      MutexLock l(output_lock_);
+      std::lock_guard l(output_lock_);
       out << Substitute("T $0 scanned $1 rows in $2 seconds\n",
                         token->tablet().id(), count, sw.elapsed().wall_seconds());
       const auto& metrics = scanner->GetResourceMetrics();
@@ -617,7 +810,7 @@ void TableScanner::ScanTask(const vector<KuduScanToken*>& tokens, Status* thread
   DCHECK(thread_status);
   *thread_status = ScanData(tokens, [&](const KuduScanBatch& batch) {
     if (out_ && FLAGS_show_values) {
-      MutexLock l(output_lock_);
+      std::lock_guard l(output_lock_);
       for (const auto& row : batch) {
         *out_ << row.ToString() << "\n";
       }
@@ -767,7 +960,7 @@ Status TableScanner::StartWork(WorkType work_type) {
     client::sp::shared_ptr<KuduTable> dst_table;
     RETURN_NOT_OK(dst_client_->get()->OpenTable(*dst_table_name_, &dst_table));
     if (dst_table->schema() != src_table->schema()) {
-      Status::InvalidArgument("source and destination tables should have the same schema");
+      return Status::InvalidArgument("source and destination tables should have the same schema");
     }
   }
 
