@@ -25,6 +25,7 @@
 #include <string>
 #include <type_traits>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -73,6 +74,7 @@
 #include "kudu/util/flag_validators.h"
 #include "kudu/util/memory/arena.h"
 #include "kudu/util/pb_util.h"
+#include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 #include "kudu/util/string_case.h"
@@ -105,6 +107,32 @@ bool ServerKeySetTogether() {
 
 GROUP_FLAG_VALIDATOR(server_key_set_together, ServerKeySetTogether);
 
+DECLARE_string(encryption_key_type);
+DECLARE_string(tenant_name);
+DECLARE_string(tenant_id);
+
+DEFINE_string(tenant_key, "",
+              "The encrypted tenant key to use in the filesystem.");
+DEFINE_string(tenant_key_iv, "",
+              "The tenant key IV to use in the filesystem.");
+DEFINE_string(tenant_key_version, "",
+              "The tenant key version to use in the filesystem.");
+
+bool TenantKeySetTogether() {
+  if (FLAGS_tenant_name.empty() != FLAGS_tenant_id.empty()
+      || FLAGS_tenant_name.empty() != FLAGS_tenant_key.empty()
+      || FLAGS_tenant_name.empty() != FLAGS_tenant_key_iv.empty()
+      || FLAGS_tenant_name.empty() != FLAGS_tenant_key_version.empty()) {
+    LOG(ERROR) << "'tenant_name', 'tenant_id', 'tenant_key', 'tenant_key_iv', and "
+                  "'tenant_key_version' must either all be set, or none of them "
+                  "must be set.";
+    return false;
+  }
+  return true;
+}
+
+GROUP_FLAG_VALIDATOR(tenant_key_set_together, TenantKeySetTogether);
+
 DEFINE_bool(repair, false,
             "Repair any inconsistencies in the filesystem.");
 
@@ -140,12 +168,55 @@ using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::unordered_map;
+using std::unordered_set;
 using std::vector;
 using strings::Substitute;
 
+constexpr const char* const kBlockIdsArg = "block_ids";
+constexpr const char* const kBlockIdsDesc = "Block ids existing in metadata files";
+
 namespace kudu {
 namespace tools {
 
+Status UpdateEncryptionKeyInfo(Env* env) {
+  FsManagerOpts fs_opts;
+  fs_opts.update_instances = UpdateInstanceBehavior::UPDATE_AND_ERROR_ON_FAILURE;
+  fs_opts.skip_block_manager = true;
+  FsManager fs_manager(env, std::move(fs_opts));
+  RETURN_NOT_OK(fs_manager.Open());
+
+  if (fs_manager.server_key().empty()) {
+    return Status::IllegalState(
+        "We should not do server key upgrade on a cluster with data rest encryption disabled.");
+  }
+
+  if (fs_manager.is_tenants_exist()) {
+    cout << "This tserver has already in tenant key version, and there is no need to upgrade again."
+         << endl;
+    return Status::OK();
+  }
+
+  unique_ptr<InstanceMetadataPB> metadata(new InstanceMetadataPB);
+  fs_manager.CopyMetadata(&metadata);
+
+  // Add the default tenant key with the server key.
+  InstanceMetadataPB::TenantMetadataPB* tenant_metadata = metadata->add_tenants();
+  tenant_metadata->set_tenant_name(fs::kDefaultTenantName);
+  tenant_metadata->set_tenant_id(fs::kDefaultTenantID);
+  tenant_metadata->set_tenant_key(fs_manager.server_key());
+  tenant_metadata->set_tenant_key_iv(fs_manager.server_key_iv());
+  tenant_metadata->set_tenant_key_version(fs_manager.server_key_version());
+  // Clear the server key info.
+  metadata->clear_server_key();
+  metadata->clear_server_key_iv();
+  metadata->clear_server_key_version();
+
+  // Write the new metadata to disk.
+  RETURN_NOT_OK(fs_manager.UpdateMetadata(metadata));
+
+  return Status::OK();
+}
+
 namespace {
 
 Status Check(const RunnerContext& /*context*/) {
@@ -245,23 +316,36 @@ Status Check(const RunnerContext& /*context*/) {
 }
 
 Status Format(const RunnerContext& /*context*/) {
-  FsManager fs_manager(Env::Default(), FsManagerOpts());
+  FsManager fs_manager(Env::Default());
   optional<string> uuid;
-  optional<string> server_key;
-  optional<string> server_key_iv;
-  optional<string> server_key_version;
+  optional<string> tenant_name;
+  optional<string> tenant_id;
+  optional<string> encryption_key;
+  optional<string> encryption_key_iv;
+  optional<string> encryption_key_version;
   if (!FLAGS_uuid.empty()) {
     uuid = FLAGS_uuid;
   }
-  if (!FLAGS_server_key.empty()
-      && !FLAGS_server_key_iv.empty()
-      && !FLAGS_server_key_version.empty()) {
-    server_key = FLAGS_server_key;
-    server_key_iv = FLAGS_server_key_iv;
-    server_key_version = FLAGS_server_key_version;
+  if (iequals(FLAGS_encryption_key_type, "server_key")) {
+    if (!FLAGS_server_key.empty()
+        && !FLAGS_server_key_iv.empty()
+        && !FLAGS_server_key_version.empty()) {
+      encryption_key = FLAGS_server_key;
+      encryption_key_iv = FLAGS_server_key_iv;
+      encryption_key_version = FLAGS_server_key_version;
+    }
+  } else {
+    if (!FLAGS_tenant_id.empty()) {
+      tenant_name = FLAGS_tenant_name;
+      tenant_id = FLAGS_tenant_id;
+      encryption_key = FLAGS_tenant_key;
+      encryption_key_iv = FLAGS_tenant_key_iv;
+      encryption_key_version = FLAGS_tenant_key_version;
+    }
   }
-  return fs_manager.CreateInitialFileSystemLayout(uuid, server_key,
-                                                  server_key_iv, server_key_version);
+
+  return fs_manager.CreateInitialFileSystemLayout(uuid, tenant_name, tenant_id, encryption_key,
+                                                  encryption_key_iv, encryption_key_version);
 }
 
 Status DumpUuid(const RunnerContext& /*context*/) {
@@ -359,6 +443,36 @@ Status DumpFsTree(const RunnerContext& /*context*/) {
   return Status::OK();
 }
 
+Status LocateBlock(const RunnerContext& context) {
+  const string& block_ids_str = FindOrDie(context.required_args, kBlockIdsArg);
+  vector<string> block_ids = strings::Split(block_ids_str, ",", strings::SkipEmpty());
+  if (block_ids.empty()) {
+    return Status::InvalidArgument("no block identifiers provided");
+  }
+  FsManagerOpts opts;
+  opts.read_only = true;
+  opts.update_instances = UpdateInstanceBehavior::DONT_UPDATE;
+  FsManager fs_manager(Env::Default(), std::move(opts));
+  RETURN_NOT_OK(fs_manager.Open());
+  DataTable output_table({ "block id", "path" });
+  for (const string& blkid_str : block_ids) {
+    uint64_t blkid = 0;
+    if (!safe_strtou64(blkid_str.c_str(), &blkid)) {
+      output_table.AddRow({blkid_str, "not an integer"});
+      continue;
+    }
+    BlockId block_id(blkid);
+    string path;
+    if (fs_manager.block_manager()->FindBlockPath(block_id, &path)) {
+      output_table.AddRow({blkid_str, path});
+    } else {
+      output_table.AddRow({blkid_str, "not found"});
+    }
+  }
+  RETURN_NOT_OK(output_table.PrintTo(cout));
+  return Status::OK();
+}
+
 Status CheckForTabletsThatWillFailWithUpdate() {
   FsManagerOpts opts;
   opts.read_only = true;
@@ -412,6 +526,24 @@ Status Update(const RunnerContext& /*context*/) {
   return fs.Open();
 }
 
+Status UpgradeEncryptionKey(const RunnerContext& /*context*/) {
+  // TODO(aserbin): if set, use TMPDIR from env instead of hardcoded /tmp
+  static const string kLockFilePath =
+      "/tmp/kudu_tserver-metadata-upgrade-instance.lock";
+
+  Env* env = Env::Default();
+  FileLock* file_lock;
+  KUDU_RETURN_NOT_OK_PREPEND(env->LockFile(kLockFilePath, &file_lock),
+                             "Could not lock instance file. Make sure that "
+                             "this tool is not already running.");
+  auto auto_unlock = MakeScopedCleanup([&]() {
+    WARN_NOT_OK(env->UnlockFile(file_lock),
+                Substitute("could not unlock file $0", kLockFilePath));
+  });
+
+  return UpdateEncryptionKeyInfo(env);
+}
+
 namespace {
 
 // The 'kudu fs list' column fields.
@@ -910,6 +1042,7 @@ unique_ptr<Mode> BuildFsMode() {
       .AddOptionalParameter("fs_metadata_dir")
       .AddOptionalParameter("fs_wal_dir")
       .AddOptionalParameter("uuid")
+      .AddOptionalParameter("encryption_key_type")
       .Build();
 
   unique_ptr<Action> update =
@@ -958,13 +1091,37 @@ unique_ptr<Mode> BuildFsMode() {
       .AddOptionalParameter("h")
       .Build();
 
+  unique_ptr<Action> upgrade_encryption_key =
+      ActionBuilder("upgrade_encryption_key", &UpgradeEncryptionKey)
+      .Description("Upgrade the encryption key info in metadata")
+      .ExtraDescription("Upgrade the server key to the tenant key which belongs to the default "
+                        "tenant. This feature only works on a cluster with data rest encryption "
+                        "enabled.\n\n"
+                        "Note: this function is exclusively for use with the Kudu CLI tool and can "
+                        "only be used once per tserver.")
+      .AddOptionalParameter("fs_data_dirs")
+      .AddOptionalParameter("fs_metadata_dir")
+      .AddOptionalParameter("fs_wal_dir")
+      .Build();
+
+  unique_ptr<Action> locate_block =
+      ActionBuilder("locate_block", &LocateBlock)
+      .Description("Find the file's path where the block locates")
+      .AddRequiredParameter({ kBlockIdsArg, kBlockIdsDesc })
+      .AddOptionalParameter("fs_data_dirs")
+      .AddOptionalParameter("fs_metadata_dir")
+      .AddOptionalParameter("fs_wal_dir")
+      .Build();
+
   return ModeBuilder("fs")
       .Description("Operate on a local Kudu filesystem")
       .AddMode(BuildFsDumpMode())
       .AddAction(std::move(check))
       .AddAction(std::move(format))
       .AddAction(std::move(list))
+      .AddAction(std::move(locate_block))
       .AddAction(std::move(update))
+      .AddAction(std::move(upgrade_encryption_key))
       .Build();
 }
 
