@@ -16,6 +16,7 @@
 // under the License.
 
 #include <algorithm>
+#include <atomic>
 #include <cstddef>
 #include <cstdint>
 #include <functional>
@@ -23,7 +24,6 @@
 #include <limits>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <set>
 #include <string>
@@ -95,6 +95,7 @@
 #include "kudu/util/status.h"
 #include "kudu/util/thread.h"
 #include "kudu/util/threadpool.h"
+#include "kudu/util/throttler.h"
 // IWYU pragma: no_include <boost/container/vector.hpp>
 
 namespace kudu {
@@ -156,8 +157,12 @@ DEFINE_string(dst_fs_metadata_dir, "",
               "metadata directory if any exists. If none exists, --dst_fs_wal_dir "
               "will be used as the metadata directory.");;
 
+DECLARE_bool(force);
 DECLARE_int32(num_threads);
+DECLARE_bool(tablet_copy_support_download_superblock_in_batch);
 DECLARE_int32(tablet_copy_download_threads_nums_per_session);
+DECLARE_int64(tablet_copy_throttler_bytes_per_sec);
+DECLARE_double(tablet_copy_throttler_burst_factor);
 DECLARE_string(tables);
 
 using kudu::consensus::ConsensusMetadata;
@@ -188,7 +193,6 @@ using kudu::tserver::TSTabletManager;
 using std::cout;
 using std::endl;
 using std::map;
-using std::move;
 using std::pair;
 using std::shared_ptr;
 using std::set;
@@ -251,10 +255,10 @@ class TabletCopier {
                FsManager* dst_fs_manager,
                scoped_refptr<ConsensusMetadataManager> dst_cmeta_manager,
                HostPort source_addr) :
-                  tablet_ids_to_copy_(move(tablet_ids_to_copy)),
+                  tablet_ids_to_copy_(std::move(tablet_ids_to_copy)),
                   dst_fs_manager_(dst_fs_manager),
-                  dst_cmeta_manager_(move(dst_cmeta_manager)),
-                  source_addr_(move(source_addr)),
+                  dst_cmeta_manager_(std::move(dst_cmeta_manager)),
+                  source_addr_(std::move(source_addr)),
                   copy_type_(CopyType::FROM_REMOTE) {
   }
 
@@ -263,11 +267,11 @@ class TabletCopier {
                scoped_refptr<ConsensusMetadataManager> dst_cmeta_manager,
                FsManager* src_fs_manager,
                set<string> src_tablet_ids_set) :
-                  tablet_ids_to_copy_(move(tablet_ids_to_copy)),
+                  tablet_ids_to_copy_(std::move(tablet_ids_to_copy)),
                   dst_fs_manager_(dst_fs_manager),
-                  dst_cmeta_manager_(move(dst_cmeta_manager)),
-                  src_fs_manager_(move(src_fs_manager)),
-                  src_tablet_ids_set_(move(src_tablet_ids_set)),
+                  dst_cmeta_manager_(std::move(dst_cmeta_manager)),
+                  src_fs_manager_(src_fs_manager),
+                  src_tablet_ids_set_(std::move(src_tablet_ids_set)),
                   copy_type_(CopyType::FROM_LOCAL) {
   }
 
@@ -302,7 +306,7 @@ class TabletCopier {
     RETURN_NOT_OK(Thread::Create("tool-tablet-copy", "check-progress",
         [&] () {
           while (!latch.WaitFor(MonoDelta::FromSeconds(10))) {
-            std::lock_guard<simple_spinlock> l(lock);
+            std::lock_guard l(lock);
             for (const auto& entry : copying_replicas_by_tablet_id) {
               LOG(INFO) << Substitute("Tablet $0 copy status: $1",
                                       entry.first,
@@ -319,28 +323,37 @@ class TabletCopier {
 
     // Create a thread pool to copy tablets.
     std::unique_ptr<ThreadPool> copy_pool;
-    ThreadPoolBuilder("tool-tablet-copy-pool")
-        .set_max_threads(FLAGS_num_threads)
-        .set_min_threads(FLAGS_num_threads)
-        .Build(&copy_pool);
+    RETURN_NOT_OK(ThreadPoolBuilder("tool-tablet-copy-pool")
+                      .set_max_threads(FLAGS_num_threads)
+                      .set_min_threads(FLAGS_num_threads)
+                      .Build(&copy_pool));
 
     shared_ptr<Messenger> messenger;
     RETURN_NOT_OK(BuildMessenger("tablet_copy_client", &messenger));
+
+    shared_ptr<Throttler> throttler;
+    if (FLAGS_tablet_copy_throttler_bytes_per_sec > 0) {
+      throttler = std::make_shared<Throttler>(0,
+                                              FLAGS_tablet_copy_throttler_bytes_per_sec,
+                                              FLAGS_tablet_copy_throttler_burst_factor);
+    }
+    std::atomic<bool> has_failed_tablets(false);
     // Start to copy tablets.
     for (const auto& tablet_id : tablet_ids_to_copy_) {
       RETURN_NOT_OK(copy_pool->Submit([&]() {
         // 'fake_replica' is used for checking copy progress only.
         scoped_refptr<TabletReplica> fake_replica(new TabletReplica());
         {
-          std::lock_guard<simple_spinlock> l(lock);
+          std::lock_guard l(lock);
           LOG(WARNING) << "Start to copy tablet " << tablet_id;
           copying_replicas_by_tablet_id[tablet_id] = fake_replica.get();
         }
         Status s;
         unique_ptr<TabletCopyClient> client;
         if (copy_type_ == CopyType::FROM_REMOTE) {
-          client.reset(new RemoteTabletCopyClient(tablet_id, dst_fs_manager_, dst_cmeta_manager_,
-                                                  messenger, &tablet_copy_client_metrics));
+          client.reset(new RemoteTabletCopyClient(
+              tablet_id, dst_fs_manager_, dst_cmeta_manager_,
+              messenger, &tablet_copy_client_metrics, throttler));
           s = client->Start(source_addr_, nullptr);
         } else {
           CHECK_EQ(copy_type_, CopyType::FROM_LOCAL);
@@ -356,7 +369,7 @@ class TabletCopier {
           return client->Finish();
         });
         {
-          std::lock_guard<simple_spinlock> l(lock);
+          std::lock_guard l(lock);
           if (!s.ok()) {
             InsertOrDie(&failed_tablet_ids, tablet_id);
             LOG(ERROR) << Substitute("Tablet $0 copy failed: $1.", tablet_id, s.ToString());
@@ -366,6 +379,10 @@ class TabletCopier {
           }
           copying_replicas_by_tablet_id.erase(tablet_id);
 
+          if (!failed_tablet_ids.empty()) {
+            has_failed_tablets.store(true);
+          }
+
           LOG(INFO) << Substitute("$0/$1 tablets, $2 bytes copied, include $3 failed tablets.",
                                   succeed_tablet_count + failed_tablet_ids.size(),
                                   total_tablet_count,
@@ -381,6 +398,9 @@ class TabletCopier {
     latch.CountDown();
     check_thread->Join();
 
+    if (has_failed_tablets.load()) {
+      return Status::RuntimeError("some tablets failed to copy: check error messages for details");
+    }
     return Status::OK();
   }
 
@@ -508,7 +528,7 @@ Status PrintReplicaUuids(const RunnerContext& context) {
 
 Status BackupConsensusMetadata(FsManager* fs_manager,
                                const string& tablet_id) {
-  Env* env = fs_manager->env();
+  Env* env = fs_manager->GetEnv();
   string cmeta_filename = fs_manager->GetConsensusMetadataPath(tablet_id);
   string backup_filename = Substitute("$0.pre_rewrite.$1", cmeta_filename, env->NowMicros());
   WritableFileOptions opts;
@@ -545,7 +565,7 @@ Status RewriteRaftConfig(const RunnerContext& context) {
   DCHECK(!peers.empty());
 
   Env* env = Env::Default();
-  FsManagerOpts fs_opts = FsManagerOpts();
+  FsManagerOpts fs_opts;
   fs_opts.skip_block_manager = true;
   FsManager fs_manager(env, std::move(fs_opts));
   RETURN_NOT_OK(fs_manager.Open());
@@ -588,9 +608,9 @@ Status SetRaftTerm(const RunnerContext& context) {
 
   // Load the current metadata from disk and verify that the intended operation is safe.
   Env* env = Env::Default();
-  FsManagerOpts fs_opts = FsManagerOpts();
+  FsManagerOpts fs_opts;
   fs_opts.skip_block_manager = true;
-  FsManager fs_manager(env, fs_opts);
+  FsManager fs_manager(env, std::move(fs_opts));
   RETURN_NOT_OK(fs_manager.Open());
   // Load the cmeta file and rewrite the raft config.
   scoped_refptr<ConsensusMetadataManager> cmeta_manager(new ConsensusMetadataManager(&fs_manager));
@@ -660,6 +680,61 @@ Status DeleteRowsets(const RunnerContext& context) {
   return Status::OK();
 }
 
+Status UnsafeRecreateCmeta(const RunnerContext& context) {
+  const string& tablet_id = FindOrDie(context.required_args, kTabletIdArg);
+  int64_t term;
+  int64_t opid_index;
+  try {
+    term = stoi(FindOrDie(context.required_args, "term"));
+    opid_index = stoi(FindOrDie(context.required_args, "index"));
+  } catch (...) {
+    return Status::InvalidArgument("term and index must be provided and must be numerical");
+  }
+
+  RaftConfigPB config;
+  config.set_opid_index(opid_index);
+  config.set_obsolete_local(false);
+
+  set<string> uuids;
+  set<string> host_ports;
+  // Parse peer arguments.
+  for (const auto& arg : context.variadic_args) {
+    RaftPeerPB* peer = config.add_peers();
+    peer->set_member_type(consensus::RaftPeerPB_MemberType_VOTER);
+    string uuid;
+    HostPort host_port;
+    RETURN_NOT_OK(ParsePeerString(arg, &uuid, &host_port));
+    if (!uuids.emplace(uuid).second) {
+      return Status::InvalidArgument("Duplicate UUID: " + uuid);
+    }
+    if (!host_ports.emplace(host_port.ToString()).second) {
+      return Status::InvalidArgument("Duplicate RPC address: " + host_port.ToString());
+    }
+    peer->set_permanent_uuid(uuid);
+    HostPortPB* hostPort = peer->mutable_last_known_addr();
+    hostPort->set_host(host_port.host());
+    hostPort->set_port(host_port.port());
+  }
+
+  // Load file system.
+  Env* env = Env::Default();
+  FsManagerOpts fs_opts;
+  fs_opts.skip_block_manager = true;
+  FsManager fs_manager(env, std::move(fs_opts));
+  RETURN_NOT_OK(fs_manager.Open());
+
+  // We need a scoped_refptr to avoid a check failure on RefCountedThreadSafe
+  // object deleted without calling Release().
+  scoped_refptr<ConsensusMetadataManager> manager(new ConsensusMetadataManager(&fs_manager));
+
+  if (FLAGS_force && manager->Load(tablet_id).ok()) {
+    RETURN_NOT_OK(manager->Delete(tablet_id));
+  }
+
+  // Write the cmeta file.
+  return manager->Create(tablet_id, config, term);
+}
+
 Status CopyFromRemote(const RunnerContext& context) {
   // Parse the tablet ID and source arguments.
   const string& tablet_ids_str = FindOrDie(context.required_args, kTabletIdsCsvArg);
@@ -671,11 +746,14 @@ Status CopyFromRemote(const RunnerContext& context) {
   HostPort hp;
   RETURN_NOT_OK(ParseHostPortString(rpc_address, &hp));
 
-  FsManager fs_manager(Env::Default(), FsManagerOpts());
+  FsManager fs_manager(Env::Default());
   RETURN_NOT_OK(fs_manager.Open());
   scoped_refptr<ConsensusMetadataManager> cmeta_manager(new ConsensusMetadataManager(&fs_manager));
 
-  TabletCopier copier(move(tablet_ids_to_copy), &fs_manager, move(cmeta_manager), move(hp));
+  TabletCopier copier(std::move(tablet_ids_to_copy),
+                      &fs_manager,
+                      std::move(cmeta_manager),
+                      std::move(hp));
   return copier.CopyTablets();
 }
 
@@ -709,11 +787,11 @@ Status CopyFromLocal(const RunnerContext& context) {
   RETURN_NOT_OK(src_fs_manager.ListTabletIds(&src_tablet_ids));
   set<string> src_tablet_ids_set(src_tablet_ids.begin(), src_tablet_ids.end());
 
-  TabletCopier copier(move(tablet_ids_to_copy),
+  TabletCopier copier(std::move(tablet_ids_to_copy),
                       &dst_fs_manager,
-                      move(dst_cmeta_manager),
+                      std::move(dst_cmeta_manager),
                       &src_fs_manager,
-                      move(src_tablet_ids_set));
+                      std::move(src_tablet_ids_set));
   return copier.CopyTablets();
 }
 
@@ -1342,6 +1420,19 @@ unique_ptr<Mode> BuildLocalReplicaMode() {
           .AddOptionalParameter("fs_wal_dir")
           .Build();
 
+  unique_ptr<Action> unsafe_recreate =
+      ActionBuilder("unsafe_recreate", &UnsafeRecreateCmeta)
+      .Description("Rewrite the consensus metadata based on the provided arguments")
+      .AddRequiredParameter({kTabletIdArg, kTabletIdArgDesc})
+      .AddRequiredParameter({"term", "Raft term"})
+      .AddRequiredParameter({"index", "OpId index"})
+      .AddRequiredVariadicParameter({kRaftPeersArg, kRaftPeersArgDesc})
+      .AddOptionalParameter("fs_data_dirs")
+      .AddOptionalParameter("fs_metadata_dir")
+      .AddOptionalParameter("fs_wal_dir")
+      .AddOptionalParameter("force")
+      .Build();
+
   unique_ptr<Mode> cmeta =
       ModeBuilder("cmeta")
       .Description("Operate on a local tablet replica's consensus "
@@ -1349,6 +1440,7 @@ unique_ptr<Mode> BuildLocalReplicaMode() {
       .AddAction(std::move(print_replica_uuids))
       .AddAction(std::move(rewrite_raft_config))
       .AddAction(std::move(set_term))
+      .AddAction(std::move(unsafe_recreate))
       .Build();
 
   unique_ptr<Mode> tmeta =
@@ -1368,6 +1460,9 @@ unique_ptr<Mode> BuildLocalReplicaMode() {
       .AddOptionalParameter("fs_wal_dir")
       .AddOptionalParameter("tablet_copy_download_threads_nums_per_session")
       .AddOptionalParameter("num_threads")
+      .AddOptionalParameter("tablet_copy_support_download_superblock_in_batch")
+      .AddOptionalParameter("tablet_copy_throttler_bytes_per_sec")
+      .AddOptionalParameter("tablet_copy_throttler_burst_factor")
       .Build();
 
   unique_ptr<Action> copy_from_local =
