@@ -31,7 +31,7 @@
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
-#include <google/protobuf/stubs/common.h>
+#include <google/protobuf/stubs/common.h> // IWYU pragma: keep
 #include <google/protobuf/stubs/status.h>
 #include <google/protobuf/stubs/stringpiece.h>
 #include <google/protobuf/util/json_util.h>
@@ -49,12 +49,15 @@
 #include "kudu/common/partial_row.h"
 #include "kudu/common/partition.h"
 #include "kudu/common/schema.h"
+#include "kudu/common/wire_protocol.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/numbers.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/master/master.pb.h"
+#include "kudu/master/master.proxy.h"
 #include "kudu/tools/table_scanner.h"
 #include "kudu/tools/tool.pb.h"
 #include "kudu/tools/tool_action.h"
@@ -86,6 +89,9 @@ using kudu::client::KuduTableStatistics;
 using kudu::client::KuduValue;
 using kudu::client::internal::ReplicaController;
 using kudu::iequals;
+using kudu::master::ListInFlightTablesRequestPB;
+using kudu::master::ListInFlightTablesResponsePB;
+using kudu::master::MasterServiceProxy;
 using kudu::tools::PartitionPB;
 using std::cerr;
 using std::cout;
@@ -104,6 +110,8 @@ using std::vector;
 using strings::Split;
 using strings::Substitute;
 
+DECLARE_string(columns);
+
 DEFINE_bool(check_row_existence, false,
             "Also check for the existence of the row on the leader replica of "
             "the tablet. If found, the full row will be printed; if not found, "
@@ -208,6 +216,7 @@ DECLARE_bool(fault_tolerant);
 DECLARE_int32(create_table_replication_factor);
 DECLARE_bool(row_count_only);
 DECLARE_bool(show_scanner_stats);
+DECLARE_bool(strict_column_id);
 
 DEFINE_string(encoding_type, "AUTO_ENCODING",
               "Type of encoding for the column including AUTO_ENCODING, PLAIN_ENCODING, "
@@ -293,6 +302,56 @@ class TableLister {
     return output;
   }
 
+  static Status ListTablesWithColumns(const vector<string>& master_addresses,
+                                      const vector<string>& columns) {
+    client::sp::shared_ptr<KuduClient> client;
+    RETURN_NOT_OK(CreateKuduClient(master_addresses, &client, true /* can_see_all_replicas */));
+    vector<KuduClient::Data::TableInfo> tables_info;
+    RETURN_NOT_OK(client->data_->ListTablesWithInfo(client.get(),
+                                                    &tables_info,
+                                                    "" /* filter */,
+                                                    FLAGS_show_tablet_partition_info,
+                                                    FLAGS_soft_deleted_only));
+
+    DataTable table(columns);
+
+    #define GET_PROPERTY(x) \
+          [](const KuduClient::Data::TableInfo& table_info) ->  \
+          string { return table_info.x;  }
+
+    #define GET_NUM_PROPERTY(x) \
+          [](const KuduClient::Data::TableInfo& table_info) -> \
+          string { return std::to_string(table_info.x); }
+
+    unordered_map<string, std::function<string(const KuduClient::Data::TableInfo& table_info)>>
+        mapping = {
+            {"id", GET_PROPERTY(id)},
+            {"name", GET_PROPERTY(table_name)},
+            {"num_tablets", GET_NUM_PROPERTY(num_tablets)},
+            {"num_replicas", GET_NUM_PROPERTY(num_replicas)},
+            {"live_row_count", GET_NUM_PROPERTY(live_row_count)},
+        };
+
+    #undef GET_PROPERTY
+    #undef GET_NUM_PROPERTY
+
+    for (const auto& tinfo : tables_info) {
+      vector<string> values;
+      for (const auto& column : columns) {
+        string lowercase_column;
+        ToLowerCase(column, &lowercase_column);
+        auto getter = mapping.find(lowercase_column);
+        if (getter == mapping.end()) {
+          return Status::InvalidArgument(Substitute("Invalid column name: $0", column));
+        }
+        values.push_back(getter->second(tinfo));
+      }
+      table.AddRow(values);
+    }
+
+    return table.PrintTo(cout);
+  }
+
   static Status ListTablets(const vector<string>& master_addresses) {
     client::sp::shared_ptr<KuduClient> client;
     RETURN_NOT_OK(CreateKuduClient(master_addresses,
@@ -859,9 +918,59 @@ Status RenameColumn(const RunnerContext& context) {
 Status ListTables(const RunnerContext& context) {
   vector<string> master_addresses;
   RETURN_NOT_OK(ParseMasterAddresses(context, &master_addresses));
+
+  if (!FLAGS_columns.empty()) {
+    vector<string> columns = Split(FLAGS_columns, ",", strings::SkipEmpty());
+    return TableLister::ListTablesWithColumns(master_addresses, columns);
+  }
+
   return TableLister::ListTablets(master_addresses);
 }
 
+Status ListInFlightTables(const RunnerContext& context) {
+  vector<string> master_addresses;
+  RETURN_NOT_OK(ParseMasterAddresses(context, &master_addresses));
+  LeaderMasterProxy proxy;
+  RETURN_NOT_OK(proxy.Init(context));
+
+  ListInFlightTablesRequestPB req;
+  ListInFlightTablesResponsePB resp;
+
+  RETURN_NOT_OK((proxy.SyncRpc<ListInFlightTablesRequestPB, ListInFlightTablesResponsePB>(
+      req, &resp, "ListInFlightTables", &MasterServiceProxy::ListInFlightTablesAsync)));
+
+  if (resp.has_error()) {
+    return StatusFromPB(resp.error().status());
+  }
+  TablesInfoPB tables_info_pb;
+  for (const auto& table : resp.tables()) {
+    TablesInfoPB::TableInfoPB* table_info_pb = tables_info_pb.add_tables();
+    table_info_pb->set_id(table.id());
+    table_info_pb->set_name(table.name());
+    table_info_pb->set_num_tablets(table.num_tablets());
+    table_info_pb->set_num_tablets_in_flight(table.num_tablets_in_flight());
+    table_info_pb->set_state(SysTablesEntryPB_State_Name(table.state()));
+  }
+  if (iequals(FLAGS_list_table_output_format, "pretty")) {
+    for (const auto& table_info : tables_info_pb.tables()) {
+      cout << Substitute("id:$0 name:$1, num_tablets:$2 num_tablets_in_flight:$3 state:$4",
+                         table_info.id(),
+                         table_info.name(),
+                         table_info.num_tablets(),
+                         table_info.num_tablets_in_flight(),
+                         table_info.state()) << endl;
+    }
+  } else {
+    DCHECK(iequals(FLAGS_list_table_output_format, "json") ||
+        iequals(FLAGS_list_table_output_format, "json_compact"));
+    auto mode = iequals(FLAGS_list_table_output_format, "json") ?
+        JsonWriter::Mode::PRETTY : JsonWriter::Mode::COMPACT;
+    cout << JsonWriter::ToJson(tables_info_pb, mode) << endl;
+  }
+
+  return Status::OK();
+}
+
 Status ScanTable(const RunnerContext &context) {
   client::sp::shared_ptr<KuduClient> client;
   RETURN_NOT_OK(CreateKuduClient(context, &client));
@@ -1133,7 +1242,7 @@ Status ModifyRangePartition(const RunnerContext& context, PartitionAction action
           hash_schema_str, &hash_schema, opts); !s.ok()) {
       return Status::InvalidArgument(
           Substitute("unable to parse JSON: $0", hash_schema_str),
-                     s.error_message().ToString());
+                     s.message().ToString());
     }
   }
 
@@ -1157,8 +1266,8 @@ Status ModifyRangePartition(const RunnerContext& context, PartitionAction action
       for (const auto& column : dimension_pb.columns()) {
         columns.emplace_back(column);
       }
-      p->add_hash_partitions(
-          columns, dimension_pb.num_buckets(), dimension_pb.seed());
+      RETURN_NOT_OK(p->add_hash_partitions(
+          columns, dimension_pb.num_buckets(), dimension_pb.seed()));
     }
     return alterer->AddRangePartition(p.release())->Alter();
   }
@@ -1714,8 +1823,8 @@ Status ParseTablePartition(const PartitionPB& partition,
         hash_columns.emplace_back(c);
       }
       const int32_t seed = hash_dimension.has_seed() ? hash_dimension.seed() : 0;
-      partition->add_hash_partitions(
-          hash_columns, hash_dimension.num_buckets(), seed);
+      RETURN_NOT_OK(partition->add_hash_partitions(
+          hash_columns, hash_dimension.num_buckets(), seed));
     }
 
     table_creator->add_custom_range_partition(partition.release());
@@ -1742,7 +1851,7 @@ Status CreateTable(const RunnerContext& context) {
   if (!google_status.ok()) {
     return Status::InvalidArgument(
         Substitute("unable to parse JSON: $0", json_str),
-                   google_status.error_message().ToString());
+                   google_status.message().ToString());
   }
 
   client::sp::shared_ptr<KuduClient> client;
@@ -1814,8 +1923,9 @@ unique_ptr<Mode> BuildTableMode() {
       .Description("Describe a table")
       .AddRequiredParameter({ kTableNameArg, "Name of the table to describe" })
       .AddOptionalParameter("show_attributes")
-      .AddOptionalParameter("show_column_comment")
       .AddOptionalParameter("show_avro_format_schema")
+      .AddOptionalParameter("show_column_comment")
+      .AddOptionalParameter("show_column_id")
       .Build();
 
   unique_ptr<Action> list_tables =
@@ -1828,6 +1938,18 @@ unique_ptr<Mode> BuildTableMode() {
       .AddOptionalParameter("show_hash_partition_info")
       .AddOptionalParameter("show_table_info")
       .AddOptionalParameter("list_table_output_format")
+      .AddOptionalParameter(
+          "columns",
+          string(""),
+          string("Comma-separated list of table info fields to "
+                 "include in output. \nPossible values: id, name, live_row_count, "
+                 "num_tablets, num_replicas"))
+      .Build();
+
+  unique_ptr<Action> list_in_flight =
+      ClusterActionBuilder("list_in_flight", &ListInFlightTables)
+      .Description("List tables in flight")
+      .AddOptionalParameter("list_table_output_format")
       .Build();
 
   unique_ptr<Action> locate_row =
@@ -1907,8 +2029,11 @@ unique_ptr<Mode> BuildTableMode() {
       .AddOptionalParameter("num_threads")
       .AddOptionalParameter("predicates")
       .AddOptionalParameter("scan_batch_size")
+      .AddOptionalParameter("strict_column_id")
       .AddOptionalParameter("tablets")
       .AddOptionalParameter("write_type")
+      .AddOptionalParameter("table_copy_throttler_bytes_per_sec")
+      .AddOptionalParameter("table_copy_throttler_burst_factor")
       .Build();
 
   unique_ptr<Action> set_extra_config =
@@ -2085,6 +2210,7 @@ unique_ptr<Mode> BuildTableMode() {
       .AddAction(std::move(describe_table))
       .AddAction(std::move(drop_range_partition))
       .AddAction(std::move(get_extra_configs))
+      .AddAction(std::move(list_in_flight))
       .AddAction(std::move(list_tables))
       .AddAction(std::move(locate_row))
       .AddAction(std::move(recall))
