@@ -22,7 +22,6 @@
 #include <initializer_list>
 #include <map>
 #include <memory>
-#include <mutex>
 #include <numeric>
 #include <random>
 #include <set>
@@ -182,7 +181,7 @@ TEST_F(TxnStatusManagerTest, TestStartTransactions) {
   }
   // Now rebuild the underlying replica and rebuild the TxnStatusManager.
   ASSERT_OK(RestartReplica());
-  NO_FATALS(ResetTxnStatusManager());
+  ASSERT_OK(ResetTxnStatusManager());
   ASSERT_EQ(expected_prts_by_txn_id,
             txn_manager_->GetParticipantsByTxnIdForTests());
   ASSERT_EQ(3, txn_manager_->highest_txn_id());
@@ -276,7 +275,7 @@ TEST_F(TxnStatusManagerTest, TestStartTransactionsConcurrently) {
         auto s = txn_manager_->BeginTransaction(
             txn_id, kOwner, &highest_seen_txn_id, &ts_error);
         if (s.ok()) {
-          std::lock_guard<simple_spinlock> l(lock);
+          std::lock_guard l(lock);
           successful_txn_ids.emplace_back(txn_id);
           CHECK_GE(highest_seen_txn_id, txn_id);
         } else {
@@ -337,7 +336,7 @@ TEST_F(TxnStatusManagerTest, TestRegisterParticipantsConcurrently) {
       TabletServerErrorPB ts_error;
       Status s = txn_manager_->RegisterParticipant(kTxnId, prt, kOwner, &ts_error);
       if (s.ok()) {
-        std::lock_guard<simple_spinlock> l(lock);
+        std::lock_guard l(lock);
         successful_participants.emplace_back(std::move(prt));
       }
     });
@@ -537,7 +536,7 @@ TEST_F(TxnStatusManagerTest, GetTransactionStatus) {
 
   // Make the TxnStatusManager start from scratch.
   ASSERT_OK(RestartReplica());
-  NO_FATALS(ResetTxnStatusManager());
+  ASSERT_OK(ResetTxnStatusManager());
 
   // Committed, aborted, and in-flight transactions should be known to the
   // TxnStatusManager even after restarting the underlying replica and
@@ -706,7 +705,7 @@ TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
 
   // Make the TxnStatusManager start from scratch.
   ASSERT_OK(RestartReplica());
-  NO_FATALS(ResetTxnStatusManager());
+  ASSERT_OK(ResetTxnStatusManager());
 
   // Committed, aborted, and in-flight transactions should be known to the
   // TxnStatusManager even after restarting the underlying replica and
