@@ -22,7 +22,6 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <string>
@@ -35,8 +34,6 @@
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
-#include <google/protobuf/stubs/common.h>
-#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/wire_protocol.h"
@@ -107,7 +104,7 @@ DEFINE_uint32(heartbeat_inject_required_feature_flag, 0,
 TAG_FLAG(heartbeat_inject_required_feature_flag, runtime);
 TAG_FLAG(heartbeat_inject_required_feature_flag, unsafe);
 
-DEFINE_int32(tserver_send_tombstoned_tablets_report_inteval_secs, 1800,
+DEFINE_int32(tserver_send_tombstoned_tablets_report_inteval_secs, -1,
              "Time interval in seconds of sending a incremental tablets report "
              "to delete the tombstoned replicas whose tablets had already been "
              "deleted. Turn off this by setting it to a value less than 0. If "
@@ -256,7 +253,7 @@ Status Heartbeater::Start() {
       for (int j = 0; j < i; j++) {
         // Ignore failures; we should try to stop every thread, and
         // 'first_failure' is the most interesting status anyway.
-        threads_[j]->Stop();
+        WARN_NOT_OK(threads_[j]->Stop(), "Unable to stop heartbeater thread");
       }
       return first_failure;
     }
@@ -498,7 +495,6 @@ Status Heartbeater::Thread::DoHeartbeat(MasterErrorPB* error,
   // Send the most recently known TSK sequence number so that the master can
   // send us knew ones if they exist.
   req.set_latest_tsk_seq_num(server_->token_verifier().GetMaxKnownKeySequenceNumber());
-  bool including_tombstoned = false;
   if (send_full_tablet_report_) {
     LOG(INFO) << Substitute(
         "Master $0 was elected leader, sending a full tablet report...",
@@ -516,10 +512,16 @@ Status Heartbeater::Thread::DoHeartbeat(MasterErrorPB* error,
     VLOG(2) << Substitute("Sending an incremental tablet report to master $0...",
                           master_address_.ToString());
     // Check if it is time to send a report with tombstoned replicas.
-    including_tombstoned = FLAGS_tserver_send_tombstoned_tablets_report_inteval_secs >= 0 &&
-        (MonoTime::Now() - last_tombstoned_report_time_).ToSeconds() >=
+    const auto now = MonoTime::Now();
+    const auto tombstoned_report_interval =
         FLAGS_tserver_send_tombstoned_tablets_report_inteval_secs;
-    GenerateIncrementalTabletReport(req.mutable_tablet_report(), including_tombstoned);
+    const auto include_tombstoned = tombstoned_report_interval >= 0 &&
+        (now - last_tombstoned_report_time_).ToSeconds() >= tombstoned_report_interval;
+    GenerateIncrementalTabletReport(req.mutable_tablet_report(), include_tombstoned);
+    // Update the timestamp of last report on tombstoned tablet replicas.
+    if (include_tombstoned) {
+      last_tombstoned_report_time_ = now;
+    }
   }
 
   req.set_num_live_tablets(server_->tablet_manager()->GetNumLiveTablets());
@@ -620,7 +622,7 @@ void Heartbeater::Thread::RunThread() {
     // Wait for either the heartbeat interval to elapse, or for an "ASAP" heartbeat,
     // or for the signal to shut down.
     {
-      MutexLock l(mutex_);
+      std::lock_guard l(mutex_);
       while (next_heartbeat > MonoTime::Now() &&
           !heartbeat_asap_ &&
           should_run_) {
@@ -695,7 +697,7 @@ bool Heartbeater::Thread::IsCurrentThread() const {
 }
 
 void Heartbeater::Thread::MarkTabletReportAcknowledged(const TabletReportPB& report) {
-  std::lock_guard<simple_spinlock> l(dirty_tablets_lock_);
+  std::lock_guard l(dirty_tablets_lock_);
 
   int32_t acked_seq = report.sequence_number();
   CHECK_LT(acked_seq, next_report_seq_.load());
@@ -730,7 +732,7 @@ Status Heartbeater::Thread::Stop() {
   }
 
   {
-    MutexLock l(mutex_);
+    std::lock_guard l(mutex_);
     should_run_ = false;
     cond_.Signal();
   }
@@ -740,14 +742,14 @@ Status Heartbeater::Thread::Stop() {
 }
 
 void Heartbeater::Thread::TriggerASAP() {
-  MutexLock l(mutex_);
+  std::lock_guard l(mutex_);
   heartbeat_asap_ = true;
   cond_.Signal();
 }
 
 void Heartbeater::Thread::MarkTabletsDirty(const vector<string>& tablet_ids,
                                            const string& /*reason*/) {
-  std::lock_guard<simple_spinlock> l(dirty_tablets_lock_);
+  std::lock_guard l(dirty_tablets_lock_);
 
   // Even though this is an atomic load, it needs to hold the lock. To see why,
   // consider this sequence:
@@ -780,7 +782,7 @@ void Heartbeater::Thread::GenerateIncrementalTabletReport(TabletReportPB* report
   report->set_is_incremental(true);
   vector<string> dirty_tablet_ids;
   {
-    std::lock_guard<simple_spinlock> l(dirty_tablets_lock_);
+    std::lock_guard l(dirty_tablets_lock_);
     AppendKeysFromMap(dirty_tablets_, &dirty_tablet_ids);
   }
   server_->tablet_manager()->PopulateIncrementalTabletReport(
