@@ -20,9 +20,9 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <numeric>
 #include <ostream>
+#include <shared_mutex>
 #include <type_traits>
 
 #include <gflags/gflags.h>
@@ -43,7 +43,6 @@
 #include "kudu/tserver/scanner_metrics.h"
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/flag_validators.h"
-#include "kudu/util/locks.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/status.h"
@@ -66,9 +65,10 @@ DEFINE_int32(completed_scan_history_count, 10,
 TAG_FLAG(completed_scan_history_count, experimental);
 
 DEFINE_bool(show_slow_scans, false,
-            "Whether to show slow scans on the /scans page of web or record it in the log. "
-            "Please note that once set to true, full table scans may occur, which may affect "
-            "the normal Kudu service unexpectedly.");
+            "Whether to report on 'slow' scans at the /scans WebUI page and "
+            "in the INFO log. If enabled, well performing full table scans "
+            "and scans run by lazily-fetching clients may be reported as "
+            "'slow' depending on the --slow_scanner_threshold_ms setting.");
 TAG_FLAG(show_slow_scans, advanced);
 TAG_FLAG(show_slow_scans, runtime);
 
@@ -100,6 +100,7 @@ METRIC_DEFINE_gauge_size(server, slow_scans,
 
 using kudu::rpc::RemoteUser;
 using kudu::tablet::TabletReplica;
+using std::shared_lock;
 using std::string;
 using std::unique_ptr;
 using std::unordered_map;
@@ -159,7 +160,7 @@ ScannerManager::ScannerManager(const scoped_refptr<MetricEntity>& metric_entity)
 
 ScannerManager::~ScannerManager() {
   {
-    MutexLock l(shutdown_lock_);
+    std::lock_guard l(shutdown_lock_);
     shutdown_ = true;
     shutdown_cv_.Broadcast();
   }
@@ -180,7 +181,7 @@ void ScannerManager::RunCollectAndRemovalThread() {
   while (true) {
     // Loop until we are shutdown.
     {
-      MutexLock l(shutdown_lock_);
+      std::lock_guard l(shutdown_lock_);
       if (shutdown_) {
         return;
       }
@@ -216,7 +217,7 @@ void ScannerManager::NewScanner(const scoped_refptr<TabletReplica>& tablet_repli
                                row_format_flags));
 
     ScannerMapStripe& stripe = GetStripeByScannerId(id);
-    std::lock_guard<RWMutex> l(stripe.lock_);
+    std::lock_guard l(stripe.lock_);
     success = InsertIfNotPresent(&stripe.scanners_by_id_, id, *scanner);
   }
 }
@@ -247,7 +248,7 @@ bool ScannerManager::UnregisterScanner(const string& scanner_id) {
   SharedScanDescriptor descriptor;
   ScannerMapStripe& stripe = GetStripeByScannerId(scanner_id);
   {
-    std::lock_guard<RWMutex> l(stripe.lock_);
+    std::lock_guard l(stripe.lock_);
     auto it = stripe.scanners_by_id_.find(scanner_id);
     if (it == stripe.scanners_by_id_.end()) {
       return false;
@@ -265,7 +266,7 @@ bool ScannerManager::UnregisterScanner(const string& scanner_id) {
   }
 
   {
-    std::lock_guard<percpu_rwlock> l(completed_scans_lock_);
+    std::lock_guard l(completed_scans_lock_);
     RecordCompletedScanUnlocked(descriptor);
   }
 
@@ -273,7 +274,7 @@ bool ScannerManager::UnregisterScanner(const string& scanner_id) {
     const MonoTime start_time = descriptor->start_time;
     if (start_time + MonoDelta::FromMilliseconds(FLAGS_slow_scanner_threshold_ms)
         < MonoTime::Now()) {
-      std::lock_guard<percpu_rwlock> l(slow_scans_lock_);
+      std::lock_guard l(slow_scans_lock_);
       RecordSlowScanUnlocked(descriptor);
     }
   }
@@ -331,7 +332,7 @@ vector<SharedScanDescriptor> ScannerManager::ListScans() const {
   }
 
   {
-    kudu::shared_lock<rw_spinlock> l(completed_scans_lock_.get_lock());
+    shared_lock<rw_spinlock> l(completed_scans_lock_.get_lock());
     // A scanner in 'scans' may have completed between the above loop and here.
     // As we'd rather have the finalized descriptor of the completed scan,
     // update over the old descriptor in this case.
@@ -356,14 +357,15 @@ vector<SharedScanDescriptor> ScannerManager::ListScans() const {
 vector<SharedScanDescriptor> ScannerManager::ListSlowScans() const {
   vector<SharedScanDescriptor> ret;
   if (!FLAGS_show_slow_scans) {
-    LOG(INFO) << "Slow scans show is disabled. Set --show_slow_scans to enable it.";
+    KLOG_EVERY_N_SECS(INFO, 60) << Substitute(
+        "slow scan reporting is disabled: set --show_slow_scans to enable");
     return ret;
   }
 
   // Get all the scans first.
   unordered_map<string, SharedScanDescriptor> scans;
   {
-    kudu::shared_lock<rw_spinlock> l(slow_scans_lock_.get_lock());
+    shared_lock<rw_spinlock> l(slow_scans_lock_.get_lock());
     for (const auto& scan : slow_scans_) {
       InsertOrUpdate(&scans, scan->scanner_id, scan);
     }
@@ -387,7 +389,7 @@ void ScannerManager::CollectSlowScanners() {
   vector<SharedScanDescriptor> descriptors;
   int32_t slow_scanner_threshold = FLAGS_slow_scanner_threshold_ms;
   for (ScannerMapStripe* stripe : scanner_maps_) {
-    std::lock_guard<RWMutex> l(stripe->lock_);
+    std::lock_guard l(stripe->lock_);
     for (auto it = stripe->scanners_by_id_.begin(); it != stripe->scanners_by_id_.end(); ++it) {
       const SharedScanner& scanner = it->second;
       if (!scanner->is_initted()) {
@@ -412,7 +414,7 @@ void ScannerManager::CollectSlowScanners() {
     }
   }
 
-  std::lock_guard<percpu_rwlock> l(slow_scans_lock_);
+  std::lock_guard l(slow_scans_lock_);
   for (auto& descriptor : descriptors) {
     if (std::find(slow_scans_.begin(), slow_scans_.end(), descriptor) == slow_scans_.end()) {
       RecordSlowScanUnlocked(descriptor);
@@ -426,7 +428,7 @@ void ScannerManager::RemoveExpiredScanners() {
 
   vector<SharedScanDescriptor> descriptors;
   for (ScannerMapStripe* stripe : scanner_maps_) {
-    std::lock_guard<RWMutex> l(stripe->lock_);
+    std::lock_guard l(stripe->lock_);
     for (auto it = stripe->scanners_by_id_.begin(); it != stripe->scanners_by_id_.end();) {
       const SharedScanner& scanner = it->second;
       MonoDelta idle_time = scanner->TimeSinceLastAccess(now);
@@ -453,7 +455,7 @@ void ScannerManager::RemoveExpiredScanners() {
     }
   }
 
-  std::lock_guard<percpu_rwlock> l(completed_scans_lock_);
+  std::lock_guard l(completed_scans_lock_);
   for (auto& descriptor : descriptors) {
     descriptor->last_access_time = now;
     descriptor->state = ScanState::kExpired;
@@ -521,9 +523,22 @@ Scanner::~Scanner() {
   }
 }
 
+void Scanner::UpdateTabletMetrics(const CpuTimes& elapsed) {
+  if (tablet_replica_) {
+    auto tablet = tablet_replica_->shared_tablet();
+    if (tablet && tablet->metrics()) {
+      // Store scan request's timings.
+      tablet->metrics()->scan_duration_wall_time->Increment(elapsed.wall_millis());
+      tablet->metrics()->scan_duration_system_time->Increment(elapsed.system_cpu_millis());
+      tablet->metrics()->scan_duration_user_time->Increment(elapsed.user_cpu_millis());
+    }
+  }
+}
+
 void Scanner::AddTimings(const CpuTimes& elapsed) {
-  std::unique_lock<RWMutex> l(cpu_times_lock_);
+  std::lock_guard l(cpu_times_lock_);
   cpu_times_.Add(elapsed);
+  UpdateTabletMetrics(elapsed);
 }
 
 void Scanner::Init(unique_ptr<RowwiseIterator> iter,
