@@ -75,11 +75,13 @@
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
+#include "kudu/util/throttler.h"
 
 DECLARE_double(env_inject_eio);
 DECLARE_double(tablet_copy_fault_crash_during_download_block);
 DECLARE_double(tablet_copy_fault_crash_during_download_wal);
 DECLARE_int32(tablet_copy_download_threads_nums_per_session);
+DECLARE_int32(tablet_copy_transfer_chunk_size_bytes);
 DECLARE_string(block_manager);
 DECLARE_string(env_inject_eio_globs);
 
@@ -91,6 +93,7 @@ using kudu::consensus::GetRaftConfigLeader;
 using kudu::consensus::RaftPeerPB;
 using kudu::fs::DataDirManager;
 using kudu::tablet::TabletMetadata;
+using std::nullopt;
 using std::shared_ptr;
 using std::string;
 using std::thread;
@@ -124,17 +127,29 @@ class TabletCopyClientTest : public TabletCopyTest {
     metric_entity_ = METRIC_ENTITY_server.Instantiate(&metric_registry_, "test");
     opts.metric_entity = metric_entity_;
     fs_manager_.reset(new FsManager(Env::Default(), opts));
-    string server_key;
-    string server_key_iv;
-    string server_key_version;
-    GetEncryptionKey(&server_key, &server_key_iv, &server_key_version);
-    if (server_key.empty()) {
+    string tenant_name;
+    string tenant_id;
+    string encryption_key;
+    string encryption_key_iv;
+    string encryption_key_version;
+    GetEncryptionKey(&tenant_name, &tenant_id, &encryption_key,
+                     &encryption_key_iv, &encryption_key_version);
+    if (tenant_name.empty() && encryption_key.empty()) {
       ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout());
+    } else if (tenant_name.empty()) {
+      ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout(nullopt,
+                                                           nullopt,
+                                                           nullopt,
+                                                           encryption_key,
+                                                           encryption_key_iv,
+                                                           encryption_key_version));
     } else {
-      ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout(std::nullopt,
-                                                           server_key,
-                                                           server_key_iv,
-                                                           server_key_version));
+      ASSERT_OK(fs_manager_->CreateInitialFileSystemLayout(nullopt,
+                                                           tenant_name,
+                                                           tenant_id,
+                                                           encryption_key,
+                                                           encryption_key_iv,
+                                                           encryption_key_version));
     }
     ASSERT_OK(fs_manager_->Open());
     ASSERT_OK(ResetTabletCopyClient());
@@ -175,7 +190,8 @@ class TabletCopyClientTest : public TabletCopyTest {
   FRIEND_TEST(TabletCopyClientBasicTest, TestSupportsLiveRowCount);
 
   Status CompareFileContents(const string& path1, const string& path2);
-  Status ResetRemoteTabletCopyClient();
+  Status ResetRemoteTabletCopyClient(
+      TabletCopyClientMetrics* tablet_copy_client_metrics = nullptr);
   Status ResetLocalTabletCopyClient();
 
   // Injection of 'supports_live_row_count' modifiers.
@@ -199,17 +215,20 @@ class TabletCopyClientTest : public TabletCopyTest {
   unique_ptr<FsManager> src_fs_manager_;
   MetricRegistry src_metric_registry_;
   scoped_refptr<MetricEntity> src_metric_entity_;
+  std::shared_ptr<Throttler> throttler_;
 };
 
 Status TabletCopyClientTest::CompareFileContents(const string& path1, const string& path2) {
-  shared_ptr<RandomAccessFile> file1, file2;
+  shared_ptr<RandomAccessFile> file1;
+  shared_ptr<RandomAccessFile> file2;
   RandomAccessFileOptions opts;
   opts.is_sensitive = true;
-  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->env(), path1, &file1));
-  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->env(), path2, &file2));
+  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->GetEnv(), path1, &file1));
+  RETURN_NOT_OK(env_util::OpenFileForRandom(opts, fs_manager_->GetEnv(), path2, &file2));
 
-  uint64_t size1, size2;
+  uint64_t size1;
   RETURN_NOT_OK(file1->Size(&size1));
+  uint64_t size2;
   RETURN_NOT_OK(file2->Size(&size2));
   size1 -= file1->GetEncryptionHeaderSize();
   size2 -= file2->GetEncryptionHeaderSize();
@@ -232,17 +251,20 @@ Status TabletCopyClientTest::CompareFileContents(const string& path1, const stri
   return Status::OK();
 }
 
-Status TabletCopyClientTest::ResetRemoteTabletCopyClient() {
+Status TabletCopyClientTest::ResetRemoteTabletCopyClient(
+    TabletCopyClientMetrics* tablet_copy_client_metrics) {
   scoped_refptr<ConsensusMetadataManager> cmeta_manager(
       new ConsensusMetadataManager(fs_manager_.get()));
 
-  tablet_replica_->WaitUntilConsensusRunning(MonoDelta::FromSeconds(10.0));
-  rpc::MessengerBuilder(CURRENT_TEST_NAME()).Build(&messenger_);
+  RETURN_NOT_OK(tablet_replica_->WaitUntilConsensusRunning(MonoDelta::FromSeconds(10.0)));
+  RETURN_NOT_OK(rpc::MessengerBuilder(CURRENT_TEST_NAME()).Build(&messenger_));
   client_.reset(new RemoteTabletCopyClient(GetTabletId(),
                                            fs_manager_.get(),
                                            cmeta_manager,
                                            messenger_,
-                                           nullptr /* no metrics */));
+                                           tablet_copy_client_metrics,
+                                           throttler_));
+
   RaftPeerPB* cstate_leader;
   ConsensusStatePB cstate;
   RETURN_NOT_OK(tablet_replica_->consensus()->ConsensusState(&cstate));
@@ -259,7 +281,7 @@ Status TabletCopyClientTest::ResetLocalTabletCopyClient() {
     tablet_id_ = tablet_replica_->tablet_id();
 
     // Prepare parameters to create source FsManager.
-    rpc::MessengerBuilder(CURRENT_TEST_NAME()).Build(&messenger_);
+    RETURN_NOT_OK(rpc::MessengerBuilder(CURRENT_TEST_NAME()).Build(&messenger_));
 
     FsManagerOpts opts;
     string wal = mini_server_->server()->fs_manager()->GetWalsRootDir();
@@ -292,6 +314,55 @@ Status TabletCopyClientTest::ResetLocalTabletCopyClient() {
   return Status::OK();
 }
 
+class TabletCopyThrottlerTest : public TabletCopyClientTest {
+ public:
+  TabletCopyThrottlerTest() {
+    mode_ = TabletCopyMode::REMOTE;
+    throttler_ = std::make_shared<Throttler>(
+        0,
+        FLAGS_tablet_copy_transfer_chunk_size_bytes,
+        2 * FLAGS_tablet_copy_transfer_chunk_size_bytes);
+  }
+
+  void SetUp() override {
+    TabletCopyClientTest::SetUp();
+  }
+};
+
+TEST_F(TabletCopyThrottlerTest, TestThrottler) {
+  scoped_refptr<MetricEntity> src_metric_entity_(
+      METRIC_ENTITY_server.Instantiate(&metric_registry_, "tablet-copy-test"));
+  TabletCopyClientMetrics tablet_copy_client_metrics(src_metric_entity_);
+  ASSERT_OK(ResetRemoteTabletCopyClient(&tablet_copy_client_metrics));
+
+  ASSERT_OK(StartCopy());
+  BlockId block_id = FirstColumnBlockId(*client_->remote_superblock_);
+  Slice slice;
+  faststring scratch;
+
+  // Ensure the block wasn't there before (it shouldn't be, we use our own FsManager dir).
+  Status s = ReadLocalBlockFile(fs_manager_.get(), block_id, &scratch, &slice);
+  ASSERT_TRUE(s.IsNotFound()) << "Expected block not found: " << s.ToString();
+
+  // Check that the client downloaded the block and verification passed.
+  BlockId new_block_id;
+  MonoTime start_time = MonoTime::Now();
+  ASSERT_OK(client_->DownloadBlock(block_id, &new_block_id));
+  MonoTime end_time = MonoTime::Now();
+  // Compute the real tablet downloading speed.
+  double download_speed = tablet_copy_client_metrics.bytes_fetched->value() /
+                          (end_time - start_time).ToSeconds();
+  // Real tablet downloading speed must be less than the defined speed.
+  ASSERT_GE(FLAGS_tablet_copy_transfer_chunk_size_bytes, download_speed);
+  ASSERT_OK(client_->transaction_->CommitCreatedBlocks());
+  // Ensure it placed the block where we expected it to.
+  ASSERT_OK(ReadLocalBlockFile(fs_manager_.get(), new_block_id, &scratch, &slice));
+  // 'client_' must be destroyed before 'tablet_copy_client_metrics', because client
+  // holds the pointer of 'tablet_copy_client_metrics', and uses 'tablet_copy_client_metrics'
+  // while being destroyed. See tablet_copy_client.cc.
+  client_.reset();
+}
+
 class TabletCopyClientBasicTest : public TabletCopyClientTest,
                                   public ::testing::WithParamInterface<TabletCopyMode> {
  public:
@@ -532,7 +603,7 @@ TEST_P(TabletCopyClientBasicTest, TestDownloadAllBlocks) {
 // stop the copy client and cause it to fail.
 TEST_P(TabletCopyClientBasicTest, TestFailedDiskStopsClient) {
   ASSERT_OK(StartCopy());
-  DataDirManager* dd_manager = fs_manager_->dd_manager();
+  scoped_refptr<DataDirManager> dd_manager = fs_manager_->dd_manager();
 
   // Repeatedly fetch files for the client.
   Status s;
@@ -546,7 +617,7 @@ TEST_P(TabletCopyClientBasicTest, TestFailedDiskStopsClient) {
   // metadata directory).
   while (true) {
     if (rand() % 10 == 0) {
-      dd_manager->MarkDirFailed(1, "injected failure in non-client thread");
+      ASSERT_OK(dd_manager->MarkDirFailed(1, "injected failure in non-client thread"));
       LOG(INFO) << "INJECTING FAILURE";
       break;
     }
@@ -615,7 +686,7 @@ void TabletCopyClientAbortTest::CreateTestBlocks(int num_blocks) {
   for (int i = 0; i < num_blocks; i++) {
     unique_ptr<fs::WritableBlock> block;
     ASSERT_OK(fs_manager_->CreateNewBlock({}, &block));
-    block->Append("Test");
+    ASSERT_OK(block->Append("Test"));
     ASSERT_OK(block->Close());
   }
 }
@@ -694,7 +765,7 @@ TEST_P(TabletCopyClientAbortTest, TestAbort) {
     ASSERT_EQ(tablet::TABLET_DATA_TOMBSTONED, meta->tablet_data_state());
     ASSERT_FALSE(fs_manager_->Exists(wal_path));
     vector<BlockId> latest_blocks;
-    fs_manager_->block_manager()->GetAllBlockIds(&latest_blocks);
+    ASSERT_OK(fs_manager_->block_manager()->GetAllBlockIds(&latest_blocks));
     ASSERT_EQ(local_block_ids.size(), latest_blocks.size());
   }
   for (const auto& block_id : local_block_ids) {
