@@ -17,20 +17,18 @@
 
 #include "kudu/tserver/tablet_copy_client.h"
 
+#include <algorithm>
 #include <cstdint>
 #include <functional>
 #include <memory>
-#include <mutex>
 #include <optional>
 #include <ostream>
 #include <type_traits>
-#include <utility>
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <google/protobuf/stubs/port.h>
 
-#include "kudu/common/common.pb.h"
 #include "kudu/common/partition.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol.h"
@@ -74,10 +72,22 @@
 #include "kudu/util/random.h"
 #include "kudu/util/random_util.h"
 #include "kudu/util/scoped_cleanup.h"
+#include "kudu/util/slice.h"
 #include "kudu/util/status.h"
+#include "kudu/util/stopwatch.h"
 #include "kudu/util/threadpool.h"
+#include "kudu/util/throttler.h" // IWYU pragma: keep
 #include "kudu/util/trace.h"
 
+DEFINE_int64(tablet_copy_throttler_bytes_per_sec, 0,
+             "Limit tablet copying speed. It limits the copying speed of all the tablets "
+             "in one server for one session. The default value is 0, which means not limiting "
+             "the speed. The unit is bytes/seconds");
+DEFINE_double(tablet_copy_throttler_burst_factor, 1.0f,
+             "Burst factor for tablet copy throttling. The maximum rate the throttler "
+             "allows within a token refill period (100ms) equals burst factor multiply "
+             "base rate. The default value is 1.0, which means the maximun rate is equals "
+             "to --tablet_copy_throttler_bytes_per_sec.");
 DEFINE_int32(tablet_copy_begin_session_timeout_ms, 30000,
              "Tablet server RPC client timeout for BeginTabletCopySession calls. "
              "Also used for EndTabletCopySession calls.");
@@ -143,6 +153,10 @@ DEFINE_int32(tablet_copy_download_threads_nums_per_session, 4,
 DEFINE_validator(tablet_copy_download_threads_nums_per_session,
      [](const char* /*n*/, int32_t v) { return v > 0; });
 
+DEFINE_bool(tablet_copy_support_download_superblock_in_batch, true,
+            "Whether to support download superblock in batch automatically when it is very large."
+            "When superblock is small, it can be downloaded once a time.");
+
 DECLARE_int32(tablet_copy_transfer_chunk_size_bytes);
 
 METRIC_DEFINE_counter(server, tablet_copy_bytes_fetched,
@@ -157,6 +171,13 @@ METRIC_DEFINE_gauge_int32(server, tablet_copy_open_client_sessions,
                           "Number of currently open tablet copy client sessions on this server",
                           kudu::MetricLevel::kInfo);
 
+METRIC_DEFINE_histogram(server, tablet_copy_duration,
+                        "Tablet Copy Duration",
+                        kudu::MetricUnit::kMilliseconds,
+                        "Duration of tablet copying as destination.",
+                        kudu::MetricLevel::kDebug,
+                        3600000LU, 1);
+
 // RETURN_NOT_OK_PREPEND() with a remote-error unwinding step.
 #define RETURN_NOT_OK_UNWIND_PREPEND(status, controller, msg) \
   RETURN_NOT_OK_PREPEND(UnwindRemoteError(status, controller), msg)
@@ -191,7 +212,8 @@ namespace tserver {
 
 TabletCopyClientMetrics::TabletCopyClientMetrics(const scoped_refptr<MetricEntity>& metric_entity)
     : bytes_fetched(METRIC_tablet_copy_bytes_fetched.Instantiate(metric_entity)),
-      open_client_sessions(METRIC_tablet_copy_open_client_sessions.Instantiate(metric_entity, 0)) {
+      open_client_sessions(METRIC_tablet_copy_open_client_sessions.Instantiate(metric_entity, 0)),
+      copy_duration(METRIC_tablet_copy_duration.Instantiate(metric_entity)) {
 }
 
 TabletCopyClient::TabletCopyClient(
@@ -199,7 +221,8 @@ TabletCopyClient::TabletCopyClient(
     FsManager* dst_fs_manager,
     scoped_refptr<ConsensusMetadataManager> cmeta_manager,
     shared_ptr<Messenger> messenger,
-    TabletCopyClientMetrics* dst_tablet_copy_metrics)
+    TabletCopyClientMetrics* dst_tablet_copy_metrics,
+    shared_ptr<Throttler> throttler)
     : tablet_id_(std::move(tablet_id)),
       dst_fs_manager_(dst_fs_manager),
       cmeta_manager_(std::move(cmeta_manager)),
@@ -209,15 +232,24 @@ TabletCopyClient::TabletCopyClient(
       session_idle_timeout_millis_(FLAGS_tablet_copy_begin_session_timeout_ms),
       start_time_micros_(0),
       rng_(GetRandomSeed32()),
-      dst_tablet_copy_metrics_(dst_tablet_copy_metrics) {
-  BlockManager* bm = dst_fs_manager->block_manager();
+      dst_tablet_copy_metrics_(dst_tablet_copy_metrics),
+      throttler_(std::move(throttler)) {
+  auto bm = dst_fs_manager->block_manager();
   transaction_ = bm->NewCreationTransaction();
   if (dst_tablet_copy_metrics_) {
     dst_tablet_copy_metrics_->open_client_sessions->Increment();
   }
-  CHECK_OK(ThreadPoolBuilder("tablet-download-pool-" + tablet_id)
+
+  // These thread pools are ephemeral, and there might be multiple pools with
+  // the same name "tablet-download-pool" running at the same time. They are not
+  // differentiated by 'tablet_id' or other dynamic information -- that's to
+  // avoid registering too many entries in the trace metrics dictionary
+  // (see trace_metrics.{h,cc} for details). So, all the trace metrics for
+  // these thread pools will be accumulated under a few thread pool metrics
+  // for all the tablet copying clients ever spawned.
+  CHECK_OK(ThreadPoolBuilder("tablet-download-pool")
+               .set_min_threads(1)
                .set_max_threads(FLAGS_tablet_copy_download_threads_nums_per_session)
-               .set_min_threads(FLAGS_tablet_copy_download_threads_nums_per_session)
                .Build(&tablet_download_pool_));
 }
 
@@ -298,6 +330,8 @@ Status RemoteTabletCopyClient::Start(const HostPort& copy_source_addr,
   BeginTabletCopySessionRequestPB req;
   req.set_requestor_uuid(dst_fs_manager_->uuid());
   req.set_tablet_id(tablet_id_);
+  req.set_auto_download_superblock_in_batch(
+      FLAGS_tablet_copy_support_download_superblock_in_batch);
 
   rpc::RpcController controller;
 
@@ -306,12 +340,36 @@ Status RemoteTabletCopyClient::Start(const HostPort& copy_source_addr,
   RETURN_NOT_OK_PREPEND(SendRpcWithRetry(&controller, [&] {
     return proxy_->BeginTabletCopySession(req, &resp, &controller);
   }), "unable to begin tablet copy session");
-
+  session_id_ = resp.session_id();
+  if (resp.superblock_is_too_large()) {
+    // Download superblock from remote and store it in local file.
+    string superblock_path;
+    // Delete the temporary superblock file finally.
+    auto deleter = MakeScopedCleanup([&]() {
+      if (!superblock_path.empty() &&
+          dst_fs_manager_->GetEnv()->FileExists(superblock_path)) {
+        WARN_NOT_OK(dst_fs_manager_->GetEnv()->DeleteFile(superblock_path),
+                    Substitute("Could not delete temporary superblock file $0",
+                               superblock_path));
+      }
+    });
+    RETURN_NOT_OK_PREPEND(DownloadSuperBlock(&superblock_path),
+                          "Download super block failed");
+    remote_superblock_.reset(new tablet::TabletSuperBlockPB);
+    // Load superblock from a local temporary file.
+    RETURN_NOT_OK_PREPEND(
+        pb_util::ReadPBContainerFromPath(dst_fs_manager_->GetEnv(), superblock_path,
+                                         remote_superblock_.get(), pb_util::SENSITIVE),
+        Substitute("Could not load superblock from $0", superblock_path));
+  } else {
+    CHECK(resp.has_superblock());
+    remote_superblock_.reset(resp.release_superblock());
+  }
   TRACE("Tablet copy session begun");
 
   string copy_peer_uuid = resp.has_responder_uuid()
       ? resp.responder_uuid() : "(unknown uuid)";
-  TabletDataState data_state = resp.superblock().tablet_data_state();
+  TabletDataState data_state = remote_superblock_->tablet_data_state();
   if (data_state != tablet::TABLET_DATA_READY) {
     Status s = Status::IllegalState(
         Substitute("Remote peer ($0) is not ready itself! state: $1",
@@ -320,13 +378,9 @@ Status RemoteTabletCopyClient::Start(const HostPort& copy_source_addr,
     return s;
   }
 
-  session_id_ = resp.session_id();
   // Update our default RPC timeout to reflect the server's session timeout.
   session_idle_timeout_millis_ = resp.session_idle_timeout_millis();
 
-  // Store a copy of the remote (old) superblock.
-  remote_superblock_.reset(resp.release_superblock());
-
   // Make a copy of the remote superblock. We first clear out the remote blocks
   // from this structure and then add them back in as they are downloaded.
   superblock_.reset(new TabletSuperBlockPB(*remote_superblock_));
@@ -504,7 +558,7 @@ Status TabletCopyClient::Finish() {
     string meta_copy_path = Substitute("$0.copy.$1$2", meta_path, start_time_micros_, kTmpInfix);
     WritableFileOptions opts;
     opts.is_sensitive = true;
-    RETURN_NOT_OK_PREPEND(env_util::CopyFile(dst_fs_manager_->env(), meta_path,
+    RETURN_NOT_OK_PREPEND(env_util::CopyFile(dst_fs_manager_->GetEnv(), meta_path,
                                              meta_copy_path, opts),
                           "Unable to make copy of tablet metadata");
   }
@@ -512,6 +566,10 @@ Status TabletCopyClient::Finish() {
   // Now that we've finished everything, complete.
   revert_activate_superblock.cancel();
   state_ = kFinished;
+  if (dst_tablet_copy_metrics_) {
+    int64_t dur = GetCurrentTimeMicros() - start_time_micros_;
+    dst_tablet_copy_metrics_->copy_duration->Increment(dur / 1000);
+  }
   return Status::OK();
 }
 
@@ -529,6 +587,10 @@ Status TabletCopyClient::Abort() {
   SCOPED_CLEANUP({
     DCHECK_EQ(tablet::TABLET_DATA_TOMBSTONED, meta_->tablet_data_state());
     state_ = kFinished;
+    if (dst_tablet_copy_metrics_) {
+      int64_t dur = GetCurrentTimeMicros() - start_time_micros_;
+      dst_tablet_copy_metrics_->copy_duration->Increment(dur / 1000);
+    }
   });
 
   // Load the in-progress superblock in-memory so that when we delete the
@@ -600,8 +662,8 @@ Status TabletCopyClient::DownloadWALs() {
   // not kept from previous copies and runs.
   RETURN_NOT_OK(log::Log::DeleteOnDiskData(dst_fs_manager_, tablet_id_));
   string path = dst_fs_manager_->GetTabletWalDir(tablet_id_);
-  RETURN_NOT_OK(dst_fs_manager_->env()->CreateDir(path));
-  RETURN_NOT_OK(dst_fs_manager_->env()->SyncDir(DirName(path))); // fsync() parent dir.
+  RETURN_NOT_OK(dst_fs_manager_->GetEnv()->CreateDir(path));
+  RETURN_NOT_OK(dst_fs_manager_->GetEnv()->SyncDir(DirName(path))); // fsync() parent dir.
 
   // Download the WAL segments.
   const int num_segments = wal_seqnos_.size();
@@ -614,14 +676,14 @@ Status TabletCopyClient::DownloadWALs() {
       SetStatusMessage(Substitute("Downloading WAL segment with seq. number $0 ($1/$2)",
                                   seg_seqno, counter.load() + 1, num_segments));
       {
-        std::lock_guard<simple_spinlock> l(simple_lock_);
+        std::lock_guard l(simple_lock_);
         if (!end_status.ok()) {
           return;
         }
       }
       Status s = DownloadWAL(seg_seqno);
       if (!s.ok()) {
-        std::lock_guard<simple_spinlock> l(simple_lock_);
+        std::lock_guard l(simple_lock_);
         if (end_status.ok()) {
           end_status = s;
         }
@@ -657,7 +719,7 @@ void TabletCopyClient::DownloadRowset(const RowSetDataPB& src_rowset,
                                       Status* end_status) {
   RowSetDataPB* dst_rowset;
   {
-    std::lock_guard<simple_spinlock> l(simple_lock_);
+    std::lock_guard l(simple_lock_);
     if (!end_status->ok()) {
       return;
     }
@@ -784,7 +846,7 @@ Status TabletCopyClient::DownloadWAL(uint64_t wal_segment_seqno) {
   opts.sync_on_close = true;
   opts.is_sensitive = true;
   unique_ptr<WritableFile> writer;
-  RETURN_NOT_OK_PREPEND(dst_fs_manager_->env()->NewWritableFile(opts, dest_path, &writer),
+  RETURN_NOT_OK_PREPEND(dst_fs_manager_->GetEnv()->NewWritableFile(opts, dest_path, &writer),
                         "Unable to open file for writing");
   RETURN_NOT_OK_PREPEND(TransferFile(data_id, writer.get()),
                         Substitute("Unable to download WAL segment with seq. number $0",
@@ -813,7 +875,7 @@ Status TabletCopyClient::WriteConsensusMetadata() {
     string cmeta_copy_path = Substitute("$0.copy.$1$2", cmeta_path, start_time_micros_, kTmpInfix);
     WritableFileOptions opts;
     opts.is_sensitive = true;
-    RETURN_NOT_OK_PREPEND(env_util::CopyFile(dst_fs_manager_->env(), cmeta_path,
+    RETURN_NOT_OK_PREPEND(env_util::CopyFile(dst_fs_manager_->GetEnv(), cmeta_path,
                                              cmeta_copy_path, opts),
                           "Unable to make copy of consensus metadata");
   }
@@ -844,12 +906,12 @@ Status TabletCopyClient::DownloadAndRewriteBlockIfEndStatusOK(const BlockIdPB& s
                                                               BlockIdPB* dest_block_id,
                                                               Status* end_status) {
   {
-    std::lock_guard<simple_spinlock> l(simple_lock_);
+    std::lock_guard l(simple_lock_);
     RETURN_NOT_OK(*end_status);
   }
   Status s = DownloadAndRewriteBlock(src_block_id, num_blocks, block_count, dest_block_id);
   if (!s.ok()) {
-    std::lock_guard<simple_spinlock> l(simple_lock_);
+    std::lock_guard l(simple_lock_);
     if (!s.ok() && end_status->ok()) {
       *end_status = s;
     }
@@ -879,12 +941,57 @@ Status TabletCopyClient::DownloadBlock(const BlockId& old_block_id,
   *new_block_id = block->id();
   RETURN_NOT_OK_PREPEND(block->Finalize(), "Unable to finalize block");
   {
-    std::lock_guard<simple_spinlock> l(simple_lock_);
+    std::lock_guard l(simple_lock_);
     transaction_->AddCreatedBlock(std::move(block));
   }
   return Status::OK();
 }
 
+Status RemoteTabletCopyClient::DownloadSuperBlock(string* superblock_path) {
+  DataIdPB data_id;
+  data_id.set_type(DataIdPB::SUPER_BLOCK);
+
+  rpc::RpcController controller;
+  controller.set_timeout(MonoDelta::FromMilliseconds(session_idle_timeout_millis_));
+  FetchDataRequestPB req;
+  req.set_session_id(session_id_);
+  req.mutable_data_id()->CopyFrom(data_id);
+  req.set_max_length(FLAGS_tablet_copy_transfer_chunk_size_bytes);
+
+  // Create a temporary file to store the superblock.
+  string tmpl = "super_block.tmp.XXXXXX";
+  unique_ptr<RWFile> tmp_file;
+  RETURN_NOT_OK_PREPEND(dst_fs_manager_->GetEnv()->NewTempRWFile(
+      RWFileOptions(), tmpl, superblock_path, &tmp_file),
+                        "could not create temporary super block data file");
+
+  bool done = false;
+  uint64_t offset = 0;
+  while (!done) {
+    req.set_offset(offset);
+
+    // Request the next data chunk.
+    FetchDataResponsePB resp;
+    RETURN_NOT_OK_PREPEND(SendRpcWithRetry(&controller, [&] {
+        return proxy_->FetchData(req, &resp, &controller);
+    }), "unable to fetch data from remote");
+
+    // Sanity-check for corruption.
+    RETURN_NOT_OK_PREPEND(VerifyData(offset, resp.chunk()),
+                          Substitute("error validating data item $0",
+                                     pb_util::SecureShortDebugString(data_id)));
+
+    // Write the data.
+    RETURN_NOT_OK(tmp_file->Write(offset, resp.chunk().data()));
+
+    auto chunk_size = resp.chunk().data().size();
+    done = offset + chunk_size == resp.chunk().total_data_length();
+    offset += chunk_size;
+  }
+  RETURN_NOT_OK_PREPEND(tmp_file->Close(), "close superblock data file failed");
+  return Status::OK();
+}
+
 template<class Appendable>
 Status RemoteTabletCopyClient::DownloadFile(const DataIdPB& data_id,
                                             Appendable* appendable) {
@@ -926,6 +1033,13 @@ Status RemoteTabletCopyClient::DownloadFile(const DataIdPB& data_id,
     if (dst_tablet_copy_metrics_) {
       dst_tablet_copy_metrics_->bytes_fetched->IncrementBy(chunk_size);
     }
+    if (throttler_) {
+      LOG_TIMING(INFO, "Tablet copy throttler") {
+        while (!throttler_->Take(0, chunk_size)) {
+          SleepFor(MonoDelta::FromMicroseconds(Throttler::kRefillPeriodMicros / 2));
+        }
+      }
+    }
   }
 
   return Status::OK();
@@ -1001,12 +1115,14 @@ RemoteTabletCopyClient::RemoteTabletCopyClient(
     FsManager* dst_fs_manager,
     scoped_refptr<ConsensusMetadataManager> cmeta_manager,
     shared_ptr<Messenger> messenger,
-    TabletCopyClientMetrics* dst_tablet_copy_metrics)
+    TabletCopyClientMetrics* dst_tablet_copy_metrics,
+    shared_ptr<Throttler> throttler)
     : TabletCopyClient(std::move(tablet_id),
                        dst_fs_manager,
                        std::move(cmeta_manager),
                        std::move(messenger),
-                       dst_tablet_copy_metrics) {
+                       dst_tablet_copy_metrics,
+                       std::move(throttler)) {
 }
 
 RemoteTabletCopyClient::~RemoteTabletCopyClient() {
