@@ -22,6 +22,7 @@
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <vector>
 
 #include <gflags/gflags_declare.h>
@@ -32,13 +33,12 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log.pb.h"
-#include "kudu/consensus/log_reader.h"
+#include "kudu/consensus/log_reader.h" // IWYU pragma: keep
 #include "kudu/consensus/log_util.h"
 #include "kudu/fs/block_id.h"
 #include "kudu/fs/data_dirs.h"
 #include "kudu/fs/fs.pb.h"
 #include "kudu/fs/fs_manager.h"
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/rpc/rpc_header.pb.h"
@@ -85,7 +85,7 @@ class TabletCopyServiceTest : public TabletCopyTest {
   }
 
  protected:
-  void SetUp() OVERRIDE {
+  void SetUp() override {
     TabletCopyTest::SetUp();
     tablet_copy_proxy_.reset(
         new TabletCopyServiceProxy(
@@ -392,7 +392,7 @@ TEST_F(TabletCopyServiceTest, TestFetchInvalidBlockOffset) {
   RpcController controller;
   // Impossible offset.
   uint64_t offset = std::numeric_limits<uint64_t>::max() -
-      mini_server_->server()->fs_manager()->env()->GetEncryptionHeaderSize();
+      mini_server_->server()->fs_manager()->GetEnv()->GetEncryptionHeaderSize();
   Status status = DoFetchData(session_id, AsDataTypeId(FirstColumnBlockId(superblock)),
                               &offset, nullptr, &resp, &controller);
   NO_FATALS(AssertRemoteError(status, controller.error_response(),
