@@ -48,6 +48,7 @@
 #include "kudu/rpc/transfer.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tablet/tablet_replica.h"
+#include "kudu/util/env.h"
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/pb_util.h"
@@ -72,6 +73,13 @@ METRIC_DEFINE_gauge_int32(server, tablet_copy_open_source_sessions,
                           "Number of currently open tablet copy source sessions on this server",
                           kudu::MetricLevel::kInfo);
 
+METRIC_DEFINE_histogram(server, tablet_copy_source_duration,
+                        "Source Tablet Copy Duration",
+                        kudu::MetricUnit::kMilliseconds,
+                        "Duration of tablet copying as source.",
+                        kudu::MetricLevel::kDebug,
+                        3600000LU, 1);
+
 DEFINE_int32(tablet_copy_session_inject_latency_on_init_ms, 0,
              "How much latency (in ms) to inject when a tablet copy session is initialized. "
              "(For testing only!)");
@@ -95,7 +103,8 @@ using tablet::TabletReplica;
 
 TabletCopySourceMetrics::TabletCopySourceMetrics(const scoped_refptr<MetricEntity>& metric_entity)
     : bytes_sent(METRIC_tablet_copy_bytes_sent.Instantiate(metric_entity)),
-      open_source_sessions(METRIC_tablet_copy_open_source_sessions.Instantiate(metric_entity, 0)) {
+      open_source_sessions(METRIC_tablet_copy_open_source_sessions.Instantiate(metric_entity, 0)),
+      copy_duration(METRIC_tablet_copy_source_duration.Instantiate(metric_entity)) {
 }
 
 TabletCopySourceSession::TabletCopySourceSession(
@@ -123,6 +132,8 @@ Status TabletCopySourceSession::Init() {
 }
 
 Status RemoteTabletCopySourceSession::InitOnce() {
+  start_time_ = MonoTime::Now();
+
   // Inject latency during Init() for testing purposes.
   if (PREDICT_FALSE(FLAGS_tablet_copy_session_inject_latency_on_init_ms > 0)) {
     TRACE("Injecting $0ms of latency due to --tablet_copy_session_inject_latency_on_init_ms",
@@ -268,7 +279,7 @@ static Status ReadFileChunkToBuf(const Info* info,
   // however any modern compiler should be compatible with it.
   // Violates the API contract, but avoids excessive copies.
   data->resize(response_data_size);
-  uint8_t* buf = reinterpret_cast<uint8_t*>(const_cast<char*>(data->data()));
+  uint8_t* buf = reinterpret_cast<uint8_t*>(data->data());
   Slice slice(buf, response_data_size);
   Status s = info->Read(offset, slice);
   if (PREDICT_FALSE(!s.ok())) {
@@ -313,6 +324,34 @@ Status TabletCopySourceSession::GetBlockPiece(const BlockId& block_id,
   return Status::OK();
 }
 
+Status TabletCopySourceSession::GetSuperBlockPiece(uint64_t offset,
+                                                   int64_t client_maxlen,
+                                                   string* data,
+                                                   int64_t* superblock_file_size,
+                                                   TabletCopyErrorPB::Code* error_code) {
+  if (offset == 0) {
+    RETURN_NOT_OK_PREPEND(pb_util::WritePBContainerToPath(
+                          fs_manager_->GetEnv(), superblock_tmp_path_, tablet_superblock_,
+                          pb_util::NO_OVERWRITE, pb_util::SYNC,
+                          pb_util::SENSITIVE),
+                          Substitute("Failed to write tablet metadata $0", tablet_id_));
+  }
+  unique_ptr<RandomAccessFile> super_block_reader;
+  RETURN_NOT_OK(fs_manager_->GetEnv()->NewRandomAccessFile(superblock_tmp_path_,
+                                                           &super_block_reader));
+  uint64_t file_size;
+  RETURN_NOT_OK(super_block_reader->Size(&file_size));
+  *superblock_file_size = file_size;
+  int64_t response_data_size = 0;
+  RETURN_NOT_OK(GetResponseDataSize(file_size, offset, client_maxlen,
+                                    error_code, &response_data_size));
+  data->resize(response_data_size);
+  uint8_t* buf = reinterpret_cast<uint8_t*>(const_cast<char*>(data->data()));
+  Slice slice(buf, response_data_size);
+  RETURN_NOT_OK_PREPEND(super_block_reader->Read(offset, slice), "read file failed");
+  return Status::OK();
+}
+
 Status TabletCopySourceSession::GetLogSegmentPiece(uint64_t segment_seqno,
                                                    uint64_t offset, int64_t client_maxlen,
                                                    string* data, int64_t* log_file_size,
@@ -457,9 +496,16 @@ RemoteTabletCopySourceSession::RemoteTabletCopySourceSession(
       tablet_replica_(std::move(tablet_replica)),
       session_id_(std::move(session_id)),
       requestor_uuid_(std::move(requestor_uuid)) {
+  // Use session id to generate the superblock temporary path.
+  // It is thread safety, beacause session id is unique.
+  superblock_tmp_path_ = Substitute("/tmp/tablet-meta-$0", session_id_);
 }
 
 RemoteTabletCopySourceSession::~RemoteTabletCopySourceSession() {
+  if (!superblock_tmp_path_.empty() &&
+      fs_manager_->GetEnv()->FileExists(superblock_tmp_path_)) {
+    CHECK_OK(fs_manager_->GetEnv()->DeleteFile(superblock_tmp_path_));
+  }
   // No lock taken in the destructor, should only be 1 thread with access now.
   CHECK_OK(UnregisterAnchorIfNeededUnlocked());
 }
@@ -468,6 +514,14 @@ Status RemoteTabletCopySourceSession::UnregisterAnchorIfNeededUnlocked() {
   return tablet_replica_->log_anchor_registry()->UnregisterIfAnchored(&log_anchor_);
 }
 
+void RemoteTabletCopySourceSession::UpdateTabletMetrics() {
+  if (PREDICT_TRUE(IsInitialized())) {
+    DCHECK(start_time_.Initialized());
+    tablet_copy_metrics_->copy_duration->Increment(
+        (MonoTime::Now() - start_time_).ToMilliseconds());
+  }
+}
+
 LocalTabletCopySourceSession::LocalTabletCopySourceSession(
     string tablet_id,
     FsManager* fs_manager,
@@ -483,7 +537,7 @@ Status LocalTabletCopySourceSession::InitOnce() {
   // Read the SuperBlock from disk.
   string path = fs_manager_->GetTabletMetadataPath(tablet_id_);
   RETURN_NOT_OK_PREPEND(
-      pb_util::ReadPBContainerFromPath(fs_manager_->env(), path, &tablet_superblock_,
+      pb_util::ReadPBContainerFromPath(fs_manager_->GetEnv(), path, &tablet_superblock_,
                                        pb_util::SENSITIVE),
       Substitute("Unable to access superblock for tablet $0", tablet_id_));
 
