@@ -27,7 +27,6 @@
 #include <snappy.h>
 #include <zlib.h>
 
-#include "kudu/gutil/port.h"
 #include "kudu/gutil/singleton.h"
 #include "kudu/gutil/stringprintf.h"
 #include "kudu/util/faststring.h"
@@ -52,11 +51,11 @@ class SlicesSource : public snappy::Source {
     available_ = TotalSize();
   }
 
-  size_t Available() const OVERRIDE {
+  size_t Available() const override {
     return available_;
   }
 
-  const char* Peek(size_t* len) OVERRIDE {
+  const char* Peek(size_t* len) override {
     if (available_ == 0) {
       *len = 0;
       return nullptr;
@@ -67,7 +66,7 @@ class SlicesSource : public snappy::Source {
     return reinterpret_cast<const char *>(data.data()) + slice_offset_;
   }
 
-  void Skip(size_t n) OVERRIDE {
+  void Skip(size_t n) override {
     DCHECK_LE(n, Available());
     if (n == 0) return;
 
@@ -115,14 +114,14 @@ class SnappyCodec : public CompressionCodec {
   }
 
   Status Compress(const Slice& input,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     snappy::RawCompress(reinterpret_cast<const char *>(input.data()), input.size(),
                         reinterpret_cast<char *>(compressed), compressed_length);
     return Status::OK();
   }
 
   Status Compress(const vector<Slice>& input_slices,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     SlicesSource source(input_slices);
     snappy::UncheckedByteArraySink sink(reinterpret_cast<char *>(compressed));
     if ((*compressed_length = snappy::Compress(&source, &sink)) <= 0) {
@@ -133,13 +132,13 @@ class SnappyCodec : public CompressionCodec {
 
   Status Uncompress(const Slice& compressed,
                     uint8_t *uncompressed,
-                    size_t /*uncompressed_length*/) const OVERRIDE {
+                    size_t /*uncompressed_length*/) const override {
     bool success = snappy::RawUncompress(reinterpret_cast<const char *>(compressed.data()),
                                          compressed.size(), reinterpret_cast<char *>(uncompressed));
     return success ? Status::OK() : Status::Corruption("unable to uncompress the buffer");
   }
 
-  size_t MaxCompressedLength(size_t source_bytes) const OVERRIDE {
+  size_t MaxCompressedLength(size_t source_bytes) const override {
     return snappy::MaxCompressedLength(source_bytes);
   }
 
@@ -155,7 +154,7 @@ class Lz4Codec : public CompressionCodec {
   }
 
   Status Compress(const Slice& input,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     *compressed_length = LZ4_compress(reinterpret_cast<const char *>(input.data()),
                                       reinterpret_cast<char *>(compressed),
                                       input.size());
@@ -163,7 +162,7 @@ class Lz4Codec : public CompressionCodec {
   }
 
   Status Compress(const vector<Slice>& input_slices,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     if (input_slices.size() == 1) {
       return Compress(input_slices[0], compressed, compressed_length);
     }
@@ -176,7 +175,7 @@ class Lz4Codec : public CompressionCodec {
 
   Status Uncompress(const Slice& compressed,
                     uint8_t *uncompressed,
-                    size_t uncompressed_length) const OVERRIDE {
+                    size_t uncompressed_length) const override {
     int n = LZ4_decompress_safe(reinterpret_cast<const char *>(compressed.data()),
                                 reinterpret_cast<char *>(uncompressed),
                                 compressed.size(), uncompressed_length);
@@ -188,7 +187,7 @@ class Lz4Codec : public CompressionCodec {
     return Status::OK();
   }
 
-  size_t MaxCompressedLength(size_t source_bytes) const OVERRIDE {
+  size_t MaxCompressedLength(size_t source_bytes) const override {
     return LZ4_compressBound(source_bytes);
   }
 
@@ -208,14 +207,14 @@ class ZlibCodec : public CompressionCodec {
   }
 
   Status Compress(const Slice& input,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     *compressed_length = MaxCompressedLength(input.size());
     int err = ::compress(compressed, compressed_length, input.data(), input.size());
     return err == Z_OK ? Status::OK() : Status::IOError("unable to compress the buffer");
   }
 
   Status Compress(const vector<Slice>& input_slices,
-                  uint8_t *compressed, size_t *compressed_length) const OVERRIDE {
+                  uint8_t *compressed, size_t *compressed_length) const override {
     if (input_slices.size() == 1) {
       return Compress(input_slices[0], compressed, compressed_length);
     }
@@ -228,13 +227,13 @@ class ZlibCodec : public CompressionCodec {
   }
 
   Status Uncompress(const Slice& compressed,
-                    uint8_t *uncompressed, size_t uncompressed_length) const OVERRIDE {
+                    uint8_t *uncompressed, size_t uncompressed_length) const override {
     int err = ::uncompress(uncompressed, &uncompressed_length,
                            compressed.data(), compressed.size());
     return err == Z_OK ? Status::OK() : Status::Corruption("unable to uncompress the buffer");
   }
 
-  size_t MaxCompressedLength(size_t source_bytes) const OVERRIDE {
+  size_t MaxCompressedLength(size_t source_bytes) const override {
     // one-time overhead of six bytes for the entire stream plus five bytes per 16 KB block
     return source_bytes + (6 + (5 * ((source_bytes + 16383) >> 14)));
   }
