@@ -130,7 +130,7 @@ class TraceBufferRingBuffer : public TraceBuffer {
     STLDeleteElements(&chunks_);
   }
 
-  virtual unique_ptr<TraceBufferChunk> GetChunk(size_t* index) OVERRIDE {
+  unique_ptr<TraceBufferChunk> GetChunk(size_t* index) override {
     // Because the number of threads is much less than the number of chunks,
     // the queue should never be empty.
     DCHECK(!QueueIsEmpty());
@@ -152,8 +152,7 @@ class TraceBufferRingBuffer : public TraceBuffer {
     return unique_ptr<TraceBufferChunk>(chunk);
   }
 
-  virtual void ReturnChunk(size_t index,
-                           unique_ptr<TraceBufferChunk> chunk) OVERRIDE {
+  void ReturnChunk(size_t index, unique_ptr<TraceBufferChunk> chunk) override {
     // When this method is called, the queue should not be full because it
     // can contain all chunks including the one to be returned.
     DCHECK(!QueueIsFull());
@@ -165,20 +164,20 @@ class TraceBufferRingBuffer : public TraceBuffer {
     queue_tail_ = NextQueueIndex(queue_tail_);
   }
 
-  virtual bool IsFull() const OVERRIDE {
+  bool IsFull() const override {
     return false;
   }
 
-  virtual size_t Size() const OVERRIDE {
+  size_t Size() const override {
     // This is approximate because not all of the chunks are full.
     return chunks_.size() * kTraceBufferChunkSize;
   }
 
-  virtual size_t Capacity() const OVERRIDE {
+  size_t Capacity() const override {
     return max_chunks_ * kTraceBufferChunkSize;
   }
 
-  virtual TraceEvent* GetEventByHandle(TraceEventHandle handle) OVERRIDE {
+  TraceEvent* GetEventByHandle(TraceEventHandle handle) override {
     if (handle.chunk_index >= chunks_.size())
       return nullptr;
     TraceBufferChunk* chunk = chunks_[handle.chunk_index];
@@ -187,7 +186,7 @@ class TraceBufferRingBuffer : public TraceBuffer {
     return chunk->GetEventAt(handle.event_index);
   }
 
-  virtual const TraceBufferChunk* NextChunk() OVERRIDE {
+  const TraceBufferChunk* NextChunk() override {
     if (chunks_.empty())
       return nullptr;
 
@@ -202,7 +201,7 @@ class TraceBufferRingBuffer : public TraceBuffer {
     return nullptr;
   }
 
-  virtual unique_ptr<TraceBuffer> CloneForIteration() const OVERRIDE {
+  unique_ptr<TraceBuffer> CloneForIteration() const override {
     unique_ptr<ClonedTraceBuffer> cloned_buffer(new ClonedTraceBuffer());
     for (size_t queue_index = queue_head_; queue_index != queue_tail_;
         queue_index = NextQueueIndex(queue_index)) {
@@ -224,26 +223,26 @@ class TraceBufferRingBuffer : public TraceBuffer {
     }
 
     // The only implemented method.
-    virtual const TraceBufferChunk* NextChunk() OVERRIDE {
+    const TraceBufferChunk* NextChunk() override {
       return current_iteration_index_ < chunks_.size() ?
           chunks_[current_iteration_index_++] : nullptr;
     }
 
-    virtual unique_ptr<TraceBufferChunk> GetChunk(size_t* /*index*/) OVERRIDE {
+    unique_ptr<TraceBufferChunk> GetChunk(size_t* /*index*/) override {
       NOTIMPLEMENTED();
       return unique_ptr<TraceBufferChunk>();
     }
-    virtual void ReturnChunk(size_t /*index*/,
-                             unique_ptr<TraceBufferChunk> /*chunk*/) OVERRIDE {
+    void ReturnChunk(size_t /*index*/,
+                     unique_ptr<TraceBufferChunk> /*chunk*/) override {
       NOTIMPLEMENTED();
     }
-    virtual bool IsFull() const OVERRIDE { return false; }
-    virtual size_t Size() const OVERRIDE { return 0; }
-    virtual size_t Capacity() const OVERRIDE { return 0; }
-    virtual TraceEvent* GetEventByHandle(TraceEventHandle /*handle*/) OVERRIDE {
+    bool IsFull() const override { return false; }
+    size_t Size() const override { return 0; }
+    size_t Capacity() const override { return 0; }
+    TraceEvent* GetEventByHandle(TraceEventHandle /*handle*/) override {
       return nullptr;
     }
-    virtual unique_ptr<TraceBuffer> CloneForIteration() const OVERRIDE {
+    unique_ptr<TraceBuffer> CloneForIteration() const override {
       NOTIMPLEMENTED();
       return unique_ptr<TraceBuffer>();
     }
@@ -301,7 +300,7 @@ class TraceBufferVector : public TraceBuffer {
     STLDeleteElements(&chunks_);
   }
 
-  virtual unique_ptr<TraceBufferChunk> GetChunk(size_t* index) OVERRIDE {
+  unique_ptr<TraceBufferChunk> GetChunk(size_t* index) override {
     // This function may be called when adding normal events or indirectly from
     // AddMetadataEventsWhileLocked(). We can not DECHECK(!IsFull()) because we
     // have to add the metadata events and flush thread-local buffers even if
@@ -314,8 +313,7 @@ class TraceBufferVector : public TraceBuffer {
         new TraceBufferChunk(static_cast<uint32_t>(*index) + 1));
   }
 
-  virtual void ReturnChunk(size_t index,
-                           unique_ptr<TraceBufferChunk> chunk) OVERRIDE {
+  void ReturnChunk(size_t index, unique_ptr<TraceBufferChunk> chunk) override {
     DCHECK_GT(in_flight_chunk_count_, 0U);
     DCHECK_LT(index, chunks_.size());
     DCHECK(!chunks_[index]);
@@ -323,20 +321,20 @@ class TraceBufferVector : public TraceBuffer {
     chunks_[index] = chunk.release();
   }
 
-  virtual bool IsFull() const OVERRIDE {
+  bool IsFull() const override {
     return chunks_.size() >= kTraceEventVectorBufferChunks;
   }
 
-  virtual size_t Size() const OVERRIDE {
+  size_t Size() const override {
     // This is approximate because not all of the chunks are full.
     return chunks_.size() * kTraceBufferChunkSize;
   }
 
-  virtual size_t Capacity() const OVERRIDE {
+  size_t Capacity() const override {
     return kTraceEventVectorBufferChunks * kTraceBufferChunkSize;
   }
 
-  virtual TraceEvent* GetEventByHandle(TraceEventHandle handle) OVERRIDE {
+  TraceEvent* GetEventByHandle(TraceEventHandle handle) override {
     if (handle.chunk_index >= chunks_.size())
       return nullptr;
     TraceBufferChunk* chunk = chunks_[handle.chunk_index];
@@ -345,7 +343,7 @@ class TraceBufferVector : public TraceBuffer {
     return chunk->GetEventAt(handle.event_index);
   }
 
-  virtual const TraceBufferChunk* NextChunk() OVERRIDE {
+  const TraceBufferChunk* NextChunk() override {
     while (current_iteration_index_ < chunks_.size()) {
       // Skip in-flight chunks.
       const TraceBufferChunk* chunk = chunks_[current_iteration_index_++];
@@ -355,7 +353,7 @@ class TraceBufferVector : public TraceBuffer {
     return nullptr;
   }
 
-  virtual unique_ptr<TraceBuffer> CloneForIteration() const OVERRIDE {
+  unique_ptr<TraceBuffer> CloneForIteration() const override {
     NOTIMPLEMENTED();
     return unique_ptr<TraceBuffer>();
   }
@@ -1571,7 +1569,7 @@ void TraceLog::Flush(const TraceLog::OutputCallback& cb) {
   {
     // Holding the active threads lock ensures that no thread will exit and
     // delete its own PerThreadInfo object.
-    MutexLock l(active_threads_lock_);
+    std::lock_guard l(active_threads_lock_);
     for (const ActiveThreadMap::value_type& entry : active_threads_) {
       int64_t tid = entry.first;
       PerThreadInfo* thr_info = entry.second;
@@ -1731,7 +1729,7 @@ TraceLog::PerThreadInfo* TraceLog::SetupThreadLocalBuffer() {
   threadlocal::internal::AddDestructor(&TraceLog::ThreadExitingCB, this);
 
   {
-    MutexLock lock(active_threads_lock_);
+    std::lock_guard lock_(active_threads_lock_);
     InsertOrDie(&active_threads_, cur_tid, thr_info);
   }
   return thr_info;
@@ -1761,7 +1759,7 @@ void TraceLog::ThreadExiting() {
   delete buf;
 
   {
-    MutexLock lock(active_threads_lock_);
+    std::lock_guard lock_(active_threads_lock_);
     active_threads_.erase(cur_tid);
   }
   delete thr_info;
