@@ -20,10 +20,10 @@
 #include <deque>
 #include <iterator>
 #include <memory>
-#include <mutex>
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -126,13 +126,15 @@ class FileCacheStressTest : public KuduTest {
                                           oid_generator.Next());
       {
         unique_ptr<WritableFile> next_file;
-        CHECK_OK(env_->NewWritableFile(next_file_name, &next_file));
+        WritableFileOptions opts;
+        opts.is_sensitive = true;
+        CHECK_OK(env_->NewWritableFile(opts, next_file_name, &next_file));
         uint8_t buf[rand.Uniform((32 * 1024) - 1) + 1];
         CHECK_OK(next_file->Append(GenerateRandomChunk(buf, sizeof(buf), &rand)));
         CHECK_OK(next_file->Close());
       }
       {
-        std::lock_guard<simple_spinlock> l(lock_);
+        std::lock_guard l(lock_);
         InsertOrDie(&available_files_, next_file_name, 0);
       }
       metrics[BaseName(next_file_name)]["create"] = 1;
@@ -241,7 +243,7 @@ class FileCacheStressTest : public KuduTest {
   // Retrieve a random file name to be either opened or deleted. If deleting,
   // the file name is made inaccessible to future operations.
   bool GetRandomFile(GetMode mode, Random* rand, string* out) {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     if (available_files_.empty()) {
       return false;
     }
@@ -268,7 +270,7 @@ class FileCacheStressTest : public KuduTest {
   // Signal that a previously in-progress open has finished, allowing the file
   // in question to be deleted.
   void FinishedOpen(const string& opened) {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     int& openers = FindOrDie(available_files_, opened);
     openers--;
   }
@@ -286,8 +288,11 @@ class FileCacheStressTest : public KuduTest {
 
     uint64_t file_size;
     RETURN_NOT_OK(file->Size(&file_size));
-    uint64_t off = file_size > 0 ? rand->Uniform(file_size) : 0;
-    size_t len = file_size > 0 ? rand->Uniform(file_size - off) : 0;
+    const uint8_t kHeaderSize = file->GetEncryptionHeaderSize();;
+    uint64_t off = file_size > kHeaderSize
+      ? rand->Uniform(file_size - kHeaderSize) + kHeaderSize
+      : kHeaderSize;
+    size_t len = file_size > kHeaderSize ? rand->Uniform(file_size - off) : 0;
     unique_ptr<uint8_t[]> scratch(new uint8_t[len]);
     RETURN_NOT_OK(file->Read(off, Slice(scratch.get(), len)));
 
@@ -307,7 +312,10 @@ class FileCacheStressTest : public KuduTest {
 
     uint64_t file_size;
     RETURN_NOT_OK(file->Size(&file_size));
-    uint64_t off = file_size > 0 ? rand->Uniform(file_size) : 0;
+    const uint8_t kHeaderSize = file->GetEncryptionHeaderSize();
+    uint64_t off = file_size > kHeaderSize
+      ? rand->Uniform(file_size - kHeaderSize) + kHeaderSize
+      : kHeaderSize;
     uint8_t buf[64];
     RETURN_NOT_OK(file->Write(off, GenerateRandomChunk(buf, sizeof(buf), rand)));
     (*metrics)[BaseName(file->filename())]["write"]++;
@@ -325,7 +333,7 @@ class FileCacheStressTest : public KuduTest {
 
   // Merge the metrics in 'new_metrics' into the global metric map.
   void MergeNewMetrics(MetricMap new_metrics) {
-    std::lock_guard<simple_spinlock> l(lock_);
+    std::lock_guard l(lock_);
     for (const auto& file_action_pair : new_metrics) {
       for (const auto& action_count_pair : file_action_pair.second) {
         metrics_[file_action_pair.first][action_count_pair.first] += action_count_pair.second;
