@@ -36,6 +36,7 @@
 #include "kudu/gutil/casts.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/split.h"
 #include "kudu/util/hdr_histogram.h"
 #include "kudu/util/jsonreader.h"
 #include "kudu/util/jsonwriter.h"
@@ -55,12 +56,25 @@ using std::vector;
 
 DECLARE_int32(metrics_retirement_age_ms);
 
-DECLARE_string(metrics_default_level);
-
 namespace kudu {
 
 METRIC_DEFINE_entity(test_entity);
 
+METRIC_DEFINE_entity(tablet);
+METRIC_DEFINE_counter(tablet, tablet_test_counter,
+                      "Tablet-wise test counter label",
+                      kudu::MetricUnit::kBytes,
+                      "Tablet-wise test counter description.",
+                      kudu::MetricLevel::kDebug);
+
+METRIC_DEFINE_entity(table);
+METRIC_DEFINE_counter(table, table_test_counter,
+                      "Table-wise test counter label",
+                      kudu::MetricUnit::kBytes,
+                      "Table-wise test counter description.",
+                      kudu::MetricLevel::kDebug);
+
+
 class MetricsTest : public KuduTest {
  public:
   void SetUp() override {
@@ -133,6 +147,70 @@ TEST_F(MetricsTest, ResetCounter) {
   ASSERT_EQ(0, c->value());
 }
 
+TEST_F(MetricsTest, TableAndTabletPrometheusTest) {
+  // Simulate two tablets in the metric registry. Write out their metric data.
+  auto tablet_metric_entity = METRIC_ENTITY_tablet.Instantiate(&registry_, "000000");
+  auto tablet_counter = METRIC_tablet_test_counter.Instantiate(tablet_metric_entity);
+  tablet_counter->IncrementBy(11);
+
+  auto tablet_metric_entity2 = METRIC_ENTITY_tablet.Instantiate(&registry_, "1111111111");
+  auto tablet_counter2 = METRIC_tablet_test_counter.Instantiate(tablet_metric_entity2);
+  tablet_counter2->IncrementBy(2);
+
+  auto table_metric_entity = METRIC_ENTITY_table.Instantiate(&registry_, "table1");
+  auto table1_counter = METRIC_table_test_counter.Instantiate(table_metric_entity);
+  table1_counter->IncrementBy(888);
+
+  auto table_metric_entity2 = METRIC_ENTITY_table.Instantiate(&registry_, "table2");
+  auto table2_counter = METRIC_table_test_counter.Instantiate(table_metric_entity2);
+  table2_counter->Increment();
+
+  auto table_metric_entity3 = METRIC_ENTITY_table.Instantiate(
+      &registry_, "55555555555555555555555555555555");
+  auto table3_counter = METRIC_table_test_counter.Instantiate(table_metric_entity3);
+  table3_counter->IncrementBy(5);
+
+  ostringstream output;
+  PrometheusWriter writer(&output);
+  ASSERT_OK(registry_.WriteAsPrometheus(&writer, {}));
+
+  // The order of elements in the output depends on the ordering in hash-map
+  // of metric entities and might be different in different STL implementations.
+  const auto& out = output.str();
+  ASSERT_STR_CONTAINS(out,
+      "# HELP kudu_tablet_000000_tablet_test_counter Tablet-wise test counter description.\n"
+      "# TYPE kudu_tablet_000000_tablet_test_counter counter\n"
+       "kudu_tablet_000000_tablet_test_counter{unit_type=\"bytes\"} 11\n"
+  );
+  ASSERT_STR_CONTAINS(out,
+      "# HELP kudu_tablet_1111111111_tablet_test_counter Tablet-wise test counter description.\n"
+      "# TYPE kudu_tablet_1111111111_tablet_test_counter counter\n"
+      "kudu_tablet_1111111111_tablet_test_counter{unit_type=\"bytes\"} 2\n"
+  );
+  ASSERT_STR_CONTAINS(out,
+      "# HELP kudu_table_table1_table_test_counter Table-wise test counter description.\n"
+      "# TYPE kudu_table_table1_table_test_counter counter\n"
+      "kudu_table_table1_table_test_counter{unit_type=\"bytes\"} 888\n"
+  );
+  ASSERT_STR_CONTAINS(out,
+      "# HELP kudu_table_table2_table_test_counter Table-wise test counter description.\n"
+      "# TYPE kudu_table_table2_table_test_counter counter\n"
+      "kudu_table_table2_table_test_counter{unit_type=\"bytes\"} 1\n"
+  );
+  ASSERT_STR_CONTAINS(out,
+      "# HELP kudu_table_55555555555555555555555555555555_table_test_counter "
+          "Table-wise test counter description.\n"
+      "# TYPE kudu_table_55555555555555555555555555555555_table_test_counter "
+          "counter\n"
+      "kudu_table_55555555555555555555555555555555_table_test_counter{unit_type=\"bytes\"} 5\n"
+  );
+
+  // The lines above and one trailing empty line is the only output expected.
+  const vector<string> lines = strings::Split(out, "\n");
+  ASSERT_EQ(3 + 3 + 3 + 3 + 3 + 1, lines.size());
+  ASSERT_TRUE(lines.back().empty());
+}
+
 TEST_F(MetricsTest, CounterPrometheusTest) {
   scoped_refptr<Counter> requests(new Counter(&METRIC_test_counter));
 
@@ -631,11 +709,13 @@ TEST_F(MetricsTest, SimpleHistogramMergeTest) {
   ASSERT_EQ(2, hist->histogram()->MinValue());
   ASSERT_EQ(4, hist->histogram()->MeanValue());
   ASSERT_EQ(6, hist->histogram()->MaxValue());
+  ASSERT_EQ(6, hist->histogram()->LastValue());
   ASSERT_EQ(2, hist->histogram()->TotalCount());
   ASSERT_EQ(8, hist->histogram()->TotalSum());
   ASSERT_EQ(1, hist_for_merge->histogram()->MinValue());
   ASSERT_EQ(3, hist_for_merge->histogram()->MeanValue());
   ASSERT_EQ(6, hist_for_merge->histogram()->MaxValue());
+  ASSERT_EQ(6, hist_for_merge->histogram()->LastValue());
   ASSERT_EQ(6, hist_for_merge->histogram()->TotalCount());
   ASSERT_EQ(18, hist_for_merge->histogram()->TotalSum());
   ASSERT_EQ(1, hist_for_merge->histogram()->ValueAtPercentile(20.0));
@@ -975,7 +1055,7 @@ TEST_F(MetricsTest, TestDumpJsonPrototypes) {
   int num_entities = d["entities"].Size();
   LOG(INFO) << "Parsed " << num_metrics << " metrics and " << num_entities << " entities";
   ASSERT_GT(num_metrics, 5);
-  ASSERT_EQ(num_entities, 2);
+  ASSERT_EQ(num_entities, 4);
 
   // Spot-check that some metrics were properly registered and that the JSON was properly
   // formed.
