@@ -33,10 +33,7 @@
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
-#include "kudu/gutil/atomic_refcount.h"
-#include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/bits.h"
-#include "kudu/gutil/dynamic_annotations.h"
 #include "kudu/gutil/hash/city.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
@@ -51,6 +48,7 @@
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
+#include "kudu/util/test_util_prod.h"
 
 #ifndef MEMKIND_PMEM_MIN_SIZE
 #define MEMKIND_PMEM_MIN_SIZE (1024 * 1024 * 16) // Taken from memkind 1.9.0.
@@ -76,6 +74,7 @@ DEFINE_double(nvm_cache_usage_ratio, 1.25,
              "the ratio.");
 TAG_FLAG(nvm_cache_usage_ratio, advanced);
 
+using kudu::util::LRUHandle;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -187,126 +186,13 @@ typedef simple_spinlock MutexType;
 
 // LRU cache implementation
 
-// An entry is a variable length heap-allocated structure.  Entries
-// are kept in a circular doubly linked list ordered by access time.
-struct LRUHandle {
-  Cache::EvictionCallback* eviction_callback;
-  LRUHandle* next_hash;
-  LRUHandle* next;
-  LRUHandle* prev;
-  size_t charge;      // TODO(opt): Only allow uint32_t?
-  uint32_t key_length;
-  uint32_t val_length;
-  Atomic32 refs;
-  uint32_t hash;      // Hash of key(); used for fast sharding and comparisons
-  uint8_t* kv_data;
-
-  Slice key() const {
-    return Slice(kv_data, key_length);
-  }
-
-  Slice value() const {
-    return Slice(&kv_data[key_length], val_length);
-  }
-
-  uint8_t* val_ptr() {
-    return &kv_data[key_length];
-  }
-};
-
-// We provide our own simple hash table since it removes a whole bunch
-// of porting hacks and is also faster than some of the built-in hash
-// table implementations in some of the compiler/runtime combinations
-// we have tested.  E.g., readrandom speeds up by ~5% over the g++
-// 4.4.3's builtin hashtable.
-class HandleTable {
- public:
-  HandleTable() : length_(0), elems_(0), list_(nullptr) { Resize(); }
-  ~HandleTable() { delete[] list_; }
-
-  LRUHandle* Lookup(const Slice& key, uint32_t hash) {
-    return *FindPointer(key, hash);
-  }
-
-  LRUHandle* Insert(LRUHandle* h) {
-    LRUHandle** ptr = FindPointer(h->key(), h->hash);
-    LRUHandle* old = *ptr;
-    h->next_hash = (old == nullptr ? nullptr : old->next_hash);
-    *ptr = h;
-    if (old == nullptr) {
-      ++elems_;
-      if (elems_ > length_) {
-        // Since each cache entry is fairly large, we aim for a small
-        // average linked list length (<= 1).
-        Resize();
-      }
-    }
-    return old;
-  }
-
-  LRUHandle* Remove(const Slice& key, uint32_t hash) {
-    LRUHandle** ptr = FindPointer(key, hash);
-    LRUHandle* result = *ptr;
-    if (result != nullptr) {
-      *ptr = result->next_hash;
-      --elems_;
-    }
-    return result;
-  }
-
- private:
-  // The table consists of an array of buckets where each bucket is
-  // a linked list of cache entries that hash into the bucket.
-  uint32_t length_;
-  uint32_t elems_;
-  LRUHandle** list_;
-
-  // Return a pointer to slot that points to a cache entry that
-  // matches key/hash.  If there is no such cache entry, return a
-  // pointer to the trailing slot in the corresponding linked list.
-  LRUHandle** FindPointer(const Slice& key, uint32_t hash) {
-    LRUHandle** ptr = &list_[hash & (length_ - 1)];
-    while (*ptr != nullptr &&
-           ((*ptr)->hash != hash || key != (*ptr)->key())) {
-      ptr = &(*ptr)->next_hash;
-    }
-    return ptr;
-  }
-
-  void Resize() {
-    uint32_t new_length = 16;
-    while (new_length < elems_ * 1.5) {
-      new_length *= 2;
-    }
-    LRUHandle** new_list = new LRUHandle*[new_length];
-    memset(new_list, 0, sizeof(new_list[0]) * new_length);
-    uint32_t count = 0;
-    for (uint32_t i = 0; i < length_; i++) {
-      LRUHandle* h = list_[i];
-      while (h != nullptr) {
-        LRUHandle* next = h->next_hash;
-        uint32_t hash = h->hash;
-        LRUHandle** ptr = &new_list[hash & (new_length - 1)];
-        h->next_hash = *ptr;
-        *ptr = h;
-        h = next;
-        count++;
-      }
-    }
-    DCHECK_EQ(elems_, count);
-    delete[] list_;
-    list_ = new_list;
-    length_ = new_length;
-  }
-};
-
 // A single shard of sharded cache.
 class NvmLRUCache {
  public:
   explicit NvmLRUCache(memkind *vmp);
   ~NvmLRUCache();
 
-  // Separate from constructor so caller can easily make an array of LRUCache
+  // Separate from constructor so caller can easily make an array of LRUCache.
   void SetCapacity(size_t capacity) { capacity_ = capacity; }
 
   void SetMetrics(CacheMetrics* metrics) { metrics_ = metrics; }
@@ -324,16 +210,15 @@ class NvmLRUCache {
   void NvmLRU_Remove(LRUHandle* e);
   void NvmLRU_Append(LRUHandle* e);
   // Just reduce the reference count by 1.
-  // Return true if last reference
-  bool Unref(LRUHandle* e);
+  // Return true if last reference.
+  static bool Unref(LRUHandle* e);
   void FreeEntry(LRUHandle* e);
 
   // Evict the LRU item in the cache, adding it to the linked list
   // pointed to by 'to_remove_head'.
   void EvictOldestUnlocked(LRUHandle** to_remove_head);
 
-  // Free all of the entries in the linked list that has to_free_head
-  // as its head.
+  // Free all the entries in the linked list that has to_free_head as its head.
   void FreeLRUEntries(LRUHandle* to_free_head);
 
   // Wrapper around memkind_malloc which injects failures based on a flag.
@@ -350,7 +235,7 @@ class NvmLRUCache {
   // lru.prev is newest entry, lru.next is oldest entry.
   LRUHandle lru_;
 
-  HandleTable table_;
+  Cache::HandleTable<LRUHandle> table_;
 
   memkind* vmp_;
 
@@ -361,7 +246,7 @@ NvmLRUCache::NvmLRUCache(memkind* vmp)
   : usage_(0),
   vmp_(vmp),
   metrics_(nullptr) {
-  // Make empty circular linked list
+  // Make empty circular linked list.
   lru_.next = &lru_;
   lru_.prev = &lru_;
 }
@@ -369,7 +254,8 @@ NvmLRUCache::NvmLRUCache(memkind* vmp)
 NvmLRUCache::~NvmLRUCache() {
   for (LRUHandle* e = lru_.next; e != &lru_; ) {
     LRUHandle* next = e->next;
-    DCHECK_EQ(e->refs, 1);  // Error if caller has an unreleased handle
+    // Error if caller has an unreleased handle.
+    DCHECK_EQ(e->refs.load(std::memory_order_relaxed), 1);
     if (Unref(e)) {
       FreeEntry(e);
     }
@@ -385,12 +271,12 @@ void* NvmLRUCache::MemkindMalloc(size_t size) {
 }
 
 bool NvmLRUCache::Unref(LRUHandle* e) {
-  DCHECK_GT(ANNOTATE_UNPROTECTED_READ(e->refs), 0);
-  return !base::RefCountDec(&e->refs);
+  DCHECK_GT(e->refs.load(std::memory_order_relaxed), 0);
+  return e->refs.fetch_sub(1) == 1;
 }
 
 void NvmLRUCache::FreeEntry(LRUHandle* e) {
-  DCHECK_EQ(ANNOTATE_UNPROTECTED_READ(e->refs), 0);
+  DCHECK_EQ(e->refs.load(std::memory_order_relaxed), 0);
   if (e->eviction_callback) {
     e->eviction_callback->EvictedEntry(e->key(), e->value());
   }
@@ -413,7 +299,7 @@ void NvmLRUCache::NvmLRU_Remove(LRUHandle* e) {
 }
 
 void NvmLRUCache::NvmLRU_Append(LRUHandle* e) {
-  // Make "e" newest entry by inserting just before lru_
+  // Make "e" newest entry by inserting just before lru_.
   e->next = &lru_;
   e->prev = lru_.prev;
   e->prev->next = e;
@@ -422,20 +308,20 @@ void NvmLRUCache::NvmLRU_Append(LRUHandle* e) {
 }
 
 Cache::Handle* NvmLRUCache::Lookup(const Slice& key, uint32_t hash, bool caching) {
- LRUHandle* e;
+  LRUHandle* e;
   {
-    std::lock_guard<MutexType> l(mutex_);
+    std::lock_guard l(mutex_);
     e = table_.Lookup(key, hash);
     if (e != nullptr) {
       // If an entry exists, remove the old entry from the cache
       // and re-add to the end of the linked list.
-      base::RefCountInc(&e->refs);
+      e->refs.fetch_add(1, std::memory_order_relaxed);
       NvmLRU_Remove(e);
       NvmLRU_Append(e);
     }
   }
 
-  // Do the metrics outside of the lock.
+  // Do the metrics outside the lock.
   if (metrics_) {
     metrics_->lookups->Increment();
     bool was_hit = (e != nullptr);
@@ -488,7 +374,8 @@ Cache::Handle* NvmLRUCache::Insert(LRUHandle* e,
   DCHECK(e);
   LRUHandle* to_remove_head = nullptr;
 
-  e->refs = 2;  // One from LRUCache, one for the returned handle
+  // One from LRUCache, one for the returned handle.
+  e->refs.store(2, std::memory_order_relaxed);
   e->eviction_callback = eviction_callback;
   if (PREDICT_TRUE(metrics_)) {
     metrics_->cache_usage->IncrementBy(e->charge);
@@ -496,7 +383,7 @@ Cache::Handle* NvmLRUCache::Insert(LRUHandle* e,
   }
 
   {
-    std::lock_guard<MutexType> l(mutex_);
+    std::lock_guard l(mutex_);
 
     NvmLRU_Append(e);
 
@@ -514,8 +401,7 @@ Cache::Handle* NvmLRUCache::Insert(LRUHandle* e,
     }
   }
 
-  // we free the entries here outside of mutex for
-  // performance reasons
+  // We free the entries here outside the mutex for performance reasons.
   FreeLRUEntries(to_remove_head);
 
   return reinterpret_cast<Cache::Handle*>(e);
@@ -525,15 +411,15 @@ void NvmLRUCache::Erase(const Slice& key, uint32_t hash) {
   LRUHandle* e;
   bool last_reference = false;
   {
-    std::lock_guard<MutexType> l(mutex_);
+    std::lock_guard l(mutex_);
     e = table_.Remove(key, hash);
     if (e != nullptr) {
       NvmLRU_Remove(e);
       last_reference = Unref(e);
     }
   }
-  // mutex not held here
-  // last_reference will only be true if e != nullptr
+  // Mutex not held here.
+  // last_reference will only be true if e != nullptr.
   if (last_reference) {
     FreeEntry(e);
   }
@@ -545,7 +431,7 @@ size_t NvmLRUCache::Invalidate(const Cache::InvalidationControl& ctl) {
   LRUHandle* to_remove_head = nullptr;
 
   {
-    std::lock_guard<MutexType> l(mutex_);
+    std::lock_guard l(mutex_);
 
     // rl_.next is the oldest entry in the recency list.
     LRUHandle* h = lru_.next;
@@ -620,7 +506,7 @@ class ShardedLRUCache : public Cache {
     }
   }
 
-  virtual ~ShardedLRUCache() {
+  ~ShardedLRUCache() override {
     shards_.clear();
     // Per the note at the top of this file, our cache is entirely volatile.
     // Hence, when the cache is destructed, we delete the underlying
@@ -628,48 +514,52 @@ class ShardedLRUCache : public Cache {
     CALL_MEMKIND(memkind_destroy_kind, vmp_);
   }
 
-  virtual UniqueHandle Insert(UniquePendingHandle handle,
-                              Cache::EvictionCallback* eviction_callback) OVERRIDE {
+  UniqueHandle Insert(UniquePendingHandle handle,
+                      Cache::EvictionCallback* eviction_callback) override {
     LRUHandle* h = reinterpret_cast<LRUHandle*>(DCHECK_NOTNULL(handle.release()));
     return UniqueHandle(
         shards_[Shard(h->hash)]->Insert(h, eviction_callback),
         Cache::HandleDeleter(this));
   }
-  virtual UniqueHandle Lookup(const Slice& key, CacheBehavior caching) OVERRIDE {
+  UniqueHandle Lookup(const Slice& key, CacheBehavior caching) override {
     const uint32_t hash = HashSlice(key);
     return UniqueHandle(
         shards_[Shard(hash)]->Lookup(key, hash, caching == EXPECT_IN_CACHE),
         Cache::HandleDeleter(this));
   }
-  virtual void Release(Handle* handle) OVERRIDE {
+  void Release(Handle* handle) override {
     LRUHandle* h = reinterpret_cast<LRUHandle*>(handle);
     shards_[Shard(h->hash)]->Release(handle);
   }
-  virtual void Erase(const Slice& key) OVERRIDE {
+  void Erase(const Slice& key) override {
     const uint32_t hash = HashSlice(key);
     shards_[Shard(hash)]->Erase(key, hash);
   }
-  virtual Slice Value(const UniqueHandle& handle) const OVERRIDE {
+  Slice Value(const UniqueHandle& handle) const override {
     return reinterpret_cast<const LRUHandle*>(handle.get())->value();
   }
-  virtual uint8_t* MutableValue(UniquePendingHandle* handle) OVERRIDE {
-    return reinterpret_cast<LRUHandle*>(handle->get())->val_ptr();
+  uint8_t* MutableValue(UniquePendingHandle* handle) override {
+    return reinterpret_cast<LRUHandle*>(handle->get())->mutable_val_ptr();
   }
 
-  virtual void SetMetrics(unique_ptr<CacheMetrics> metrics) OVERRIDE {
+  void SetMetrics(unique_ptr<CacheMetrics> metrics,
+                  Cache::ExistingMetricsPolicy metrics_policy) override {
+    if (metrics_ && metrics_policy == Cache::ExistingMetricsPolicy::kKeep) {
+      CHECK(IsGTest()) << "Metrics should only be set once per Cache";
+      return;
+    }
     metrics_ = std::move(metrics);
     for (const auto& shard : shards_) {
       shard->SetMetrics(metrics_.get());
     }
   }
-  virtual UniquePendingHandle Allocate(Slice key, int val_len, int charge) OVERRIDE {
+  UniquePendingHandle Allocate(Slice key, int val_len, int charge) override {
     int key_len = key.size();
     DCHECK_GE(key_len, 0);
     DCHECK_GE(val_len, 0);
 
     // Try allocating from each of the shards -- if memkind is tight,
-    // this can cause eviction, so we might have better luck in different
-    // shards.
+    // this can cause eviction, so we might have better luck in different shards.
     for (const auto& shard : shards_) {
       UniquePendingHandle ph(static_cast<PendingHandle*>(
           shard->Allocate(sizeof(LRUHandle) + key_len + val_len)),
@@ -694,7 +584,7 @@ class ShardedLRUCache : public Cache {
     return UniquePendingHandle(nullptr, Cache::PendingHandleDeleter(this));
   }
 
-  virtual void Free(PendingHandle* ph) OVERRIDE {
+  void Free(PendingHandle* ph) override {
     CALL_MEMKIND(memkind_free, vmp_, ph);
   }
 
