@@ -17,16 +17,18 @@
 
 #include "kudu/util/test_util.h"
 
-#include <limits.h>
 #include <unistd.h>
 
+#include <algorithm>
 #include <cerrno>
+#include <climits>
 #include <cstdlib>
 #include <limits>
 #include <map>
 #include <memory>
 #include <ostream>
 #include <string>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -59,17 +61,20 @@
 #include "kudu/util/status.h"
 #include "kudu/util/string_case.h"
 #include "kudu/util/subprocess.h"
+#include "kudu/util/test_macros.h"
 
 DEFINE_string(test_leave_files, "on_failure",
               "Whether to leave test files around after the test run. "
-              " Valid values are 'always', 'on_failure', or 'never'");
+              "Valid values are 'always', 'on_failure', or 'never'");
 
 DEFINE_int32(test_random_seed, 0, "Random seed to use for randomized tests");
 
 DECLARE_string(time_source);
+DECLARE_bool(enable_multi_tenancy);
 DECLARE_bool(encrypt_data_at_rest);
 
 using std::string;
+using std::unordered_map;
 using std::vector;
 using strings::Substitute;
 
@@ -88,10 +93,17 @@ static const uint8_t kEncryptionKey[kEncryptionKeySize] =
   {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 42};
 static const uint8_t kEncryptionKeyIv[kEncryptionKeySize] =
   {42, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0};
-static const char* const kEncryptionKeyVersion = "kuduclusterkey@0";
+static const char* const kEncryptionKeyVersion = "kudutenantkey@0";
+static const char* const kEncryptionTenantName = "default_tenant_kudu";
+static const char* const kEncryptionTenantID = "00000000000000000000000000000000";
 
 static const uint64_t kTestBeganAtMicros = Env::Default()->NowMicros();
 
+static const char* const kContentTypeTextPlain = "text/plain";
+static const char* const kContentTypeTextHtml = "text/html";
+static const char* const kContentTypeApplicationOctet = "application/octet-stream";
+static const char* const kContentTypeApplicationJson = "application/json";
+
 // Global which production code can check to see if it is running
 // in a GTest environment (assuming the test binary links in this module,
 // which is typically a good assumption).
@@ -167,9 +179,9 @@ KuduTest::~KuduTest() {
   if (FLAGS_test_leave_files == "always") {
     LOG(INFO) << "-----------------------------------------------";
     LOG(INFO) << "--test_leave_files specified, leaving files in " << test_dir_;
-  } else if (FLAGS_test_leave_files == "on_failure" && HasFatalFailure()) {
+  } else if (FLAGS_test_leave_files == "on_failure" && HasFailure()) {
     LOG(INFO) << "-----------------------------------------------";
-    LOG(INFO) << "Had fatal failures, leaving test files at " << test_dir_;
+    LOG(INFO) << "Had failures, leaving test files at " << test_dir_;
   } else {
     VLOG(1) << "Cleaning up temporary test files...";
     WARN_NOT_OK(env_->DeleteRecursively(test_dir_),
@@ -196,9 +208,9 @@ void KuduTest::OverrideKrb5Environment() {
   //
   // NOTE: we don't simply *unset* the variables, because then we'd still pick up
   // the user's /etc/krb5.conf and other default locations.
-  setenv("KRB5_CONFIG", kInvalidPath, 1);
-  setenv("KRB5_KTNAME", kInvalidPath, 1);
-  setenv("KRB5CCNAME", kInvalidPath, 1);
+  PCHECK(setenv("KRB5_CONFIG", kInvalidPath, 1) == 0);
+  PCHECK(setenv("KRB5_KTNAME", kInvalidPath, 1) == 0);
+  PCHECK(setenv("KRB5CCNAME", kInvalidPath, 1) == 0);
 }
 
 void KuduTest::SetEncryptionFlags(bool enable_encryption) {
@@ -208,12 +220,21 @@ void KuduTest::SetEncryptionFlags(bool enable_encryption) {
   }
 }
 
-void KuduTest::GetEncryptionKey(string* key, string* iv, string* version) {
+void KuduTest::GetEncryptionKey(string* name, string* id, string* key, string* iv,
+                                string* version) {
   if (FLAGS_encrypt_data_at_rest) {
+    if (FLAGS_enable_multi_tenancy && name && id) {
+      *name = kEncryptionTenantName;
+      *id = kEncryptionTenantID;
+    }
     strings::b2a_hex(kEncryptionKey, key, kEncryptionKeySize);
     strings::b2a_hex(kEncryptionKeyIv, iv, kEncryptionKeySize);
     *version = kEncryptionKeyVersion;
   } else {
+    if (name && id) {
+      *name = "";
+      *id = "";
+    }
     *key = "";
     *iv = "";
     *version = "";
@@ -440,7 +461,8 @@ int CountOpenFds(Env* env, const string& path_pattern) {
 namespace {
 const vector<string> kWildcard = { "0.0.0.0" };
 
-Status WaitForBind(pid_t pid, uint16_t* port,
+Status WaitForBind(pid_t pid,
+                   uint16_t* port,
                    const vector<string>& addresses,
                    const char* kind,
                    MonoDelta timeout) {
@@ -454,7 +476,7 @@ Status WaitForBind(pid_t pid, uint16_t* port,
   RETURN_NOT_OK(FindExecutable("lsof", {"/sbin", "/usr/sbin"}, &lsof));
 
   const vector<string> cmd = {
-    lsof, "-wbnP", "-Ffn",
+    lsof, "-wnP", "-Ffn",
     "-p", std::to_string(pid),
     "-a", "-i", kind
   };
@@ -482,10 +504,10 @@ Status WaitForBind(pid_t pid, uint16_t* port,
   const auto& addresses_to_check = addresses.empty() ? kWildcard : addresses;
   for (int64_t i = 1; ; ++i) {
     for (const auto& addr : addresses_to_check) {
-      string addr_pattern = Substitute("n$0:", addr == "0.0.0.0" ? "*" : addr);
+      const string addr_pattern = Substitute("n$0:", addr == "0.0.0.0" ? "*" : addr);
       string lsof_out;
       int32_t p = -1;
-      Status s = Subprocess::Call(cmd, "", &lsof_out).AndThen([&] () {
+      const auto s = Subprocess::Call(cmd, "", &lsof_out).AndThen([&] () {
         StripTrailingNewline(&lsof_out);
         vector<string> lines = strings::Split(lsof_out, "\n");
         for (int index = 2; index < lines.size(); index += 2) {
@@ -494,18 +516,21 @@ Status WaitForBind(pid_t pid, uint16_t* port,
               !cur_line.contains("->")) {
             cur_line.remove_prefix(addr_pattern.size());
             if (!safe_strto32(cur_line.data(), cur_line.size(), &p)) {
-              return Status::RuntimeError("unexpected lsof output", lsof_out);
+              return Status::RuntimeError(Substitute(
+                  "could not parse port number in string '$0' from lsof output",
+                  string(cur_line.data(), cur_line.size())), lsof_out);
             }
 
             return Status::OK();
           }
         }
 
-        return Status::RuntimeError("unexpected lsof output", lsof_out);
+        return Status::NotFound(
+            "could not find pattern of a bound port in lsof output", lsof_out);
       });
 
       if (s.ok()) {
-        CHECK(p > 0 && p < std::numeric_limits<uint16_t>::max())
+        CHECK(p > 0 && p <= std::numeric_limits<uint16_t>::max())
             << "parsed invalid port: " << p;
         VLOG(1) << "Determined bound port: " << p;
         *port = static_cast<uint16_t>(p);
@@ -513,10 +538,15 @@ Status WaitForBind(pid_t pid, uint16_t* port,
         return Status::OK();
       }
       if (deadline < MonoTime::Now()) {
-        return s;
+        return Status::TimedOut(Substitute(
+            "process with PID $0 is not yet bound to any port at the specified "
+            "addresses; last attempt running lsof returned '$1'",
+            pid, s.ToString()));
       }
     }
-    SleepFor(MonoDelta::FromMilliseconds(i * 10));
+    auto time_left_ms = std::max<int64_t>(
+        (deadline - MonoTime::Now()).ToMilliseconds(), 0);
+    SleepFor(MonoDelta::FromMilliseconds(std::min<int64_t>(i * 10, time_left_ms)));
   }
 
   // Should not reach here.
@@ -530,7 +560,7 @@ Status WaitForBindAtPort(const vector<string>& addresses,
                          MonoDelta timeout) {
   string lsof;
   RETURN_NOT_OK(FindExecutable("lsof", {"/sbin", "/usr/sbin"}, &lsof));
-  const vector<string> cmd = { lsof, "-wbnP", "-Fpfn", "-a", "-i", kind };
+  const vector<string> cmd = { lsof, "-wnP", "-Fpfn", "-a", "-i", kind };
 
   // The '-Fpfn' flag gets lsof to output something like:
   //   p2133
@@ -562,7 +592,7 @@ Status WaitForBindAtPort(const vector<string>& addresses,
           "n$0:$1", addr == "0.0.0.0" ? "*" : addr, port);
       for (const auto& l : lines) {
         if (l.empty()) {
-          return Status::RuntimeError("unexpected lsof output", lsof_out);
+          return Status::RuntimeError("empty line in lsof output", lsof_out);
         }
         if (l[0] == 'p' || l[0] == 'f') {
           continue;
@@ -580,7 +610,9 @@ Status WaitForBindAtPort(const vector<string>& addresses,
     if (deadline < MonoTime::Now()) {
       break;
     }
-    SleepFor(MonoDelta::FromMilliseconds(i * 10));
+    auto time_left_ms = std::max<int64_t>(
+        (deadline - MonoTime::Now()).ToMilliseconds(), 0);
+    SleepFor(MonoDelta::FromMilliseconds(std::min<int64_t>(i * 10, time_left_ms)));
   }
 
   return Status::TimedOut(
@@ -628,4 +660,102 @@ Status FindHomeDir(const string& name, const string& bin_dir, string* home_dir)
   return Status::OK();
 }
 
+const unordered_map<string, string>& GetCommonWebserverEndpoints() {
+  static const unordered_map<string, string> common_endpoints = {
+      {"logs", kContentTypeTextHtml},
+      {"varz", kContentTypeTextHtml},
+      {"config", kContentTypeTextHtml},
+      {"memz", kContentTypeTextHtml},
+      {"mem-trackers", kContentTypeTextHtml},
+      {"stacks", kContentTypeTextPlain},
+      {"version", kContentTypeTextPlain},
+      {"healthz", kContentTypeTextPlain},
+      {"metrics", kContentTypeApplicationJson},
+      {"jsonmetricz", kContentTypeApplicationJson},
+      {"metrics_prometheus", kContentTypeTextPlain},
+      {"rpcz", kContentTypeApplicationJson},
+      {"startup", kContentTypeTextHtml},
+      {"pprof/cmdline", kContentTypeTextPlain},
+      {"pprof/heap", kContentTypeTextPlain},
+      {"pprof/growth", kContentTypeTextPlain},
+      {"pprof/profile", kContentTypeTextPlain},
+      {"pprof/symbol", kContentTypeTextPlain},
+      {"pprof/contention", kContentTypeTextPlain},
+      {"tracing/json/begin_monitoring", kContentTypeApplicationJson},
+      {"tracing/json/end_monitoring", kContentTypeApplicationJson},
+      {"tracing/json/capture_monitoring", kContentTypeApplicationJson},
+      {"tracing/json/get_monitoring_status", kContentTypeApplicationJson},
+      {"tracing/json/categories", kContentTypeApplicationJson},
+      {"tracing/json/begin_recording", kContentTypeApplicationJson},
+      {"tracing/json/get_buffer_percent_full", kContentTypeApplicationJson},
+      {"tracing/json/end_recording", kContentTypeApplicationJson},
+      {"tracing/json/end_recording_compressed", kContentTypeApplicationJson},
+      {"tracing/json/simple_dump", kContentTypeApplicationJson}};
+  return common_endpoints;
+}
+
+// Add necessary query params to get 200 response in tests.
+const unordered_map<string, string>& GetTServerWebserverEndpoints(const string& tablet_id) {
+  static const unordered_map<string, string> tserver_endpoints = {
+      {"scans", kContentTypeTextHtml},
+      {"tablets", kContentTypeTextHtml},
+      {Substitute("tablet?id=$0", tablet_id), kContentTypeTextHtml},
+      {"transactions", kContentTypeTextHtml},
+      {Substitute("tablet-rowsetlayout-svg?id=$0", tablet_id), kContentTypeTextHtml},
+      {Substitute("tablet-consensus-status?id=$0", tablet_id), kContentTypeTextHtml},
+      {Substitute("log-anchors?id=$0", tablet_id), kContentTypeTextHtml},
+      {"dashboards", kContentTypeTextHtml},
+      {"maintenance-manager", kContentTypeTextHtml}};
+  return tserver_endpoints;
+}
+
+// Add necessary query params to get 200 response in tests.
+const unordered_map<string, string>& GetMasterWebserverEndpoints(const string& table_id) {
+  static unordered_map<string, string> master_endpoints = {
+      {"tablet-servers", kContentTypeTextHtml},
+      {"tables", kContentTypeTextHtml},
+      {Substitute("table?id=$0", table_id), kContentTypeTextHtml},
+      {"masters", kContentTypeTextHtml},
+      {"ipki-ca-cert", kContentTypeTextPlain},
+      {"ipki-ca-cert-pem", kContentTypeTextPlain},
+      {"ipki-ca-cert-der", kContentTypeApplicationOctet},
+      {"dump-entities", kContentTypeApplicationJson}};
+  return master_endpoints;
+}
+
+void CheckPrometheusOutput(const string& prometheus_output) {
+  vector<string> lines = strings::Split(prometheus_output, "\n", strings::SkipEmpty());
+  vector<vector<string>> metric_groups;
+  // Split the lines into groups. Every group contains a help line, a type line and
+  // then lines with the actual metric values in this order.
+  for (const auto& line : lines) {
+    if (HasPrefixString(line, "# HELP")) {
+      metric_groups.push_back({line});
+    } else if (HasPrefixString(line, "# TYPE")) {
+      metric_groups.back().push_back(line);
+    } else {
+      metric_groups.back().push_back(line);
+    }
+  }
+
+  std::unordered_set<string> metric_names;
+  for (const auto& group : metric_groups) {
+    ASSERT_GE(group.size(), 3);
+    ASSERT_STR_MATCHES(group[0], "^# HELP ");
+    ASSERT_STR_MATCHES(group[1], "^# TYPE ");
+    vector<string> help_line_split(strings::Split(group[0], " "));
+    vector<string> type_line_split(strings::Split(group[1], " "));
+    ASSERT_GE(help_line_split.size(), 3);
+    ASSERT_GE(type_line_split.size(), 3);
+    string name_from_help_line = help_line_split[2];
+    string name_from_type_line = type_line_split[2];
+    ASSERT_EQ(name_from_type_line, name_from_help_line);
+    ASSERT_TRUE(metric_names.emplace(name_from_help_line).second)
+        << "Duplicate metric: " << name_from_help_line;
+    for (int i = 2; i < group.size(); i++) {
+      ASSERT_TRUE(HasPrefixString(group[i], name_from_help_line))
+          << "Every line should start with the expected metric name: " << name_from_help_line;
+    }
+  }
+}
 } // namespace kudu
