@@ -26,6 +26,7 @@
 #include <memory>
 #include <string>
 #include <vector>
+#include <unordered_map>
 
 #include <gtest/gtest.h>
 
@@ -34,8 +35,7 @@
 
 #define SKIP_IF_SLOW_NOT_ALLOWED() do { \
   if (!AllowSlowTests()) { \
-    LOG(WARNING) << "test is skipped; set KUDU_ALLOW_SLOW_TESTS=1 to run"; \
-    GTEST_SKIP(); \
+    GTEST_SKIP() << "test is skipped; set KUDU_ALLOW_SLOW_TESTS=1 to run"; \
   } \
 } while (0)
 
@@ -63,9 +63,9 @@ class KuduTest : public ::testing::Test {
  public:
   KuduTest();
 
-  virtual ~KuduTest();
+  ~KuduTest() override;
 
-  virtual void SetUp() OVERRIDE;
+  void SetUp() override;
 
   // Tests assume that they run with no outside-provided kerberos credentials, and if the
   // user happened to have some credentials available they might fail due to being already
@@ -73,10 +73,14 @@ class KuduTest : public ::testing::Test {
   // variables so that we don't pick up the user's credentials.
   static void OverrideKrb5Environment();
 
-  // Returns the encryption key, IV, and version used by the test.
-  static void GetEncryptionKey(std::string* key, std::string* iv, std::string* version);
+  // Returns the encryption tenant name, tenant id, key, IV, and version used by the test.
+  static void GetEncryptionKey(std::string* name, std::string* id,
+                               std::string* key, std::string* iv, std::string* version);
 
  protected:
+  // Sets the flags to enable encryption if 'enable_encryption' is true.
+  static void SetEncryptionFlags(bool enable_encryption);
+
   // Returns absolute path based on a unit test-specific work directory, given
   // a relative path. Useful for writing test files that should be deleted after
   // the test ends.
@@ -88,9 +92,6 @@ class KuduTest : public ::testing::Test {
   // (and the flags reset) before test_dir_ is deleted.
   std::unique_ptr<google::FlagSaver> flag_saver_;
 
-  // Sets the flags to enable encryption if 'enable_encryption' is true.
-  void SetEncryptionFlags(bool enable_encryption);
-
   std::string test_dir_;
 };
 
@@ -201,5 +202,22 @@ Status FindHomeDir(const std::string& name,
                    const std::string& bin_dir,
                    std::string* home_dir) WARN_UNUSED_RESULT;
 
+// Contains the endpoints which are to be found on Master and TServer as well.
+// Key: endpoint name, value: content-type header for that particular endpoint.
+const std::unordered_map<std::string, std::string>& GetCommonWebserverEndpoints();
+
+// Contains the endpoints which are to be found on TServer only.
+// Key: endpoint name, value: content-type header for that particular endpoint.
+const std::unordered_map<std::string, std::string>& GetTServerWebserverEndpoints(
+    const std::string& tablet_id);
+
+// Contains the endpoints which are to be found on Master only.
+// Key: endpoint name, value: content-type header for that particular endpoint.
+const std::unordered_map<std::string, std::string>& GetMasterWebserverEndpoints(
+    const std::string& table_id);
+
+// Prometheus metrics output sanity check. This check is used in both the tablet_server-test
+// and the master-test, so it is placed here.
+void CheckPrometheusOutput(const std::string& prometheus_output);
 } // namespace kudu
 #endif
