@@ -31,8 +31,9 @@
 #include <cerrno>
 #include <cstring>
 #include <memory>
-#include <mutex>
+#include <shared_mutex>
 #include <sstream>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -66,6 +67,7 @@
 using std::atomic;
 using std::ostringstream;
 using std::pair;
+using std::shared_lock;
 using std::shared_ptr;
 using std::string;
 using std::vector;
@@ -335,7 +337,7 @@ Status ThreadMgr::StartInstrumentation(const scoped_refptr<MetricEntity>& metric
                                       WebCallbackRegistry::WebResponse* resp) {
           this->ThreadPathHandler(req, resp);
         },
-        /* is_styled= */ true,
+        StyleMode::STYLED,
         /* is_on_nav_bar= */ true);
   }
   return Status::OK();
@@ -371,7 +373,7 @@ void ThreadMgr::AddThread(const pthread_t& pthread_id, const string& name,
     //   global static container, but bind the container with the life cycle
     //   of some top-level object that uses the ThreadMgr as a singleton.
     {
-      std::lock_guard<decltype(lock_)> l(lock_);
+      std::lock_guard l(lock_);
       thread_categories_[category][pthread_id] = ThreadDescriptor(category, name, tid);
     }
     ++threads_running_metric_;
@@ -385,7 +387,7 @@ void ThreadMgr::RemoveThread(const pthread_t& pthread_id, const string& category
   ANNOTATE_IGNORE_SYNC_BEGIN();
   ANNOTATE_IGNORE_READS_AND_WRITES_BEGIN();
   {
-    std::lock_guard<decltype(lock_)> l(lock_);
+    std::lock_guard l(lock_);
     auto& threads = FindOrDie(thread_categories_, category);
     auto num_erased = threads.erase(pthread_id);
     CHECK_EQ(1, num_erased);
@@ -572,6 +574,11 @@ Thread::~Thread() {
   }
 }
 
+void Thread::Join() {
+  WARN_NOT_OK(ThreadJoiner(this).Join(),
+              Substitute("$0 failed joining", ToString()));
+}
+
 string Thread::ToString() const {
   return Substitute("Thread $0 (name: \"$1\", category: \"$2\")", tid(), name_, category_);
 }
