@@ -89,11 +89,21 @@ fixup_libtool() {
 }
 
 build_cmake() {
-  # Use system curl if it is available: this shortens build times a bit.
-  local use_system_curl=""
-  if pkg-config --exists libcurl; then
-    use_system_curl="--system-curl"
-  fi
+  # It would be nice to use system curl if it is available to shorten build
+  # times, but with curl v8.13.0 and newer some of the CURL_NETRC_OPTION
+  # enumerators were made constants of type long [1], and compiling
+  # cmake v3.25.3 fails when trying to assign those constants to strings.
+  #
+  # [1] https://github.com/curl/curl/commit/2ec00372a
+  local use_system_curl="--no-system-curl"
+
+  # TODO(aserbin): switch to using system curl when migrating to newer
+  #                cmake version that's able to build with curl v8.13.0
+  #                and newer
+  #if pkg-config --exists libcurl; then
+  #  use_system_curl="--system-curl"
+  #fi
+
   CMAKE_BDIR=$TP_BUILD_DIR/$CMAKE_NAME$MODE_SUFFIX
   mkdir -p $CMAKE_BDIR
   pushd $CMAKE_BDIR
@@ -212,6 +222,10 @@ build_llvm() {
       CLANG_TOOL_CLANG_RENAME_BUILD \
       CLANG_TOOL_DIAGTOOL_BUILD \
       COMPILER_RT_BUILD_LIBFUZZER \
+      COMPILER_RT_INCLUDE_TESTS \
+      LLVM_BUILD_BENCHMARKS \
+      LLVM_ENABLE_OCAMLDOC \
+      LLVM_INCLUDE_BENCHMARKS \
       LLVM_INCLUDE_GO_TESTS \
       LLVM_POLLY_BUILD \
       LLVM_TOOL_BUGPOINT_BUILD \
@@ -341,7 +355,7 @@ build_llvm() {
     -DLLVM_INCLUDE_EXAMPLES=OFF \
     -DLLVM_INCLUDE_TESTS=OFF \
     -DLLVM_INCLUDE_UTILS=OFF \
-    -DLLVM_TARGETS_TO_BUILD="X86;AArch64;PowerPC" \
+    -DLLVM_TARGETS_TO_BUILD=host \
     -DLLVM_ENABLE_RTTI=ON \
     -DCMAKE_CXX_FLAGS="$CLANG_CXXFLAGS" \
     -DCMAKE_EXE_LINKER_FLAGS="$CLANG_LDFLAGS" \
@@ -567,21 +581,30 @@ build_protobuf() {
 }
 
 build_snappy() {
-  SNAPPY_BDIR=$TP_BUILD_DIR/$SNAPPY_NAME$MODE_SUFFIX
-  mkdir -p $SNAPPY_BDIR
-  pushd $SNAPPY_BDIR
-  rm -Rf CMakeCache.txt CMakeFiles/
-  CFLAGS="$EXTRA_CFLAGS -fPIC" \
-    CXXFLAGS="$EXTRA_CXXFLAGS -fPIC" \
-    cmake \
-    -DCMAKE_BUILD_TYPE=release \
-    -DBUILD_STATIC_LIBS=ON \
-    -DCMAKE_INSTALL_PREFIX:PATH=$PREFIX \
-    -DSNAPPY_BUILD_TESTS=OFF \
-    $EXTRA_CMAKE_FLAGS \
-    $SNAPPY_SOURCE
-  ${NINJA:-make} -j$PARALLEL $EXTRA_MAKEFLAGS install
-  popd
+  SNAPPY_SHARED_BDIR=$TP_BUILD_DIR/$SNAPPY_NAME.shared$MODE_SUFFIX
+  SNAPPY_STATIC_BDIR=$TP_BUILD_DIR/$SNAPPY_NAME.static$MODE_SUFFIX
+  for SHARED in ON OFF; do
+    if [ $SHARED = "ON" ]; then
+      SNAPPY_BDIR=$SNAPPY_SHARED_BDIR
+    else
+      SNAPPY_BDIR=$SNAPPY_STATIC_BDIR
+    fi
+    mkdir -p $SNAPPY_BDIR
+    pushd $SNAPPY_BDIR
+    rm -Rf CMakeCache.txt CMakeFiles/
+    CFLAGS="$EXTRA_CFLAGS -fPIC" \
+      CXXFLAGS="$EXTRA_CXXFLAGS -fPIC" \
+      LDFLAGS="$EXTRA_LDFLAGS" \
+      cmake \
+      -DCMAKE_BUILD_TYPE=release \
+      -DBUILD_SHARED_LIBS=$SHARED \
+      -DCMAKE_INSTALL_PREFIX:PATH=$PREFIX \
+      -DSNAPPY_BUILD_TESTS=OFF \
+      $EXTRA_CMAKE_FLAGS \
+      $SNAPPY_SOURCE
+    ${NINJA:-make} -j$PARALLEL $EXTRA_MAKEFLAGS install
+    popd
+  done
 }
 
 build_zlib() {
@@ -739,9 +762,7 @@ build_curl() {
   # In the scope of using libcurl in Kudu tests and other simple scenarios,
   # not so much functionality is needed as of now, so configure for a fairly
   # minimal install. For testing, we need HTTP/HTTPS with GSSAPI support
-  # (GSSAPI is needed for SPNEGO testing). Also, cookies might be useful
-  # to test the new functionality introduced in Impala's embedded Web server
-  # recently if Kudu is to pick up the new functionality as well.
+  # (GSSAPI is needed for SPNEGO testing).
   #
   # NOTE: curl shows a message asking for CPPFLAGS to be used for include
   #       directories, not CFLAGS.
@@ -753,32 +774,64 @@ build_curl() {
     $CURL_SOURCE/configure \
     --prefix=$PREFIX \
     --disable-alt-svc \
+    --disable-aws \
+    --disable-cookies \
+    --disable-dateparse \
     --disable-dict \
+    --disable-docs \
     --disable-doh \
     --disable-file \
+    --disable-form-api  \
     --disable-ftp \
+    --disable-headers-api \
+    --disable-hsts \
+    --disable-httpsrr \
     --disable-gopher \
     --disable-imap \
+    --disable-ipfs \
     --disable-ipv6 \
     --disable-ldap \
     --disable-ldaps \
     --disable-libcurl-option \
     --disable-manual \
     --disable-mime \
+    --disable-mqtt \
     --disable-netrc \
+    --disable-ntlm \
     --disable-pop3 \
     --disable-progress-meter \
     --disable-rtsp \
+    --disable-sha512-256 \
     --disable-smb \
     --disable-smtp \
+    --disable-sspi \
     --disable-telnet \
     --disable-tftp \
+    --disable-tls-srp \
+    --disable-unix-sockets \
+    --disable-websockets \
+    --enable-basic-auth \
+    --enable-bearer-auth \
+    --enable-digest-auth \
+    --enable-http-auth \
+    --enable-kerberos-auth \
+    --enable-negotiate-auth \
+    --without-apple-idn \
     --without-brotli \
+    --without-fish-functions-dir \
     --without-libidn2 \
     --without-libpsl \
     --without-librtmp \
+    --without-libssh \
     --without-libssh2 \
+    --without-libuv \
+    --without-msh3 \
     --without-nghttp2 \
+    --without-nghttp3 \
+    --without-ngtcp2 \
+    --without-openssl-quic \
+    --without-quiche \
+    --without-zsh-functions-dir \
     --without-zstd \
     --with-gssapi \
     --with-openssl
@@ -966,26 +1019,30 @@ build_thrift() {
   # Configure for a very minimal install - only the C++ client libraries are needed.
   # Thrift requires C++11 when compiled on Linux against libc++ (see cxxfunctional.h).
   CFLAGS="$EXTRA_CFLAGS" \
-    CXXFLAGS="$EXTRA_CXXFLAGS -std=c++11" \
+    CFLAGS="$EXTRA_CFLAGS -fPIC" \
+    CXXFLAGS="$EXTRA_CXXFLAGS -fPIC -std=c++17" \
     LDFLAGS="$EXTRA_LDFLAGS" \
     LIBS="$EXTRA_LIBS" \
     cmake \
     -DBOOST_ROOT=$PREFIX \
+    -DBUILD_AS3=OFF \
     -DBUILD_C_GLIB=OFF \
     -DBUILD_COMPILER=ON \
     -DBUILD_CPP=ON \
-    -DBUILD_EXAMPLES=OFF \
-    -DBUILD_HASKELL=OFF \
     -DBUILD_JAVA=OFF \
+    -DBUILD_JAVASCRIPT=OFF \
+    -DBUILD_NODEJS=OFF \
     -DBUILD_PYTHON=OFF \
+    -DBUILD_SHARED_LIBS=OFF \
     -DBUILD_TESTING=OFF \
     -DBUILD_TUTORIALS=OFF \
     -DCMAKE_BUILD_TYPE=Release \
     -DCMAKE_INSTALL_PREFIX=$PREFIX \
-    -DWITH_BOOSTTHREADS=OFF \
     -DWITH_LIBEVENT=OFF \
     -DWITH_OPENSSL=OFF \
     -DWITH_PLUGIN=OFF \
+    -DWITH_QT5=OFF \
+    -DWITH_ZLIB=OFF \
     $EXTRA_CMAKE_FLAGS \
     $THRIFT_SOURCE
 
@@ -1117,11 +1174,12 @@ build_postgres() {
   mkdir -p $POSTGRES_BDIR
   pushd $POSTGRES_BDIR
 
-  # We don't need readline and zlib, so let's simplify build.
+  # We don't need readline, zlib and icu, so let's simplify build.
   CFLAGS="$EXTRA_CFLAGS" \
     LDFLAGS="$EXTRA_LDFLAGS" \
     $POSTGRES_SOURCE/configure \
     --prefix=$PREFIX \
+    --without-icu \
     --without-readline \
     --without-zlib
 
@@ -1186,6 +1244,10 @@ build_rocksdb() {
     CXXFLAGS="$EXTRA_CXXFLAGS -fPIC" \
     cmake \
     -DROCKSDB_BUILD_SHARED=ON \
+    -DPORTABLE=$PORTABLE \
+    -DWITH_SNAPPY=ON \
+    -Dsnappy_ROOT_DIR=$PREFIX \
+    -DUSE_RTTI=ON \
     -DFAIL_ON_WARNINGS=OFF \
     -DWITH_BENCHMARK_TOOLS=OFF \
     -DWITH_CORE_TOOLS=OFF \
@@ -1193,10 +1255,9 @@ build_rocksdb() {
     -DWITH_TRACE_TOOLS=OFF \
     -DWITH_JNI=OFF \
     -DWITH_LIBURING=OFF \
-    -DWITH_LZ4=ON \
     -DWITH_ZSTD=OFF \
-    -DWITH_SNAPPY=ON \
     -DWITH_BZ2=OFF \
+    -DWITH_LZ4=OFF \
     -DWITH_TESTS=OFF \
     -DWITH_GFLAGS=OFF \
     -DCMAKE_BUILD_TYPE=release \
