@@ -260,21 +260,22 @@ fetch_and_patch \
  $ZLIB_SOURCE \
  $ZLIB_PATCHLEVEL
 
-LIBEV_PATCHLEVEL=1
+LIBEV_PATCHLEVEL=0
 fetch_and_patch \
  libev-${LIBEV_VERSION}.tar.gz \
  $LIBEV_SOURCE \
- $LIBEV_PATCHLEVEL \
- "patch -p1 < $TP_DIR/patches/libev-c17.patch"
+ $LIBEV_PATCHLEVEL
 
-RAPIDJSON_PATCHLEVEL=3
+RAPIDJSON_PATCHLEVEL=5
 fetch_and_patch \
  rapidjson-${RAPIDJSON_VERSION}.zip \
  $RAPIDJSON_SOURCE \
  $RAPIDJSON_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/rapidjson-fix-signed-unsigned-conversion-error.patch" \
  "patch -p1 < $TP_DIR/patches/rapidjson-assertions-for-clang-warnings.patch" \
- "patch -p1 < $TP_DIR/patches/rapidjson-avoid-pointer-arithmetic-on-null-pointer.patch"
+ "patch -p1 < $TP_DIR/patches/rapidjson-avoid-pointer-arithmetic-on-null-pointer.patch" \
+ "patch -p1 < $TP_DIR/patches/rapidjson-document-assignment-operator-00.patch" \
+ "patch -p1 < $TP_DIR/patches/rapidjson-document-assignment-operator-01.patch"
 
 SQUEASEL_PATCHLEVEL=1
 fetch_and_patch \
@@ -302,13 +303,14 @@ fetch_and_patch \
  $GCOVR_SOURCE \
  $GCOVR_PATCHLEVEL
 
-CURL_PATCHLEVEL=2
+CURL_PATCHLEVEL=3
 fetch_and_patch \
  curl-${CURL_VERSION}.tar.gz \
  $CURL_SOURCE \
  $CURL_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/curl-custom-openssl-library.patch" \
  "patch -p1 < $TP_DIR/patches/curl-handle-openssl-errors.patch" \
+ "patch -p1 < $TP_DIR/patches/curl-eventfd-double-close.patch" \
  "autoreconf -fvi"
 
 CRCUTIL_PATCHLEVEL=0
@@ -330,16 +332,26 @@ fetch_and_patch \
  $PYTHON_SOURCE \
  $PYTHON_PATCHLEVEL
 
-LLVM_PATCHLEVEL=5
+LLVM_PATCHLEVEL=8
 fetch_and_patch \
  llvm-${LLVM_VERSION}-iwyu-${IWYU_VERSION}.src.tar.gz \
  $LLVM_SOURCE \
  $LLVM_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/llvm-add-iwyu.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-iwyu-718e69875.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-iwyu-0de60d8a2.patch" \
  "patch -d projects -p1 < $TP_DIR/patches/llvm-remove-cyclades-inclusion-in-sanitizer.patch" \
  "patch -p2 < $TP_DIR/patches/llvm-fix-missing-include.patch" \
  "patch -d projects -p1 < $TP_DIR/patches/llvm-Sanitizer-built-against-glibc-2_34-doesnt-work.patch" \
- "patch -d tools -p1 < $TP_DIR/patches/llvm-ignore-flto-values.patch"
+ "patch -d tools -p1 < $TP_DIR/patches/llvm-ignore-flto-values.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-nostdinc-nostdlib-00.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-nostdinc-nostdlib-01.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-nostdinc-nostdlib-02.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-include-llvm-support-signals.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-is-convertible-00.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-is-convertible-01.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-chrono-duration-00.patch" \
+ "patch -p1 < $TP_DIR/patches/llvm-chrono-duration-01.patch"
 
 LZ4_PATCHLEVEL=0
 fetch_and_patch \
@@ -366,14 +378,18 @@ fetch_and_patch \
  $BOOST_PATCHLEVEL \
  "patch -p0 < $TP_DIR/patches/boost-bootstrap.patch"
 
-BREAKPAD_PATCHLEVEL=3
+BREAKPAD_PATCHLEVEL=7
 fetch_and_patch \
  breakpad-${BREAKPAD_VERSION}.tar.gz \
  $BREAKPAD_SOURCE \
  $BREAKPAD_PATCHLEVEL \
  "patch -p1 < $TP_DIR/patches/breakpad-add-basic-support-for-dwz-dwarf-extension.patch" \
  "patch -p1 < $TP_DIR/patches/breakpad-syscall-rsp-clobber-fix.patch" \
- "patch -p0 < $TP_DIR/patches/breakpad-SIGSTKSZ-error.patch"
+ "patch -p1 < $TP_DIR/patches/breakpad-SIGSTKSZ-error.patch" \
+ "patch -p1 < $TP_DIR/patches/breakpad-fclose.patch" \
+ "patch -p1 < $TP_DIR/patches/breakpad-fread.patch" \
+ "patch -p1 < $TP_DIR/patches/breakpad-minidump-descriptor.patch" \
+ "patch -p1 < $TP_DIR/patches/breakpad-guid-creator.patch"
 
 SPARSEHASH_PATCHLEVEL=3
 fetch_and_patch \
@@ -421,12 +437,11 @@ fetch_and_patch \
  $YAML_SOURCE \
  $YAML_PATCHLEVEL
 
-CHRONY_PATCHLEVEL=2
+CHRONY_PATCHLEVEL=1
 fetch_and_patch \
  $CHRONY_NAME.tar.gz \
  $CHRONY_SOURCE \
  $CHRONY_PATCHLEVEL \
- "patch -p1 < $TP_DIR/patches/chrony-no-superuser.patch" \
  "patch -p1 < $TP_DIR/patches/chrony-reuseport.patch"
 
 GUMBO_PARSER_PATCHLEVEL=1
@@ -481,21 +496,22 @@ fetch_and_patch \
 
 JWT_CPP_PATCHLEVEL=0
 fetch_and_patch \
-  $JWT_CPP_NAME.tar.gz \
-  $JWT_CPP_SOURCE \
-  $JWT_CPP_PATCHLEVEL
+ $JWT_CPP_NAME.tar.gz \
+ $JWT_CPP_SOURCE \
+ $JWT_CPP_PATCHLEVEL
 
 RANGER_KMS_PATCHLEVEL=0
 fetch_and_patch \
-  $RANGER_KMS_NAME.tar.gz \
-  $RANGER_KMS_SOURCE \
-  $RANGER_KMS_PATCHLEVEL
+ $RANGER_KMS_NAME.tar.gz \
+ $RANGER_KMS_SOURCE \
+ $RANGER_KMS_PATCHLEVEL
 
-ROCKSDB_PATCHLEVEL=0
+ROCKSDB_PATCHLEVEL=1
 fetch_and_patch \
-  $ROCKSDB_NAME.tar.gz \
-  $ROCKSDB_SOURCE \
-  $ROCKSDB_PATCHLEVEL
+ $ROCKSDB_NAME.tar.gz \
+ $ROCKSDB_SOURCE \
+ $ROCKSDB_PATCHLEVEL \
+ "patch -p1 < $TP_DIR/patches/rocksdb-gcc13.patch"
 
 echo "---------------"
 echo "Thirdparty dependencies downloaded successfully"
